'
' ####################
' #####  PROLOG  #####
' ####################
'
' Bounce - Creates a new thread each time the letter 'a' is typed.
'  Each thread bounces a happy face of a different color around the screen.
'  All threads are terminated when the letter 'Q' is entered.
'
VERSION "0.0001"
CONSOLE
'
	IMPORT "xst"				' Standard library : required by most programs
	IMPORT "xsx"				' Extended standard library
	IMPORT "xio"				' Console input/ouput library

'	IMPORT "xst_s.lib"
' IMPORT "xsx_s.lib"
' IMPORT "xio_s.lib"

'	IMPORT "gdi32"			' gdi32.dll
'	IMPORT "user32"		  ' user32.dll
	IMPORT "kernel32"	  ' kernel32.dll
	IMPORT "msvcrt"		  ' msvcrt.dll

'
DECLARE FUNCTION Entry ()
DECLARE FUNCTION ShutDown ()
DECLARE FUNCTION KbdFunc ()
DECLARE FUNCTION BounceProc (myID)
DECLARE FUNCTION WriteTitle (threadNum)
DECLARE FUNCTION ClearScreen ()

$$MAX_THREADS = 32
'
'
' ######################
' #####  Entry ()  #####
' ######################
'
FUNCTION Entry ()

	SHARED CONSOLE_SCREEN_BUFFER_INFO csbiInfo
	SHARED hConsoleOut
	SHARED hRunMutex, hScreenMutex, threadNr

	' Get display screen information & clear the screen.
	hConsoleOut = GetStdHandle ($$STD_OUTPUT_HANDLE)
	XioSetConsoleBufferSize (hConsoleOut, 80, 25)
	XioSetCursorType (hConsoleOut, $$NOCURSOR)

	GetConsoleScreenBufferInfo (hConsoleOut, &csbiInfo)
	ClearScreen ()
	WriteTitle (0)

	' Create the mutexes and reset thread count.
	hScreenMutex = CreateMutexA (NULL, 0, NULL)
	hRunMutex = CreateMutexA (NULL, 1, NULL)
	threadNr = 0

	' Start waiting for keyboard input to dispatch threads or exit.
	KbdFunc ()

	' All threads done. Clean up handles.
	CloseHandle (hScreenMutex)
	CloseHandle (hRunMutex)
	CloseHandle (hConsoleOut)

'	a$ = INLINE$ ("Press Enter to quit >")

END FUNCTION

FUNCTION ShutDown ()

	SHARED hRunMutex, hScreenMutex, threadNr

	DO WHILE (threadNr > 0)
		' Tell thread to die and record its death.
		ReleaseMutex (hRunMutex)
		DEC threadNr
	LOOP
	' Clean up display when done
	WaitForSingleObject (hScreenMutex, $$INFINITE)
	ClearScreen ()

END FUNCTION

FUNCTION KbdFunc ()

	SHARED threadNr

	DO
		keyInfo = _getch ()
		IF (tolower (keyInfo) == 'a' && threadNr < $$MAX_THREADS) THEN
			INC threadNr
			_beginthread (&BounceProc (), 0, threadNr)
			WriteTitle (threadNr)
		END IF
	LOOP WHILE (tolower (keyInfo) <> 'q')

	ShutDown ()

END FUNCTION

FUNCTION BounceProc (myID)

	SHARED CONSOLE_SCREEN_BUFFER_INFO csbiInfo
	SHARED hConsoleOut
	SHARED hRunMutex, hScreenMutex, threadNr

	COORD Coords, Delta, Old
	UBYTE MyCell, OldCell

	BlankCell = 0x20

	' Generate update increments and initial display coordinates.
	Coords.x = XstRandomRange (0, csbiInfo.dwSize.x - 1)
	Coords.y = XstRandomRange (0, csbiInfo.dwSize.y - 1)
	Delta.x = XstRandomRange (-3, 3)
	Delta.y = XstRandomRange (-3, 3)

	' Set up "happy face" & generate color attribute from thread number.
	IF (myID > 16) THEN
		MyCell = 0x01		' outline face
	ELSE
		MyCell = 0x02		' solid face
	END IF
	MyAttrib = myID & 0x0F		' force black background

	DO

		' Wait for display to be available, then lock it.
		WaitForSingleObject (hScreenMutex, $$INFINITE)

		' If we still occupy the old screen position, blank it out.
		old = MAKELONG (Old.x, Old.y)
		ReadConsoleOutputCharacterA (hConsoleOut, &OldCell, 1, old, &Dummy)
		ReadConsoleOutputAttribute (hConsoleOut, &OldAttrib, 1, old, &Dummy)
		IF ((OldCell == MyCell) && (OldAttrib == MyAttrib)) THEN
			WriteConsoleOutputCharacterA (hConsoleOut, &BlankCell, 1, old, &Dummy)
		END IF

		' Draw new face, then clear screen lock
		coords = MAKELONG (Coords.x, Coords.y)
		WriteConsoleOutputCharacterA (hConsoleOut, &MyCell, 1, coords, &Dummy)
		WriteConsoleOutputAttribute (hConsoleOut, &MyAttrib, 1, coords, &Dummy)
		ReleaseMutex (hScreenMutex)

		' Increment the coordinates for next placement of the block.
		Old.x = Coords.x
		Old.y = Coords.y
		Coords.x = Coords.x + Delta.x
		Coords.y = Coords.y + Delta.y

		' If we are about to go off the screen, reverse direction
		IF (Coords.x < 0 || Coords.x >= csbiInfo.dwSize.x) THEN
			Delta.x = -Delta.x
			Beep (400, 50)
		END IF
		IF (Coords.y < 0 || Coords.y > csbiInfo.dwSize.y) THEN
			Delta.y = -Delta.y
			Beep (600, 50)
		END IF

		' Repeat while RunMutex is still taken.
	LOOP WHILE (WaitForSingleObject (hRunMutex, 75) == $$WAIT_TIMEOUT)


END FUNCTION

FUNCTION WriteTitle (threadNum)

  msg$ = "Threads running: " + STRING$(threadNum) + ".  Press 'A' to start a thread, 'Q' to quit."
  SetConsoleTitleA (&msg$)

END FUNCTION

FUNCTION ClearScreen ()

	SHARED CONSOLE_SCREEN_BUFFER_INFO csbiInfo
	SHARED hConsoleOut

	home = 0
  FillConsoleOutputCharacterA ( hConsoleOut, ' ', csbiInfo.dwSize.x * csbiInfo.dwSize.y, home, &dummy )

END FUNCTION
END PROGRAM