'
' ####################
' #####  PROLOG  #####
' ####################
'
' Genetic algorithm to determine best integer constant multiplier expansion
' The GA optimizes for 4 operations using shl, sub, add, and lea.
'
VERSION "0.0003"
CONSOLE
'
	IMPORT "xst"				' Standard library : required by most programs
	IMPORT "xsx"				' Extended standard library
	IMPORT "xio"				' Console input/ouput library

'	IMPORT "xst_s.lib"
' IMPORT "xsx_s.lib"
' IMPORT "xio_s.lib"

'	IMPORT "msvcrt"		  ' msvcrt.dll

' OPS
$$NOOP = 0
$$SHL1 = 1
$$SHL2 = 2
$$SHL3 = 3
$$SHL4 = 4
$$SHL5 = 5
$$SHL6 = 6
$$SHL7 = 7
$$SHL8 = 8
$$SHL9 = 9
$$SHL10 = 10
$$SHL11 = 11
$$SHL12 = 12
$$SHL13 = 13
$$SHL14 = 14
$$SHL15 = 15
$$SHL16 = 16
$$SHL17 = 17
$$SHL18 = 18
$$SHL19 = 19
$$SHL20 = 20
$$SHL21 = 21
$$SHL22 = 22
$$SHL23 = 23
$$SHL24 = 24
$$SHL25 = 25
$$SHL26 = 26
$$SHL27 = 27
$$SHL28 = 28
$$SHL29 = 29
$$SHL30 = 30
$$SHL31 = 31

$$NOOP2 = 32

' 33 - 63 are SHL
$$LEA2 = 64
$$LEA4 = 65
$$LEA8 = 66
$$LEA2Y = 67
$$LEA4Y = 68
$$LEA8Y = 69
$$ADD = 70
$$SUB = 71
$$ADDXY = 72
$$SUBXY = 73

$$MAXOP = 73

$$MAXXLONG = 2147483647

$$CHROMES = 3  ' upper bound of an array of 4 chromosomes

TYPE GENE
	XLONG .gene[$$CHROMES]
	ULONG .fitness
	XLONG .alive
END TYPE

TYPE POPULATION
	GENE .g
END TYPE

TYPE BESTFIT
	GENE .bestFitGene
END TYPE

TYPE OPDATA
	XLONG .fitnessValue
	STRING * 32 .opStr
END TYPE

DECLARE FUNCTION Entry ()
DECLARE FUNCTION Initialize ()
DECLARE FUNCTION Genetic (nPop, nGen, DOUBLE rCr, DOUBLE rMut, ULONG constant)
DECLARE FUNCTION Fitness (GENE g, ULONG constant)
DECLARE FUNCTION GENE Crossover (GENE a, GENE b)
DECLARE FUNCTION Mutate (GENE g)
DECLARE FUNCTION PrintGene (GENE g, fMutant, fKilled)
'
'
' ######################
' #####  Entry ()  #####
' ######################
'
FUNCTION Entry ()

	SHARED nPop   			' size of parent population
	SHARED nGen   			' number of generations
	SHARED DOUBLE rCr   ' rate of crossover (0.0 - 1.0)
	SHARED DOUBLE rMut	' rate of mutation (0.0 - 1.0)
	ULONG constant      ' multiplier to replace with shl, add, sub, and lea instructions
	SHARED BESTFIT bestfit[] ' results of running ga (best gene from each generation)
	SHARED fPrint				' flag to print details of each generation

	GENE g

	hStdOut = XioGetStdOut ()
	XioSetConsoleBufferSize (hStdOut, 0, 2000)

' Crossover: Crossover randomly selects and combines cells from
' two parents genes to form the gene of the offspring.

' Selective Breeding: Parents are selected to breed with
' probability proportional to their fitnesses.

' Mutation: Mutation consists of randomly changing a single
' op within a gene in a newly generated child.

' Each run of the GA will provide a unique set of solutions
' since the starting set of genes is generated by random

	' initialize op data
'	Initialize ()

	fPrint = 0		' flag to print details of each generation

	' run genetic algorithm

	nPop 			= 600				' initial population size
	nGen 			= 600				' number of generations
	rCr 			= 0.35			' crossover factor
	rMut 			= 0.20			' mutation factor
	constant 	= 517				' integer constant multiplier
	Genetic (nPop, nGen, rCr, rMut, constant)

	' Print list of the fittest gene from each generation
	' Note that several different genes may have the same fitness in each generation.
	' A fitness of 4 or less means that a correct solution has been found
	' with the fitness value equaling the number of operations required.

	upp = UBOUND (bestfit[])
	? "Generation Gene        Fitness"
	? "---------- ----------- ----------"
	f$ = "##########"
	FOR i = 0 TO upp
		g = bestfit[i].bestFitGene
		? FORMAT$(f$, i);
		PrintGene (@g, 0, 0)
	NEXT i

  a$ = INLINE$ ("Press Enter to quit >")

END FUNCTION

'
' #####################
' #####  Genetic  #####
' #####################
'
'
'
FUNCTION Genetic (nPop, nGen, DOUBLE rCr, DOUBLE rMut, ULONG constant)

	POPULATION population[]
	ULONG seed
	GENE g, ga, gb
	SHARED BESTFIT bestfit[]
	SHARED fPrint

	DIM bestfit[nGen-1]
	DIM parents[5]
	DIM fitness[5]

'	fPrint = 0		' flag to print details of each generation

  psize = ((1.0 + rCr) * nPop)	' population size
  nkids = psize - nPop					' number of children

	f1$ = "#####"
	f2$ = "########"
	f3$ = "######"

  ' allocate space for the population
  DIM population[psize-1]

	' initialize rng
	' uncomment this if the same seed is wanted each time
'	XstRandomSeed (123456789)

	' randomly generate the intial population and get fitness of each gene

	IF fPrint THEN
		? "***** Initial population *****"
		? "Gene         Fitness"
		? "------------ ----------"
	END IF

	FOR i = 0 TO psize-1
		IF (i < nPop ) THEN

			' randomly select from all ops to create each gene
			FOR j = 0 TO $$CHROMES
				population[i].g.gene[j] = XstRandom() MOD ($$MAXOP+1)
			NEXT j

			g = population[i].g
			Fitness (@g, constant)
			population[i].g = g

			population[i].g.alive = $$TRUE

			' print initial population
			IF fPrint THEN
				g = population[i].g
				PrintGene (@g, 0, 0)
			END IF

		ELSE
			population[i].g.alive = $$FALSE
			population[i].g.fitness = $$MAXXLONG
		END IF
	NEXT i

  ' begin genetic loop
  FOR current_gen = 0 TO nGen-1
		IF fPrint THEN
			?
			? "* Starting Generation"; current_gen+1
			? "Child  Parent A  Parent B  Gene        Fitness"
			? "-----  --------  --------  ----------- ----------"
		END IF

    FOR i = 0 TO nkids - 1
      ' Select unique, living parents a and b
			' Let the tournament begin. Randomly select 6 possible
			' parents from among the living, and sort them by fitness.
			' The two fittest become parents.
			a = -1
			b = -1
			count = 0
			FOR k = 0 TO 5
				parents[k] = -1
			NEXT k
			DO
				DO
					p = XstRandom() MOD ULONG((psize+1))
					IF (population[p].g.alive) THEN
						' is it unique
						FOR k = 0 TO 5
							IF p = parents[k] THEN DO LOOP  'nope, find another one
						NEXT k
						parents[count] = p
						fitness[count] = population[p].g.fitness
						INC count
						EXIT DO
					END IF
				LOOP
			LOOP UNTIL count = 6

			' sort the parents based on fitness (lower values are more fit)
			DIM order[0]
			XstQuickSort (@fitness[], @order[], 0, UBOUND(parents[]), $$SortIncreasing)

			' so now get the parents
			a = parents[order[0]]
			b = parents[order[1]]

      ' find a spot for the kid -- open spots have fitness = $$MAXXLONG
      FOR kid = 0 TO psize - 1
        IF population[kid].g.fitness = $$MAXXLONG THEN EXIT FOR
      NEXT kid

      IF (kid == psize) THEN ? "***** No room for child in population! *****"

      ' create the kid from parents
			ga = population[a].g
			gb = population[b].g
      population[kid].g = Crossover (@ga, @gb)

      ' consider mutating the kid
      fMutant = $$FALSE
      IF ( XstRandomUniform() < rMut ) THEN
				g = population[kid].g
        Mutate (@g)
				population[kid].g = g
        fMutant = $$TRUE
      END IF

			g = population[kid].g
			Fitness (@g, constant)
			population[kid].g = g

			IF fPrint THEN
				? FORMAT$(f1$, i+1);;;FORMAT$(f2$, a+1);;;FORMAT$(f2$, b+1);;
				PrintGene(@g, fMutant, 0)
			END IF

    NEXT i

    ' now set all as alive
    FOR i = 0 TO psize-1
			population[i].g.alive = $$TRUE
		NEXT i


    ' SURVIVAL OF THE FITTEST
    FOR i = 0 TO nkids - 1
      ' kill the weakest living member
      b = 0 							' b is min fitness
      a = -1 							' a is index of member with min fitness
			best = $$MAXXLONG		'	best is max fitness
			bi = -1     				' bi is index of member with best fitness

      FOR kid = 0 TO psize - 1
        IFZ (population[kid].g.alive) THEN DO NEXT   	' skip dead members
        IF (population[kid].g.fitness > b) THEN				' larger values are less fit
          b = population[kid].g.fitness
          a = kid
				END IF
				IF (population[kid].g.fitness < best) THEN
					bi = kid
				END IF
      NEXT kid

      ' now kill "a"
      population[a].g.alive = $$FALSE

			' save best fit gene of generation
			bestfit[current_gen].bestFitGene = population[bi].g

    NEXT i

		IF fPrint THEN
			' print the population
			?	"***** Starting Survival of the Fittest *****"
			?	"Member  Gene        Fitness"
			? "------  ----------- ----------"
		END IF

		FOR i = 0 TO psize -1
			fKilled = 0

			IF fPrint THEN
				? FORMAT$(f3$, i+1);;
			END IF

			IFZ population[i].g.alive THEN fKilled = $$TRUE

			IF fPrint THEN
				g = population[i].g
				PrintGene(@g, 0, fKilled)
			END IF
		NEXT i

		' set fitnesses of the dead to $$MAXXLONG
    FOR i = 0 TO psize - 1
      IF (population[i].g.alive) THEN DO NEXT
      population[i].g.fitness = $$MAXXLONG
    NEXT i

	NEXT current_gen

END FUNCTION
'
' #####################
' #####  Fitness  #####
' #####################
'
' Smaller values for fitness are better.
' A fitness greater than 4 means multiplier is not correct.
' 4 or less means the multiply factor is correct
' with 1, 2, 3, 4 giving number of required ops (discounting noops).
' Fitness is 'distance' from correct value
'
FUNCTION Fitness (GENE g, ULONG constant)

	GIANT x, y

	x = 1
	g.fitness = 0

	FOR i = 0 TO $$CHROMES
		op = g.gene[i]
		SELECT CASE TRUE
			CASE  op = 0 : 															' noop
			CASE  op > 0 && op < 32 : x = x << op      	' shl
			CASE  op = 32 :															' noop
			CASE  op > 32 && op < 64 : y = x << (op - 32) ' shl save x in y
			CASE  op = 64 : x = x * 3										' lea all x
			CASE  op = 65 : x = x * 5										' lea all x
			CASE  op = 66 : x = x * 9										' lea all x
			CASE  op = 67 : y = x * 3										' lea save x in y
			CASE  op = 68 : y = x * 5										' lea save x in y
			CASE  op = 69 : y = x * 9										' lea save x in y
			CASE  op = 70 : x = x + 1										' add x
			CASE  op = 71 : x = x - 1										' sub x
			CASE  op = 72 : x = x + y										' add x + y
			CASE  op = 73 : x = x - y										' x - y
		END SELECT
		IF x <= 0 THEN x = $$MAXXLONG		' watch out for overflows
	NEXT i

	IF x > $$MAXXLONG THEN x = $$MAXXLONG

	IF x <> constant THEN
		g.fitness = $$CHROMES + 1 + ABS(x-constant)
	ELSE
		FOR i = 0 TO $$CHROMES
			IF g.gene[i] <> 0 && g.gene[i] <> 32 THEN INC g.fitness   ' skip noops
		NEXT i
	END IF

END FUNCTION
'
' #######################
' #####  Crossover  #####
' #######################
'
'
'
FUNCTION GENE Crossover (GENE a, GENE b)

	GENE c

'	PRINT "***** Crossover *****"
'	PrintGene (@a, 0, 0)
'	PrintGene (@b, 0, 0)

	' shuffle cells of genes a & b
	FOR i = 0 TO $$CHROMES
		where = XstRandom () MOD 4&&
		SWAP a.gene[where], a.gene[i]
	NEXT i

	FOR i = 0 TO $$CHROMES
		where = XstRandom () MOD 4&&
		SWAP b.gene[where], b.gene[i]
	NEXT i

	' take cells from parents to create child gene
	FOR i = 0 TO $$CHROMES
		IF (XstRandom() MOD 2&&) THEN
			c.gene[i] = a.gene[i]
		ELSE
			c.gene[i] = b.gene[i]
		END IF
	NEXT i

'	PrintGene (@c, 0, 0)

	RETURN c

END FUNCTION
'
' ####################
' #####  Mutate  #####
' ####################
'
'
'
FUNCTION Mutate (GENE g)

	' modify one cell of gene or exchange two randomly selected cells

	IF XstRandomUniform() < 0.10 THEN
		x = XstRandom() MOD 4&&
		op = XstRandom () MOD (ULONG($$MAXOP+1))
		g.gene[x] = op
	ELSE
		DO
			c1 = XstRandom () MOD 4&&
			c2 = XstRandom () MOD 4&&
			IF c1 <> c2 THEN EXIT DO
		LOOP
		x = g.gene[c1]
		g.gene[c1] = g.gene[c2]
		g.gene[c2] = x
	END IF

END FUNCTION
'
' #######################
' #####  PrintGene  #####
' #######################
'
'
'
FUNCTION PrintGene (GENE g, fMutant, fKilled)

	STATIC f$

	IFZ f$ THEN f$ = " ##########"

	FOR j = 0 TO $$CHROMES
		op = g.gene[j]
		s$ = FORMAT$("###", op)
		? s$;
	NEXT j
	IF fMutant = 0 && fKilled = 0 THEN
		? FORMAT$(f$, g.fitness)
	ELSE
		IF fMutant THEN
			? FORMAT$(f$, g.fitness);;"<== Mutant"
		ELSE
			? FORMAT$(f$, g.fitness);;"<== Killed"
		ENDIF
	END IF

END FUNCTION
'
' ########################
' #####  Initialize  #####
' ########################
'
'
'
FUNCTION Initialize ()

'	SHARED OPDATA opdata[]
'	DIM opdata[$$MAXOP]

'	opdata[0].fitnessValue = 0
'	opdata[0].opStr = "; noop"

'	FOR i = 1 TO 31
'		opdata[i].fitnessValue = 2 ** i
'		opdata[i].opStr = "shl " + STRING$(i)
'	NEXT i

'	opdata[32].fitnessValue = 3
'	opdata[32].opStr = "lea eax, [eax + eax * 2]"

'	opdata[33].fitnessValue = 5
'	opdata[33].opStr = "lea eax, [eax + eax * 4]"

'	opdata[34].fitnessValue = 9
'	opdata[34].opStr = "lea eax, [eax + eax * 8]"




END FUNCTION
END PROGRAM