PROGRAM "enumerate"
VERSION "1.00"
'
'
' ##############################
' #####  Import Libraries  #####
' ##############################
'
'
' Win32API DLL headers
'
	IMPORT "kernel32"   ' operating system
'
	IMPORT "gdi32"      ' Graphic Device Interface
	IMPORT "user32"     ' Windows management
	IMPORT "comctl32"   ' common controls; ==> initialize w/ InitCommonControlsEx ()
	IMPORT "xst"        ' xblite Standard Library
	IMPORT "xsx"        ' xblite Extended Standard Library

'
' ############################################
' #####  Internal Function Declarations  #####
' ############################################
'
'
DECLARE FUNCTION Entry () ' program entry point


DECLARE FUNCTION CreateWindows () ' create the windows of the application

DECLARE FUNCTION GetNotifyMsg (lParam, @hwndFrom, @idFrom, @code) ' get event that has occurred in a control

DECLARE FUNCTION InitGui () ' initialize win32 controls and libraries
DECLARE FUNCTION InitWindows () ' for initializations after CreateWindows()

DECLARE FUNCTION MessageLoop () ' main message loop

DECLARE FUNCTION NewChild (className$, text$, style, x, y, w, h, parent, id, exStyle) ' create a child window
DECLARE FUNCTION NewWindow (className$, titleBar$, style, x, y, w, h, exStyle) ' create a window

DECLARE FUNCTION RegisterWinClass (className$, addrWndProc, icon$, menu$) ' register the window class

DECLARE FUNCTION StartUp () ' application setup

DECLARE FUNCTION WapiNewFont (fontName$, pointSize, weight, italic, underline, strikeOut) ' create a new logical font
DECLARE FUNCTION WapiSetFont (hCtr, hFont) ' apply font to control

DECLARE FUNCTION WndProc (hWnd, wMsg, wParam, lParam) ' callback function for window #frmMain
DECLARE FUNCTION WndEnumProc (hwnd, lParam)
'
'
' #########################################
' #####  Shared Constant Definitions  #####
' #########################################
'
'
' ***** Constants used as control identificators for window #frmMain *****
'
' control identificators for #frmMain
$$btnEnd      = 101	' push button 'E&nd'
$$btnEnumThem = 102	' push button '&Enumerate Windows'
$$lstOutput   = 103	' list box
'
' ***** Shared Program Constants *****
'
'
FUNCTION Entry ()
	STATIC entry            ' ensure Entry is entered only one time

	IF entry THEN RETURN    ' enter once
	entry = $$TRUE          ' enter occured

	StartUp ()              ' initialize program and libraries
	InitGui ()              ' initialize the GUI
	CreateWindows ()        ' create windows and other child controls
	InitWindows ()          ' for initializations after CreateWindows()
	MessageLoop ()          ' monitor the events
	QUIT (0)

END FUNCTION

FUNCTION CreateWindows ()	' create the windows of the application
	SHARED hInst

	' ***************** Begin window #frmMain setup *****************
	' register the window class for window #frmMain
	#frmMain_class$ = "frmMainClass"
	addrWndProc = &WndProc ()
	RegisterWinClass (#frmMain_class$, addrWndProc, "", "") ' register window class \"frmMainClass\"

	' create window #frmMain
	titleBar$ = "Enumerate Windows demo"
	style = $$WS_OVERLAPPEDWINDOW
	#frmMain = NewWindow (#frmMain_class$, titleBar$, style, 0, 0, 352, 346, 0) ' create window #frmMain
	IFZ #frmMain THEN ' fail: null handle
		msg$ = "NewWindow: Can't create window #frmMain"
		XstAlert (msg$)
		RETURN $$TRUE ' fail
	ENDIF

	' ***************** End window #frmMain setup *****************

	XstCenterWindow (#frmMain) ' auto-center the window
	ShowWindow (#frmMain, $$SW_SHOWNORMAL) ' show the window

END FUNCTION

FUNCTION GetNotifyMsg (lParam, @hwndFrom, @idFrom, @code) ' get event that has occurred in a control

	NMHDR nmhdr

'	XstCopyMemory (lParam, &nmhdr, SIZE(nmhdr))	'Xsx library function
	RtlMoveMemory (&nmhdr, lParam, SIZE(nmhdr))	'kernel32 library function
	hwndFrom = nmhdr.hwndFrom
	idFrom   = nmhdr.idFrom
	code     = nmhdr.code

END FUNCTION

FUNCTION InitGui () ' initialize win32 controls and libraries
	' Returns
	' - an error flag: $$TRUE = fail, $$FALSE = success

	SHARED hInst
	SHARED hFontDefault		' default control font

	INITCOMMONCONTROLSEX icc

	hInst = GetModuleHandleA (0)  ' get current instance handle
	IFZ hInst THEN XstAbend ("Application TERMINATED! InitGui: Can't get current instance handle")

	' initialize the specific common controls classes
	' from the common control dynamic-link library
	icc.dwSize = SIZE(icc)

	' $$ICC_ANIMATE_CLASS      : animate
	' $$ICC_BAR_CLASSES        : toolbar, statusbar, trackbar, tooltips
	' $$ICC_COOL_CLASSES       : rebar (coolbar) control
	' $$ICC_DATE_CLASSES       : month picker, date picker, time picker, updown
	' $$ICC_HOTKEY_CLASS       : hotkey
	' $$ICC_INTERNET_CLASSES   : WIN32_IE >= 0x0400
	' $$ICC_LISTVIEW_CLASSES   : listview, header
	' $$ICC_PAGESCROLLER_CLASS : page scroller (WIN32_IE >= 0x0400)
	' $$ICC_PROGRESS_CLASS     : progress
	' $$ICC_TAB_CLASSES        : tab, tooltips
	' $$ICC_TREEVIEW_CLASSES   : treeview, tooltips
	' $$ICC_UPDOWN_CLASS       : updown
	' $$ICC_USEREX_CLASSES     : comboex
	' $$ICC_WIN95_CLASSES      : everything else
	icc.dwICC  = $$ICC_ANIMATE_CLASS      | _
	             $$ICC_BAR_CLASSES        | _
	             $$ICC_COOL_CLASSES       | _
	             $$ICC_DATE_CLASSES       | _
	             $$ICC_HOTKEY_CLASS       | _
	             $$ICC_INTERNET_CLASSES   | _
	             $$ICC_LISTVIEW_CLASSES   | _
	             $$ICC_NATIVEFNTCTL_CLASS | _
	             $$ICC_PAGESCROLLER_CLASS | _
	             $$ICC_PROGRESS_CLASS     | _
	             $$ICC_TAB_CLASSES        | _
	             $$ICC_TREEVIEW_CLASSES   | _
	             $$ICC_UPDOWN_CLASS       | _
	             $$ICC_USEREX_CLASSES     | _
	             $$ICC_WIN95_CLASSES

	InitCommonControlsEx (&icc) ' initialize comctl32.dll library

	hFontDefault = GetStockObject ($$ANSI_VAR_FONT) ' get the ANSI default font
	IFZ hFontDefault THEN ' fail
		msg$ = "InitGui: Can't get the ANSI default font with 'GetStockObject ($$ANSI_VAR_FONT)'"
		XstAlert (msg$)
	ENDIF

END FUNCTION

FUNCTION InitWindows () ' for initializations after CreateWindows()

	ShowWindow (#frmMain, $$SW_SHOW) ' show window #frmMain

END FUNCTION

FUNCTION MessageLoop ()		' main message loop
	'
	' Returns
	' - an error flag: $$TRUE = fail, $$FALSE on quit

	MSG wMsg		' will be sent to window callback function when an event occurs

	' reserved to a stand-alone application
'	IF LIBRARY (0) THEN RETURN ' main program executes message loop

	' supervise system messages until
	' - the user decides to leave the application (RETURN $$FALSE)
	' - an error occurred (RETURN $$TRUE)
	DO		' the message loop
		' retrieve next message from queue
		ret = GetMessageA (&wMsg, 0, 0, 0)
		SELECT CASE ret
			CASE  0 : RETURN ' received a quit message
			CASE -1 : RETURN $$TRUE ' fail
			CASE ELSE
				' deal with window messages
				hWnd = GetActiveWindow ()
				'
				IF (!IsWindow (hWnd)) || (!IsDialogMessageA (hWnd, &wMsg)) THEN
					' send only non-dialog messages
					' translate virtual-key messages into character messages
					' ex.: SHIFT + a is translated as "A"
					TranslateMessage (&wMsg)
					'
					' send message to window callback function
					DispatchMessageA (&wMsg)
				ENDIF
		END SELECT
	LOOP		' forever

END FUNCTION

FUNCTION NewChild (className$, text$, style, x, y, w, h, hParent, idCtr, exStyle) ' create a child window
	'
	' Arguments
	' - className$: control's class
	' - text$     : title
	' - style     : style
	' - x         : Left
	' - y         : Top
	' - w         : Width
	' - h         : Height
	' - hParent   : window or parent control
	' - idCtr       : control ID
	' - exStyle   : extended style
	'
	' Returns
	' - The child window's handle if OK!, 0 = error

	SHARED hInst

	IFZ TRIM$ (className$) THEN
		msg$ = "NewChild: Can't create the child window " + text$ + ", with an empty control class"
		XstAlert (msg$)
		RETURN ' fail
	ENDIF

	IFZ hParent THEN
		msg$ = "NewChild: Can't create the child window " + text$ + ", control class " + className$ + " without a parent"
		XstAlert (msg$)
		RETURN ' fail
	ENDIF

	' create the child window
	newStyle = style | $$WS_CHILD | $$WS_VISIBLE

	SetLastError (0)
	hCtr = CreateWindowExA (exStyle, &className$, &text$, newStyle, x, y, w, h, hParent, idCtr, hInst, 0)
	IFZ hCtr THEN
		msg$ = "NewChild: Can't create the child window " + text$ + " with CreateWindowExA, control class " + className$
		XstAlert (msg$)
	ENDIF

	RETURN hCtr

END FUNCTION

FUNCTION NewWindow (className$, titleBar$, style, x, y, w, h, exStyle) ' create a window
	'
	' Arguments
	' - className$: window's class
	' - titleBar$ : title
	' - style     : style
	' - x         : Left
	' - y         : Top
	' - w         : Width
	' - h         : Height
	' - exStyle   : extended style
	'
	' Returns
	' - The window's handle if OK!, 0 = error

	SHARED hInst

	SetLastError (0)
	hWin = CreateWindowExA (exStyle, &className$, &titleBar$, style, x, y, w, h, 0, 0, hInst, 0)
	IFZ hWin THEN
		msg$ = "NewWindow: Can't create the window " + titleBar$ + " with CreateWindowExA, window class " + className$
		XstAlert (msg$)
	ENDIF

	RETURN hWin

END FUNCTION

FUNCTION RegisterWinClass (className$, addrWndProc, icon$, menu$) ' register the window class
	SHARED hInst

	WNDCLASS wc		' window class

	wc.style           = $$CS_HREDRAW | $$CS_VREDRAW | $$CS_OWNDC
	wc.lpfnWndProc     = addrWndProc
	wc.cbClsExtra      = 0                 ' no extra bytes after the window class
	wc.cbWndExtra      = 0                 ' structure or the window instance
	wc.hInstance       = hInst

	IF icon$ THEN wc.hIcon = LoadIconA (hInst, &icon$)

	wc.hCursor         = LoadCursorA (0, $$IDC_ARROW)
	wc.hbrBackground   = $$COLOR_BTNFACE + 1
	IF menu$ THEN wc.lpszMenuName = &menu$
	wc.lpszClassName   = &className$

	SetLastError (0)
	ret = RegisterClassA (&wc)
	IFZ ret THEN
		msg$ = "RegisterWinClass: Can't register the window class " + className$
		XstAlert (msg$)
		RETURN $$TRUE ' fail
	ENDIF

END FUNCTION

FUNCTION StartUp () ' application setup
	'
	' Returns
	' - an error flag: $$TRUE = fail, $$FALSE = success

	SHARED hInst

	hInst = GetModuleHandleA (0) ' get current instance handle
	IFZ hInst THEN ' fail: null handle
		msg$ = "GetModuleHandleA: Can't get current instance handle"
		XstAlert (msg$)
		RETURN $$TRUE ' fail
	ENDIF

END FUNCTION

FUNCTION WapiNewFont (fontName$, pointSize, weight, italic, underline, strikeOut) ' create a new logical font
	' Returns the font handle if success, 0 = fail

	LOGFONT oLogFont

	' check fontName$ not empty
	fontName$ = TRIM$ (fontName$)
	IFZ fontName$ THEN
		XstAlert ("WapiNewFont: empty font face")
		RETURN ' fail
	ENDIF

	hfontToClone = GetStockObject ($$DEFAULT_GUI_FONT) ' get a font to clone
	IFZ hfontToClone THEN ' fail: null handle
		msg$ = "GetStockObject: Can't get a font to clone"
		XstAlert (msg$)
		RETURN 0 ' invalid handle
	ENDIF

	' hfontToClone provides with a well-formed font structure
	bytes = GetObjectA (hfontToClone, SIZE (oLogFont), &oLogFont) ' allocate structure font
	IFZ bytes THEN ' fail: null handle
		msg$ = "GetObjectA: Can't allocate structure font"
		XstAlert (msg$)
		RETURN 0 ' invalid handle
	ENDIF

	' release the cloned font
	DeleteObject (hfontToClone)
	hfontToClone = 0

	' set the cloned font structure with the passed parameters
	oLogFont.faceName = fontName$

	IFZ pointSize THEN
		oLogFont.height = 0
	ELSE
		' character height is specified (in points)
		IF pointSize > 0 THEN
			pointH = pointSize
		ELSE
			pointH = -pointSize ' make it positive
		ENDIF
		'
		' convert pointSize to pixels
		hdc = GetDC ($$HWND_DESKTOP) ' get the desktop context's handle
		IFZ hdc THEN ' fail: null handle
			msg$ = "GetDC: Can't get the desktop context's handle"
			XstAlert (msg$)
			RETURN 0 ' invalid handle
		ENDIF
		'
		' Windows expects the font height to be in pixels and negative
		oLogFont.height = MulDiv (pointH, GetDeviceCaps (hdc, $$LOGPIXELSY), -72)
		ReleaseDC ($$HWND_DESKTOP, hdc) ' release the handle of the desktop context
	ENDIF

	SELECT CASE weight
		CASE $$FW_THIN, $$FW_EXTRALIGHT, $$FW_LIGHT, $$FW_NORMAL, $$FW_MEDIUM, _
		     $$FW_SEMIBOLD, $$FW_BOLD, $$FW_EXTRABOLD, $$FW_HEAVY, $$FW_DONTCARE
			oLogFont.weight = weight
			'
		CASE ELSE : oLogFont.weight = $$FW_NORMAL
	END SELECT

	IF italic    THEN oLogFont.italic    = 1 ELSE oLogFont.italic    = 0
	IF underline THEN oLogFont.underline = 1 ELSE oLogFont.underline = 0
	IF strikeOut THEN oLogFont.strikeOut = 1 ELSE oLogFont.strikeOut = 0

	hFont = CreateFontIndirectA (&oLogFont) ' create logical font hFont
	IFZ hFont THEN ' fail: null handle
		msg$ = "CreateFontIndirectA: Can't create logical font hFont"
		XstAlert (msg$)
		RETURN 0 ' invalid handle
	ENDIF

	RETURN hFont

END FUNCTION

FUNCTION WapiSetFont (hCtr, hFont) ' apply font to control

	IFZ hCtr  THEN RETURN ' ignore null control handle
	IFZ hFont THEN RETURN $$TRUE ' fail

	' $$WM_SETFONT does not return a value
	SendMessageA (hCtr, $$WM_SETFONT, hFont, 0)

END FUNCTION

FUNCTION WndProc (hWnd, wMsg, wParam, lParam) ' callback function for window #frmMain

	SHARED hInst
	SHARED hFontDefault ' default control font

	RECT rect

	SELECT CASE wMsg
		CASE $$WM_CREATE
			GOSUB frmMain_Create
			RETURN
			'
		CASE $$WM_CLOSE ' closed by User
			DestroyWindow (#frmMain) ' destroy the main window
			RETURN
			'
		CASE $$WM_DESTROY ' being destroyed
			PostQuitMessage ($$WM_QUIT) ' end execution
			#frmMain = 0
			RETURN
			'
		CASE $$WM_COMMAND
			notifyCode = HIWORD (wParam)
			idCtr      = LOWORD (wParam)
			hCtr       = lParam
			'
			SELECT CASE idCtr
				CASE $$btnEnd
					IF notifyCode = $$BN_CLICKED THEN
						DestroyWindow (hWnd)
						RETURN
					ENDIF
					'
				CASE $$btnEnumThem
					IF notifyCode = $$BN_CLICKED THEN
						SendMessageA (#lstOutput, $$LB_RESETCONTENT, 0, 0)
						ret = EnumWindows (&WndEnumProc(), #lstOutput)
						RETURN
					ENDIF
					'
			END SELECT
			'
	END SELECT

	RETURN DefWindowProcA (hWnd, wMsg, wParam, lParam) ' message not processed
'
'
SUB frmMain_Create

	#frmMain = hWnd ' set the global handle

	' *************** Begin Controls setup **************

	' creating push button hCtr
	hCtr = NewChild ($$BUTTON, "E&nd", $$WS_TABSTOP, 176, 264, 153, 33, #frmMain, $$btnEnd, 0) ' create push button $$btnEnd
	IFZ hCtr THEN ' fail: null handle
		msg$ = "NewChild: Can't create push button $$btnEnd"
		XstAlert (msg$)
	ENDIF

	WapiSetFont (hCtr, hFontDefault)

	' creating push button hCtr
	hCtr = NewChild ($$BUTTON, "&Enumerate Windows", $$WS_TABSTOP, 8, 264, 161, 33, #frmMain, $$btnEnumThem, 0) ' create push button $$btnEnumThem
	IFZ hCtr THEN ' fail: null handle
		msg$ = "NewChild: Can't create push button $$btnEnumThem"
		XstAlert (msg$)
	ENDIF

	WapiSetFont (hCtr, hFontDefault)

	' creating list box #lstOutput
	style = $$LBS_NOTIFY | $$LBS_HASSTRINGS | $$LBS_NOINTEGRALHEIGHT | $$WS_TABSTOP | $$WS_HSCROLL | $$WS_VSCROLL
	#lstOutput = NewChild ($$LISTBOX, "", style, 8, 8, 321, 228, #frmMain, $$lstOutput, 0) ' create list box $$lstOutput
	IFZ #lstOutput THEN ' fail: null handle
		msg$ = "NewChild: Can't create list box $$lstOutput"
		XstAlert (msg$)
	ENDIF

	WapiSetFont (#lstOutput, hFontDefault)
	' **************** End Controls setup ***************

END SUB ' frmMain_Create

END FUNCTION
'
' #########################
' #####  WndEnumProc  #####
' #########################
'
'
'
FUNCTION WndEnumProc (hwnd, lParam)

	szTitle$ = NULL$ (512)
	ret = GetWindowTextA (hwnd, &szTitle$, 512)

	' Only show those that have titles.
	IF ret THEN
		index = SendMessageA (lParam, $$LB_ADDSTRING, 0, &szTitle$)
	END IF

	RETURN 1

END FUNCTION

END PROGRAM
