
' A demo of drawing letters with various colors.

	IMPORT	"xst"   		' Standard library : required by most programs
	IMPORT  "xsx"
	IMPORT	"gdi32"     ' gdi32.dll
	IMPORT  "user32"    ' user32.dll
	IMPORT  "kernel32"  ' kernel32.dll

DECLARE FUNCTION  Entry ()
DECLARE FUNCTION  WndProc (hwnd, msg, wParam, lParam)
DECLARE FUNCTION  MessageLoop ()
DECLARE FUNCTION  NewFont (fontName$, pointSize, weight, italic, underline, angle#)
DECLARE FUNCTION  GetLineHeight (hDC, @lineSpace, @pixels, @leading)
DECLARE FUNCTION  DrawCustomFontText (hDC, fontName$, pointSize, weight, italic, underline, angle#, x, y, text$, @lineSpace)
DECLARE FUNCTION  CreateWindow (title$, x, y, w, h, color)
DECLARE FUNCTION  DrawLetters (hDC)
'
'
' ######################
' #####  Entry ()  #####
' ######################
'
FUNCTION  Entry ()

	CreateWindow ("Letter Drawing Demo.", 120, 120, 230, 336, RGB(0, 0, 0))
	MessageLoop ()

END FUNCTION
'
'
' ########################
' #####  WndProc ()  #####
' ########################
'
FUNCTION  WndProc (hWnd, msg, wParam, lParam)

	PAINTSTRUCT ps

	SELECT CASE msg

		CASE $$WM_PAINT :
			hDC = BeginPaint (hWnd, &ps)
			
			' draw text onto window
			DrawLetters (hDC)

			EndPaint (hWnd, &ps)
			
		CASE $$WM_DESTROY :
			PostQuitMessage(0)

		CASE ELSE :
			RETURN DefWindowProcA (hWnd, msg, wParam, lParam)

	END SELECT
END FUNCTION
'
'
' ############################
' #####  MessageLoop ()  #####
' ############################
'
FUNCTION  MessageLoop ()

	STATIC MSG msg

' main message loop

	DO
		ret = GetMessageA (&msg, 0, 0, 0)

		SELECT CASE ret
			CASE  0 : RETURN msg.wParam
			CASE -1 : RETURN $$TRUE
			CASE ELSE:
        hwnd = GetActiveWindow ()
        IF (!IsWindow (hwnd)) || (!IsDialogMessageA (hwnd, &msg)) THEN
  				TranslateMessage (&msg)
  				DispatchMessageA (&msg)
				END IF
		END SELECT
	LOOP

END FUNCTION
'
'
' ########################
' #####  NewFont ()  #####
' ########################
'
FUNCTION  NewFont (fontName$, pointSize, weight, italic, underline, angle#)

	LOGFONT lf
	hDC = GetDC ($$HWND_DESKTOP)
	hFont = GetStockObject ($$DEFAULT_GUI_FONT)	' get a font handle
	bytes = GetObjectA (hFont, SIZE(lf), &lf)		' fill LOGFONT struct lf
	lf.faceName = fontName$											' set font name
	lf.italic = italic													' set italic
	lf.weight = weight													' set weight
	lf.underline = underline										' set underlined
	lf.escapement = angle# * 10									' set text rotation
	lf.height = -1 * pointSize * GetDeviceCaps(hDC, $$LOGPIXELSY) / 72
	ReleaseDC ($$HWND_DESKTOP, hDC)
	RETURN CreateFontIndirectA (&lf)						' create a new font and get handle

END FUNCTION
'
'
' ##############################
' #####  GetLineHeight ()  #####
' ##############################
'
FUNCTION  GetLineHeight (hDC, @lineSpace, @pixels, @leading)

	TEXTMETRIC tm

	GetTextMetricsA (hDC, &tm)
	lineSpace 	= tm.height + tm.externalLeading
	pixels 			= tm.height
	leading 		= tm.externalLeading

END FUNCTION
'
'
' ###################################
' #####  DrawCustomFontText ()  #####
' ###################################
'
FUNCTION  DrawCustomFontText (hDC, fontName$, pointSize, weight, italic, underline, angle#, x, y, text$, @lineSpace)

	hCustFont = NewFont (fontName$, pointSize, weight, italic, underline, angle#)
	hOldFont = SelectObject (hDC, hCustFont)
	GetLineHeight (hDC, @lineSpace, @pixels, @leading)
	TextOutA (hDC, x, y, &text$, LEN(text$))
	SelectObject (hDC, hOldFont)
	DeleteObject (hCustFont)

END FUNCTION
'
' ##########################
' #####  CreateWindow  #####
' ##########################
'
FUNCTION CreateWindow (title$, x, y, w, h, color)

	WNDCLASS wc
	
	SHARED className$

	className$ = title$
	XstReplace (@className$, " ", "_", 0) 

' register window class

	menu$ 			= ""
	
	wc.style           = $$CS_HREDRAW | $$CS_VREDRAW | $$CS_OWNDC
	wc.lpfnWndProc     = &WndProc()
	wc.cbClsExtra      = 0
	wc.cbWndExtra      = 0
	wc.hInstance       = GetModuleHandleA (0)
	wc.hIcon           = LoadIconA (GetModuleHandleA (0), &"scrabble")
	wc.hCursor         = LoadCursorA (0, $$IDC_ARROW)
	wc.hbrBackground   = CreateSolidBrush (color)
	wc.lpszMenuName    = &menu$
	wc.lpszClassName   = &className$

	IFZ RegisterClassA (&wc) THEN RETURN ($$TRUE)

' create main window
	style 			= $$WS_OVERLAPPEDWINDOW
	exStyle			= 0
	hInst 			= GetModuleHandleA (0)
	hWin = CreateWindowExA (0, &className$, &title$, style, x, y, w, h, 0, 0, hInst, 0)

	IFZ hWin THEN RETURN ($$TRUE)

	ShowWindow (hWin, $$SW_SHOWNORMAL)
	
	RETURN hWin

END FUNCTION
'
' #########################
' #####  DrawLetters  #####
' #########################
'
FUNCTION DrawLetters (hDC)
	
	SetBkColor (hDC, RGB(0, 0, 0))	' set text background color to black
	
	counter = 0
	y = 0
	
	FOR i = 0 TO 5
		x = 10
		FOR j = 0 TO 5
			
			count = 65 + (i*6) + j
			IF count <= 90 THEN
				letter = 65 + counter
				SELECT CASE letter
					CASE 'A', 'E', 'I', 'O', 'U' 	: SetTextColor (hDC, RGB(204, 204, 0))
					CASE ELSE 										: SetTextColor (hDC, RGB(255, 255, 255))
				END SELECT
				
			ELSE
				letter = 48 + counter
				SetTextColor (hDC, RGB(153, 153, 153))
			END IF	
			
			' draw a letter using courier new font
			letter$ = CHR$(letter)
			DrawCustomFontText (hDC, "courier new", 32, $$FW_BOLD, $$FALSE, $$FALSE,  0, x, y, letter$, @spacing)
			x = x + 34 
			
			'increment the counter
			INC counter
			IF counter >= 26 THEN counter = 0
			
		NEXT j
		y = y + spacing
	NEXT i	

END FUNCTION

END PROGRAM
