'
'
' Print preview library for xblite. It can be used to 
' preview (WYSIWYG format) text, change printer settings,
' and then print the text.
'
' Use:  PrintPreview (text$, header$)
'
' This is based on a Feng Yuan example codeprint from his 
' book Windows Graphics Programming.
'
'
PROGRAM "preview"
VERSION	"0.0000"
'
'	IMPORT  "xst_s.lib"
'	IMPORT  "xsx_s.lib"
'	IMPORT  "xio_s.lib"
	IMPORT	"xst"   		' standard library	: required by most programs
	IMPORT  "xsx"				' extended std library
'	IMPORT	"xio"				' console io library
	IMPORT	"gdi32"     ' gdi32.dll
	IMPORT  "user32"    ' user32.dll
	IMPORT  "kernel32"  ' kernel32.dll
	IMPORT  "shell32"   ' shell32.dll
	IMPORT  "comctl32"	' comctl32.dll			: common controls library
	IMPORT	"comdlg32"  ' comdlg32.dll	    : common dialog library

TYPE PAINT_STRUCT
    XLONG  .hdc 
    XLONG .fErase 
    RECT .rcPaint 
    XLONG .fRestore 
    XLONG .fIncUpdate 
    UBYTE .rgbReserved[31]
END TYPE

DECLARE FUNCTION Entry            ()
DECLARE FUNCTION WndProc          (hwnd, msg, wParam, lParam)
DECLARE FUNCTION InitGui          ()
DECLARE FUNCTION CreateWindows    ()
DECLARE FUNCTION NewWindow        (titleBar$, style, x, y, w, h, exStyle, addrWndProc, icon$, menu$)
DECLARE FUNCTION NewChild         (className$, text$, style, x, y, w, h, parent, id, exStyle)
DECLARE FUNCTION MessageLoop      ()

EXPORT
' user functions
DECLARE FUNCTION PrintPreview (text$, header$)
DECLARE FUNCTION SetPreviewFontSize (size)
DECLARE FUNCTION SetPreviewFont (fontName$)
END EXPORT

' internal support print preview functions
DECLARE FUNCTION px (x, col)
DECLARE FUNCTION py (y, row)
DECLARE FUNCTION UponDraw (hDC, RECT rcPaint)
DECLARE FUNCTION UponInitialize (hWnd)
DECLARE FUNCTION RefreshPaper ()
DECLARE FUNCTION SetPaper ()
DECLARE FUNCTION GetPageCount ()
DECLARE FUNCTION GetColumnCount ()
DECLARE FUNCTION GetDrawableWidth ()
DECLARE FUNCTION GetDrawableHeight ()
DECLARE FUNCTION STRING GetDocumentName ()
DECLARE FUNCTION GetDimension ()
DECLARE FUNCTION CalculateSize ()
DECLARE FUNCTION CountLine ()
DECLARE FUNCTION UponSetZoom (zoom)
DECLARE FUNCTION SetZoom (zoom)
DECLARE FUNCTION DrawPaper (hDC, RECT rcPaint, col, row)
DECLARE FUNCTION SetupULCS (hDC, zoom)
DECLARE FUNCTION UponDrawPage (hDC, RECT rcPaint, width, height, pageno, maxpages, fRect)
DECLARE FUNCTION DrawFrame (hDC, x0, y0, x1, y1, crPen, crBrush, frame)
DECLARE FUNCTION gcd (m, n)
DECLARE FUNCTION SINGLE GetLineSpace ()
DECLARE FUNCTION SetupPixel (hDC, hFont, SINGLE pixelsize)
DECLARE FUNCTION Setup (hDC, hFont, SINGLE pointsize)
DECLARE FUNCTION LogFont (height, typeface$, LOGFONT lf)
DECLARE FUNCTION CreateFont (LOGFONT lf)
DECLARE FUNCTION Response (hWnd, rslt)
DECLARE FUNCTION GetPixelWidth ()
DECLARE FUNCTION GetPixelHeight ()
DECLARE FUNCTION PreviewProc (hWnd, msg, wParam, lParam)
DECLARE FUNCTION CreateToolbar (hWnd)
DECLARE FUNCTION CreateCanvas (hWnd)
DECLARE FUNCTION CanvasProc (hWnd, msg, wParam, lParam)
DECLARE FUNCTION UponFilePrint ()
DECLARE FUNCTION UponFilePageSetup ()
DECLARE FUNCTION UponPrint (hDC, docName$)
DECLARE FUNCTION CanvasTextOut (hDC, x, y, text$, width, height)

' printer setup and page setup functions
DECLARE FUNCTION OutputSetup ()
DECLARE FUNCTION OutputSetDefault (hwndOwner, minpage, maxpage)
DECLARE FUNCTION OutputDeletePrinterDC ()
DECLARE FUNCTION OutputPrintDialog (flag)
DECLARE FUNCTION OutputCreatePrinterDC ()
DECLARE FUNCTION OutputGetDeviceName ()
DECLARE FUNCTION OutputGetDevMode ()
DECLARE FUNCTION OutputGetPrinterDC ()
DECLARE FUNCTION OutputGetMinMargin (RECT rect)
DECLARE FUNCTION OutputGetMargin (RECT rect)
DECLARE FUNCTION OutputGetPaperSize (POINT p)
DECLARE FUNCTION OutputPageSetup (flag)

' scrollbar functions
DECLARE FUNCTION SetSize (hWnd, width, height, linedx, linedy, resize)
DECLARE FUNCTION SetScrollBar (hWnd, side, maxsize, pagesize)
DECLARE FUNCTION OnScroll (hWnd, nBar, nScrollCode, nPos)
DECLARE FUNCTION OnZoom (hWnd, x, y, mul, div)
DECLARE FUNCTION _GCD (x, y)
DECLARE FUNCTION OnKeyDown (hWnd, msg, wParam, lParam)
DECLARE FUNCTION OnMouseWheel (hWnd, msg, wParam, lParam)
DECLARE FUNCTION IsScrollBarEnabled (hWnd)

' open file dialog functions
DECLARE FUNCTION OpenFileDialog (hWnd, Caption$, Filespec$, InitialDir$, Filter$, DefExtension$, Flags)
DECLARE FUNCTION OnFileOpen (hWnd, @f$)

$$ONEINCH = 7200
$$BASE_DPI = $$ONEINCH
$$MARGIN_X = 16
$$MARGIN_Y = 16
$$MaxCharNo = 256

$$View_NoChange = 0
$$View_Redraw = 1
$$View_Resize = 2

$$Menu_File_Open = 101
$$Menu_File_Exit = 102
$$Menu_File_Dialog = 103

$$IDC_PREVIEW_PRINT = 110
$$IDC_PREVIEW_PAGE_SETUP = 111
$$IDC_PREVIEW_SCALE = 112
$$IDC_PREVIEW_CLOSE = 113


$$IDC_TOOLBAR = 200
$$IDC_CANVAS = 300

$$IDM_SCALESTART = 400
$$IDM_SCALE400 = 401
$$IDM_SCALE200 = 402
$$IDM_SCALE100 = 403
$$IDM_SCALE75 = 404
$$IDM_SCALE50 = 405
$$IDM_SCALE25 = 406
$$IDM_SCALE10 = 407
'
'
' ######################
' #####  Entry ()  #####
' ######################
'
FUNCTION Entry ()

	STATIC	entry
	
	IF entry THEN RETURN							' enter once
	entry =  $$TRUE										' enter occured

'	XioCreateConsole (title$, 2500)			' create console, if console is not wanted, comment out this line
	InitGui ()												' initialize program and libraries
	IF CreateWindows () THEN QUIT (0)	' create windows and other child controls
	MessageLoop ()										' the main message loop

END FUNCTION
'
'
' ########################
' #####  WndProc ()  #####
' ########################
'
FUNCTION WndProc (hWnd, msg, wParam, lParam)

	SHARED hInst

	SELECT CASE msg

		CASE $$WM_DESTROY :
			PostQuitMessage(0)

		CASE $$WM_COMMAND :
			id         = LOWORD(wParam)
			hwndCtl    = lParam
			notifyCode = HIWORD(wParam)

			SELECT CASE id
				
				CASE $$Menu_File_Open :
					IF OnFileOpen (hWnd, @file$) THEN 
						XstLoadString (file$, @s$)
						XstReplace (@s$, "\t", "  ", 0)	
						XstGetDateAndTimeFormatted (0, 1, @date$, 5, @time$)
						XstGetPathComponents (file$, @path$, @drive$, @dir$, @filename$, @attributes)
						header$ = filename$
						header$ = header$ + " - Printed on " + date$ + " " + time$
	'					SetPreviewFontSize (11)
	'					SetPreviewFont ("Times New Roman")
						err = PrintPreview (@s$, @header$)
					END IF
					
				CASE $$Menu_File_Exit :
					PostQuitMessage(0)

'				CASE $$Menu_File_Dialog :
'					file$ = "c:\\temp\\ftptest.x"	
''					file$ = "c:\\temp\\preview.x"	
'					XstLoadString (file$, @s$)
'					XstReplace (@s$, "\t", "  ", 0)	
''					SetPreviewFontSize (11)
''					SetPreviewFont ("Times New Roman")
'					XstGetDateAndTimeFormatted (0, 1, @date$, 5, @time$)
'					XstGetPathComponents (file$, @path$, @drive$, @dir$, @filename$, @attributes)
'					header$ = filename$
'					header$ = header$ + " - Printed on " + date$ + " " + time$
'					err = PrintPreview (@s$, @header$)
			END SELECT

		CASE ELSE :
			RETURN DefWindowProcA (hWnd, msg, wParam, lParam)

	END SELECT

END FUNCTION
'
'
' ########################
' #####  InitGui ()  #####
' ########################
'
FUNCTION InitGui ()

	SHARED hInst
	INITCOMMONCONTROLSEX icc
	
	hInst = GetModuleHandleA (0) 	' get current instance handle
	IFZ hInst THEN QUIT (0)
	InitCommonControls ()					' initialize comctl32.dll library
	
	' initialize the common control classes
  icc.dwSize = SIZE(icc)
  icc.dwICC  = $$ICC_ANIMATE_CLASS      OR _
               $$ICC_BAR_CLASSES        OR _
               $$ICC_COOL_CLASSES       OR _
               $$ICC_DATE_CLASSES       OR _
               $$ICC_HOTKEY_CLASS       OR _
               $$ICC_INTERNET_CLASSES   OR _
               $$ICC_LISTVIEW_CLASSES   OR _
               $$ICC_NATIVEFNTCTL_CLASS OR _
               $$ICC_PAGESCROLLER_CLASS OR _
               $$ICC_PROGRESS_CLASS     OR _
               $$ICC_TAB_CLASSES        OR _
               $$ICC_TREEVIEW_CLASSES   OR _
               $$ICC_UPDOWN_CLASS       OR _
               $$ICC_USEREX_CLASSES     OR _
               $$ICC_WIN95_CLASSES
  InitCommonControlsEx (&icc)

END FUNCTION
'
'
' ##############################
' #####  CreateWindows ()  #####
' ##############################
'
FUNCTION CreateWindows ()

	' create main window
	titleBar$ 	= "Print Preview Demo"
	style 			= $$WS_OVERLAPPEDWINDOW
	w 					= 300
	h 					= 150
	exStyle			= 0
	addrWndProc = &WndProc()
	icon$ 			= "printer"
	menu$ 			= ""
	#winMain = NewWindow (titleBar$, style, x, y, w, h, exStyle, addrWndProc, icon$, menu$)
	IFZ #winMain THEN RETURN ($$TRUE)

' build a main menu
	#mainMenu = CreateMenu()				' create main menu

' build dropdown submenus
	#fileMenu = CreateMenu()				' create dropdown file menu
	InsertMenuA (#mainMenu, $$TRUE, $$MF_BYPOSITION | $$MF_POPUP, #fileMenu, &"&File")
	AppendMenuA (#fileMenu, $$MF_STRING, $$Menu_File_Open, &"&Open File")
	AppendMenuA (#fileMenu, $$MF_STRING, $$Menu_File_Exit, &"&Exit")
'	AppendMenuA (#fileMenu, $$MF_STRING, $$Menu_File_Dialog, &"&Show Dialog")

	SetMenu (#winMain, #mainMenu)						' activate the menu

	XstCenterWindow (#winMain)							' center window position
	ShowWindow (#winMain, $$SW_SHOWNORMAL)	' show window

END FUNCTION
'
'
' ##########################
' #####  NewWindow ()  #####
' ##########################
'
FUNCTION NewWindow (titleBar$, style, x, y, w, h, exStyle, addrWndProc, icon$, menu$)

	SHARED hInst
	WNDCLASS wc
	
	className$ = titleBar$
	IFZ className$ THEN className$ = "Main Program"
	IF INSTR(className$, " ") THEN
		XstReplace (@className$, " ", "_", 0)
	END IF
	
	IFZ icon$ THEN icon$ = "scrabble"
	
	wc.style           = $$CS_HREDRAW | $$CS_VREDRAW | $$CS_OWNDC
	wc.lpfnWndProc     = addrWndProc
	wc.cbClsExtra      = 0
	wc.cbWndExtra      = 0
	wc.hInstance       = hInst
	wc.hIcon           = LoadIconA (hInst, &icon$)
	wc.hCursor         = LoadCursorA (0, $$IDC_ARROW)
	wc.hbrBackground   = $$COLOR_BTNFACE + 1
	wc.lpszMenuName    = &menu$
	wc.lpszClassName   = &className$

	IFZ RegisterClassA (&wc) THEN RETURN
	RETURN CreateWindowExA (exStyle, &className$, &titleBar$, style, x, y, w, h, 0, 0, hInst, 0)

END FUNCTION
'
'
' #########################
' #####  NewChild ()  #####
' #########################
'
FUNCTION  NewChild (className$, text$, style, x, y, w, h, parent, id, exStyle)

	SHARED hInst

	style = style | $$WS_CHILD | $$WS_VISIBLE
	RETURN CreateWindowExA (exStyle, &className$, &text$, style, x, y, w, h, parent, id, hInst, 0)

END FUNCTION
'
'
' ############################
' #####  MessageLoop ()  #####
' ############################
'
FUNCTION MessageLoop ()

	MSG msg

' main message loop

	DO																			' the message loop
		ret = GetMessageA (&msg, NULL, 0, 0)	' retrieve next message from queue

		SELECT CASE ret
			CASE  0 : RETURN msg.wParam					' WM_QUIT message
			CASE -1 : RETURN $$TRUE							' error
			CASE ELSE:
				hwnd = GetActiveWindow ()
				IF (!IsWindow (hwnd)) || (!IsDialogMessageA (hwnd, &msg)) THEN		' send only non-dialog messages
  				TranslateMessage (&msg)					' translate virtual-key messages into character messages
  				DispatchMessageA (&msg)					' send message to window callback function WndProc()
				END IF
		END SELECT
	LOOP

END FUNCTION
'
' #############################
' #####	OpenFileDialog ()	#####
' #############################
'
' Open a file open dialog window to select filename.
'
FUNCTION OpenFileDialog (hWnd, Caption$, Filespec$, InitialDir$, Filter$, DefExtension$, Flags)

	SHARED CURDIR$
	OPENFILENAME ofn
	SHARED hInst

	XstReplace (@Filter$, "|", CHR$ (0), 0)

	IFZ InitialDir$ THEN
		XstGetCurrentDirectory (@CURDIR$)
		InitialDir$ = CURDIR$
	END IF

	IFZ Filespec$ THEN
		Filespec$ = NULL$ (255)
	ELSE
		Filespec$ = Filespec$ + NULL$ (255 - LEN (Filespec$))
	END IF

	FileTitle$ 					= NULL$ (255)

	ofn.lStructSize 		= SIZE (ofn)
	ofn.hwndOwner 			= hWnd
	ofn.hInstance 			= hInst
	ofn.lpstrFilter 		= &Filter$
	ofn.nFilterIndex 		= 1
	ofn.lpstrFile 			= &Filespec$
	ofn.nMaxFile 				= LEN (Filespec$)
	ofn.lpstrFileTitle 	= &FileTitle$
	ofn.nMaxFileTitle 	= LEN (FileTitle$)
	ofn.lpstrInitialDir = &InitialDir$
	IF Caption$ THEN
		ofn.lpstrTitle 		= &Caption$
	END IF
	ofn.flags 					= Flags
	ofn.lpstrDefExt 		= &DefExtension$

	' call dialog
	IFZ GetOpenFileNameA (&ofn) THEN
		Flags = ofn.flags
		Filespec$ = ""
		RETURN
	ELSE
		Flags = ofn.flags
		Filespec$ = CSIZE$ (Filespec$)
		RETURN ($$TRUE)
	END IF

END FUNCTION

'
' #########################
' #####	OnFileOpen ()	#####
' #########################
'
' Open a file in a print preview
'
FUNCTION OnFileOpen (hWnd, @f$)

	XstGetCurrentDirectory (@CURDIR$)
	' Path	= CURDIR$

	fOptions$ = "XBLite Code Files (*.x, *.xl, *.xbl)|*.x;*.xl;*.xbl| _
XBLite DEC Files (*.dec)|*.dec| _
Resource Files (*.rc)|*.rc| _
Text Files (*.txt)|*.txt| _
HTML Files (*.html, *.htm)|*.html;*.htm| _
Assembly files (*.s, *.asm)|*.s;*.asm| _
Template files (*.xxx)|*.xxx| _
All Files (*.*)|*.*"

	f$ = "*.x;*.xl;*.xbl;*.dec;*.asm"
	Style = $$OFN_EXPLORER OR $$OFN_FILEMUSTEXIST OR $$OFN_HIDEREADONLY
	RETURN OpenFileDialog (hWnd, "", @f$, CURDIR$, fOptions$, "x", Style)

END FUNCTION

FUNCTION OutputSetup ()

SHARED PRINTDLG m_pd
SHARED PAGESETUPDLG m_psd

	RtlZeroMemory (&m_pd, SIZE(m_pd))
	m_pd.lStructSize = SIZE (m_pd)
	
	RtlZeroMemory (&m_psd, SIZE(m_psd))
	m_psd.lStructSize = SIZE (m_psd)

END FUNCTION

FUNCTION OutputSetDefault (hwndOwner, minpage, maxpage)

SHARED PRINTDLG m_pd
SHARED PAGESETUPDLG m_psd
SHARED m_hInst

	m_psd.hwndOwner  = hwndOwner
	m_psd.hInstance  = m_hInst

	OutputPrintDialog ($$PD_RETURNDEFAULT)

	m_pd.nFromPage = minpage
	m_pd.nToPage   = maxpage
	m_pd.nMinPage  = minpage
	m_pd.nMaxPage  = maxpage

	m_psd.hwndOwner  = hwndOwner

	m_psd.rtMargin.left  = 1250 ' 1.25 inch
	m_psd.rtMargin.right = 1250 ' 1.25 inch
	m_psd.rtMargin.top   = 1000 ' 1.25 inch
	m_psd.rtMargin.bottom= 1000 ' 1.25 inch

	hDC = OutputCreatePrinterDC ()

	IF hDC THEN dpix = GetDeviceCaps(hDC, $$LOGPIXELSX) ELSE dpix = 300
	IF hDC THEN dpiy = GetDeviceCaps(hDC, $$LOGPIXELSY) ELSE dpix = 300

	IF hDC THEN pw = GetDeviceCaps(hDC, $$PHYSICALWIDTH) ELSE pw = 2500
	m_psd.ptPaperSize.x = pw * 1000. / DOUBLE(dpix)

	IF hDC THEN ph = GetDeviceCaps(hDC, $$PHYSICALHEIGHT) ELSE ph = 3300
	m_psd.ptPaperSize.y = ph * 1000. / DOUBLE(dpiy)

	IF hDC THEN poffx = GetDeviceCaps(hDC, $$PHYSICALOFFSETX) ELSE poffx = 75
	m_psd.rtMinMargin.left = poffx * 1000. / DOUBLE(dpix)
	
	IF hDC THEN poffy = GetDeviceCaps(hDC, $$PHYSICALOFFSETY) ELSE poffy = 12
	m_psd.rtMinMargin.top = poffy * 1000. / DOUBLE(dpiy)
	 
	IF hDC THEN hr = GetDeviceCaps(hDC, $$HORZRES) ELSE hr = 203
	IF hDC THEN vr = GetDeviceCaps(hDC, $$VERTRES) ELSE vr = 266
	
	m_psd.rtMinMargin.right  = m_psd.ptPaperSize.x - m_psd.rtMinMargin.left - hr * 1000. / DOUBLE(dpix)
	m_psd.rtMinMargin.bottom = m_psd.ptPaperSize.y - m_psd.rtMinMargin.top  - vr * 1000. / DOUBLE(dpiy)
		
	IF hDC THEN DeleteObject (hDC)
END FUNCTION

FUNCTION OutputDeletePrinterDC ()

SHARED PRINTDLG m_pd

	IF m_pd.hdc THEN
		DeleteObject (m_pd.hdc)
		m_pd.hdc = 0
	END IF

END FUNCTION

FUNCTION OutputPrintDialog (flag)

SHARED PRINTDLG m_pd

	m_pd.flags = flag
	
	m_pd.nFromPage = 1
	m_pd.nToPage = GetPageCount ()
	m_pd.nMinPage = 1
	m_pd.nMaxPage = m_pd.nToPage
	
	m_pd.nCopies = $$PD_USEDEVMODECOPIES
	
	RETURN PrintDlgA (&m_pd)

END FUNCTION

FUNCTION OutputCreatePrinterDC ()

	pdn = OutputGetDeviceName ()
	RETURN CreateDCA (0, pdn, 0, OutputGetDevMode ())

END FUNCTION

FUNCTION OutputGetDeviceName ()

SHARED PRINTDLG m_pd
DEVNAMES dnms

	pDevNames = GlobalLock (m_pd.hDevNames)
	XstCopyMemory (pDevNames, &dnms, SIZE(dnms))
	IF pDevNames THEN
		RETURN pDevNames + dnms.wDeviceOffset
	END IF

END FUNCTION

FUNCTION OutputGetDevMode ()

SHARED PRINTDLG m_pd
	RETURN GlobalLock (m_pd.hDevMode)

END FUNCTION

FUNCTION OutputGetPrinterDC ()

SHARED PRINTDLG m_pd
	RETURN m_pd.hdc

END FUNCTION

FUNCTION OutputGetMinMargin (RECT rect)

SHARED PAGESETUPDLG m_psd
	rect = m_psd.rtMinMargin

END FUNCTION

FUNCTION OutputGetMargin (RECT rect)

SHARED PAGESETUPDLG m_psd
	rect = m_psd.rtMargin

END FUNCTION

FUNCTION OutputGetPaperSize (POINT p)

SHARED PAGESETUPDLG m_psd
	p = m_psd.ptPaperSize
	
END FUNCTION

FUNCTION OutputPageSetup (flag)

SHARED PAGESETUPDLG m_psd
SHARED PRINTDLG m_pd

	m_psd.hDevMode  = m_pd.hDevMode 
	m_psd.hDevNames = m_pd.hDevNames
	m_psd.flags     = flag | $$PSD_INTHOUSANDTHSOFINCHES

	RETURN PageSetupDlgA (&m_psd)

END FUNCTION

FUNCTION px (x, col)
SHARED SIZEAPI m_Paper 
SHARED m_nScale
	RETURN (x + m_Paper.cx * col) * m_nScale /DOUBLE($$BASE_DPI) + $$MARGIN_X * (col+1)
END FUNCTION

FUNCTION py (y, row)
SHARED SIZEAPI m_Paper 
SHARED m_nScale
	RETURN (y + m_Paper.cy * row) * m_nScale /DOUBLE($$BASE_DPI) + $$MARGIN_Y * (row+1)
END FUNCTION

FUNCTION UponDraw (hDC, RECT rcPaint)

SHARED m_nZoom
SHARED RECT m_Margin

	nPage = GetPageCount ()
	nCol  = GetColumnCount ()
	nRow  = (nPage + nCol - 1)/nCol
	maxPage = nPage - 1

	FOR p = 0 TO maxPage

		SaveDC (hDC)

		col = p MOD nCol
		row = p/nCol
		DrawPaper (hDC, @rcPaint, col, row)
		SetupULCS (hDC, m_nZoom)
		OffsetViewportOrgEx (hDC, px(m_Margin.left, col), py(m_Margin.top, row), 0)
		UponDrawPage (hDC, @rcPaint, GetDrawableWidth(), GetDrawableHeight(), p, maxPage, $$TRUE)

		RestoreDC (hDC, -1)
	NEXT p
END FUNCTION

FUNCTION UponInitialize (hWnd)

SHARED m_nPixelWidth, m_nPixelHeight
SHARED m_nZoom

	OutputSetDefault (hWnd, 1, GetPageCount ())
	m_nZoom = 100
	RefreshPaper ()
	GetDimension ()
	SetSize (hWnd, m_nPixelWidth, m_nPixelHeight, 20, 20, $$TRUE)

END FUNCTION

FUNCTION RefreshPaper ()

SHARED m_nZoom
SHARED m_hFont
SHARED LOGFONT m_lf
SHARED m_fontSize
SHARED m_fontName$

	IFZ m_hFont THEN
	
		' set font size
		IF m_fontSize THEN
			pointsize = m_fontSize
		ELSE
			pointsize = 9
		END IF
		
		' set font name
		IF m_fontName$ THEN
			font$ = m_fontName$
		ELSE
			font$ = "Courier New"
		END IF

		LogFont (-pointsize * $$ONEINCH /DOUBLE(72), font$, @m_lf)
		m_hFont = CreateFont (@m_lf)
		hDC		  = GetDC(0)
		hOld    = SelectObject (hDC, m_hFont)
		SetupPixel (hDC, m_hFont, (pointsize * $$ONEINCH/SINGLE(72)))
		SelectObject(hDC, hOld)
		ReleaseDC (0, hDC)
	END IF

	zoom = m_nZoom
	m_nZoom = 0
	SetPaper ()
	UponSetZoom (zoom)

END FUNCTION

FUNCTION SetPaper ()

	POINT paper
	RECT  margin
	RECT  minmargin 
	
	SHARED SIZEAPI m_Paper
	SHARED RECT m_Margin
	SHARED RECT m_MinMargin	

	OutputGetPaperSize (@paper)
	OutputGetMargin (@margin)
	OutputGetMinMargin (@minmargin)

	m_Paper.cx     			= paper.x * $$BASE_DPI / 1000.
	m_Paper.cy     			= paper.y * $$BASE_DPI / 1000.

	m_Margin.left  			= margin.left   * $$BASE_DPI / 1000.
	m_Margin.right 			= margin.right  * $$BASE_DPI / 1000.
	m_Margin.top   			= margin.top    * $$BASE_DPI / 1000.
	m_Margin.bottom			= margin.bottom * $$BASE_DPI / 1000.

	m_MinMargin.left  	= minmargin.left   * $$BASE_DPI / 1000.
	m_MinMargin.right 	= minmargin.right  * $$BASE_DPI / 1000.
	m_MinMargin.top   	= minmargin.top    * $$BASE_DPI / 1000.
	m_MinMargin.bottom 	= minmargin.bottom * $$BASE_DPI / 1000.
END FUNCTION

FUNCTION GetPageCount ()
SHARED m_hFont
SHARED m_nLinePerPage
SHARED m_nLineCount

	m_nLineCount = CountLine ()

	IFZ m_hFont
		RETURN 1
	ELSE
		m_nLinePerPage = GetDrawableHeight()/GetLineSpace()
		IF (m_nLinePerPage <= 0) THEN RETURN 1
		RETURN (m_nLineCount + m_nLinePerPage-1 )/m_nLinePerPage
	END IF
END FUNCTION

FUNCTION GetColumnCount ()
RECT rect
SHARED m_hWnd
SHARED SIZEAPI m_Paper
SHARED m_nScale

	GetClientRect (m_hWnd, &rect)
	clientwidth = rect.right - rect.left - $$MARGIN_X
	pagewidth   = m_Paper.cx * m_nScale/DOUBLE($$BASE_DPI) + $$MARGIN_X
	RETURN MAX (1, clientwidth/pagewidth)
END FUNCTION

FUNCTION GetDrawableWidth ()
SHARED SIZEAPI m_Paper
SHARED RECT m_Margin
 RETURN m_Paper.cx - m_Margin.left - m_Margin.right
END FUNCTION

FUNCTION GetDrawableHeight ()
SHARED SIZEAPI m_Paper
SHARED RECT m_Margin
	RETURN m_Paper.cy - m_Margin.top  - m_Margin.bottom
END FUNCTION

FUNCTION STRING GetDocumentName ()
	RETURN "XPreview - Document"
END FUNCTION

FUNCTION GetDimension ()
SHARED m_nPixelWidth, m_nPixelHeight
SHARED m_nSurfaceWidth, m_nSurfaceHeight

	m_nPixelWidth  = m_nSurfaceWidth
	m_nPixelHeight = m_nSurfaceHeight

END FUNCTION

FUNCTION CalculateSize ()
SHARED SIZEAPI m_Paper
SHARED m_nSurfaceWidth, m_nSurfaceHeight

	nPage = GetPageCount()
	nCol  = GetColumnCount()
	nRow  = (nPage + nCol - 1) / nCol
	
	m_nSurfaceWidth  = px(m_Paper.cx, 0) * nCol + $$MARGIN_X
	m_nSurfaceHeight = py(m_Paper.cy, 0) * nRow + $$MARGIN_Y 

	GetDimension () 
END FUNCTION

FUNCTION CountLine ()

SHARED m_text$
SHARED m_nSize
SHARED m_text$[]
SHARED m_nLineCount
SHARED m_fWordWrap

	IFZ m_text$ THEN RETURN

	IFZ m_nLineCount THEN
		m_nSize = LEN(m_text$)
		XstStringToStringArray (@m_text$, @m_text$[])
		m_nLineCount = UBOUND(m_text$[]) + 1
	END IF
	
 RETURN m_nLineCount

END FUNCTION

FUNCTION SetSize (hWnd, width, height, linedx, linedy, resize)

RECT rect
SHARED m_width, m_height
SHARED m_linedx, m_linedy

	m_width  = width
	m_height = height
	m_linedx = linedx
	m_linedy = linedy

	IF resize THEN
		GetClientRect (hWnd, &rect)
		SetScrollBar (hWnd, $$SB_HORZ, m_width,  rect.right)
		SetScrollBar (hWnd, $$SB_VERT, m_height, rect.bottom)
	END IF

END FUNCTION

FUNCTION UponSetZoom (zoom)

SHARED m_nZoom
SHARED m_nScale

	IF zoom = m_nZoom THEN RETURN 
	
	m_nZoom  = zoom
	hDC      = GetDC (0)
	m_nScale = zoom * GetDeviceCaps (hDC, $$LOGPIXELSX) / 100.0 'DOUBLE(100)

	DeleteDC (hDC)
	CalculateSize ()
	RETURN (1)
END FUNCTION

FUNCTION DrawPaper (hDC, RECT rcPaint, col, row)

SHARED SIZEAPI m_Paper
SHARED RECT m_Margin
SHARED RECT m_MinMargin	

	' paper frame
	DrawFrame (hDC, px(0, col), py(0, row), px(m_Paper.cx, col), py(m_Paper.cy, row), _ 
		RGB(0, 0, 0), RGB(0xE0, 0xE0, 0xE0), 1)

	' minimum margin : printable margin
	DrawFrame (hDC, px(m_MinMargin.left, col), py(m_MinMargin.top, row), _
		px(m_Paper.cx - m_MinMargin.right, col), _
		py(m_Paper.cy - m_MinMargin.bottom, row), _
		RGB(0xF0, 0xF0, 0xF0), RGB(0xF0, 0xF0, 0xF0), 0)

	' margin
	DrawFrame (hDC, px(m_Margin.left, col), py(m_Margin.top, row), _
		px(m_Paper.cx - m_Margin.right, col), py(m_Paper.cy - m_Margin.bottom, row), _ 
		RGB(0xFF, 0xFF, 0xFF), RGB(0xFF, 0xFF, 0xFF), 0)
END FUNCTION

FUNCTION SetupULCS (hDC, zoom)

	SetMapMode (hDC, $$MM_ANISOTROPIC)

	mul = $$ONEINCH	* 100
	div = GetDeviceCaps (hDC, $$LOGPIXELSX) * zoom
	fac = gcd (mul, div)

	mul = mul/fac
	div = div/fac

	SetWindowExtEx (hDC, mul, mul, NULL)
	SetViewportExtEx (hDC, div, div, NULL)

END FUNCTION

FUNCTION UponDrawPage (hDC, RECT rcPaint, width, height, pageno, maxpages, fRect)

RECT rect
SHARED m_hFont
SHARED m_nLinePerPage
SHARED m_text$[]
SINGLE ls
SHARED RECT m_MinMargin	
SHARED RECT m_Margin
SHARED m_header$

	' get offsets for printing header & footer
	x = -(m_Margin.left - m_MinMargin.left)
	y = -(m_Margin.top - m_MinMargin.top)
	b = m_Margin.bottom - m_MinMargin.bottom

	' skip painting if current page does not intersect with rcPaint
	IF fRect THEN	
		rect.left 	= x 
		rect.top 		= y
		rect.right 	= width
		rect.bottom = height + b
		LPtoDP (hDC, &rect, 2)
		IF !IntersectRect (&rect, &rcPaint, &rect) THEN RETURN
	END IF

	hOld = SelectObject (hDC, m_hFont)

	SetBkMode (hDC, $$TRANSPARENT)
	SetTextAlign (hDC, $$TA_LEFT | $$TA_TOP)
	
	' draw header
	IF m_header$ THEN CanvasTextOut (hDC, x, y, m_header$, 0, 0)

	start = pageno * m_nLinePerPage
	upp   = UBOUND(m_text$[])
	
	ls = GetLineSpace ()

	FOR i = 0 TO m_nLinePerPage - 1
		IF start + i > upp THEN EXIT FOR
		text$ = m_text$[start + i]
		CanvasTextOut (hDC, 0, ls * i + 0.5, text$, width, height)
	NEXT i
	
	' draw footer
	footer$ = "Page " + STRING$(pageno+1) + " of " + STRING$(maxpages+1)
	CanvasTextOut (hDC, x, height + b - ls, footer$, 0, 0)

	SelectObject (hDC, hOld)

END FUNCTION

FUNCTION DrawFrame (hDC, x0, y0, x1, y1, crPen, crBrush, frame)

	RECT rect

	hPen   = CreatePen ($$PS_SOLID, 0, crPen)
	hBrush = CreateSolidBrush (crBrush)

	hOldPen = SelectObject(hDC, hPen)
	hOldBrush = SelectObject(hDC, hBrush)

	Rectangle (hDC, x0-1, y0-1, x1+1, y1+1)
	
	SelectObject (hDC, hOldPen)
	SelectObject (hDC, hOldBrush)
	DeleteObject (hPen)
	DeleteObject (hBrush)

	IF frame THEN
		rect.left   = x1+1
		rect.top    = y0+1
		rect.right  = x1+3
		rect.bottom = y1+3
		FillRect (hDC, &rect, GetStockObject ($$BLACK_BRUSH))

		rect.left   = x0+1
		rect.top    = y1+1
		rect.right  = x1+3
		rect.bottom = y1+3
		FillRect (hDC, &rect, GetStockObject ($$BLACK_BRUSH))
	END IF
END FUNCTION

FUNCTION gcd (m, n)

	IFZ m THEN
		RETURN n
	ELSE
		RETURN gcd (n MOD m, m)
	END IF

END FUNCTION

FUNCTION SINGLE GetLineSpace ()
SHARED SINGLE m_fLinespace
	RETURN m_fLinespace
END FUNCTION

FUNCTION SetupPixel (hDC, hFont, SINGLE pixelsize)

	LOGFONT lf
	OUTLINETEXTMETRIC otm[] 
	SHARED SINGLE m_fCharWidth[]
	SHARED SINGLE m_fHeight
	SHARED SINGLE m_fLinespace
	
	DIM otm[2]

	IFZ GetOutlineTextMetricsA (hDC, SIZE(otm[]), &otm[]) THEN RETURN 

	GetObjectA (hFont, SIZE(lf), &lf)

	emsquare  = otm[0].otmEMSquare  ' get EM square size
	lf.height = -emsquare		  			' font size for 1:1 mapping
	lf.width  = 0				  					' original proportion
	
	hRefFont  = CreateFontIndirectA (&lf)
	hRefDC    = CreateCompatibleDC (hDC)
	hOld      = SelectObject (hRefDC, hRefFont)
		
	DIM nCharWidth[$$MaxCharNo-1]

	GetCharWidth32A (hRefDC, 0, $$MaxCharNo-1, &nCharWidth[])
		
	IFZ GetOutlineTextMetricsA (hRefDC, SIZE(otm[]), &otm[]) THEN RETURN

	SelectObject (hRefDC, hOld)
	DeleteObject (hRefDC)
	DeleteObject (hRefFont)

	m_fHeight    = otm[0].otmTextMetrics.height * pixelsize / SINGLE(emsquare)
	m_fLinespace = (otm[0].otmTextMetrics.height + otm[0].otmTextMetrics.externalLeading) * pixelsize / SINGLE(emsquare)

	DIM m_fCharWidth[$$MaxCharNo-1]
	
	FOR i = 0 TO $$MaxCharNo-1 
		m_fCharWidth[i] = nCharWidth[i] * pixelsize / SINGLE(emsquare)
	NEXT i
		
	RETURN 1

END FUNCTION

FUNCTION Setup (hDC, hFont, SINGLE pointsize)

	RETURN SetupPixel (hDC, hFont, pointsize * GetDeviceCaps(hDC, $$LOGPIXELSY) / SINGLE(72))

END FUNCTION

FUNCTION SetScrollBar (hWnd, side, maxsize, pagesize)

	SCROLLINFO si

	IF pagesize < maxsize THEN

		si.cbSize = SIZE (SCROLLINFO)
		si.fMask  = $$SIF_ALL
		si.nMin   = 0
		si.nMax   = maxsize - 1
		si.nPage  = pagesize
		si.nPos   = 0
	
		EnableScrollBar (hWnd, side, $$ESB_ENABLE_BOTH)
		SetScrollInfo (hWnd, side, &si, 1)
	ELSE
		SetScrollPos (hWnd, side, 0, 0)
		EnableScrollBar (hWnd, side, $$ESB_DISABLE_BOTH)
	END IF

END FUNCTION

FUNCTION LogFont (height, typeface$, LOGFONT lf)

	lf.height         = height
	lf.width          = 0
	lf.escapement     = 0
	lf.orientation    = 0
	lf.weight         = $$FW_NORMAL
	lf.italic         = $$FALSE
	lf.underline      = $$FALSE
	lf.strikeOut      = $$FALSE
	lf.charSet        = $$ANSI_CHARSET
	lf.outPrecision   = $$OUT_TT_PRECIS
	lf.clipPrecision  = $$CLIP_DEFAULT_PRECIS
	lf.quality        = $$DEFAULT_QUALITY
	lf.pitchAndFamily = $$DEFAULT_PITCH | $$FF_DONTCARE

	IF typeface$ THEN 
		lf.faceName = typeface$
	ELSE
		lf.faceName = ""
	END IF

END FUNCTION

FUNCTION CreateFont (LOGFONT lf)
	RETURN CreateFontIndirectA (&lf)
END FUNCTION

FUNCTION OnScroll (hWnd, nBar, nScrollCode, nPos)
	SCROLLINFO si
	SHARED m_linedy, m_linedx
	
  si.cbSize = SIZE(SCROLLINFO)
  si.fMask  = $$SIF_ALL
 
	GetScrollInfo (hWnd, nBar, &si)
	nPageSize = si.nPage

	SELECT CASE nScrollCode
		
		CASE $$SB_TOP:									' Scroll to top
			nDis = -GetScrollPos (hWnd, nBar)

		CASE $$SB_BOTTOM:								' Scroll to bottom
			nDis = si.nMax - GetScrollPos (hWnd, nBar)

		CASE $$SB_LINEDOWN:             ' One line right/down.
			IF nBar = $$SB_VERT THEN
				nDis = m_linedy
			ELSE
				nDis = m_linedx
			END IF

		CASE $$SB_LINEUP:               ' One line left/up.
			IF nBar = $$SB_VERT THEN
				nDis = -m_linedy
			ELSE
				nDis = -m_linedx
			END IF

		CASE $$SB_PAGEDOWN:             ' One page right/down.
			nDis = nPageSize

		CASE $$SB_PAGEUP:               ' One page left/up.
			nDis = -nPageSize

'		CASE $$SB_THUMBPOSITION:        ' Absolute position.
'			nDis = nPos - GetScrollPos (hWnd, nBar)
			
		CASE $$SB_THUMBTRACK:
			nDis = si.nTrackPos - si.nPos

		CASE ELSE:                      ' No change.
			nDis = 0

  END SELECT

	IF nDis THEN

		GetScrollRange (hWnd, nBar, &nMin, &nMax)

		nPos = GetScrollPos (hWnd, nBar) + nDis
		nPos = MAX (nMin, nPos)
		nPos = MIN (nMax-nPageSize, nPos)

		nDis = nPos - GetScrollPos (hWnd, nBar)

		IF nDis THEN
 			SetScrollPos (hWnd, nBar, nPos, 1)
			IF (nBar = $$SB_HORZ) THEN
				ScrollWindow (hWnd, -nDis, 0, NULL, NULL)
			ELSE
				ScrollWindow (hWnd, 0, -nDis, NULL, NULL)
			END IF
		END IF
	END IF
END FUNCTION

FUNCTION OnZoom (hWnd, x, y, mul, div)

RECT rect
SHARED m_zoommul, m_zoomdiv
SHARED m_width, m_height

	m_zoommul = m_zoommul * mul
	m_zoomdiv = m_zoomdiv * div

	factor = _GCD (m_zoommul, m_zoomdiv)

	m_zoommul = m_zoommul / factor
	m_zoomdiv = m_zoomdiv / factor

	' add scrollbar offset
	x = ( x + GetScrollPos (hWnd, $$SB_HORZ) ) * mul / DOUBLE(div)
	y = ( y + GetScrollPos (hWnd, $$SB_VERT) ) * mul / DOUBLE(div)

	' update canvas
	m_width  = m_width  * mul / DOUBLE(div)
	m_height = m_height * mul / DOUBLE(div)

	GetClientRect (hWnd, &rect)

	' reset scrollbars
	SetScrollBar (hWnd, $$SB_HORZ, m_width,  rect.right)
	SetScrollBar (hWnd, $$SB_VERT, m_height, rect.bottom)

	' x in center the center of window if needed
	x = x - rect.right/2.0
	IF x < 0 THEN x = 0
	IF ( x > m_width - rect.right ) THEN x = m_width - rect.right
	SetScrollPos (hWnd, $$SB_HORZ, x, 0)
	
	y = y - rect.bottom/2.0
	IF y < 0 THEN  y = 0
	IF ( y > m_height - rect.bottom ) THEN y = m_height - rect.bottom

	SetScrollPos (hWnd, $$SB_VERT, y, 0)

	' repaint			
	InvalidateRect (hWnd, NULL, 1)
	UpdateWindow (hWnd)
END FUNCTION

FUNCTION _GCD (x, y)

	DO WHILE x != y
		IF x > y THEN x = x - y ELSE y = y - x
	LOOP

	RETURN x

END FUNCTION

FUNCTION OnKeyDown (hWnd, msg, wParam, lParam)

	virtKey = wParam
	
	SELECT CASE wParam
		CASE $$VK_PRIOR : SendMessageA (hWnd, $$WM_VSCROLL, $$SB_PAGEUP, 0) 
		CASE $$VK_NEXT  : SendMessageA (hWnd, $$WM_VSCROLL, $$SB_PAGEDOWN, 0) 
		CASE $$VK_END   : SendMessageA (hWnd, $$WM_VSCROLL, $$SB_BOTTOM, 0)
											SendMessageA (hWnd, $$WM_HSCROLL, $$SB_BOTTOM, 0)
		CASE $$VK_HOME  :	SendMessageA (hWnd, $$WM_VSCROLL, $$SB_TOP, 0)
											SendMessageA (hWnd, $$WM_HSCROLL, $$SB_TOP, 0)
		CASE $$VK_LEFT	:	SendMessageA (hWnd, $$WM_HSCROLL, $$SB_LINEUP, 0)
		CASE $$VK_RIGHT	:	SendMessageA (hWnd, $$WM_HSCROLL, $$SB_LINEDOWN, 0)
		CASE $$VK_UP		: SendMessageA (hWnd, $$WM_VSCROLL, $$SB_LINEUP, 0)
		CASE $$VK_DOWN	: SendMessageA (hWnd, $$WM_VSCROLL, $$SB_LINEDOWN, 0)
			
	END SELECT

END FUNCTION

FUNCTION OnMouseWheel (hWnd, msg, wParam, lParam)

SHARED hCanvas

  IFZ IsScrollBarEnabled (hWnd) THEN RETURN

	delta = HIWORD(wParam)

	SELECT CASE TRUE
		CASE wParam & $$MK_SHIFT   : RETURN DefWindowProcA (hWnd, msg, wParam, lParam) ' datazoom
		CASE wParam & $$MK_CONTROL : RETURN DefWindowProcA (hWnd, msg, wParam, lParam) ' zoom image?? 
		CASE ELSE : 
	END SELECT
	
	IF delta < 0 THEN
		wP = $$SB_LINEDOWN 
	ELSE
		wP = $$SB_LINEUP
	END IF
	SendMessageA (hWnd, $$WM_VSCROLL, wP, 0) 

END FUNCTION

FUNCTION SetZoom (zoom)

		IF UponSetZoom (zoom) THEN
			GetDimension ()
			RETURN $$View_Resize
		ELSE
			RETURN $$View_NoChange
		END IF

END FUNCTION

FUNCTION Response (hWnd, rslt)

'? "Response", rslt, GetPixelWidth(), GetPixelHeight()
	IF (rslt & $$View_Resize) THEN
		SetSize (hWnd, GetPixelWidth(), GetPixelHeight(), 20, 20, 1)
		rslt = rslt | $$View_Redraw
	END IF

	IF (rslt & $$View_Redraw) THEN
		InvalidateRect (hWnd, NULL, 1)
	END IF

END FUNCTION

FUNCTION GetPixelWidth ()
SHARED m_nPixelWidth
	RETURN m_nPixelWidth
END FUNCTION

FUNCTION GetPixelHeight ()
SHARED m_nPixelHeight
 RETURN m_nPixelHeight
END FUNCTION

FUNCTION PreviewProc (hWnd, msg, wParam, lParam)

	RECT rect, rc
	SHARED hToolbar
	SHARED hCanvas
	SHARED hIml
	SHARED hPopupMenu
	SHARED m_hInst
	SHARED m_hWnd
	
	SELECT CASE msg

		CASE $$WM_INITDIALOG :
			
			m_hWnd = hWnd
			
			' set icon in title bar
			icon$ = "printer"
			hIcon = LoadIconA (m_hInst, &icon$)
			IF hIcon THEN
				SendMessageA (hWnd, $$WM_SETICON, $$ICON_BIG, hIcon)
				SendMessageA (hWnd, $$WM_SETICON, $$ICON_SMALL, hIcon)
			END IF
			 
			' create scrollable canvas window
			CreateCanvas (hWnd)
			
			' create toolbar at top of window
			CreateToolbar (hWnd)
			
			XstCenterWindow (hWnd)
			
			' create popup menu for scale button
			hPopupMenu = CreatePopupMenu ()
			DIM scale$[6]
			scale$[0] = "400"
			scale$[1] = "200"
			scale$[2] = "100"
			scale$[3] = "75"
			scale$[4] = "50"
			scale$[5] = "25"	
			scale$[6] = "10"	
			FOR i = 0 TO 6
				text$ = scale$[i] + "%"
				AppendMenuA (hPopupMenu, $$MF_ENABLED, $$IDM_SCALESTART + i + 1, &text$)
			NEXT
			CheckMenuRadioItem (hPopupMenu, $$IDM_SCALE400, $$IDM_SCALE10, $$IDM_SCALE100, $$MF_BYCOMMAND)

		CASE $$WM_COMMAND :
			notifyCode = HIWORD(wParam)
			ID         = LOWORD(wParam)
			hwndCtl    = lParam
			SELECT CASE notifyCode
				CASE $$BN_CLICKED :
					SELECT CASE ID :
						
						CASE $$IDC_PREVIEW_PRINT : 
							UponFilePrint ()
							GetDimension ()
							Response (hCanvas, $$View_Resize) 
							
						CASE $$IDC_PREVIEW_PAGE_SETUP :
							UponFilePageSetup ()
							GetDimension ()
							Response (hCanvas, $$View_Resize) 

						CASE $$IDCANCEL, $$IDC_PREVIEW_CLOSE :
							IF hIml THEN ImageList_Destroy (hImlNor)
							IF hPopupMenu THEN DestroyMenu (hPopupMenu)
							EndDialog (hWnd, ID)
						
						CASE $$IDC_PREVIEW_SCALE :
							SendMessageA (hToolbar, $$TB_GETITEMRECT, 2, &rc)
							MapWindowPoints (hToolbar, $$HWND_DESKTOP, &rc, 2)
							TrackPopupMenu (hPopupMenu, 0, rc.left, rc.bottom, 0, hWnd, NULL)
							
						CASE $$IDM_SCALE400 : 
							result = SetZoom (400)
							Response (hCanvas, result)
							CheckMenuRadioItem (hPopupMenu, $$IDM_SCALE400, $$IDM_SCALE10, ID, $$MF_BYCOMMAND)
						CASE $$IDM_SCALE200 : 
							result = SetZoom (200)
							Response (hCanvas, result)
							CheckMenuRadioItem (hPopupMenu, $$IDM_SCALE400, $$IDM_SCALE10, ID, $$MF_BYCOMMAND)
						CASE $$IDM_SCALE100 : 
							result = SetZoom (100)
							Response (hCanvas, result)
							CheckMenuRadioItem (hPopupMenu, $$IDM_SCALE400, $$IDM_SCALE10, ID, $$MF_BYCOMMAND)
						CASE $$IDM_SCALE75  : 
							result = SetZoom (75)
							Response (hCanvas, result)
							CheckMenuRadioItem (hPopupMenu, $$IDM_SCALE400, $$IDM_SCALE10, ID, $$MF_BYCOMMAND)
						CASE $$IDM_SCALE50  : 
							result = SetZoom (50)
							Response (hCanvas, result)
							CheckMenuRadioItem (hPopupMenu, $$IDM_SCALE400, $$IDM_SCALE10, ID, $$MF_BYCOMMAND)
						CASE $$IDM_SCALE25  : 
							result = SetZoom (25)
							Response (hCanvas, result)
							CheckMenuRadioItem (hPopupMenu, $$IDM_SCALE400, $$IDM_SCALE10, ID, $$MF_BYCOMMAND)	
						CASE $$IDM_SCALE10  : 
							result = SetZoom (10)
							Response (hCanvas, result)
							CheckMenuRadioItem (hPopupMenu, $$IDM_SCALE400, $$IDM_SCALE10, ID, $$MF_BYCOMMAND)
							
							
					END SELECT
			END SELECT

		CASE $$WM_SIZE :
			wc = LOWORD(lParam)
			hc = HIWORD(lParam)
			SendMessageA (hToolbar, $$WM_SIZE, wParam, lParam)
			GetClientRect (hToolbar, &rect)
			ht = rect.bottom
			SetWindowPos (hCanvas, 0, 0, ht+4, wc, hc-ht-4, $$SWP_NOZORDER)

		CASE ELSE : RETURN ($$FALSE)

	END SELECT
	
	RETURN ($$TRUE)

END FUNCTION

FUNCTION PrintPreview (text$, header$)

SHARED m_hInst
SHARED m_text$
SHARED m_header$

	m_text$ = text$
	m_header$ = header$
	m_hInst = GetModuleHandleA (0)
	RETURN DialogBoxParamA (m_hInst, 100, 0, &PreviewProc(), 0)

END FUNCTION

FUNCTION CreateToolbar (hWnd)

TBBUTTON tbb 
SHARED hToolbar
SHARED hIml
BITMAP bm
SHARED m_hInst

	hBmp = LoadImageA (m_hInst, &"TBNOR", $$IMAGE_BITMAP, 0, 0, $$LR_LOADTRANSPARENT OR $$LR_LOADMAP3DCOLORS OR $$LR_DEFAULTSIZE)	
	GetObjectA (hBmp, SIZE (bm), &bm)		' Get the Bitmap's sizes.
	lBmpSize = bm.height	
	
	hIml = ImageList_Create (lBmpSize, lBmpSize, $$ILC_COLORDDB OR $$ILC_MASK, 1, 0)
	ImageList_AddMasked (hIml, hBmp, RGB (255, 0, 255))
	ImageList_Add (hIml, hBmp, RGB (255, 0, 255))
	
	hToolbar = NewChild ($$TOOLBARCLASSNAME, "", $$TBSTYLE_TOOLTIPS | $$TBSTYLE_FLAT, 0, 0, 0, 0, hWnd, $$IDC_TOOLBAR, $$WS_EX_STATICEDGE)
	SendMessageA (hToolbar, $$TB_SETEXTENDEDSTYLE, 0, $$TBSTYLE_EX_DRAWDDARROWS)	
	SendMessageA (hToolbar, $$TB_BUTTONSTRUCTSIZE, SIZE(tbb), 0)
	
	SendMessageA (hToolbar, $$TB_SETIMAGELIST, 0, hIml)
	
	tbb.iBitmap = 9
	tbb.idCommand = $$IDC_PREVIEW_PRINT
	tbb.fsState = $$TBSTATE_ENABLED
	tbb.fsStyle = $$TBSTYLE_BUTTON
	tbb.iString = SendMessageA (hToolbar, $$TB_ADDSTRING, 0, &"Print...")
	SendMessageA (hToolbar, $$TB_INSERTBUTTON, 0, &tbb)

	tbb.iBitmap = 7
	tbb.idCommand = $$IDC_PREVIEW_PAGE_SETUP
	tbb.fsState = $$TBSTATE_ENABLED
	tbb.fsStyle = $$TBSTYLE_BUTTON
	tbb.iString = SendMessageA (hToolbar, $$TB_ADDSTRING, 0, &"Page Setup...")
	SendMessageA (hToolbar, $$TB_INSERTBUTTON, 1, &tbb)	
	
	tbb.iBitmap = 78
	tbb.idCommand = $$IDC_PREVIEW_SCALE
	tbb.fsState = $$TBSTATE_ENABLED
	tbb.fsStyle = $$TBSTYLE_BUTTON
	tbb.iString = SendMessageA (hToolbar, $$TB_ADDSTRING, 0, &"Scale")
	SendMessageA (hToolbar, $$TB_INSERTBUTTON, 2, &tbb)
	
	tbb.iBitmap = 63
	tbb.idCommand = $$IDC_PREVIEW_CLOSE
	tbb.fsState = $$TBSTATE_ENABLED
	tbb.fsStyle = $$TBSTYLE_BUTTON
	tbb.iString = SendMessageA (hToolbar, $$TB_ADDSTRING, 0, &"Close")
	SendMessageA (hToolbar, $$TB_INSERTBUTTON, 3, &tbb)
	
	
	SendMessageA (hToolbar, $$TB_AUTOSIZE, 0, 0)
	IF hBmp THEN DeleteObject (hBmp)
	
	RETURN hToolbar

END FUNCTION

FUNCTION CreateCanvas (hWnd)

	SHARED m_hInst
	SHARED hCanvas
	WNDCLASS wc
	
	className$ 				 = "PREVIEWCANVASCLASS"
	wc.style           = $$CS_HREDRAW | $$CS_VREDRAW
	wc.lpfnWndProc     = &CanvasProc()
	wc.cbClsExtra      = 0
	wc.cbWndExtra      = 0
	wc.hInstance       = m_hInst
	wc.hIcon           = 0 'LoadIconA (hInst, &icon$)
	wc.hCursor         = LoadCursorA (0, $$IDC_ARROW)
	wc.hbrBackground   = $$COLOR_BTNFACE + 1
	wc.lpszMenuName    = 0 '&menu$
	wc.lpszClassName   = &className$

	IFZ RegisterClassA (&wc) THEN RETURN ($$TRUE)
	
	style = $$WS_VSCROLL | $$WS_HSCROLL | $$WS_TABSTOP
	hCanvas = NewChild (className$, "", style, 0, 0, 400, 400, hWnd, $$IDC_CANVAS, 0)


END FUNCTION

FUNCTION CanvasProc (hWnd, msg, wParam, lParam)

	SHARED hInst
	PAINT_STRUCT ps
	RECT rect
	
	SHARED m_hWnd
	SHARED m_nSurfaceWidth		' total surface width  in pixel
	SHARED m_nSurfaceHeight		' total surface height in pixel

	SHARED SIZEAPI m_Paper		' in BASE_DPI
	SHARED RECT m_Margin			' in BASE_DPI
	SHARED RECT m_MinMargin		' in BASE_DPI

	SHARED m_nZoom						' 100 for 1:1
	SHARED m_nScale						' GetDeviceCaps(hDC, LOGPIXELSX) * zoom / 100
	
	SHARED m_width, m_height
	SHARED m_linedx, m_linedy
	SHARED m_zoommul, m_zoomdiv
	
	SHARED m_nPixelWidth, m_nPixelHeight
	
	SHARED SINGLE m_fCharWidth[]
	SHARED SINGLE m_fHeight
	SHARED SINGLE m_fLinespace
	
	SHARED hToolbar
	SHARED hCanvas

	SELECT CASE msg
		
		CASE $$WM_CREATE :
			m_hWnd = hWnd
			m_width   = 0 : m_height  = 0
			m_linedx  = 0 : m_linedy  = 0
			m_zoommul = 1 : m_zoomdiv = 1
			OutputSetup ()
			UponInitialize (hWnd)
			
		CASE $$WM_SIZE :
			SetScrollBar (hWnd, $$SB_HORZ, m_width,  LOWORD(lParam))
			SetScrollBar (hWnd, $$SB_VERT, m_height, HIWORD(lParam))
			
		CASE $$WM_PAINT :
			hDC = BeginPaint (hWnd, &ps)
			SetWindowOrgEx (hDC, 0, 0, NULL)
			SetViewportOrgEx (hDC, -GetScrollPos (hWnd, $$SB_HORZ), -GetScrollPos (hWnd, $$SB_VERT), NULL)
			UponDraw (hDC, @ps.rcPaint)
			EndPaint (hWnd, &ps)
			
'		CASE $$WM_RBUTTONDOWN:
'			OnZoom (hWnd, LOWORD(lParam), HIWORD(lParam), 1, 2)

'		CASE $$WM_LBUTTONDOWN:
'			OnZoom (hWnd, LOWORD(lParam), HIWORD(lParam), 2, 1)


		CASE $$WM_HSCROLL:
			OnScroll (hWnd, $$SB_HORZ, LOWORD(wParam), HIWORD(wParam))

		CASE $$WM_VSCROLL:
			OnScroll (hWnd, $$SB_VERT, LOWORD(wParam), HIWORD(wParam))
			
		CASE $$WM_KEYDOWN :
			OnKeyDown (hWnd, msg, wParam, lParam)
			
		CASE $$WM_MOUSEWHEEL: 
			OnMouseWheel (hWnd, msg, wParam, lParam)
			
		CASE $$WM_GETDLGCODE: 
			RETURN $$DLGC_WANTARROWS


		CASE ELSE :
			RETURN DefWindowProcA (hWnd, msg, wParam, lParam)

	END SELECT

END FUNCTION

FUNCTION IsScrollBarEnabled (hWnd)

	SCROLLINFO si
	
	enabled = EnableScrollBar (hWnd, $$SB_VERT, $$ESB_ENABLE_BOTH)
	IFZ enabled THEN
	  RETURN ($$TRUE)
	ELSE
		EnableScrollBar (hWnd, $$SB_VERT, $$ESB_DISABLE_BOTH)
	END IF
	 
END FUNCTION

FUNCTION UponFilePrint ()

	rslt = OutputPrintDialog ($$PD_RETURNDC | $$PD_PAGENUMS | $$PD_DISABLEPRINTTOFILE) '$$PD_SELECTION
	IF rslt = $$IDOK THEN
		UponPrint (OutputGetPrinterDC(), GetDocumentName())
	ENDIF

	OutputDeletePrinterDC ()

END FUNCTION

FUNCTION UponFilePageSetup ()

	IF OutputPageSetup ($$PSD_MARGINS) THEN
		RefreshPaper ()
		RETURN ($$TRUE)
	END IF

END FUNCTION

FUNCTION UponPrint (hDC, docName$)

DOCINFO docinfo
SHARED RECT m_Margin
SHARED RECT m_MinMargin	
RECT rc
SHARED PRINTDLG m_pd

	scale = GetDeviceCaps (hDC, $$LOGPIXELSX)
		
	SetMapMode (hDC, $$MM_ANISOTROPIC)
	SetWindowExtEx (hDC, $$BASE_DPI, $$BASE_DPI, NULL)
	SetViewportExtEx (hDC, scale, scale, NULL)

	OffsetViewportOrgEx (hDC, (m_Margin.left - m_MinMargin.left) * scale / DOUBLE($$BASE_DPI), (m_Margin.top  - m_MinMargin.top ) * scale / DOUBLE($$BASE_DPI), NULL)

	docinfo.size     = SIZE(docinfo)
	docinfo.docName  = &docName$
	docinfo.output   = NULL 
	docinfo.dataType = &"EMF"
	docinfo.type		 = 0 

	IF StartDocA (hDC, &docinfo) <= 0 THEN RETURN

	SELECT CASE TRUE
		' all pages selected
		CASE !(m_pd.flags & $$PD_SELECTION) && !(m_pd.flags & $$PD_PAGENUMS) : 
			nFrom = m_pd.nMinPage - 1
			nTo = m_pd.nMaxPage - 1
			
		CASE m_pd.flags & $$PD_SELECTION : 
			nFrom = m_pd.nMinPage - 1
			nTo = m_pd.nMaxPage - 1
			
		CASE m_pd.flags & $$PD_PAGENUMS  :
			nFrom = m_pd.nFromPage - 1
			nTo = m_pd.nToPage - 1
		
	END SELECT

'	nFrom = 0
'	nTo   = GetPageCount ()

	FOR pageno = nFrom TO nTo
		IF StartPage (hDC) < 0 THEN
			AbortDoc (hDC)
			RETURN
		END IF

		UponDrawPage (hDC, rc, GetDrawableWidth(), GetDrawableHeight(), pageno, nTo, $$FALSE)
		EndPage(hDC)
		
	NEXT pageno

	EndDoc (hDC)

	RETURN ($$TRUE) 

END FUNCTION

FUNCTION CanvasTextOut (hDC, x, y, text$, width, height)

	RECT rc
	SINGLE sum
	SHARED SINGLE m_fCharWidth[]
	upp = LEN(text$) - 1
	DIM Dx[upp]

	FOR i = 0 TO upp
		sum = sum + m_fCharWidth[text${i}]
		newx = INT (sum + 0.5)
		Dx[i] = newx - lastx
		lastx = newx
	NEXT i
	
	IF width > 0 && height > 0 THEN
		' clip text at margin border
		rc.left = -1
		rc.top = -1
		rc.right = width + 1
		rc.bottom = height + 1
		options = $$ETO_CLIPPED
	END IF

	RETURN ExtTextOutA (hDC, x, y, options, &rc, &text$, LEN(text$), &Dx[0])

END FUNCTION

FUNCTION SetPreviewFontSize (size)

SHARED m_fontSize

	m_fontSize = size

END FUNCTION

FUNCTION SetPreviewFont (fontName$)

SHARED m_fontName$

	m_fontName$ = fontName$

END FUNCTION


END PROGRAM
