'
'
' ####################
' #####  PROLOG  #####
' ####################
'
' A demo of changing the default text and background colors for items in a toolbar control
' by intercepting WM_NOTIFY messages for various stages of the control's drawing process
' If a theme is active (you have used a manifest file), it is necessary to disable the theme for the control
' Disabling the theme for a control does not affect how the theme works on other controls in your program or other running apps.
'
VERSION	"0.0000"
'
	IMPORT  "xst_s.lib"
	IMPORT  "xsx_s.lib"
'	IMPORT  "xio_s.lib"
'	IMPORT	"xst"   		' standard library	: required by most programs
'	IMPORT  "xsx"				' extended std library
'	IMPORT	"xio"				' console io library
	IMPORT	"gdi32"     ' gdi32.dll
	IMPORT  "user32"    ' user32.dll
	IMPORT  "kernel32"  ' kernel32.dll
	IMPORT  "shell32"   ' shell32.dll
	IMPORT  "comctl32"	' comctl32.dll			: common controls library
	IMPORT	"comdlg32"  ' comdlg32.dll	    : common dialog library
'
DECLARE FUNCTION Entry            ()
DECLARE FUNCTION WndProc          (hwnd, msg, wParam, lParam)
DECLARE FUNCTION InitGui          ()
DECLARE FUNCTION CreateWindows    ()
DECLARE FUNCTION NewWindow        (titleBar$, style, x, y, w, h, exStyle, addrWndProc, icon$, menu$)
DECLARE FUNCTION MessageLoop      ()
DECLARE FUNCTION InsertToolbarButton (hToolbar, iconID, iCommand, string$)
DECLARE FUNCTION DisableWindowTheme (hWnd)

$$IDC_TOOLBAR   = 200
$$IDC_TB1       = 201
$$IDC_TB2       = 202
$$IDC_TB3       = 203
$$IDC_TB4       = 204
$$IDC_TB5       = 205
$$IDC_TB6       = 206
'
'
' ######################
' #####  Entry ()  #####
' ######################
'
FUNCTION Entry ()

	STATIC	entry
'
	IF entry THEN RETURN							' enter once
	entry =  $$TRUE										' enter occured

'	XioCreateConsole (title$, 50)			' create console, if console is not wanted, comment out this line
	InitGui ()												' initialize program and libraries
	IF CreateWindows () THEN QUIT (0)	' create windows and other child controls
	MessageLoop ()										' the main message loop
'	XioFreeConsole ()									' free console

END FUNCTION
'
'
' ########################
' #####  WndProc ()  #####
' ########################
'
FUNCTION WndProc (hWnd, msg, wParam, lParam)

	PAINTSTRUCT ps
	RECT rect
	SHARED hToolbar
	STATIC hBrushA, hBrushB, hBrushC
	NMTBCUSTOMDRAW tbcd

	SELECT CASE msg

		CASE $$WM_CREATE :
			' create background color brushes
			hBrushA = CreateSolidBrush (RGB(231, 200, 112))
      hBrushB = CreateSolidBrush (RGB(150, 200, 100))
      hBrushC = CreateSolidBrush (RGB(231, 200, 255))
			
		CASE $$WM_COMMAND :
			id         = LOWORD(wParam)
			hwndCtl    = lParam
			notifyCode = HIWORD(wParam)
			SELECT CASE id
				CASE $$IDC_TB1, $$IDC_TB2, $$IDC_TB3, $$IDC_TB4, $$IDC_TB5, $$IDC_TB6 :
					txt$ = "Toolbar button " + STRING$(id) + " clicked."
					MessageBoxA (0, &txt$, &"Button Pressed", $$MB_ICONINFORMATION)
      END SELECT

		CASE $$WM_DESTROY :
			IF hBrushA THEN DeleteObject (hBrushA)
      IF hBrushB THEN DeleteObject (hBrushB)
      IF hBrushC THEN DeleteObject (hBrushC)
			PostQuitMessage(0)

		CASE $$WM_NOTIFY :
			idCtrl = wParam
			SELECT CASE idCtrl
				CASE $$IDC_TOOLBAR :
					' copy from lParam to nmcd type
					RtlMoveMemory (&tbcd, lParam, SIZE (tbcd))

				SELECT CASE tbcd.nmcd.dwDrawStage
						' draws the background of entire toolbar control
						CASE $$CDDS_PREPAINT
								FillRect (tbcd.nmcd.hdc, &tbcd.nmcd.rc, GetStockObject ($$LTGRAY_BRUSH))
								RETURN $$CDRF_NOTIFYITEMDRAW

						' draws the background of individual buttons
						CASE $$CDDS_ITEMPREPAINT
								SELECT CASE tbcd.nmcd.dwItemSpec
									CASE $$IDC_TB1, $$IDC_TB2
												FillRect (tbcd.nmcd.hdc, &tbcd.nmcd.rc, hBrushC)
												tbcd.clrText = $$BrightRed
									CASE $$IDC_TB3, $$IDC_TB4
												FillRect (tbcd.nmcd.hdc, &tbcd.nmcd.rc, hBrushA)
												tbcd.clrText = $$BrightGreen
									CASE $$IDC_TB5, $$IDC_TB6
												FillRect (tbcd.nmcd.hdc, &tbcd.nmcd.rc, hBrushB)
												tbcd.clrText = $$DarkBlue
								END SELECT
								RtlMoveMemory (lParam, &tbcd, SIZE (tbcd))
								RETURN $$CDRF_NEWFONT
					END SELECT 
				END SELECT 

		CASE ELSE :
			RETURN DefWindowProcA (hWnd, msg, wParam, lParam)

	END SELECT

END FUNCTION
'
'
' ########################
' #####  InitGui ()  #####
' ########################
'
FUNCTION InitGui ()

	SHARED hInst
	INITCOMMONCONTROLSEX icc
	
	hInst = GetModuleHandleA (0) 	' get current instance handle
	IFZ hInst THEN QUIT (0)
	InitCommonControls ()					' initialize comctl32.dll library
	
	' initialize the common control classes
  icc.dwSize = SIZE(icc)
  icc.dwICC  = $$ICC_ANIMATE_CLASS      OR _
               $$ICC_BAR_CLASSES        OR _
               $$ICC_COOL_CLASSES       OR _
               $$ICC_DATE_CLASSES       OR _
               $$ICC_HOTKEY_CLASS       OR _
               $$ICC_INTERNET_CLASSES   OR _
               $$ICC_LISTVIEW_CLASSES   OR _
               $$ICC_NATIVEFNTCTL_CLASS OR _
               $$ICC_PAGESCROLLER_CLASS OR _
               $$ICC_PROGRESS_CLASS     OR _
               $$ICC_TAB_CLASSES        OR _
               $$ICC_TREEVIEW_CLASSES   OR _
               $$ICC_UPDOWN_CLASS       OR _
               $$ICC_USEREX_CLASSES     OR _
               $$ICC_WIN95_CLASSES
  InitCommonControlsEx (&icc)

END FUNCTION
'
'
' ##############################
' #####  CreateWindows ()  #####
' ##############################
'
FUNCTION CreateWindows ()

	SHARED hToolbar

	' create main window
	titleBar$ 	= "Toolbar Custom Colors"
	style 			= $$WS_OVERLAPPEDWINDOW
	w 					= 250
	h 					= 200
	exStyle			= 0
	addrWndProc = &WndProc()
	icon$ 			= "xblite_icon"
	menu$ 			= ""
	hWnd = NewWindow (titleBar$, style, 300, 300, w, h, exStyle, addrWndProc, icon$, menu$)
	IFZ hWnd THEN RETURN ($$TRUE)
	
	' create toolbar
	style = $$WS_CHILD | $$WS_VISIBLE | $$TBSTYLE_FLAT | $$TBSTYLE_TOOLTIPS						 
	hToolbar = CreateWindowExA (0, &"ToolbarWindow32", NULL, style, 0, 0, 0, 0, hWnd, $$IDC_TOOLBAR, hInst, NULL)
	
	' if you have used a manifest file, then disable the theme for the toolbar
	' DisableWindowTheme (hToolbar)

	' load system icons
	DIM hIcon[5]
	hIcon[0] = LoadIconA (0, $$IDI_APPLICATION)
	hIcon[1] = LoadIconA (0, $$IDI_HAND)
	hIcon[2] = LoadIconA (0, $$IDI_QUESTION)
	hIcon[3] = LoadIconA (0, $$IDI_EXCLAMATION)
	hIcon[4] = LoadIconA (0, $$IDI_ASTERISK)
	hIcon[5] = LoadIconA (0, $$IDI_SHIELD) 
	
	' check on system icon size, should be 32
	wIcon = GetSystemMetrics ($$SM_CXICON)
	' PRINT "wIcon= "; wIcon

	' create image list, icon size is 32x32, 6 items
	hImageList = ImageList_Create (32, 32, $$ILC_COLOR24 OR $$ILC_MASK, 6, 0)

	' add icons to image list, note use of ImageList_ReplaceIcon
	FOR i = 0 TO 5
		' add each icon to image list, return value is icon id 
		ret = ImageList_ReplaceIcon (hImageList, -1, hIcon[i])
	NEXT i

	' attach image list to the toolbar
	SendMessageA (hToolbar, $$TB_SETIMAGELIST, 0, hImageList)
	
	' add each item from image list to toolbar as a button with text 
	InsertToolbarButton (hToolbar, 0,  $$IDC_TB1, "1")
	InsertToolbarButton (hToolbar, 1,  $$IDC_TB2, "2")
	InsertToolbarButton (hToolbar, 2,  $$IDC_TB3, "3")
	InsertToolbarButton (hToolbar, 3,  $$IDC_TB4, "4")
	InsertToolbarButton (hToolbar, 4,  $$IDC_TB5, "5")
	InsertToolbarButton (hToolbar, 5,  $$IDC_TB6, "6")
			
	XstCenterWindow (hWnd)							' center window position
	ShowWindow (hWnd, $$SW_SHOWNORMAL)	' show window
	
END FUNCTION
'
' #################################
' #####  InsertToolbarButton  #####
' #################################
'
'
'
FUNCTION InsertToolbarButton (hToolbar, iconID,  iCommand, string$)

TBBUTTON tbb

	tbb.iBitmap 	= iconID
	tbb.idCommand = iCommand
	tbb.fsState 	= $$TBSTATE_ENABLED
	tbb.fsStyle 	= $$TBSTYLE_BUTTON OR $$TBSTYLE_AUTOSIZE
	tbb.dwData 		= 0
	tbb.iString 	= &string$

	SendMessageA (hToolbar, $$TB_BUTTONSTRUCTSIZE, SIZE(TBBUTTON), 0)
	SendMessageA (hToolbar, $$TB_ADDBUTTONS, 1, &tbb) 
	SendMessageA (hToolbar, $$TB_AUTOSIZE, 0, 0)

END FUNCTION
'
'
' ##########################
' #####  NewWindow ()  #####
' ##########################
'
FUNCTION NewWindow (titleBar$, style, x, y, w, h, exStyle, addrWndProc, icon$, menu$)

	SHARED hInst
	WNDCLASS wc
	
	className$ = titleBar$
	IFZ className$ THEN className$ = "MainProgram"
	
	IFZ icon$ THEN icon$ = "xblite_icon"
	
	wc.style           = $$CS_HREDRAW | $$CS_VREDRAW | $$CS_OWNDC
	wc.lpfnWndProc     = addrWndProc
	wc.cbClsExtra      = 0
	wc.cbWndExtra      = 0
	wc.hInstance       = hInst
	wc.hIcon           = LoadIconA (hInst, &icon$)
	wc.hCursor         = LoadCursorA (0, $$IDC_ARROW)
	wc.hbrBackground   = $$COLOR_BTNFACE + 1
	wc.lpszMenuName    = &menu$
	wc.lpszClassName   = &className$

	IFZ RegisterClassA (&wc) THEN RETURN
	RETURN CreateWindowExA (exStyle, &className$, &titleBar$, style, x, y, w, h, 0, 0, hInst, 0)

END FUNCTION
'
'
' ############################
' #####  MessageLoop ()  #####
' ############################
'
FUNCTION MessageLoop ()

	MSG msg

' main message loop

	IF LIBRARY(0) THEN RETURN								' main program executes message loop

	DO																			' the message loop
		ret = GetMessageA (&msg, NULL, 0, 0)	' retrieve next message from queue

		SELECT CASE ret
			CASE  0 : RETURN msg.wParam					' WM_QUIT message
			CASE -1 : RETURN $$TRUE							' error
			CASE ELSE:
				hwnd = GetActiveWindow ()
				IF (!IsWindow (hwnd)) || (!IsDialogMessageA (hwnd, &msg)) THEN		' send only non-dialog messages
  				TranslateMessage (&msg)					' translate virtual-key messages into character messages
  				DispatchMessageA (&msg)					' send message to window callback function WndProc()
				END IF
		END SELECT
	LOOP

END FUNCTION
'
'
' ###############################
' #####  DisableWindowTheme  #####
' ###############################
'
' Set theme of window control theme to null, ie, turn off current theme
'
FUNCTION DisableWindowTheme (hWnd)

	' Call SetWindowTheme to cancel aero color settings
	DIM args[2]
	a$ = " "
	awSize = (LEN (a$) * 2) + 2
	lpaW = GlobalAlloc ($$GMEM_FIXED, awSize)
	MultiByteToWideChar (0, 0, &a$, -1, lpaW, awSize)
	args[0] = hWnd
	args[1] = lpaW
	args[2] = lpaW
	ret = XstCall ("SetWindowTheme", "UxTheme.dll", @args[])
	GlobalFree (lpaW)

END FUNCTION

END PROGRAM
