'
'
' ####################
' #####  PROLOG  #####
' ####################
'
' Demo of creating a tracking tooltip which follows the position of the cursor
' In this case, the entire window is the control which will show the tooltip, but
' the tooltip can also be confined to a specific control
'
VERSION	"0.0001"
'
	IMPORT  "xst_s.lib"
	IMPORT  "xsx_s.lib"
'	IMPORT  "xio_s.lib"
'	IMPORT	"xst"   		' standard library	: required by most programs
'	IMPORT  "xsx"				' extended std library
'	IMPORT	"xio"				' console io library
	IMPORT	"gdi32"     ' gdi32.dll
	IMPORT  "user32"    ' user32.dll
	IMPORT  "kernel32"  ' kernel32.dll
	IMPORT  "shell32"   ' shell32.dll
	IMPORT  "comctl32"	' comctl32.dll			: common controls library
	IMPORT	"comdlg32"  ' comdlg32.dll	    : common dialog library
'	IMPORT	"xma"   		' math library			: Sin/Asin/Sinh/Asinh/Log/Exp/Sqrt...
'	IMPORT  "msvcrt"		' msvcrt.dll				: C function library
'
DECLARE FUNCTION Entry            ()
DECLARE FUNCTION WndProc          (hwnd, msg, wParam, lParam)
DECLARE FUNCTION InitGui          ()
DECLARE FUNCTION CreateWindows    ()
DECLARE FUNCTION NewWindow        (titleBar$, style, x, y, w, h, exStyle, addrWndProc, icon$, menu$)
DECLARE FUNCTION MessageLoop      ()
DECLARE FUNCTION CreateTrackingToolTip (toolID, hWnd, text$)

$$IDC_BUTTON1 = 100 ' not used
'
'
' ######################
' #####  Entry ()  #####
' ######################
'
FUNCTION Entry ()

	STATIC	entry
'
	IF entry THEN RETURN							' enter once
	entry =  $$TRUE										' enter occured

'	XioCreateConsole (title$, 50)			' create console, if console is not wanted, comment out this line
	InitGui ()												' initialize program and libraries
	IF CreateWindows () THEN QUIT (0)	' create windows and other child controls
	MessageLoop ()										' the main message loop
'	XioFreeConsole ()									' free console

END FUNCTION
'
'
' ########################
' #####  WndProc ()  #####
' ########################
'
FUNCTION WndProc (hWnd, msg, wParam, lParam)

	PAINTSTRUCT ps
	RECT rect
	SHARED hTrackingTT
	SHARED TOOLINFO toolinfo
	SHARED g_TrackingMouse
	POINT pt
	TRACKMOUSEEVENT tme
	STATIC init

	SELECT CASE msg

		CASE $$WM_CREATE :

			hTrackingTT = CreateTrackingToolTip ($$IDC_BUTTON1, hWnd, "")


		CASE $$WM_MOUSELEAVE :
			' The mouse pointer has left our window. Deactivate the tooltip.
			SendMessageA (hTrackingTT, $$TTM_TRACKACTIVATE, 0, &toolinfo)
			g_TrackingMouse = 0

		CASE $$WM_MOUSEMOVE :

			' Activate the tooltip.
			SendMessageA (hTrackingTT, $$TTM_TRACKACTIVATE, 1, &toolinfo)

			' Track mouse for tooltip movement.
			pt.x = LOWORD(lParam) : pt.y = HIWORD(lParam)

			' The mouse has just entered the window.  Request notification when the mouse leaves.
			IFZ g_TrackingMouse THEN
				tme.cbSize 					= SIZE(tme)
        tme.hwndTrack       = hWnd
        tme.dwFlags         = $$TME_LEAVE
        ret = TrackMouseEvent (&tme)
        ' Activate the tooltip.
        ' SendMessageA (hTrackingTT, $$TTM_TRACKACTIVATE, 1, &toolinfo)
        g_TrackingMouse = $$TRUE
    END IF

    newX = pt.x
    newY = pt.y

    ' Make sure the mouse has actually moved. The presence of the tooltip
    ' causes Windows to send the message continuously.

    IF ((newX != oldX) || (newY != oldY)) THEN

			oldX = newX
			oldY = newY

			' Update the text
			szText$ = STRING$(newX) + ", " + STRING$(newY)
			toolinfo.lpszText = &szText$
			SendMessageA (hTrackingTT, $$TTM_SETTOOLINFO, 0, &toolinfo)

			' Position the tooltip. The coordinates are adjusted so that the tooltip does not overlap the mouse pointer.
			pt.x = newX
			pt.y = newY
			ClientToScreen (hWnd, &pt)
			SendMessageA (hTrackingTT, $$TTM_TRACKPOSITION, 0, MAKELONG(pt.x + 10, pt.y + 20))
    END IF

		RETURN ($$TRUE)

		CASE $$WM_DESTROY :
			PostQuitMessage(0)

		CASE ELSE :
			RETURN DefWindowProcA (hWnd, msg, wParam, lParam)

	END SELECT

END FUNCTION
'
'
' ########################
' #####  InitGui ()  #####
' ########################
'
FUNCTION InitGui ()

	SHARED hInst
	INITCOMMONCONTROLSEX icc

	hInst = GetModuleHandleA (0) 	' get current instance handle
	IFZ hInst THEN QUIT (0)
	InitCommonControls ()					' initialize comctl32.dll library

	' initialize the common control classes
  icc.dwSize = SIZE(icc)
  icc.dwICC  = $$ICC_ANIMATE_CLASS      OR _
               $$ICC_BAR_CLASSES        OR _
               $$ICC_COOL_CLASSES       OR _
               $$ICC_DATE_CLASSES       OR _
               $$ICC_HOTKEY_CLASS       OR _
               $$ICC_INTERNET_CLASSES   OR _
               $$ICC_LISTVIEW_CLASSES   OR _
               $$ICC_NATIVEFNTCTL_CLASS OR _
               $$ICC_PAGESCROLLER_CLASS OR _
               $$ICC_PROGRESS_CLASS     OR _
               $$ICC_TAB_CLASSES        OR _
               $$ICC_TREEVIEW_CLASSES   OR _
               $$ICC_UPDOWN_CLASS       OR _
               $$ICC_USEREX_CLASSES     OR _
               $$ICC_WIN95_CLASSES
  InitCommonControlsEx (&icc)

END FUNCTION
'
'
' ##############################
' #####  CreateWindows ()  #####
' ##############################
'
FUNCTION CreateWindows ()

	' create main window
	titleBar$ 	= "Tracking Tooltip Demo"
	style 			= $$WS_OVERLAPPEDWINDOW
	w 					= 300
	h 					= 300
	exStyle			= 0
	addrWndProc = &WndProc()
	icon$ 			= "scrabble"
	menu$ 			= ""
	#winMain = NewWindow (titleBar$, style, x, y, w, h, exStyle, addrWndProc, icon$, menu$)
	IFZ #winMain THEN RETURN ($$TRUE)

	XstCenterWindow (#winMain)							' center window position
	ShowWindow (#winMain, $$SW_SHOWNORMAL)	' show window

END FUNCTION
'
'
' ##########################
' #####  NewWindow ()  #####
' ##########################
'
FUNCTION NewWindow (titleBar$, style, x, y, w, h, exStyle, addrWndProc, icon$, menu$)

	SHARED hInst
	WNDCLASS wc

	className$ = titleBar$
	IFZ className$ THEN className$ = "MainProgram"

	IFZ icon$ THEN icon$ = "scrabble"

	wc.style           = $$CS_HREDRAW | $$CS_VREDRAW | $$CS_OWNDC
	wc.lpfnWndProc     = addrWndProc
	wc.cbClsExtra      = 0
	wc.cbWndExtra      = 0
	wc.hInstance       = hInst
	wc.hIcon           = LoadIconA (hInst, &icon$)
	wc.hCursor         = LoadCursorA (0, $$IDC_ARROW)
	wc.hbrBackground   = $$COLOR_BTNFACE + 1
	wc.lpszMenuName    = &menu$
	wc.lpszClassName   = &className$

	IFZ RegisterClassA (&wc) THEN RETURN
	RETURN CreateWindowExA (exStyle, &className$, &titleBar$, style, x, y, w, h, 0, 0, hInst, 0)

END FUNCTION
'
' ############################
' #####  MessageLoop ()  #####
' ############################
'
FUNCTION MessageLoop ()

	MSG msg

' main message loop

	IF LIBRARY(0) THEN RETURN								' main program executes message loop

	DO																			' the message loop
		ret = GetMessageA (&msg, NULL, 0, 0)	' retrieve next message from queue

		SELECT CASE ret
			CASE  0 : RETURN msg.wParam					' WM_QUIT message
			CASE -1 : RETURN $$TRUE							' error
			CASE ELSE:
				hwnd = GetActiveWindow ()
				IF (!IsWindow (hwnd)) || (!IsDialogMessageA (hwnd, &msg)) THEN		' send only non-dialog messages
  				TranslateMessage (&msg)					' translate virtual-key messages into character messages
  				DispatchMessageA (&msg)					' send message to window callback function WndProc()
				END IF
		END SELECT
	LOOP

END FUNCTION
'
' ######################################
' #####  CreateTrackingToolTip ()  #####
' ######################################
'
'
FUNCTION CreateTrackingToolTip (toolID, hWnd, text$)

	SHARED TOOLINFO toolinfo
	RECT rc
	SHARED hInst

	' Create tooltip control. Note use of $$WS_EX_TOPMOST for exStyle.
	hWndTT = CreateWindowExA ($$WS_EX_TOPMOST, &$$TOOLTIPS_CLASS$, &"", $$WS_POPUP | $$TTS_NOPREFIX | $$TTS_ALWAYSTIP, $$CW_USEDEFAULT, $$CW_USEDEFAULT, $$CW_USEDEFAULT, $$CW_USEDEFAULT, hWnd, NULL, hInst, NULL)

	IFZ hWndTT THEN RETURN

	hControl = GetDlgItem (hWnd, toolID)

	' Set up the tool information. In this case, the "tool" is the entire parent window.

	toolinfo.cbSize   = SIZE(TOOLINFO)
	toolinfo.uFlags   = $$TTF_TRACK | $$TTF_ABSOLUTE | $$TTF_IDISHWND
	toolinfo.hwnd     = hWnd
	toolinfo.hinst		= hInst
	toolinfo.lpszText = &text$
	toolinfo.uId      = hWnd  ' hControl - usually set to control (toolID) to track

	GetClientRect (hWnd, &rc)
	toolinfo.rect = rc

	' Associate the tooltip with the tool window.
	SendMessageA (hWndTT, $$TTM_ADDTOOL, 0, &toolinfo)
	RETURN hWndTT

END FUNCTION
END PROGRAM
