' ####################
' #####  PROLOG  #####
' ####################
'
' This is a demonstration of how I use WinX onPaint() event handler for graphics operations.
' Current functions can display lines, rectangles, text and images in multiple windows.
' As of Apr 2013, there is a bug in WinX draw functions. They fail after 4,194,304 operations.
' This is the reason for presenting this alternative.
' I still use the excellent WinX functions for windows, callbacks, controls etc. allowing for very
' compact coding.

' Use initAutoDraw (9999) to initialise these drawing functions. The value indicates the maximum #
' of objects you can have on screen at once. The larger the number, the more memory is locked up.
' Array space is initialised just once at the beginning. There should be some speed advantage as this
' code does not periodically REDIM[] large arrays.
' Linked lists populate the array. A stack keeps record of where there are empty spaces.
' If ever the maximum # of object spaces are used up, drawing functions return -1.


VERSION	"1.00"

	IMPORT	"gdi32"     ' gdi32.dll
	IMPORT  "user32"    ' Windows management
	IMPORT	"xst"   		' standard library	: required by most programs
	IMPORT  "xsx"				' extended std library
	IMPORT	"xio"				' console io library
	IMPORT	"WinX"			' The Xwin GUI library


$$ESC=27
$$PB1 = 100: $$PB2 = 101: $$PB3 = 102: $$PB4 = 103
$$ID_CW1 = 500: $$ID_CW2 = 501: $$ID_CW3 = 502: $$ID_CW4 = 503


TYPE LINKEDLISTINFO
	XLONG	.hWnd			'Each window has a linked list
	SSHORT .head		'Linked list head for hWnd
	SSHORT .tail		'Linked list tail for hWnd
END TYPE

TYPE DRAWRECT
	XLONG	.x1
	XLONG	.y1
	XLONG	.x2
	XLONG	.y2
	XLONG	.hPen
	XLONG	.hBrush
END TYPE
TYPE DRAWTEXT
	XLONG	.x
	XLONG	.y
	XLONG	.iString
	XLONG	.fCol
	XLONG	.bCol
	XLONG	.hFont
END TYPE
TYPE DRAWIMAGE
	XLONG	.x
	XLONG	.y
	XLONG	.w
	XLONG	.h
	XLONG	.xSrc
	XLONG	.ySrc
	XLONG	.hImage
END TYPE
TYPE AUTODRAW
	SSHORT .head    'Node head
	SSHORT .tail    'Node tail
	XLONG	.hWnd
	UBYTE .type     '1=Line, 2=Rectange, 3=Filled Rectangle, 4=Text, 5=Image
	UNION
		DRAWRECT	.rect
		DRAWTEXT	.text
		DRAWIMAGE	.image
	END UNION
END TYPE


DECLARE FUNCTION Entry ()
DECLARE FUNCTION initWindow ()
DECLARE FUNCTION onCommand (id, code, hWnd)
DECLARE FUNCTION onClose (hWnd)
DECLARE FUNCTION Timer (timer, count, msec, time)
DECLARE FUNCTION DrawObjects()

DECLARE FUNCTION initAutoDraw (objects)
DECLARE FUNCTION onPaint(hWnd, hDC)
DECLARE FUNCTION DrawClear (hWnd)
DECLARE FUNCTION InitEmptySpace(hWnd)
DECLARE FUNCTION DrawLine (hWnd, hPen, x1, y1, x2, y2)
DECLARE FUNCTION DrawRect (hWnd, hPen, x1, y1, x2, y2)
DECLARE FUNCTION DrawFillRect (hWnd, hPen, hBrush, x1, y1, x2, y2)
DECLARE FUNCTION DrawText (hWnd, hFont, text$, x, y, bCol, fCol)
DECLARE FUNCTION DrawImage (hWnd, hImage, x, y, w, h, xSrc, ySrc)
DECLARE FUNCTION DrawModifyPen (id, hPen)
DECLARE FUNCTION DrawModifyText (id, text$)
DECLARE FUNCTION RGB (UBYTE r, UBYTE g, UBYTE b)


FUNCTION Entry ()
	XstGetCurrentDirectory(@cd$)
	'XioCreateConsole (title$, 50)
	IF WinX() THEN QUIT(0)
	initAutoDraw (9999) 	' Max. number of drawing objects you expect to have displayed at once
	IF initWindow () THEN QUIT(0)
  'cWnd = XioGetConsoleWindow (): MoveWindow (cWnd, 1100, 0, 300, 800, $$TRUE)

  XstStartTimer (@timer1, 1, 200, &Timer())   	'You might try changing the timer speed!

	WinXDoEvents()
	WinXCleanUp ()
	XioFreeConsole ()
	QUIT (0)
END FUNCTION


FUNCTION initAutoDraw (objects)
	LINKEDLISTINFO #gLink[]
	AUTODRAW #gRecord[]
	SSHORT #gStack[]

	IF objects>32000 THEN objects=32000
	#gTop=objects																'Tracks the last item in a stack denoting empty spaces
	DIM #gRecord[#gTop]													'Drawing object record
	DIM #gStack[#gTop] 													'This array tells us where to find an empty space
	FOR i=0 TO #gTop: #gStack[i]=#gTop-i: NEXT  'Initialise empty space array pointers
	DIM #gText$[#gTop] 													'Text object strings go into a array parallel to gRecord[]
	DIM #gLink[-1] 															'Stores head & tail of linked lists for each window
END FUNCTION


FUNCTION initWindow ()
	SHARED LOGFONT logFont
	ACCEL accel[]

	#anim1=1: #anim2=1: #anim3=1: #anim4=1

	#hMain = WinXNewWindow (0, "Graphics Demo", 0, 0, 1070, 960,$$XWSS_APPNORESIZE,0,0,#mnuRoot) '$$XWSS_APPNORESIZE
	WinXSetWindowColour (#hMain, $$LightGrey)

	#hCW1 = WinXNewChildWindow (#hMain, "CW1", 0, 0, $$ID_CW1)
	WinXAutoSizer_SetInfo (#hCW1, -1,0,0, 20, 40, 500, 400, 0): WinXSetWindowColour (#hCW1, $$Black)
	#hCW2 = WinXNewChildWindow (#hMain, "CW2", 0, 0, $$ID_CW2)
	WinXAutoSizer_SetInfo (#hCW2, -1,0,0, 540, 40, 500, 400, 0): WinXSetWindowColour (#hCW2, $$Black)
	#hCW3 = WinXNewChildWindow (#hMain, "CW3", 0, 0, $$ID_CW3)
	WinXAutoSizer_SetInfo (#hCW3, -1,0,0, 20, 510, 500, 400, 0): WinXSetWindowColour (#hCW3, $$Black)
	#hCW4 = WinXNewChildWindow (#hMain, "CW4", 0, 0, $$ID_CW4)
	WinXAutoSizer_SetInfo (#hCW4, -1,0,0, 540, 510, 500, 400, 0): WinXSetWindowColour (#hCW4, $$Black)

  DIM accel[]
  WinXAddAccelerator (@accel[], $$ESC, $$VK_ESCAPE, $$FALSE, $$FALSE, $$FALSE)
  #hAccel = WinXAddAcceleratorTable (@accel[])
  WinXAttachAccelerators (#hMain, #hAccel)

	logFont = WinXDraw_MakeLogFont ("Arial", 17, 0): #hFont1 = CreateFontIndirectA (&logFont)
	logFont = WinXDraw_MakeLogFont ("Arial", 40, 0): #hFont2 = CreateFontIndirectA (&logFont)

	#hTick = WinXDraw_LoadImage ("Tick2.bmp", $$FILETYPE_WINBMP)
	#hCross = WinXDraw_LoadImage ("Cross.bmp", $$FILETYPE_WINBMP)

	#pen1=CreatePen ($$PS_SOLID, 1, 0x00FF6060): #brush1=CreateSolidBrush (0x00FF6060)
	#pen2=CreatePen ($$PS_SOLID, 1, 0x0000FF00): #brush2=CreateSolidBrush (0x0000FF00)
	#pen3=CreatePen ($$PS_DOT, 4, 0x00F03030)
	#pen4=CreatePen ($$PS_DOT, 4, 0x00F0F060)

  #PB1=WinXAddButton (#hMain, "WINDOW 1", 0, $$PB1): MoveWindow (#PB1, 20, 18, 100, 23, 0)
  #PB2=WinXAddButton (#hMain, "WINDOW 2", 0, $$PB2): MoveWindow (#PB2, 540, 18, 100, 23, 0)
  #PB3=WinXAddButton (#hMain, "WINDOW 3", 0, $$PB3): MoveWindow (#PB3, 20, 488, 100, 23, 0)
  #PB4=WinXAddButton (#hMain, "WINDOW 4", 0, $$PB4): MoveWindow (#PB4, 540, 488, 100, 23, 0)

	DrawText (#hCW2, #hFont1, "OBJECTS IN THIS WINDOW ARE MODIFIED", 20, 30, -1, RGB(100,100,100))
	DrawText (#hCW2, #hFont1, "(NOT DELETED AND REDRAWN)", 20, 50, -1, RGB(100,100,100))

	#id1=DrawLine (#hCW2, #pen3, 20, 100, 480, 100)
	#id2=DrawRect (#hCW2, #pen4, 20, 150, 480, 300)
	#id3=DrawText (#hCW2, #hFont2, "0", 20, 320, -1, RGB(255,255,255))

	WinXRegOnPaint (#hCW1, &onPaint())
	WinXRegOnPaint (#hCW2, &onPaint())
	WinXRegOnPaint (#hCW3, &onPaint())
	WinXRegOnPaint (#hCW4, &onPaint())
	WinXRegOnCommand (#hMain, &onCommand())
	WinXRegOnClose (#hMain, &onClose())
	WinXEnableDialogInterface(#hMain, $$TRUE)

	DrawObjects()
  WinXDisplay (#hMain)

	RETURN 0
END FUNCTION


FUNCTION DrawObjects()
	IF #anim1=1
		DrawClear(#hCW1)
		FOR y=5 TO 380 STEP 40
			FOR x=10 TO 485 STEP 4
				DrawFillRect (#hCW1, #pen1, #brush1, x, XstRandomRange(5,30)+y, x+2, y+30)
			NEXT
		NEXT
	END IF

	IF #anim2=1
		#tog2=-#tog2+1
		IF #tog2=0 THEN pen=#pen3 ELSE pen=#pen4
		DrawModifyPen (#id1, pen)
		DrawModifyPen (#id2, pen)
		DrawModifyText (#id3, STRING(#count))
		#count=#count+1: IF #count>999 THEN #count=0
	END IF

	IF #anim3=1
		DrawClear(#hCW3)
		FOR y=10 TO 380 STEP 30
			a$="": FOR i=0 TO 6: a$=a$+CHR$(XstRandomRange(65,90)): NEXT
			FOR x=10 TO 485 STEP 100
				DrawText(#hCW3, #hFont1, a$, x, y, -1, RGB(0,0,255))
			NEXT
		NEXT
	END IF

	IF #anim4=1
		DrawClear(#hCW4)
		#tog=-#tog+1
		FOR y=20 TO 350 STEP 100
			#tog=-#tog+1
			FOR x=20 TO 450 STEP 160
				IF #tog=1 THEN x1=80: x2=0 ELSE x1=0: x2=80
				DrawImage (#hCW4, #hTick, x+x1, y, 50, 50, 0, 0)
				DrawImage (#hCW4, #hCross, x+x2, y, 50, 50, 0, 0)
			NEXT
		NEXT
	END IF
END FUNCTION


FUNCTION onCommand (id, code, hWnd)
  SELECT CASE id
		CASE $$PB1: #anim1=-#anim1+1
		CASE $$PB2: #anim2=-#anim2+1
		CASE $$PB3: #anim3=-#anim3+1
		CASE $$PB4: #anim4=-#anim4+1
    CASE $$ESC: QUIT(0)
 	END SELECT
END FUNCTION


FUNCTION onPaint(hWnd, hDC)
	LINKEDLISTINFO #gLink[]
  AUTODRAW #gRecord[]
	STATIC POINT pt[]

	n=UBOUND(#gLink[])
	FOR i=0 TO n
		IF #gLink[i].hWnd=hWnd THEN EXIT FOR
	NEXT
	IF i<=n
		id=#gLink[i].head: pen=-2
		DO WHILE id>-1
			IF pen<>#gRecord[id].rect.hPen THEN pen=#gRecord[id].rect.hPen: SelectObject (hDC, pen)
			SELECT CASE #gRecord[id].type
				CASE 1:
					MoveToEx (hDC, #gRecord[id].rect.x1, #gRecord[id].rect.y1, 0): LineTo (hDC, #gRecord[id].rect.x2, #gRecord[id].rect.y2)
				CASE 2:
					DIM pt[4]
					pt[0].x = #gRecord[id].rect.x1: pt[0].y = #gRecord[id].rect.y1
					pt[1].x = #gRecord[id].rect.x2: pt[1].y = #gRecord[id].rect.y1
					pt[2].x = #gRecord[id].rect.x2: pt[2].y = #gRecord[id].rect.y2
					pt[3].x = #gRecord[id].rect.x1: pt[3].y = #gRecord[id].rect.y2: pt[4] = pt[0]
					Polyline (hDC, &pt[0], 5)
				CASE 3:
					SelectObject (hDC, #gRecord[id].rect.hBrush)
					Rectangle (hDC, #gRecord[id].rect.x1, #gRecord[id].rect.y1, #gRecord[id].rect.x2, #gRecord[id].rect.y2)
				CASE 4:
					SelectObject (hDC, #gRecord[id].text.hFont)
					IF #gRecord[id].text.bCol=-1 THEN
						SetBkMode (hDC, $$TRANSPARENT)
					ELSE
						SetBkMode (hDC, $$OPAQUE): SetBkColor (hDC, #gRecord[id].text.bCol)
					END IF
					SetTextColor (hDC, #gRecord[id].text.fCol)
					text$=#gText$[id]
					ExtTextOutA (hDC, #gRecord[id].text.x, #gRecord[id].text.y, 0, 0, &text$, LEN(text$), 0)
				CASE 5:
					hdcSrc = CreateCompatibleDC (0)
					hOld = SelectObject (hdcSrc, #gRecord[id].image.hImage)
					BitBlt (hDC, #gRecord[id].image.x, #gRecord[id].image.y, #gRecord[id].image.w, #gRecord[id].image.h, hdcSrc, _
					#gRecord[id].image.xSrc, #gRecord[id].image.ySrc, $$SRCCOPY)
					SelectObject (hdcSrc, hOld)
					DeleteObject(hdcSrc)
			END SELECT
			id=#gRecord[id].tail
		LOOP
	END IF
END FUNCTION


FUNCTION InitEmptySpace(hWnd)
	LINKEDLISTINFO #gLink[]
  AUTODRAW #gRecord[]
	SSHORT #gStack[]

	id=-1
	IF #gTop>-1
		id=#gStack[#gTop]: #gStack[#gTop]=-1: #gTop=#gTop-1 'Pick an empty slot from the stack
		n=UBOUND(#gLink[])
		FOR i=0 TO n
			IF #gLink[i].hWnd=hWnd THEN EXIT FOR
		NEXT
		IF i>n THEN																					'New linked list for a new window
			REDIM #gLink[i]: #gLink[i].hWnd=hWnd
			#gLink[i].head=-1: #gLink[i].tail=-1
		END IF
		IF #gLink[i].head=-1																'Initialising a linked list
			#gRecord[id].head=id: #gRecord[id].tail=-1
			#gLink[i].head=id: #gLink[i].tail=id
		ELSE 																								'Add to an existing linked list
			#gRecord[#gLink[i].tail].tail=id                	'Get previous record to point to this record
			#gRecord[id].head=id: #gRecord[id].tail=-1
			#gLink[i].tail=id
		END IF
	END IF
	RETURN id
END FUNCTION


FUNCTION DrawLine (hWnd, hPen, x1, y1, x2, y2)
  AUTODRAW #gRecord[]

	id=InitEmptySpace(hWnd)
	IF id>-1
		#gRecord[id].hWnd=hWnd: #gRecord[id].type=1: #gRecord[id].rect.hPen=hPen
		#gRecord[id].rect.x1=x1: #gRecord[id].rect.y1=y1: #gRecord[id].rect.x2=x2: #gRecord[id].rect.y2=y2
	END IF
	RETURN id
END FUNCTION


FUNCTION DrawRect (hWnd, hPen, x1, y1, x2, y2)
  AUTODRAW #gRecord[]

	id=InitEmptySpace(hWnd)
	IF id>-1
		#gRecord[id].hWnd=hWnd: #gRecord[id].type=2: #gRecord[id].rect.hPen=hPen
		#gRecord[id].rect.x1=x1: #gRecord[id].rect.y1=y1: #gRecord[id].rect.x2=x2: #gRecord[id].rect.y2=y2
	END IF
	RETURN id
END FUNCTION


FUNCTION DrawFillRect (hWnd, hPen, hBrush, x1, y1, x2, y2)
  AUTODRAW #gRecord[]

	id=InitEmptySpace(hWnd)
	IF id>-1
		#gRecord[id].hWnd=hWnd: #gRecord[id].type=3: #gRecord[id].rect.hPen=hPen: #gRecord[id].rect.hBrush=hBrush
		#gRecord[id].rect.x1=x1: #gRecord[id].rect.y1=y1: #gRecord[id].rect.x2=x2: #gRecord[id].rect.y2=y2
	END IF
	RETURN id
END FUNCTION


FUNCTION DrawText(hWnd, hFont, text$, x, y, bCol, fCol)
  AUTODRAW #gRecord[]

	id=InitEmptySpace(hWnd)
	IF id>-1
		#gRecord[id].hWnd=hWnd: #gRecord[id].type=4
		#gRecord[id].text.x=x: #gRecord[id].text.y=y
		#gRecord[id].text.hFont=hFont: #gText$[id]=text$
		#gRecord[id].text.bCol=bCol: #gRecord[id].text.fCol=fCol
	END IF
	RETURN id
END FUNCTION


FUNCTION DrawImage(hWnd, hImage, x, y, w, h, xSrc, ySrc)
  AUTODRAW #gRecord[]

	id=InitEmptySpace(hWnd)
	IF id>-1
		#gRecord[id].hWnd=hWnd: #gRecord[id].type=5
		#gRecord[id].image.hImage=hImage
		#gRecord[id].image.x=x: #gRecord[id].image.y=y
		#gRecord[id].image.w=w: #gRecord[id].image.h=h
		#gRecord[id].image.xSrc=xSrc: #gRecord[id].image.ySrc=ySrc
	END IF
	RETURN id
END FUNCTION


FUNCTION DrawClear (hWnd)
	LINKEDLISTINFO #gLink[]
  AUTODRAW #gRecord[]
	SSHORT #gStack[]

	n=UBOUND(#gLink[])
	FOR i=0 TO n
		IF #gLink[i].hWnd=hWnd THEN EXIT FOR
	NEXT
	IF i<=n
		id=#gLink[i].head
		DO WHILE id>-1
			#gTop=#gTop+1: #gStack[#gTop]=id   															'Mark slot as empty, put on the stack
			#gRecord[id].head=0: #gRecord[id].hWnd=0: #gRecord[id].type=0   'Clear main record items
			#gText$[id]=""
			pid=id: id=#gRecord[id].tail: #gRecord[pid].tail=0
		LOOP
		#gLink[i].head=-1: #gLink[i].tail=-1
	END IF
	GetClientRect (hWnd, &rect): InvalidateRect (hWnd, 0, $$TRUE)
END FUNCTION


FUNCTION DrawModifyPen(id, hPen)
  AUTODRAW #gRecord[]
	RECT rect

	n=UBOUND(#gRecord[])
	IF (id>=0 && id<=n)
		hWnd=#gRecord[id].hWnd
		IF hWnd>0
			#gRecord[id].rect.hPen=hPen
			x1=#gRecord[id].rect.x1: y1=#gRecord[id].rect.y1: x2=#gRecord[id].rect.x2: y2=#gRecord[id].rect.y2
			rect.left=MIN(x1,x2)-3: rect.top=MIN(y1,y2)-3
			rect.right=MAX(x1,x2)+3: rect.bottom=MAX(y1,y2)+3
			InvalidateRect (hWnd, &rect, $$TRUE)
		END IF
	END IF
END FUNCTION


FUNCTION DrawModifyText(id, text$)
  AUTODRAW #gRecord[]
	RECT rect
	SIZEAPI size

	n=UBOUND(#gRecord[])
	IF (id>=0 && id<=n)
		hWnd=#gRecord[id].hWnd
		IF (hWnd>0 && #gRecord[id].type=4)
			#gText$[id]=text$
			hDC = CreateCompatibleDC (0)
			SelectObject (hDC, #gRecord[id].text.hFont)
			GetTextExtentPoint32A (hDC, &text$, LEN (text$)+2, &size)  'Increased size by 2 chars to cover old text
			DeleteDC (hDC)
			x=#gRecord[id].text.x: y=#gRecord[id].text.y
			rect.left=x-1: rect.top=y-1
			rect.right=x+size.cx+1: rect.bottom=y+size.cy+1
			InvalidateRect (hWnd, &rect, $$TRUE)
		END IF
	END IF
END FUNCTION


FUNCTION RGB (UBYTE r, UBYTE g, UBYTE b)
	RETURN 0 << 24 + r << 16 + g << 8 + b
END FUNCTION


FUNCTION  Timer (timer, count, msec, time)
  count=1
	DrawObjects()
END FUNCTION


FUNCTION onClose (hWnd)
  QUIT(0)
END FUNCTION


END PROGRAM