' **************************************************************************
'Subject: Basic library for large-integer arithmetic.
'Author : Sjoerd.J.Schaper
'Date   : 10-15-2005
'Code   : XBasic adaptation
PROGRAM	"bignumXB"
VERSION	"11.11"
IMPORT	"xst" ' standard
IMPORT  "xsx"
IMPORT	"xma" ' mathematics
EXPORT
' **************************************************************************
'Subject: Declaration file for large-integer arithmetic Basic library.
'Author : Sjoerd.J.Schaper - vspickelen@zonnet.nl
'Date   : 10-15-2005
'Code   : XBasic version bignumXB.x
' `[..] there is a bound to the number of symbols which the computer
' can observe at one moment. If he wishes to observe more, he must use
' successive operations.' Alan Turing, On computable numbers [..], 1936.
'
' **************************************************************************
'                      Initialization and assignment
' **************************************************************************
'The LargeInt class is initialized with a 'LargeInit( name$ )' call at
'module level. You have to declare constant indices for referencing the
'large integers you're planning to use ('p = 0: q = 1, ..'), so that
'each number is associated with its own distinct pointer. These pointers
'are passed to the procedures. Don't call with same pointers ('Squ(p, p)'),
'and beware of duplicate references, for they will crash your program.
'
'In the following prototypes,
'the letters p, q, r, d, m  denote 32-bit pointers to large integers;
'variables a, c, k, sw  are 32-bit signed integers;
'g is a number string; f and h are any string.
'
END EXPORT
DECLARE FUNCTION SLONG DLLMain ()
'Initialize the number-arrays, return value = -1.
EXPORT
DECLARE FUNCTION VOID LargeInit (STRING f)
'Open the primetable and logfile, to be called with f = "Program".
'If f = "", no logfile is opened.
'
DECLARE FUNCTION VOID Letf (SLONG p, SLONG c)
'Assign long integer c to large integer p.
'
DECLARE FUNCTION VOID Rndf (SLONG p, SLONG k)
'Assign to p a random positive number with length Abs(k).
'
DECLARE FUNCTION VOID Term ()
'Close the primetable and logfile.
'
' **************************************************************************
'                          Conversion functions
' **************************************************************************
DECLARE FUNCTION VOID Readst (SLONG p, STRING g)
'Convert decimal number string g to large integer p.
'Except for a prefixed minus sign, there's no check
'on non-digit characters in the string.
'
DECLARE FUNCTION SLONG Decf (SLONG p)
'Convert large integer p to base MD in array ln(),
'the bytelength of p is returned. Call before CnvSt.
'
DECLARE FUNCTION VOID CnvSt (STRING g)
'Convert base MD number to string. First call k = Decf(p),
'create stringbuffer g = CHR$('0', k) and pass g by reference to CnvSt.
'
DECLARE FUNCTION DOUBLE Logf (SLONG p)
'Double precision natural logarithm of large integer Abs(p).
'
DECLARE FUNCTION VOID Ratdec (STRING g, SLONG p, SLONG q)
'Return the decimal expansion of rational p / q in [0, 1) in g.
'Note that string g must be passed by reference.
'
DECLARE FUNCTION SLONG Cvl (STRING g)
'Convert 4-byte Ascii string g to long integer.
'
DECLARE FUNCTION STRING Mkl (SLONG c)
'Convert long integer c to 4-byte Ascii string.
'
' **************************************************************************
'                             Obtaining output
' **************************************************************************
END EXPORT
DECLARE FUNCTION VOID Printf (STRING f, STRING g, STRING h, SLONG sw)
'Print prefix f, number string g and suffix h to screen and the logfile,
'set switch = 1 to attach a CrLf, switch = 2 to also attach length(g).
EXPORT
DECLARE FUNCTION VOID Printn (SLONG p, STRING f, STRING h, SLONG sw)
'Print large integer p with prefix f and suffix h using Printf switches.
'
DECLARE FUNCTION VOID Printr (SLONG p, SLONG q, SLONG r, STRING f, STRING h)
'Print rational p / q, pass decimal length in number r with 'Setl(r, len)'.
'
DECLARE FUNCTION VOID Prints (STRING f, SLONG sw)
'Print string f. Set switch = 1 to attach a CrLf, switch = 2 for two.
'
' **************************************************************************
'                        Basic arithmetic functions
' **************************************************************************
DECLARE FUNCTION SLONG Absf (SLONG p)
'Set the sign of number p to 1, returns the current value.
'
DECLARE FUNCTION VOID Add (SLONG p, SLONG q)
'Add p and q, the result is in p.
'
DECLARE FUNCTION VOID Chs (SLONG p)
'Change the sign of number p.
'
DECLARE FUNCTION VOID Dcr (SLONG p, SLONG a)
'Decrease large integer p with small positive a.
'
DECLARE FUNCTION VOID Divd (SLONG p, SLONG d, SLONG q)
'Divide p by d. The quotient is in q, the remainder in p.
'
DECLARE FUNCTION VOID Inc (SLONG p, SLONG a)
'Increase large integer p with small positive a.
'
DECLARE FUNCTION VOID Mult (SLONG p, SLONG q, SLONG r)
'Multiply p and q. The result is in p by default,
' 'Swp(p, r)' puts it in r.
'
DECLARE FUNCTION VOID Powr (SLONG p, SLONG k)
'Raise p to the power k, the result is in p.
'
DECLARE FUNCTION VOID Squ (SLONG p, SLONG q)
'Return the square of p in q.
'
DECLARE FUNCTION VOID Subt (SLONG p, SLONG q)
'Subtract q from p, the result is in p.
'To retain p, use 'Subt(q, p): Chs(q)'.
'
' **************************************************************************
'                          Comparison and copying
' **************************************************************************
DECLARE FUNCTION SLONG Cmp (SLONG p, SLONG q)
'Compare returns -1 if p < q, 0 if p = q, or 1 if p > q.
'
DECLARE FUNCTION SLONG Isf (SLONG p, SLONG a)
'Boolean: check if large integer p equals one-word value a.
'
DECLARE FUNCTION SLONG Ismin1 (SLONG p, SLONG m)
'Boolean: check if p = m - 1
'
DECLARE FUNCTION VOID Copyf (SLONG p, SLONG q)
'Copy number p to q.
'
DECLARE FUNCTION VOID Swp (SLONG p, SLONG q)
'Exchange numbers p and q.
'
' **************************************************************************
'                             Bit manipulation
' **************************************************************************
DECLARE FUNCTION VOID Andf (SLONG p, SLONG q)
'Bitwise logical p AND q, the result is in p.
'
DECLARE FUNCTION VOID Lsft (SLONG p, SLONG k)
'Large integer p is shifted left by k bits.
'
DECLARE FUNCTION VOID Rsft (SLONG p, SLONG k)
'Large integer p is shifted right by k bits.
'
' **************************************************************************
'                       Modular arithmetic functions
' **************************************************************************
DECLARE FUNCTION VOID Modbal (SLONG p, SLONG m)
'Balanced residue p modulo m: Abs(p) <= m \ 2.
'
DECLARE FUNCTION VOID Moddiv (SLONG p, SLONG m)
'Compute positive residue p modulo m.
'
DECLARE FUNCTION VOID Modmlt (SLONG p, SLONG q, SLONG m)
'Multiply p and q, then reduce modulo m. The result is in p.
'
DECLARE FUNCTION VOID Modpwr (SLONG p, SLONG q, SLONG m)
'Modular exponentiation: base p, exponent q, modulus m.
'The result is in p.
'
DECLARE FUNCTION VOID Modsqu (SLONG p, SLONG m)
'Square p, then reduce modulo m.
'
DECLARE FUNCTION SLONG Mp2 (SLONG p, SLONG a)
'Return large integer p modulo one-word power-of-2.
'
' **************************************************************************
'                        Number theoretic functions
' **************************************************************************
DECLARE FUNCTION VOID Bezout (SLONG p, SLONG q, SLONG d)
'Solution of the Diophantine equation px + qy = gcd(p, q).
'Returns positive x in p, y in q and gcd(p, q) in d.
'
DECLARE FUNCTION VOID Euclid (SLONG p, SLONG q, SLONG d)
'The extended Euclidean algorithm computes
'p^ -1 Mod q in p, q/ gcd in q and gcd(p, q) in d.
'
DECLARE FUNCTION VOID Fctl (SLONG p, SLONG a)
'Computes factorial a(a-1)21 in large integer p.
'
DECLARE FUNCTION VOID Gcd (SLONG p, SLONG q, SLONG d)
'Returns greatest common divisor(p, q) in d.
'
DECLARE FUNCTION SLONG IsPPrm (SLONG p, SLONG d, SLONG k)
'Boolean: check if p is probably prime with the Miller-Rabin test.
'k is the number of witnesses, d returns a small divisor, if any.
'
DECLARE FUNCTION SLONG IsSqr (SLONG p, SLONG q)
'Boolean: check if p is square. If true, the root is in q.
'
DECLARE FUNCTION VOID Isqrt (SLONG p, SLONG q)
'Integer square root of p with Heron's algorithm. Result is in q.
'
DECLARE FUNCTION SLONG Kronec (SLONG p, SLONG q, SLONG d)
'Kronecker's quadratic residuosity symbol (p/q) = 0, 1, or -1.
'Returns an odd gcd(p, q) in d, and 2 if it's even.
'
DECLARE FUNCTION VOID Lcm (SLONG p, SLONG q, SLONG d)
'Returns least common multiple(p, q) in d.
'
DECLARE FUNCTION SLONG Nxtprm (SLONG sw)
'Loop through primetable PrimFlgs.bin. Initiate with sw = 0
'and get the next prime with sw <> 0 for each successive call.
'
DECLARE FUNCTION VOID Triald (SLONG q, SLONG p, SLONG c)
'Trial divide p up to limit c. Primedivisors w/multiplicities
'are stored in list {q}, the remaining cofactor is returned in p.
'
' **************************************************************************
'                              Direct access
' **************************************************************************
DECLARE FUNCTION VOID EnQ (SLONG q, SLONG p, SLONG a)
'Store small p and one-word Abs(a) in sequential list {q}.
'As {q} contains negative record separators, an attempt to
'simply 'Printn q, ...' will crash your program. Instead use:
'
DECLARE FUNCTION SLONG ExQ (SLONG p, SLONG a, SLONG q, SLONG k)
'Get successive numbers p and Abs(a) from list {q} at offset k.
'Set k = 0 to initialize reading, returns zero if @end-of-list.
'Note that both a and k must be passed by reference.
'
DECLARE FUNCTION SLONG Gete (SLONG p, SLONG k)
'Return array element k of number p. This is a base $$MB digit.
'
DECLARE FUNCTION SLONG Getl (SLONG p)
'Return the length of number p measured in array elements.
'
DECLARE FUNCTION SLONG Gets (SLONG p)
'Return the sign of number p, Gets = -1 if p < 0, 1 otherwise.
'
DECLARE FUNCTION VOID Sete (SLONG p, SLONG k, SLONG a)
'Set binary array element k of number p to Abs(a).
'
DECLARE FUNCTION VOID Setl (SLONG p, SLONG a)
'Set the length of number array p to Abs(a).
'
DECLARE FUNCTION VOID Sets (SLONG p, SLONG a)
'Set the sign of number p to Sgn(a + 0.5).
'
' **************************************************************************
'                            Internal functions
' **************************************************************************
END EXPORT
DECLARE FUNCTION VOID Errorh (STRING f)
'Print an error message.
EXPORT
DECLARE FUNCTION SLONG Hp2 (SLONG p)
'Return 2^ (highest set bit in the leftmost word of p).
'
DECLARE FUNCTION VOID Lftj (SLONG p, SLONG k)
'Left-justify large integer p on from array element k.
'
DECLARE FUNCTION SLONG PrimCeil ()
'Return the upper limit of primetable PrimFlgs.bin.
'
DECLARE FUNCTION SINGLE Rnd (SLONG c)
'Initiate with c > 0, get the next random number in [0, 1) with c = 0.
'
END EXPORT
' **************************************************************************
'                             Internal variables
' **************************************************************************
SHARED SLONG i[], n[], lt, sg
'n[] holds the numbers, n[.., lt] their lengths,
'n[.., sg] the signs, the i[]'s are row indices.
SHARED SLONG ln[]
'Used in Decf and CnvSt only.
SHARED SLONG t0, t1, t2, t3 '          pointers to swap-rows
SHARED XLONG Lognr, Prmnr '            filehandles
EXPORT
' **************************************************************************
'                             Global constants
' **************************************************************************
$$ui = 172: $$uj = 5119 '              number array n[ui, uj] upper bounds
$$LB = 15: $$MB = 32768: $$MH = 16384 'binary storage base
$$M1 = 32767: $$M2 = 1073709056 '      masks MB - 1, (MB - 1)* MB
$$L10 = 0.221461873# '                 1/ 10log(MB)
$$LD = 8: $$MD = 100000000 '           decimal in/out base, MY < MB
$$MY = 10000&: $$Fx = 2147483629 '     fixed limit
' Maximal number length = uj / L10, approximately 9 * uj / 2.
'
' **************************************************************************
END EXPORT

FUNCTION SLONG DLLMain ()
SHARED SLONG i[], ln[], n[] '          called by default
SHARED SLONG t0, t1, t2, t3, lt, sg
   DIM n[$$ui + 4, $$uj + 2]
   DIM i[$$ui + 4], ln[$$uj + 2]
   FOR t = 0 TO $$ui + 4 '             initialize indices,
      i[t] = t: Letf(t, 0) '           set numbers to zero
   NEXT t
   t0 = $$ui + 1: t1 = $$ui + 2 '      set pointers
   t2 = $$ui + 3: t3 = $$ui + 4
   sg = $$uj + 1: lt = $$uj + 2
RETURN -1
END FUNCTION

FUNCTION VOID LargeInit (STRING g)
SHARED XLONG Lognr, Prmnr
   XstGetEnvironmentVariable(@"LARGEINT", @s$)
   IF s$ = "" THEN s$ = ".\\" '        working directory
   IF RIGHT$(s$, 1) <> "\\" THEN s$ = s$ + "\\"
   '
   Lognr = 0
   Prmnr = OPEN(s$ + "PrimFlgs.bin", $$RDSHARE)
   IF LEN(g) THEN
      Lognr = OPEN(s$ + g + ".log", $$WRNEW)
   END IF
   '
   XstGetSystemTime(@t)
   dum! = Rnd(t) '                     randomize(timer)
END FUNCTION

FUNCTION VOID Letf (SLONG p, SLONG c)
SHARED SLONG i[], n[], lt, sg
   n[i[p], sg] = SIGN(c)
   j = 0: c0 = ABS(c) '                SGN(0) = 1
   DO
      n[i[p], j] = c0 AND $$M1
      c0 = c0 >> $$LB: j = j + 1 '     split DWord c
   LOOP WHILE c0
   n[i[p], lt] = j
   n[i[p], j] = 0
END FUNCTION

FUNCTION VOID Rndf (SLONG p, SLONG k)
SHARED SLONG i[], n[], lt, sg
   k = ABS(k): f! = k * $$L10
   lp = INT(f!): m = INT((f! - lp) * $$LB)
   IF m THEN
      lp = lp + 1
   ELSE
      m = $$LB
   END IF
   IF lp >= $$uj THEN
      Errorh("overflow in sub Rndf")
      Letf(p, 1): EXIT FUNCTION
   END IF
   FOR j = 0 TO lp - 2
      n[i[p], j] = INT(Rnd(0) * $$MB)
   NEXT j
   m = m - 1: r = 1
   FOR j = 1 TO m: r = r * 2: NEXT
   m = r * 2: r = r + INT(Rnd(0) * (m - r))
   n[i[p], lt] = lp
   n[i[p], sg] = 1
   n[i[p], lp - 1] = r
   n[i[p], lp] = 0
END FUNCTION

FUNCTION VOID Term ()
SHARED XLONG Lognr, Prmnr
   CLOSE(Lognr) ' the files, then terminate
   CLOSE(Prmnr)
END FUNCTION

FUNCTION VOID Readst (SLONG p, STRING g)
SHARED SLONG i[], n[], t1, t3, lt, sg
   g = TRIM$(g)
   Letf(p, 0): k = LEN(g)
   IF (g = "0") OR (k = 0) THEN EXIT FUNCTION
   sw = 0: t = 0
   IF LEFT$(g, 1) = "-" THEN
      sw = -1: t = 1: k = k - 1
   END IF
   lp = k \ $$LD: m = k - lp * $$LD
   IF m = 0 THEN m = $$LD: lp = lp - 1
   k = t + 1: t = m
   Letf(t3, $$MD)
   FOR j = 0 TO lp
      IF j THEN
         Mult(p, t3, t1): t = $$LD '   decimal base t3
      END IF
      Letf(t1, SLONG(MID$(g, k, t)))
      Add(p, t1): k = k + t '          convert to base MB
   NEXT j
   Lftj(p, n[i[p], lt] - 1)
   IF sw THEN n[i[p], sg] = -1
END FUNCTION

FUNCTION SLONG Decf (SLONG p)
SHARED SLONG i[], ln[], n[]
SHARED SLONG t0, t1, t3, lt, sg
   ln[sg] = n[i[p], sg]
   Letf(t3, $$MY)
   Copyf(p, t0): j = 0
   DO
      Divd(t0, t3, t1) '               decimal base t3
      c = n[i[t0], 0] '                unrolled single digit division
      Divd(t1, t3, t0)
      ln[j] = n[i[t1], 0] * $$MY + c
      j = j + 1
   LOOP UNTIL Isf(t0, 0)
   ln[j] = 0: ln[lt] = j
   j = j - 1: c = ln[j]: j = j * $$LD
   DO
      c = c \ 10: j = j + 1
   LOOP WHILE c
   IF ln[sg] = -1 THEN j = j + 1
END FUNCTION j

FUNCTION VOID CnvSt (STRING g)
SHARED SLONG ln[], sg
   k = LEN(g): t = 0
   IF ln[sg] = -1 THEN
      g = STUFF$(g, "-", 1)
      t = 1: k = k - 1
   END IF
   lp = k \ $$LD: m = k - lp * $$LD
   IF m = 0 THEN m = $$LD: lp = lp - 1
   k = k + t + 1: t = $$LD
   FOR j = 0 TO lp
      s$ = LTRIM$(STR$(ln[j]))
      IF j = lp THEN t = m
      g = STUFF$(g, s$, k - LEN(s$), t)
      k = k - $$LD
   NEXT j
END FUNCTION

FUNCTION DOUBLE Logf (SLONG p)
SHARED SLONG i[], n[], lt
   t = n[i[p], lt] - 1
   nr# = n[i[p], t]
   FOR j = t - 1 TO t - 2 STEP -1
      IF j < 0 THEN EXIT FOR
      nr# = nr# * $$MB + n[i[p], j]
   NEXT j
RETURN Log(nr#) + (j + 1) * 10.3972077083992#
END FUNCTION

FUNCTION VOID Ratdec (STRING g, SLONG p, SLONG q)
SHARED SLONG i[], n[], t0, t1, t3
   c = LEN(g): lp = c \ $$LD: m = c - lp * $$LD
   IF m = 0 THEN m = $$LD: lp = lp - 1
   Letf(t3, $$MY): c = 1
   FOR j = 0 TO lp
      c0 = 0
      FOR t = 0 TO 1
	 Mult(p, t3, t1) '             dividend * MY
	 Divd(p, q, t0) '              partial quotient t0
	 Divd(t0, t3, t1) '            base MY digit
	 c0 = c0 * $$MY + n[i[t0], 0]
      NEXT t
      s$ = LTRIM$(STR$(c0))
      IF j < lp THEN
         c = c + $$LD
      ELSE
         s$ = CHR$('0', $$LD - LEN(s$)) + s$
         s$ = LEFT$(s$, m): c = c + m 'last digit
      END IF
      g = STUFF$(g, s$, c - LEN(s$), $$LD)
   NEXT j
END FUNCTION

FUNCTION SLONG Cvl (STRING g)
   s$ = NULL$(4): c = 0
   s$ = STUFF$(s$, g, 1, 4)
   FOR k = 3 TO 0 STEP -1
      c = c << 8 + s${k}
   NEXT k
END FUNCTION c

FUNCTION STRING Mkl (SLONG c)
   s$ = NULL$(4): c0 = c
   FOR k = 1 TO 4
      s$ = STUFF$(s$, CHR$(c0 AND 255), k)
      c0 = c0 >> 8
   NEXT k
END FUNCTION s$

FUNCTION VOID Printf (STRING f, STRING g, STRING h, SLONG sw)
SHARED XLONG Lognr
   SELECT CASE sw
   CASE 0
      PRINT f; g; h;
      IF Lognr THEN PRINT [Lognr], f; g; h;
   CASE 1
      PRINT f; g; h; CHR$(13)
      IF Lognr THEN PRINT [Lognr], f; g; h; CHR$(13)
   CASE ELSE
      k = LEN(g)
      IF LEFT$(g, 1) = "-" THEN k = k - 1
      s$ = "  [" + LTRIM$(STR$(k)) + "]"
      PRINT f; g; h; s$; CHR$(13)
      IF Lognr THEN PRINT [Lognr], f; g; h; s$; CHR$(13)
   END SELECT
END FUNCTION

FUNCTION VOID Printn (SLONG p, STRING f, STRING h, SLONG sw)
   k = Decf(p)
   g$ = CHR$('0', k)
   CnvSt(@g$)
   Printf(f, g$, h, sw)
END FUNCTION

FUNCTION VOID Printr (SLONG p, SLONG q, SLONG r, STRING f, STRING h)
SHARED SLONG i[], n[], t0, t1, lt
   lx = n[i[r], lt]
   Divd(p, q, r): k = Decf(r)
   g$ = CHR$('0', k): CnvSt(@g$)
   k = 1 - (LEN(h) = 0)
   IF Isf(p, 0) THEN
      Printf(f, g$, "", k)
   ELSE
      Printf(f, g$, "", 0)
      g$ = CHR$('0', lx)
      Ratdec(@g$, p, q)
      Printf(".", g$, h, k)
   END IF
END FUNCTION

FUNCTION VOID Prints (STRING f, SLONG sw)
SHARED XLONG Lognr
   SELECT CASE sw
   CASE 0
      PRINT f;
      IF Lognr THEN PRINT [Lognr], f;
   CASE 1
      PRINT f; CHR$(13)
      IF Lognr THEN PRINT [Lognr], f; CHR$(13)
   CASE ELSE
      PRINT f; CHR$(13): PRINT CHR$(13)
      IF Lognr THEN
	 PRINT [Lognr], f; CHR$(13): PRINT [Lognr], CHR$(13)
      END IF
   END SELECT
END FUNCTION

FUNCTION SLONG Absf (SLONG p)
SHARED SLONG i[], n[], sg
   x = n[i[p], sg]
   n[i[p], sg] = 1
END FUNCTION x

FUNCTION VOID Add (SLONG p, SLONG q)
SHARED SLONG i[], n[], sg
   n[i[q], sg] = -n[i[q], sg]
   Subt(p, q)
   n[i[q], sg] = -n[i[q], sg]
END FUNCTION

FUNCTION VOID Chs (SLONG p)
SHARED SLONG i[], n[], lt, sg
   r = n[i[p], 1] + n[i[p], 0] = 0
   IF NOT (r AND (n[i[p], lt] = 1)) THEN
      n[i[p], sg] = -n[i[p], sg]
   END IF
END FUNCTION

FUNCTION VOID Dcr (SLONG p, SLONG a)
SHARED SLONG i[], n[], t2, sg
   a = ABS(a)
   c = n[i[p], 0] - n[i[p], sg] * a
   IF (c > 0) AND (c < $$MB) THEN
      n[i[p], 0] = c
   ELSE
      IF c = 0 THEN
         n[i[p], 0] = 0
         IF Isf(p, 0) THEN n[i[p], sg] = 1
      ELSE
         Letf(t2, a)
         Subt(p, t2)
      END IF
   END IF
END FUNCTION

FUNCTION VOID Divd (SLONG p, SLONG d, SLONG q)
SHARED SLONG i[], n[], t2, lt, sg
GIANT nr
   s = n[i[p], sg] * n[i[d], sg]
   n[i[q], sg] = s
   l0 = n[i[p], lt]: l1 = n[i[d], lt]
   m = l0 - l1
   IF m < 0 THEN
      Letf(q, 0)
      n[i[q], sg] = s: EXIT FUNCTION
   END IF
   IF l1 = 1 THEN
      IF Isf(d, 0) THEN
         Errorh("div. by zero in sub Divd")
         Letf(q, 0): EXIT FUNCTION
      END IF
      t = l0 - 1: c = 0
      cs = n[i[d], 0] '                short divisor
      FOR j = t TO 0 STEP -1
         c0 = c << $$LB OR n[i[p], j] 'paste carry w/dividend
         c = c0 \ cs: n[i[q], j] = c ' save quotient
         c = c0 - c * cs '             remainder
      NEXT j
      n[i[p], lt] = 1
      n[i[p], 0] = c
      n[i[p], 1] = 0: Lftj(q, t)
   ELSE
      cs = n[i[d], l1 - 2] + n[i[d], l1 - 1] << $$LB
      n[i[t2], sg] = n[i[p], sg]
      FOR j = 0 TO m '                 MSB(divisor)
         n[i[t2], j] = n[i[p], j]
      NEXT j
      '
      FOR k = m TO 0 STEP -1
         t = l1 + k
         nr = n[i[p], t] '             MSB(intermediate dividend)
         FOR j = t - 1 TO t - 2 STEP -1
            nr = nr << $$LB + n[i[p], j]
         NEXT j
         c2 = INT(nr / cs) '           estimate partial quotient
         IF c2 > $$M1 THEN c2 = $$M1
         n[i[q], k] = c2
         IF c2 THEN
            DO
               c = $$MB
               FOR j = 0 TO l1 '       subtract scaled divisor,
                  c = c + $$M2 - c2 * n[i[d], j] + n[i[p], j + k]
                  n[i[t2], j + k] = c AND $$M1: c = c >> $$LB
               NEXT j
               IF c = $$MB THEN
                  SWAP i[p], i[t2] '   swap dividend & remainder
                  EXIT DO
               ELSE
                  IF c2 > 1 THEN '     or decrease quotient
                     c2 = c2 - 1
                     n[i[q], k] = c2
                  ELSE
                     n[i[q], k] = 0: EXIT DO
                  END IF
               END IF
            LOOP
         END IF
      NEXT k
      Lftj(p, l1 - 1)
      Lftj(q, m)
   END IF
END FUNCTION

FUNCTION VOID Inc (SLONG p, SLONG a)
SHARED SLONG i[], n[], t2, sg
   a = ABS(a)
   c = n[i[p], 0] + n[i[p], sg] * a
   IF (c > 0) AND (c < $$MB) THEN
      n[i[p], 0] = c
   ELSE
      IF c = 0 THEN
         n[i[p], 0] = 0
         IF Isf(p, 0) THEN n[i[p], sg] = 1
      ELSE
         Letf(t2, a): Add(p, t2)
      END IF
   END IF
END FUNCTION

FUNCTION VOID Mult (SLONG p, SLONG q, SLONG r)
SHARED SLONG i[], n[], lt, sg
   l0 = n[i[p], lt]: l1 = n[i[q], lt]
   lx = l0 + l1 - 1
   IF lx >= $$uj THEN
      Errorh("overflow in sub Mult")
      Letf(r, 1): EXIT FUNCTION
   END IF
   sw = l0 < l1
   IF sw THEN
      SWAP i[p], i[q]: SWAP l0, l1
   END IF
   '
   c = 0: c2 = n[i[q], 0]
   FOR j = 0 TO l0
      c = c + c2 * n[i[p], j] '        initiate destination
      n[i[r], j] = c AND $$M1: c = c >> $$LB
   NEXT j
   FOR j = l0 + 1 TO lx: n[i[r], j] = 0: NEXT
   '
   FOR m = 1 TO l1 - 1 STEP 2
      FOR t = 0 TO 1
         k = t + m: c = n[i[q], k]
         FOR j = 0 TO l0 - 1 '         multiply,
            n[i[r], j + k] = c * n[i[p], j] + n[i[r], j + k]
         NEXT j
         IF k = l1 - 1 THEN EXIT FOR
      NEXT t
      c = 0
      FOR j = m TO k + l0
         c = c + n[i[r], j] '          normalize,
         n[i[r], j] = c AND $$M1: c = c >> $$LB
      NEXT j
   NEXT m
   '
   Lftj(r, lx) '                       and resize
   n[i[r], sg] = n[i[p], sg] * n[i[q], sg]
   IF sw THEN SWAP i[p], i[q]
   SWAP i[p], i[r]
END FUNCTION

FUNCTION VOID Powr (SLONG p, SLONG k)
SHARED SLONG t2, t3
   Letf(t3, k): Letf(t2, 0)
   Modpwr(p, t3, t2)
END FUNCTION

FUNCTION VOID Squ (SLONG p, SLONG q)
SHARED SLONG i[], n[], lt, sg
   l0 = n[i[p], lt]
   lx = l0 + l0 - 1
   IF lx >= $$uj THEN
      Errorh("overflow in sub Squ")
      Letf(q, 1): EXIT FUNCTION
   END IF
   IF l0 = 1 THEN
      c = n[i[p], 0] * n[i[p], 0]
      Letf(q, c)
   ELSE
      j = 0: n[i[q], sg] = 1
      FOR k = 0 TO l0 - 1 '            initialize destination
         c = n[i[p], k] * n[i[p], k]
         n[i[q], j] = c AND $$M1: c = c >> $$LB
         n[i[q], j + 1] = c: j = j + 2
      NEXT k
      '
      FOR k = 1 TO l0 - 1
         c = 0: c2 = n[i[p], k] * 2& ' add mixed terms
         FOR j = 0 TO k - 1
            c = c + c2 * n[i[p], j] + n[i[q], j + k]
            n[i[q], j + k] = c AND $$M1: c = c >> $$LB
         NEXT j
         m = k + k
         FOR j = m TO m + 1
            c = c + n[i[q], j]
            n[i[q], j] = c AND $$M1: c = c >> $$LB
         NEXT j
      NEXT k
      Lftj(q, lx)
   END IF
END FUNCTION

FUNCTION VOID Subt (SLONG p, SLONG q)
SHARED SLONG i[], n[], lt, sg
   IF n[i[p], sg] = n[i[q], sg] THEN ' subtract
      ix = i[p]: im = i[q]
      s = n[ix, lt] - n[im, lt]
      IF s = 0 THEN
         FOR j = n[ix, lt] - 1 TO 0 STEP -1
            s = n[ix, j] - n[im, j]
            IF s THEN EXIT FOR
         NEXT j
         IF s = 0 THEN Letf(p, 0): EXIT FUNCTION
      END IF
      IF s < 0 THEN '                     p:= -(q - p)
         SWAP ix, im
         n[i[p], sg] = -n[i[p], sg]
      END IF
      lx = n[ix, lt]: lm = n[im, lt]
      FOR j = lm + 1 TO lx: n[im, j] = 0: NEXT
      c = $$MB
      FOR j = 0 TO lx
         c = c + n[ix, j] + $$M2 - n[im, j]
         n[i[p], j] = c AND $$M1: c = c >> $$LB
      NEXT j
   ELSE '                                  add
      lx = n[i[p], lt]: lm = n[i[q], lt]: im = i[q]
      IF lx < lm THEN SWAP lx, lm: im = i[p]
      FOR j = lm + 1 TO lx: n[im, j] = 0: NEXT
      c = 0
      FOR j = 0 TO lx
         c = c + n[i[p], j] + n[i[q], j]
         n[i[p], j] = c AND $$M1: c = c >> $$LB
      NEXT j
   END IF
   Lftj(p, lx)
END FUNCTION

FUNCTION SLONG Cmp (SLONG p, SLONG q)
SHARED SLONG i[], n[], lt, sg
   x = n[i[p], sg]
   IF x = n[i[q], sg] THEN '           equal signs
      s = n[i[p], lt] - n[i[q], lt]
      IF s = 0 THEN '                  equal lengths
         FOR j = n[i[p], lt] - 1 TO 0 STEP -1
            s = n[i[p], j] - n[i[q], j] ' L=>R check
            IF s THEN EXIT FOR
         NEXT j
         IF s = 0 THEN x = 0
      END IF
      IF s < 0 THEN x = -x
   END IF
END FUNCTION x

FUNCTION SLONG Isf (SLONG p, SLONG a)
SHARED SLONG i[], n[], lt, sg
   r = n[i[p], 1] + n[i[p], 0] = ABS(a)
   s = SGN(a): IF a = 0 THEN s = n[i[p], sg]
   r = r AND (n[i[p], sg] = s)
   x = r AND (n[i[p], lt] = 1)
END FUNCTION x

FUNCTION SLONG Ismin1 (SLONG p, SLONG m)
SHARED SLONG i[], n[], lt, sg
   IF (n[i[p], lt] > n[i[m], lt]) THEN
      IF n[i[p], sg] = 1 THEN
         RETURN 0: EXIT FUNCTION
      END IF
   END IF
   Inc(p, 1)
   x = Cmp(p, m) = 0
   Dcr(p, 1)
END FUNCTION x

FUNCTION VOID Copyf (SLONG p, SLONG q)
SHARED SLONG i[], n[], lt, sg
   FOR j = 0 TO n[i[p], lt]
      n[i[q], j] = n[i[p], j]
   NEXT j
   n[i[q], sg] = n[i[p], sg]
   n[i[q], lt] = n[i[p], lt]
END FUNCTION

FUNCTION VOID Swp (SLONG p, SLONG q)
SHARED SLONG i[]
   SWAP i[p], i[q]
END FUNCTION

FUNCTION VOID Andf (SLONG p, SLONG q)
SHARED SLONG i[], n[], lt, sg
   lx = n[i[p], lt] - 1
   lm = n[i[q], lt] - 1
   IF lx < lm THEN SWAP lx, lm
   FOR j = 0 TO lm
      n[i[p], j] = n[i[p], j] AND n[i[q], j]
   NEXT j
   Lftj(p, lm)
   IF n[i[p], sg] = -1 THEN
      n[i[p], sg] = n[i[q], sg]
   END IF
END FUNCTION

FUNCTION VOID Lsft (SLONG p, SLONG r)
SHARED SLONG i[], n[], lt
   k = r \ $$LB: m = r - k * $$LB
   t = n[i[p], lt]
   IF m > 0 THEN
      c = 0
      FOR j = 0 TO t
         c = n[i[p], j] << m OR c '    ShL element, paste carry
         n[i[p], j] = c AND $$M1 '     mask off high bits
         c = c >> $$LB
      NEXT j
      IF n[i[p], t] THEN t = t + 1
      n[i[p], t] = 0
   END IF
   IF k > 0 THEN
      t = t + k
      IF t >= $$uj THEN
         Errorh("overflow in sub Lsft")
         Letf(p, 1): EXIT FUNCTION
      END IF
      IF Isf(p, 0) THEN EXIT FUNCTION
      FOR j = t TO k STEP -1
         n[i[p], j] = n[i[p], j - k] ' ShL array
      NEXT j
      FOR j = 0 TO k - 1: n[i[p], j] = 0: NEXT
   END IF
   n[i[p], lt] = t
END FUNCTION

FUNCTION VOID Rsft (SLONG p, SLONG r)
SHARED SLONG i[], n[], lt
   k = r \ $$LB: m = r - k * $$LB
   t = n[i[p], lt] - 1
   IF m > 0 THEN
      c = 0
      FOR j = t TO 0 STEP -1
         c0 = c << $$LB OR n[i[p], j] ' paste carry
         c = c0 AND $$M1 '              save next
         n[i[p], j] = c0 >> m AND $$M1 'ShR element
      NEXT j
      IF (n[i[p], t] = 0) AND (t > 0) THEN t = t - 1
   END IF
   IF k > 0 THEN
      t = t - k
      IF t < 0 THEN Letf(p, 0): EXIT FUNCTION
      FOR j = 0 TO t
         n[i[p], j] = n[i[p], j + k] '  ShR array
      NEXT j
   END IF
   n[i[p], lt] = t + 1
   n[i[p], t + 1] = 0
END FUNCTION

FUNCTION VOID Modbal (SLONG p, SLONG m)
SHARED SLONG i[], n[], t1, sg
   s = n[i[p], sg]
   z = n[i[m], sg]
   n[i[p], sg] = 1
   n[i[m], sg] = 1
   Divd(p, m, t1)
   Copyf(m, t1): Rsft(t1, 1)
   IF NOT Isf(t1, 0) THEN
      r = Cmp(p, t1)
      SELECT CASE r
      CASE 1 '                         Abs(p) > m \ 2
	 Subt(p, m): s = -s '          balance p mod m
      CASE 0
	 IF (n[i[m], 0] AND 1) = 0 THEN s = z
      END SELECT
   END IF
   IF Isf(p, 0) THEN s = 1
   n[i[p], sg] = s
   n[i[m], sg] = z
END FUNCTION

FUNCTION VOID Moddiv (SLONG p, SLONG m)
SHARED SLONG i[], n[], t1, sg
   Divd(p, m, t1)
   IF n[i[p], sg] = -1 THEN '          make positive residue
      IF Isf(p, 0) THEN
         n[i[p], sg] = 1
      ELSE
         s = n[i[m], sg]
         n[i[m], sg] = -1
         Subt(p, m)
         n[i[m], sg] = s
      END IF
   END IF
END FUNCTION

FUNCTION VOID Modmlt (SLONG p, SLONG q, SLONG m)
SHARED SLONG t1
   Mult(p, q, t1)
   Divd(p, m, t1)
END FUNCTION

FUNCTION VOID Modpwr (SLONG p, SLONG q, SLONG m)
SHARED SLONG i[], n[], t0, t1, lt, sg
   IF Isf(q, 0) THEN
      Letf(p, 1): EXIT FUNCTION
   END IF
   sw = NOT Isf(m, 0) '                enable reduction mod m
   IF sw THEN
      s = n[i[m], sg]: n[i[m], sg] = 1
      Moddiv(p, m) '                   initial reduction
   ELSE
      s = n[i[p], sg]: n[i[p], sg] = 1
      IF (n[i[q], 0] AND 1) = 0 THEN s = 1
   END IF
   '
   k = n[i[q], lt] - 1
   FOR j = k TO 0 STEP -1 '           L=>R binary exponentiation
      a = n[i[q], j]: a0 = $$MH '      unsigned bitvector q
      IF j = k THEN
         Copyf(p, t0): a0 = Hp2(q) '   handle highest set bit(q)
         a = a AND (a0 - 1): a0 = a0 \ 2
      END IF
      DO WHILE a0
         Squ(t0, t1)
         SWAP i[t0], i[t1] '           square t0
         IF sw THEN Divd(t0, m, t1) '  reduce Mod m
         IF a AND a0 THEN
            Mult(t0, p, t1) '          t0 times base p
            IF sw THEN Divd(t0, m, t1)
         END IF
         a = a AND (a0 - 1): a0 = a0 \ 2 ' read bit
      LOOP
   NEXT j
   SWAP i[p], i[t0]
   IF sw THEN
      n[i[m], sg] = s
   ELSE
      n[i[p], sg] = s
   END IF
END FUNCTION

FUNCTION VOID Modsqu (SLONG p, SLONG m)
SHARED SLONG i[], t1
   Squ(p, t1)
   SWAP i[p], i[t1]
   Divd(p, m, t1)
END FUNCTION

FUNCTION SLONG Mp2 (SLONG p, SLONG a)
SHARED SLONG i[], n[]
   m = n[i[p], 0] AND (a - 1)
   IF n[i[p], -1] = -1 THEN
      m = (a - m) AND (a - 1)
   END IF
END FUNCTION m

FUNCTION VOID Bezout (SLONG p, SLONG q, SLONG d)
SHARED SLONG i[], t0, t3
   Copyf(p, t3)
   Euclid(p, q, d)
   IF NOT Isf(d, 1) THEN
      Divd(t3, d, t0)
      SWAP i[t3], i[t0]
   END IF
   Mult(t3, p, t0) '                       p * p^-1)
   Dcr(t3, 1): Chs(t3) '              (1 -
   Divd(t3, q, t0) '           q^-1 =                / q
   SWAP i[q], i[t0]
END FUNCTION

FUNCTION VOID Euclid (SLONG p, SLONG q, SLONG d)
SHARED SLONG i[], n[], t0, t1, t2, sg
   IF Isf(q, 0) THEN
      Copyf(p, d): EXIT FUNCTION
   END IF
   s = n[i[q], sg]: n[i[q], sg] = 1
   IF n[i[p], sg] = -1 THEN Moddiv(p, q)
   IF Isf(p, 0) THEN
      SWAP i[q], i[d]
      Letf(q, 1): EXIT FUNCTION
   END IF
   '
   Letf(d, 1): Letf(t0, 0)
   DO
      SWAP i[p], i[q]
      SWAP i[d], i[t0]
      Divd(p, q, t1) '                 Euclidean division & inversion
      Mult(t1, t0, t2): Subt(d, t1) '  r_t = r_t-2 - q_t * r_t-1
   LOOP UNTIL Isf(p, 0)
   SWAP i[p], i[t0]
   SWAP i[q], i[d]
   IF n[i[p], sg] = -1 THEN Add(p, q)
   n[i[q], sg] = s
   n[i[d], sg] = 1
END FUNCTION

FUNCTION VOID Fctl (SLONG p, SLONG a)
SHARED SLONG i[], n[], t1, t2, lt
GIANT nr
   IF a < 3 THEN
      IF a < 0 THEN
         Errorh("illegal argument sub Fctl")
         Letf(p, 1): EXIT FUNCTION
      END IF
      IF a < 2 THEN a = 1
      Letf(p, a): EXIT FUNCTION
   END IF
   Letf(p, 1): r = 1
   DO
      nr = r
      DO
         r = r + 1
         c = SLONG(nr): nr = nr * r
      LOOP UNTIL (nr > $$Fx) OR (r > a)
      Letf(t1, c) '                     partial product
      IF n[i[p], lt] + n[i[t1], lt] >= $$uj THEN
         Errorh("overflow in sub Fctl")
         Letf(p, 1): EXIT FUNCTION
      END IF
      Mult(p, t1, t2)
   LOOP UNTIL r > a
END FUNCTION

FUNCTION VOID Gcd (SLONG p, SLONG q, SLONG d)
SHARED SLONG i[], n[], t0, t1, sg
   Copyf(p, d): Copyf(q, t0)
   DO UNTIL Isf(t0, 0)
      SWAP i[t0], i[d]
      Divd(t0, d, t1)
   LOOP
   n[i[d], sg] = 1
END FUNCTION

FUNCTION SLONG IsPPrm (SLONG p, SLONG d, SLONG w)
SHARED SLONG i[], n[], t0, t1, t3, lt, sg
   x = -1
   IF (n[i[p], sg] = -1) OR Isf(p, 1) THEN
      x = 0: sw = -1: GOTO jmp
   END IF
   sw = (n[i[d], sg] = -128) AND (n[i[p], lt] > 1)
   IF NOT sw THEN '                       try small divisors
      Letf(d, 2): a = 2: df = 1
      k = n[i[p], lt] * 51
      IF k > 1547 THEN k = 1547
      FOR j = 1 TO k
         Copyf(p, t0)
         Divd(t0, d, t1)
         IF Isf(t0, 0) THEN '             d | N
            x = Isf(t1, 1)
            sw = 0: GOTO jmp
         END IF
         IF Cmp(d, t1) = 1 THEN GOTO jmp 'd > sqrt(N)
         IF sw THEN
            DO
               df = 6 - df: a = a + df '  sieve multiples of 2, 3 and 5
	    LOOP UNTIL a MOD 5
         ELSE
            a = a + df: df = df * 2: sw = a = 5
         END IF
         n[i[d], 0] = a
      NEXT j
   END IF
   '
   Copyf(p, d) '                       Miller-Rabin test
   Dcr(d, 1): k = 0 '                  determine N - 1 = (2^ k) * odd(m)
   DO
      k = k + 1: Rsft(d, 1)
   LOOP UNTIL n[i[d], 0] AND 1
   a = 5 + INT(Rnd(0) * 313) '         random witness
   r = a MOD 6
   IF r < 3 THEN
      a = a + 1 - r: df = 2
   ELSE
      a = a + 5 - r: df = 4
   END IF
   FOR r = 1 TO w
      Letf(t3, a)
      Modpwr(t3, d, p) '               if N is prime then
      x = Isf(t3, 1) OR Ismin1(t3, p) 'a^ m Mod N = 1 or
      IF x = 0 THEN '         (a^ m)^ (2^ j) Mod N = -1
         FOR j = 1 TO k - 1 '   for some j in [0, k - 1]
            Modsqu(t3, p)
            IF Ismin1(t3, p) THEN x = -1: EXIT FOR
         NEXT j
         IF x = 0 THEN GOTO jmp
      END IF
      DO
         df = 6 - df: a = a + df
      LOOP UNTIL a MOD 5
   NEXT r
jmp:
   IF x OR sw THEN Letf(d, 1) '        Else return small factor d
END FUNCTION x

FUNCTION SLONG IsSqr (SLONG p, SLONG q)
'Square test from Henri Cohen's `Computational Number Theory', 1.7.2
SHARED SLONG i[], n[], t0, t1
STATIC SLONG q11[], q63[], q64[], q65[], sw
IF NOT sw THEN
   DIM q11[10], q63[62], q64[63], q65[64]
   FOR k = 0 TO 32
      k2 = k * k
      q11[k2 MOD 11] = -1
      q63[k2 MOD 63] = -1
      q64[k2 AND 63] = -1
      q65[k2 MOD 65] = -1
   NEXT k: sw = -1
END IF
   x = 0
   IF q64[n[i[p], 0] AND 63] THEN
      Copyf(p, t0)
      Letf(q, 45045)
      Divd(t0, q, t1)
      c = n[i[t0], 0] + n[i[t0], 1] << $$LB
      IF q63[c MOD 63] THEN
         IF q65[c MOD 65] THEN
            IF q11[c MOD 11] THEN
               Isqrt(p, q): Squ(q, t1) ' p is possibly square
               x = Cmp(p, t1) = 0
            END IF
         END IF
      END IF
   END IF
END FUNCTION x

FUNCTION VOID Isqrt (SLONG p, SLONG q)
SHARED SLONG i[], n[], t0, t1, lt, sg
   IF n[i[p], sg] = -1 THEN
      Errorh("illegal argument Isqrt")
      Letf(q, 0): EXIT FUNCTION
   END IF
   IF Isf(p, 0) OR Isf(p, 1) THEN
      Copyf(p, q): EXIT FUNCTION
   END IF
   k = n[i[p], lt] - 1
   c = n[i[p], k]: r! = 32 '           argument a in p
   DO
      tr! = c / r!
      r! = (r! + tr!) / 2 '            compute MSB(sqrt)
   LOOP UNTIL ABS(tr! - r!) < 1
   l1 = 1 + k \ 2
   n[i[q], lt] = l1 '                  compose seed x0 in q
   n[i[q], sg] = 1
   FOR j = 0 TO l1: n[i[q], j] = 0: NEXT
   s = (k AND 1) * 180 '               int(sqrt($$MB))
   n[i[q], l1 - 1] = SLONG(r! * (1 + s))
   '
   DO '                                Heron's algorithm
      Copyf(p, t0)
      Divd(t0, q, t1) '                a / x0
      Add(q, t1): Rsft(q, 1) '         x1:= (x0 + a/x0) / 2
      Subt(t1, q)
      s = n[i[t1], 1] + n[i[t1], 0] < 2
      s = s AND (n[i[t1], lt] = n[i[t1], sg])
   LOOP UNTIL s '                      If a/x0 - x1 = 0 Or 1 Then Exit
END FUNCTION

FUNCTION SLONG Kronec (SLONG p, SLONG q, SLONG d)
SHARED SLONG i[], n[], t0, t1, sg
x = 1
   Copyf(p, t0)
   Copyf(q, t1)
   IF (n[i[t1], 0] AND 1) = 0 THEN '   even(N)
      IF Isf(t1, 0) THEN '             (a/0)
         n[i[t0], sg] = 1
	 IF NOT Isf(t0, 1) THEN x = 0
	 SWAP i[d], i[t0]: GOTO jump
      END IF
      IF (n[i[t0], 0] AND 1) = 0 THEN ' both even
	 Letf(d, 2) '                  least common divisor 2
	 IF Isf(t0, 0) THEN SWAP i[d], i[t1]
	 x = 0: GOTO jump
      END IF
      '
      r = 0
      DO
	 r = r + 1: Rsft(t1, 1)
      LOOP UNTIL n[i[t1], 0] AND 1 '   make odd(N)
      IF r AND 1 THEN
	 r = n[i[t0], 0] AND 7
	 IF (r = 3) OR (r = 5) THEN x = -x
      END IF
   END IF
   '
   IF n[i[t0], sg] = -1 THEN '         negative(a)
      r = n[i[t1], 0] AND 3
      IF n[i[t1], sg] = -1 THEN r = (4 - r) AND 3
      IF r = 3 THEN x = -x '           N mod 4 = 3
      n[i[t0], sg] = 1 '               Abs(a)
   END IF
   n[i[t1], sg] = 1 '                  Abs(N)
   '
   DO
      IF (n[i[t0], 0] AND 1) = 0 THEN ' even(a)
	 IF Isf(t0, 0) THEN
	    IF NOT Isf(t1, 1) THEN x = 0
	    SWAP i[d], i[t1]: EXIT DO ' return N in d
	 END IF
	 r = 0
	 DO
	    r = r + 1: Rsft(t0, 1)
	 LOOP UNTIL n[i[t0], 0] AND 1 ' make odd(a)
	 IF r AND 1 THEN
	    r = n[i[t1], 0] AND 7
	    IF (r = 3) OR (r = 5) THEN x = -x
	 END IF
      END IF
      r = n[i[t0], 0] AND n[i[t1], 0]
      IF (r AND 2) = 2 THEN x = -x '   reciprocity
      SWAP i[t0], i[t1]
      Divd(t0, t1, d)
   LOOP
jump:
END FUNCTION x

FUNCTION VOID Lcm (SLONG p, SLONG q, SLONG d)
SHARED SLONG i[], t0, t1
   Gcd(p, q, d)
   IF Isf(d, 1) THEN
      Mult(p, q, d): SWAP i[p], i[d]
   ELSE
      Copyf(p, t0)
      Divd(t0, d, t1)
      Mult(t1, q, d): SWAP i[t1], i[d]
   END IF
END FUNCTION

FUNCTION SLONG Nxtprm (SLONG sw)
SHARED XLONG Prmnr
STATIC STRING b4
STATIC SLONG c, cb, cpl, dc, fl, j
IF sw = 0 THEN
   b4 = NULL$(4)
   c = 2: dc = 1: fl = 0: j = 0 '      initialize
   cpl = LOF(Prmnr): SEEK(Prmnr, 0)
ELSE
   IF fl THEN
      DO
         dc = 6 - dc: c = c + dc '     skip multiples of 2 and 3
         IF j = 0 THEN
            j = 30
            IF POF(Prmnr) < cpl THEN
               READ [Prmnr], b4 '      next bitvector in PrimFlgs.bin
               cb = Cvl(@b4)
            ELSE
               cb = 469171647 '        5-folds excluded
            END IF
         END IF
         j = j - 1
         r = cb AND 1: cb = cb \ 2 '   read bit
      LOOP UNTIL r
   ELSE
      c = c + dc: dc = dc * 2: fl = c = 5
   END IF
END IF
END FUNCTION c

FUNCTION VOID Triald (SLONG q, SLONG p, SLONG c)
SHARED SLONG i[], n[], t0, t1, t3, lt
   n[i[q], lt] = 0 '                      clear list {q}
   Letf(t3, Nxtprm(0))
   DO
      FOR k = 0 TO 2147483646
	 Copyf(p, t0): Divd(t0, t3, t1) ' trial divide p
	 IF NOT Isf(t0, 0) THEN EXIT FOR ' remainder > 0
	 SWAP i[p], i[t1] '               running quotient p:= p/dr
      NEXT k
      IF k > 0 THEN
	 IF k < 2147483647 THEN
	    EnQ(q, t3, k) '               store pfactor
	 ELSE
	    Errorh("exp overflow in sub Triald")
	    EXIT FUNCTION
	 END IF
      END IF
      r = Cmp(t3, t1) = 1 '               divisor > Isqrt(p)
      cp = Nxtprm(1): Letf(t3, cp) '      next prime
   LOOP UNTIL r OR (cp > c)
   IF r AND NOT Isf(p, 1) THEN
      EnQ(q, p, 1): Letf(p, 1)
   END IF
END FUNCTION

FUNCTION VOID EnQ (SLONG q, SLONG p, SLONG k)
SHARED SLONG i[], n[], lt
   lq = n[i[q], lt]
   lx = lq + n[i[p], lt]
   IF lx < $$uj THEN
      FOR j = lq TO lx - 1
	 n[i[q], j] = n[i[p], j - lq]
      NEXT j
      IF k = 0 THEN k = 1
      n[i[q], lx] = -ABS(k) '          end-of-record
      n[i[q], lt] = lx + 1
   ELSE
      Errorh("list overflow in sub EnQ")
   END IF
END FUNCTION

FUNCTION SLONG ExQ (SLONG p, SLONG k, SLONG q, SLONG j)
SHARED SLONG i[], n[], lt, sg
   IF j < n[i[q], lt] THEN
      x = -1: t = j + 1
      DO WHILE j < n[i[q], lt]
         s = n[i[q], j]: j = j + 1
         IF s < 0 THEN EXIT DO '       end-of-record
         n[i[p], j - t] = s
      LOOP
      n[i[p], sg] = 1: k = ABS(s)
      Lftj(p, j - t - 1)
   ELSE
      x = 0
   END IF
END FUNCTION x

FUNCTION SLONG Gete (SLONG p, SLONG j)
SHARED SLONG i[], n[]
END FUNCTION n[i[p], j]

FUNCTION SLONG Getl (SLONG p)
SHARED SLONG i[], n[], lt
END FUNCTION n[i[p], lt]

FUNCTION SLONG Gets (SLONG p)
SHARED SLONG i[], n[], sg
END FUNCTION n[i[p], sg]

FUNCTION VOID Sete (SLONG p, SLONG j, SLONG a)
SHARED SLONG i[], n[]
   n[i[p], j] = ABS(a)
END FUNCTION

FUNCTION VOID Setl (SLONG p, SLONG a)
SHARED SLONG i[], n[], lt
   n[i[p], lt] = ABS(a)
END FUNCTION

FUNCTION VOID Sets (SLONG p, SLONG a)
SHARED SLONG i[], n[], sg
   n[i[p], sg] = SIGN(a)
END FUNCTION

FUNCTION VOID Errorh (STRING g)
   Printf(CHR$(13), "Error: ", g, 1)
END FUNCTION

FUNCTION SLONG Hp2 (SLONG p)
SHARED SLONG i[], n[], lt
   a = n[i[p], n[i[p], lt] - 1]
   a0 = $$MH
   DO UNTIL a AND a0
      a0 = a0 \ 2
   LOOP
END FUNCTION a0

FUNCTION VOID Lftj (SLONG p, SLONG k)
SHARED SLONG i[], n[], lt
   t = 1
   FOR j = k TO 0 STEP -1
      IF n[i[p], j] THEN t = j + 1: EXIT FOR
   NEXT j
   n[i[p], lt] = t
   n[i[p], t] = 0
END FUNCTION

FUNCTION SLONG PrimCeil ()
SHARED XLONG Prmnr
STRING b4
   c = 0
   IF LOF(Prmnr) > 3 THEN
      b4 = NULL$(4)
      SEEK(Prmnr, 0)
      READ [Prmnr], b4
      IF Cvl(@b4) = 192331199 THEN '   valid primelist
         c = 5 + (LOF(Prmnr) \ 4) * 90
      END IF
   END IF
RETURN c
END FUNCTION

FUNCTION SINGLE Rnd (SLONG c)
STATIC ULONG s
   IF c THEN s = ULONG(c)
   s = (1366 * s + 150889) MOD 714025
END FUNCTION SINGLE(s) / 714025
END PROGRAM
