'Subject: Approximates a real root of integral polynomial f(x) = 0
'         with Lagrange's 1769 continued fractions method.
'Author : Sjoerd.J.Schaper
'Date   : 21-12-2003
'Code   : XBasic
PROGRAM	"cnfcroot"
VERSION	"01.00"
IMPORT	"bignumXB"
IMPORT	"xst"

DECLARE FUNCTION VOID Main()
DECLARE FUNCTION VOID Coeff()
DECLARE FUNCTION VOID Evalf (SLONG p, SLONG q)

FUNCTION VOID Main()
SHARED SLONG m[], n, t2
t0 = 0: t1 = 1: t2 = 2: a = 3
p0 = 4: p1 = 5: q0 = 6: q1 = 7
ix = 8: e = 100 '                      customize precision e
k = LargeInit("cnfcroot")
XstGetEnvironmentVariable(@"LARGEINT", @WrkD$)
XstGetSystemTime(@Tim)
XstClearConsole(): PRINT

Fl = 0 '                               verbosity flag
DIM m[30]
Readnr = OPEN(WrkD$ + "\\cnfcroot", $$RD)

begin:
'Ni$ = INLINE$ (" N? ")
Ni$ = INFILE$ (Readnr)
IF INSTR(Ni$, "'") THEN GOTO begin
n = 0
IF XLONG(Ni$) < 31 THEN n = XLONG(Ni$) 'degree n
IF n < 1 THEN GOTO eind
'
FOR i = n TO 0 STEP -1
   g$ = "c(" + LTRIM$(STR$(i)) + ")"
   'Ni$ = INLINE$ (g$)
   PRINT g$; : Ni$ = INFILE$ (Readnr)
   m[i] = ix + i: Readst(m[i], Ni$) '  read coefficients
NEXT i
'
'Ni$ = INLINE$ ("[x]")
PRINT "[x]"; : Ni$ = INFILE$ (Readnr)
Readst(a, Ni$) '                       read initial a
XstClearConsole(): PRINT
'
Coeff()
Letf(p0, 0): Letf(p1, 1) '             initiate recurrences
Letf(q0, 1): Letf(q1, 0)
'
t = 0
DO
   t = t + 1
   Evalf(a, t0): i = Isf(t0, 0)
   DO
      Inc(a, 1)
      Evalf(a, t1): j = Isf(t1, 0)
      IF i OR j THEN '                 root found
         IF i THEN Dcr(a, 1)
         IF j THEN Swp(t0, t1)
         EXIT DO
      END IF
      IF Gets(t0) <> Gets(t1) THEN '   root bracketed
         Dcr(a, 1): EXIT DO
      END IF
      Swp(t0, t1): i = j '             t0:= t1
      IF Getl(a) > 1 THEN '            partial quotients < 32,768
         Prints("search aborted", 2) ' bracketing fails if there
         GOTO begin '                  are 2k roots in [a, a + 1[
      END IF
   LOOP
   Sw = Isf(t0, 0)
   Printn(a, "", ",", 0)
   '
   Mult(a, p1, t2) '                  Euclid's algorithm:
   Swp(a, t2): Add(p0, t2) '           p_t = p_t-2 + a_t * p_t-1
   Mult(a, q1, t2)
   Swp(a, t2): Add(q0, t2) '           q_t = q_t-2 + a_t * q_t-1
   Swp(p0, p1): Swp(q0, q1)
   '
   FOR i = 0 TO n - 1
      FOR j = n - 1 TO i STEP -1
         Mult(a, m[j + 1], t2) '       Horner substitution y:= x - a
         Swp(a, t2): Add(m[j], t2)
      NEXT j
   NEXT i
   FOR i = 0 TO (n - 1) \ 2 '          substitute x:= 1 / y
      Swp(m[i], m[n - i]) '            and multiply by highest power
   NEXT i
   Letf(a, 1) '                        reset a
   IF Fl THEN
      Prints("", 1): Coeff() '         display all transformed polynomials
   END IF
   i = Getl(p1) + 2 > $$uj '           precision exhausted
   j = 2 * Getl(q1) / $$L10 > e
   j = j OR Sw '                       error < 1/ q^2 or root found
LOOP UNTIL i OR j
Prints(" [" + LTRIM$(STR$(t)) + "]", 1) ' length of convergent
'
Printn(p1, " p = ", "", 2)
Printn(q1, " q = ", "", 2)
Setl(a, e)
Printr(p1, q1, a, " r = ", "")
Prints("", 1)
GOTO begin

eind:
XstGetSystemTime(@Now)
g$ = STR$(SINGLE(Now - Tim) / 1000)
Prints("Timer:" + g$ + " s", 1)
CLOSE(Readnr)
Term ()
END FUNCTION

FUNCTION VOID Coeff()
SHARED SLONG m[], n
   Prints("C: [", 0)
   FOR i = n TO 0 STEP -1
      Printn(m[i], " ", " ", 0) '      print coefficient vector c[n..0]
   NEXT i
   Prints("]", 1)
END FUNCTION

FUNCTION VOID Evalf (SLONG p, SLONG q)
SHARED SLONG m[], n, t2
   Copyf(m[n], q)
   FOR i = n - 1 TO 0 STEP -1 '        evaluate polynomial, i1:= f(i0)
      Mult(q, p, t2)
      Add(q, m[i])
   NEXT i
END FUNCTION
END PROGRAM
