'Subject: Lenstra's elliptic curve method finds factor p of N if
'         some random elliptic curve group has a smooth order mod p.
'         Uses homogeneous Montgomery form BY^2Z = X^3 + AX^2Z + XZ^2.
'         Recursive double-step version
'Author : Sjoerd.J.Schaper
'Date   : 03-13-2005
'Code   : XBasic adaptation demo
PROGRAM	"ellcrvfr"
VERSION	"01.10"
IMPORT	"bignumXB"
IMPORT	"xma"
IMPORT	"xst"

DECLARE FUNCTION VOID Main()
DECLARE FUNCTION VOID Factor (SLONG d, SLONG N)
DECLARE FUNCTION SLONG Splitf (SLONG d, SLONG N)
DECLARE FUNCTION VOID Eadd (SLONG xm, SLONG zm, SLONG xn, SLONG zn, SLONG xd, SLONG zd)
'Add points P_m and P_n, using P_(m-n) = P_d
DECLARE FUNCTION ECM (SLONG d, SLONG N)
'Elliptic curve engine attempts to
'split N in N/ d and d with bound B.
DECLARE FUNCTION VOID Edbl (SLONG xn, SLONG zn)
'Double point P_n
DECLARE FUNCTION VOID Ellmlt (SLONG xv, SLONG zv, SLONG c)
'Calculate point c*P in (xv: :zv)
$$dq = 153: $$stax = 6
$$lim = 131071 '                       prime trial limit

FUNCTION VOID Main()
SHARED SLONG t0, t1, t3
SHARED SLONG N, a, xs, zs, xt, zt, xu, zu
SHARED SLONG B, p, qu, i[], St[], x0, z0
t0 = $$ui + 1: t1 = $$ui + 2
t2 = $$ui + 3: t3 = $$ui + 4
N = 0: d = 1: a = 2: p = 3: qu = 4 '   pointers
x0 = 5: z0 = 6: xs = 7: zs = 8
xt = 9: zt = 10: xu = 11: zu = 12
qfl = zu + 1: qcl = qfl + $$dq
sfl = qcl: scl = sfl + $$stax
DIM i[$$dq], St[$$stax] '              difference table, stack
FOR q = qfl TO qcl: i[q - qfl] = q: NEXT
FOR t = sfl TO scl: St[t - sfl] = t: NEXT
k = LargeInit("ellcrvfr")
XstGetEnvironmentVariable(@"LARGEINT", @WrkD$)
XstGetSystemTime(@Tim)
XstClearConsole(): fl = 0 '           set flag for file input

Readnr = OPEN(WrkD$ + "\\ellcrvfr", $$RD)

begin:
IF fl THEN
   Ni$ = INFILE$ (Readnr)
ELSE
   Ni$ = INLINE$ (" N? ")
END IF
IF INSTR(Ni$, "'") THEN GOTO begin
IF XLONG(LEFT$(Ni$, 5)) < 2 THEN GOTO eind
Readst(N, Ni$)

IF fl THEN
   bi$ = INFILE$ (Readnr)
ELSE
   bi$ = INLINE$ (" B? ")
END IF
B = 0 '                                smoothness bound
IF ABS(XLONG(bi$)) < $$Fx \ 1000 THEN
  B = ABS(XLONG(bi$))
END IF
IF B < 10 THEN B = 1000 '              default B

Printn(N, " N = ", "", 2)
Factor(d, N)
Prints("", 1)
GOTO begin

eind:
XstGetSystemTime(@Now)
g$ = STR$(SINGLE(Now - Tim) / 1000)
Prints("Timer:" + g$ + " s", 1)
CLOSE(Readnr)
Term ()
END FUNCTION

FUNCTION VOID Factor (SLONG d, SLONG N)
SHARED SLONG qu
   pr = 0: sq = 0
   IF Getl(N) > 2 THEN '               preliminary tests
      sq = IsSqr(N, d)
      IF sq THEN Swp(N, d) '           N is square
      pr = IsPPrm(N, d, 7) '           N or sqrt(N) is pprime
   END IF

   IF pr THEN
      Setl(qu, 0)
      EnQ(qu, N, 1)
   ELSE
      Triald(qu, N, $$lim) '           trial divide N
      IF NOT Isf(N, 1) THEN
         IF NOT Splitf(d, N) THEN EXIT FUNCTION
      END IF
   END IF

   k = 1: t = 0
   DO WHILE ExQ(N, @k, qu, @t) '       read factors
      g$ = "": IF sq THEN k = k * 2
      IF k > 1 THEN g$ = "^ " + LTRIM$(STR$(k))
      Printn(N, SPACE$(5), g$, 1)
   LOOP
END FUNCTION

FUNCTION SLONG Splitf (SLONG d, SLONG N)
SHARED SLONG t0, t1, St[], qu
   t = 0
   DO
      Sets(d, -128) '                      skip search for tinyfactors
      IF IsPPrm(N, d, 5) THEN '            N is probably prime
         k = 1
         FOR j = t TO 1 STEP -1
            DO
               Copyf(St[j], t0) '          trace possible common
               Divd(t0, N, t1) '           factors in the stack
               IF NOT Isf(t0, 0) THEN EXIT DO
               Swp(St[j], t1): k = k + 1
            LOOP
            IF Isf(t0, 1) AND Isf(t1, 0) THEN
               Swp(St[t], St[j]): t = t - 1 ' pop empty entry
            END IF
         NEXT j
         EnQ(qu, N, k)
         Swp(St[t], N): t = t - 1 '        pop pprime
         '
      ELSE
         ECM(d, N): t = t + 1 '            make search for divisor
         Printn(N, "N/f = ", "", 2)
         IF t > $$stax THEN
            Prints("  out of stack space", 1)
            EXIT FUNCTION
         END IF
         Swp(N, St[t]): Swp(d, N) '        push N/ d, check d
      END IF
   LOOP UNTIL t < 0
END FUNCTION -1

FUNCTION VOID Eadd (SLONG xm, SLONG zm, SLONG xn, SLONG zn, SLONG xd, SLONG zd)
SHARED SLONG t0, t1, t3, N
   Mult(xm, xn, t0): Mult(zm, zn, t1)
   Swp(xm, t0): Swp(zm, t1): Subt(t0, t1)
   Divd(t0, N, t1): Modsqu(t0, N) '    t0:= (XmXn - ZmZn)^ 2 (mod N)
   Mult(xm, zn, t3): Mult(zm, xn, t1)
   Swp(xm, t3): Swp(zm, t1): Subt(t3, t1)
   Divd(t3, N, t1): Modsqu(t3, N) '    t3:= (XmZn - ZmXn)^ 2 (mod N)
   Mult(t0, zd, xm): Swp(t0, xm) '     xm:= t0 * Z|m-n| (mod N)
   Mult(t3, xd, zm): Swp(t3, zm) '     zm:= t3 * X|m-n| (mod N)
   Moddiv(zm, N): Moddiv(xm, N)
END FUNCTION

FUNCTION VOID ECM (SLONG d, SLONG N)
SHARED SLONG t0, a, B, p, i[]
SHARED SLONG x0, z0, xs, zs
B1 = B * 62
IF B1 > $$Fx THEN B1 = $$Fx '          second bound
LnB! = LOG(B)
'
IF IsSqr(N, d) THEN
   Copyf(d, N): EXIT FUNCTION
END IF
'
Stp1:
PRINT "  k =";
Letf(a, 2): k = 0
DO
   k = k + 1: PRINT k;
   t = 0: Sets(a, 1)
   Inc(a, 1 + INT(Rnd(0) * 5))
   Sets(a, SIGN(Rnd(0) - .5))
   x = 2 + INT(Rnd(0) * 10) '          draw elements A, X
   Letf(xs, x): Letf(zs, 1) '          P0 = (X: :1)
   IF Kronec(xs, N, d) = 0 THEN GOTO jump
   '
   sw = 0: fr = 1
   r = 0: Kr = 4: t = 1
   DO
      IF sw THEN '                     prime-power
         c = POWER(fr, INT(LnB! / LOG(fr)))
      ELSE
         c = 43545600 '                initial factor 10! * 12
      END IF
      Ellmlt(xs, zs, c)
      IF Isf(zs, 0) THEN GOTO Stp1
      IF r = Kr THEN
         IF Kr < 32 THEN Kr = Kr * 2
         IF Kronec(zs, N, d) = 0 THEN GOTO jump
         r = 0
      END IF
      fr = Nxtprm(sw): sw = -1
      r = r + 1
   LOOP UNTIL fr > B
   '
   Copyf(xs, i[0]): Copyf(zs, i[1])
   Edbl(i[0], i[1]) '                  2*P
   Copyf(i[0], i[2]): Copyf(i[1], i[3])
   Edbl(i[2], i[3]) '                  4*P
   FOR q = 4 TO $$dq - 1 STEP 2 '      store even points 2*P,.., 154*P
      Copyf(i[q - 2], i[q])
      Copyf(i[q - 1], i[q + 1])
      Eadd(i[q], i[q + 1], i[0], i[1], i[q - 4], i[q - 3])
   NEXT q
   '
   Copyf(xs, x0): Copyf(zs, z0)
   t = 2: Letf(p, 1)
   DO
      Copyf(x0, xs): Copyf(z0, zs) '   make search for one last big factor
      f0 = fr: Ellmlt(xs, zs, fr) '    giant step
      Copyf(zs, d)
      DO
         Modmlt(p, d, N)
         fr = Nxtprm(1)
         q = fr - f0 - 2
         IF q >= $$dq THEN EXIT DO
         Mult(xs, i[q + 1], d): Swp(xs, d)
         Mult(zs, i[q], t0): Swp(zs, t0)
         Subt(d, t0): Sets(d, 1) '     baby step:
         Divd(d, N, t0) '              |XmZn - ZmXn| mod N
      LOOP
      IF Kronec(p, N, d) = 0 THEN GOTO jump
   LOOP UNTIL fr > B1
LOOP
'
jump:
PRINT
Divd(N, d, p) '                        gcd in d
IF Isf(d, 1) OR Isf(p, 1) THEN
  IF Isf(p, 1) THEN Swp(p, d)
  Swp(p, N): GOTO Stp1
END IF
IF t > 0 THEN
   g$ = "  x =" + STR$(x) '            log curve elements,
   c = Gets(a) * Gete(a, 0)
   Prints(g$ + "  a =" + STR$(c), 0)
   c = x * (x * (x + c) + 1)
   Prints("  by=" + STR$(c), 1)
   g$ = "  k =" + STR$(k) '            curve number, final factor
   Prints(g$ + "  fr =" + STR$(fr), 1)
END IF
Swp(p, N)
END FUNCTION

FUNCTION VOID Edbl (SLONG xn, SLONG zn)
SHARED SLONG t0, t1, t3, N, a
   Mult(xn, zn, t0): Swp(xn, t0)
   Divd(t0, N, t1) '                   t0:= Xn * Zn (mod N)
   Modsqu(xn, N): Modsqu(zn, N) '      xn:= Xn^ 2 (mod N), zn:= Zn^ 2 (mod N)
   Mult(a, t0, t3): Swp(a, t3)
   Add(t3, xn): Add(t3, zn) '          t3:= Xn^ 2 + A * XnZn + Zn^ 2
   Subt(xn, zn): Modsqu(xn, N) '       xn:= (Xn^ 2 - Zn^ 2)^ 2 (mod N)
   Lsft(t0, 2): Mult(t0, t3, zn) '     zn:= 4 * XnZn * t3 (mod N)
   Swp(t0, zn): Moddiv(zn, N)
END FUNCTION

FUNCTION VOID Ellmlt (SLONG xv, SLONG zv, SLONG c)
SHARED SLONG xt, zt, xu, zu
   c0 = ABS(c)
   DO UNTIL -(c0 AND 1)
      Edbl(xv, zv): c0 = c0 \ 2
   LOOP '                              c0 = (2^ i) * odd(k)
   Copyf(xv, xt): Copyf(zv, zt)
   Copyf(xv, xu): Copyf(zv, zu) '      V = T = -U = (2^ i) * P
   DO UNTIL c0 = 1 '                  R=>L Montgomery ladder
      Edbl(xt, zt): c0 = c0 \ 2 '      T:= 2T
      IF -(c0 AND 1) THEN
         Eadd(xv, zv, xt, zt, xu, zu) 'V:= V + T, using V - T = U
      ELSE
         Eadd(xu, zu, xt, zt, xv, zv) 'U:= U - T, using U + T = V
      END IF
   LOOP
END FUNCTION
END PROGRAM
