'
' ####################
' #####  PROLOG  #####
' ####################
'
' Simple, general purpose XML parser for Xblite written by Rhett Thompson. Probably bound by the terms within the LGPL,
'		but you can do whatever you want.
'
' - - - - - - - - - -
'
' 5-10-08 :: 2.0   :: TODO: Implement some kind of binary searching on node pool.
'								   :: First build of the 2.0 branch.
'
' 5-13-08 :: 2.1   :: Added a few options to the parser. FIX USER NODES!
' 5-16-08 :: 2.2   :: Greatly improved the way user nodes are handled. Added another event system. There is no longer a limit to
'		the number of nodes you can have. Entities were also updated.
' 6-6-08  :: 2.3   :: Nodes no longer carry their attributes with them, instead they carry an array of pointers to a resizable pool.
'		Not unlike nodes.  I experimented further with unicode, and I couldn't get it to fail (or even not work) with a unicode input file.
'		It is somewhat expected, as the unicode codes that I need to use are in fact equivalent to ANSI (i.e. the '<' is 60 in both encodings). 
'		However I've been using the Xblite direct access thing instead of the unicode address method, which is puzzling why it doesn't fail.
' 6-13-08 :: 2.3.1 :: Comments are now properly removed from retrieved data. Added support for the space between entity declarations, even
'		though I'm against it. However it does make the parser WC3 syntactically compliant (technically it isn't because it doesn't validate)
'   , which is cool.
' 6-15-08 :: 2.3.1 :: Fixed a bug where XML nodes were not getting added to the node pool. Added argument to XmlGetChildFromPath(),  which allows
'		a start node to be specified. Added XmlGetNodes() to obtain how many nodes there are (Note. that that the return value is one based).
'   Fixed a bug in XmlGetAbsoluteParent() where it was getting stuck in an infinite loop if the only possible parent was the root node.
' 6-18-08 :: 2.3.1 :: Completely removed user nodes for now.
' 6-19-08 :: 2.3.2 :: Random bug fixes I forgot to note.
' 6-30-08 :: 2.3.3 :: Greatly improved core function XmlGetChild(). Added XmlGetChildB() which uses binary searching instead of linear searching.
'		However, I was unable to create an XML tree (or couldn't see the reason) large enough to witness a speed difference, in XmlGetChildB()'s favor.
'		All my tests show that XmlGetChild() is about 18.5x faster than the binary searcher with an above average sized XML file.  Owell. Also
'		XmlGetChildFromPath() is sadly kinda slow, and it gets exponentially worse the more path parts there are.
' 7-3-08 	:: 2.3.4 :: Several small bugfixes associated with the return values of funtions, and things like that. Fixed several functions that should have
'		taken a pointer instead of a copy of the XMLNODE structure.
' 7-4-08 	:: 2.3.4 :: Squeezed in a few bugfixes before the 4th's usual parties:) Fixed XmlSetParent to actually work:P  Renamed XmlTranslateReferences to
' 	XmlTranslateData.  Cleaned up XmlLoadFromFile to use XmlLoadFromString, will keep maintenance easy. Also added a few options to the parser. More to come
'   in 2.3.5.
' 7-6-08 	:: 2.3.5 :: In XmlSetListener if func is 0, then the default listener is reset. Improved and optimized attribute/entity parsing greatly. Padded
'		space and non-printable characters should no longer get removed. Added concept of mnemonics, which are sort of like entities except get executed before
'		anything is parsed. I'm going to wait and document this later as it isn't very mature yet. Fixed a bug in XmlTranslateData concerning CDATA. I'm a little
'		concerned why it hasn't affected anything thus far. I discovered it purely by looking at the code, not from practice.
' 7-9-08 	:: 2.3.5 :: Fixed a bug in commenting, where the file pointer wasn't getting advanced correctly.
' 7-23-08	:: 2.3.6 :: Fixed a bug in which attributes where the containing symbols were right next to each other, would cause the value to be the same as the 
'		attribute name.
' 8-7-08 :: 2.3.7 :: Completely removed mnemonic support. Lowered maximum number of attributes that belong to a single node to 20. Renamed the .this member
'		of XMLNODE to .id. Added XmlGetNodeAddress() and XmlGetNodePoolAddress() for easy direct node access.
' 8-11-08 :: 2.3.8 :: Fixed a bug in internal utility function CSTRING(), where I forgot to append a terminating null character at the end of a string...Oops.
' 9-3-08 :: 2.3.9 :: Added XmlCompile$() which compiles the currently loaded tree into a string.  When changing a nodes data, the .dataStart/.dataEnd methods are now
'		correctly broken. Fixed a bug in XmlGetChildren(). Fixed a bug in XmlGetNodePoolAddress (). XmlAddAttribute() now rightly replaces the specified attribute 
'		if it already exists.
'
PROGRAM "gxml"
VERSION "2.3.9"
CONSOLE
'
	IMPORT "xst"
	IMPORT "xsx"
'
EXPORT

' represents a node attribute
	TYPE XMLATTRIBUTE
		XLONG		.name		'address of the name of the attribute
		XLONG		.value	'address of the value of the attribute
	END TYPE

' represents a node
	TYPE XMLNODE
		XLONG		 			.name						'address of the name of the node
		XLONG					.type						'node type
		XLONG					.data						'address of the data of the node
		XLONG					.parent					'points to the parent of this node
		XLONG					.attribute[19]	'$$XmlMaxAttributes
		XLONG					.dataStart			'point in the file when the node data block begins (Note. This is only used if the node is of type $$XmlNodeType_Regular)
		XLONG					.dataEnd				'point in the file when the node data block ends
		XLONG					.id							'points to this node
	END TYPE
'
	DECLARE FUNCTION gxml ()
' gxml API
	DECLARE FUNCTION XMLNODE XmlLoadFromFile (fileName$)
	DECLARE FUNCTION XMLNODE XmlLoadFromString (string$)
	DECLARE FUNCTION XMLNODE XmlGetChild (XMLNODE parent, child$, instance)
	DECLARE FUNCTION XmlGetData$ (XMLNODE node)
	DECLARE FUNCTION XmlGetAttribute$ (XMLNODE node, name$)
	DECLARE FUNCTION XmlGetChildInstances (XMLNODE parent, child$)
	DECLARE FUNCTION XMLNODE XmlAddNode (XMLNODE parent, name$, type)
	DECLARE FUNCTION XmlDeleteNode (lpnode)
	DECLARE FUNCTION XmlAddNodeData (lpnode, data$)
	DECLARE FUNCTION XmlAddNodeAttribute (lpnode, name$, value$)
	DECLARE FUNCTION XmlDeleteNodeAttribute (lpnode, name$)
	DECLARE FUNCTION XmlSetParent (lpnode,XMLNODE parent)
	DECLARE FUNCTION XmlGetAttributes (XMLNODE node)
	DECLARE FUNCTION XmlChildExist (XMLNODE parent,child$)
	DECLARE FUNCTION XmlAttributeExist (XMLNODE node,name$)
	DECLARE FUNCTION XmlNodeList$ ()
	DECLARE FUNCTION XmlDrop ()
	DECLARE FUNCTION XmlGetChildren (XMLNODE parent)
	DECLARE FUNCTION XMLNODE XmlGetChildFromPath (lpstart, path$)
	DECLARE FUNCTION XMLNODE XmlGetNode (node)
	DECLARE FUNCTION XmlGetEntity$ (name$)
	DECLARE FUNCTION XmlEntityExist (name$)
	DECLARE FUNCTION XmlAddEntity (name$,value$)
	DECLARE FUNCTION XmlDeleteEntity (name$)
	DECLARE FUNCTION XmlTranslateData (@data$)
	DECLARE FUNCTION XMLNODE XmlGetAbsoluteParent (XMLNODE this)
	DECLARE FUNCTION XmlSetFlag (flag, value)
	DECLARE FUNCTION XmlSetListener (func)
	DECLARE FUNCTION XmlGetNodes ()
	DECLARE FUNCTION XmlDefaultSettings ()
	DECLARE FUNCTION XMLNODE XmlGetChildB (XMLNODE parent,child$,instance)
	DECLARE FUNCTION XmlGetNodeAddress (XMLNODE node)
	DECLARE FUNCTION XmlGetNodePoolAddress ()
	DECLARE FUNCTION XmlCompile$ (useIndention)
END EXPORT
' internal gxml API
DECLARE FUNCTION AddNode (name$,parent,type)
DECLARE FUNCTION ParseAttributeString (XMLNODE node,string$)
DECLARE FUNCTION DetectNodeType (node$)
DECLARE FUNCTION CleanNode$ (node$)
DECLARE FUNCTION GetNode$ (node$)
DECLARE FUNCTION ParseData (data$)
DECLARE FUNCTION HasChildren (XMLNODE node)
DECLARE FUNCTION ParseEntityString (string$)
DECLARE FUNCTION AddEntity (name$,value$)
DECLARE FUNCTION EntityExist (name$)
DECLARE FUNCTION GetAttribute$ (XMLNODE node,name$)
DECLARE FUNCTION GetEntity$ (name$)
DECLARE FUNCTION isValidName (string$)
DECLARE FUNCTION CSTRING (string$)
DECLARE FUNCTION DestroyNode (node)
DECLARE FUNCTION Hello (code$)
DECLARE FUNCTION DefaultListener (a,b)
DECLARE FUNCTION AddAttribute (name$, value$)
DECLARE FUNCTION DestroyAttribute (attribute)
DECLARE FUNCTION BinarySearch_STRING (STRING array[], STRING value, matches[])
DECLARE FUNCTION FindNode (name$, instance)
DECLARE FUNCTION ChangeParent (child, newparent)
DECLARE FUNCTION Compile (parent, useIndention)
DECLARE FUNCTION BuildAttributeString$ (node)
'
'node types used internally
$$RegularStart=0
$$RegularEnd=1
$$Irregular=2
$$Xml=3
$$Entity=4
$$User=5
$$Invalid=6
EXPORT
'gxml version
$$XmlParserVersion="2.3.9"
'max attributes
$$XmlMaxAttributes=19
'node types
$$XmlNodeType_Undefined=-1
$$XmlNodeType_Regular=0
$$XmlNodeType_Irregular=1
$$XmlNodeType_Xml=2
'events
$$XmlEvent_AddNode=0x01
$$XmlEvent_Start=0x02
$$XmlEvent_End=0x03
'option flags
$$XmlOption_CollectEntities=0x01
$$XmlOption_UseEvents=0x02
$$XmlOption_CollectXml=0x03
$$XmlOption_CollectData=0x04
END EXPORT
'
'
'
' #####################
' #####  gxml ()  #####
' #####################
'
FUNCTION gxml ()

	'XMLNODE root,a,b
	'DOUBLE c,d
	
	'set default xml settings
	XmlDefaultSettings()

	'XmlSetListener (&DefaultListener())
  'root=XmlLoadFromFile ("testapp.xml")
	
	'a=XmlGetChildFromPath (0, "TestApp/font")
	'XmlAddNodeAttribute (&a, "name", "asd")
	'? XmlGetAttribute$ (a, "name")

'	b=XmlGetChildFromPath (0, "Shader/Context/VertexShader/DefCode")
'	XmlSetParent (&a,b)

	'XstSaveString ("asd.xml", XmlCompile$ (1))
	
	
	'a$=INLINE$(">")
	IF LIBRARY(0) THEN RETURN
	
END FUNCTION
'
' #####################
' #####  AddNode  #####
' #####################
'
'
'
FUNCTION AddNode (name$,parent,type)

	SHARED XMLNODE Node[]
	SHARED Node$[]
	SHARED FUNCADDR XmlListener(XLONG,XLONG)
	SHARED useEvents
	SHARED xmldata$
	
	nodeName$=CleanNode$(name$)
	IFT isValidName (nodeName$)

		'create a new index for this node
		upp=UBOUND(Node[])+1
		REDIM Node[upp]
		REDIM Node$[upp]
		
		'clear attributes
		FOR i=0 TO $$XmlMaxAttributes
			Node[upp].attribute[i]=-1
		NEXT i
	
		'cut name off at first space, and get list of attributes
		attributes$=""
		len=LEN(name$)
		FOR i=0 TO len
			IF name${i}==' ' THEN attributes$=TRIM$(MID$(name$,i+2,len-i-1)):name$=LEFT$(name$,i):EXIT FOR
		NEXT i

		'create this node
		Node$[upp]=nodeName$
		Node[upp].name=CSTRING(nodeName$)
		Node[upp].parent=parent
		Node[upp].id=upp
		Node[upp].type=type
		
		IF parent>-1
		
			Node[upp].data=0
			Node[upp].dataStart=-1
			Node[upp].dataEnd=-1
		
			'add attributes, if any
			IF attributes$ THEN ParseAttributeString(Node[upp],attributes$)

			'fire event
			IFT useEvents THEN @XmlListener($$XmlEvent_AddNode,upp+1)

		END IF
		RETURN upp
	ELSE
		RETURN -1
	END IF

END FUNCTION
'
' ##################################
' #####  ParseAttributeString  #####
' ##################################
'
'
'
FUNCTION ParseAttributeString (XMLNODE node,string$)

	SHARED XMLNODE Node[]
	
	string$=TRIM$(string$)
	
	'tokenize line
	quote=$$FALSE
	len=LEN(string$)
	DIM table$[0]
	token=0
	FOR c=0 TO len
		char$=CHR$(string${c})
		SELECT CASE char$
			CASE "=":
				IFT quote THEN table$[token]=table$[token]+char$
			CASE CHR$('"'),CHR$('''):
				INC token:REDIM table$[token]
				IFF quote THEN quote=$$TRUE ELSE quote=$$FALSE
			CASE ELSE:
				table$[token]=table$[token]+char$
		END SELECT
	NEXT c

	'add attributes
	upp=UBOUND(table$[])
	FOR i=0 TO upp STEP 2
		IF i+1>upp THEN EXIT FOR
		n$=TRIM$(table$[i])
		v$=table$[i+1]
		'find a free slot for attribute
		FOR t=0 TO $$XmlMaxAttributes
			IF Node[node.id].attribute[t]==-1
				Node[node.id].attribute[t]=AddAttribute (n$, v$)
				EXIT FOR
			END IF
		NEXT t
	NEXT i
	
END FUNCTION
'
' ############################
' #####  DetectNodeType  #####
' ############################
'
'
'
FUNCTION DetectNodeType (node$)

	node$=TRIM$(node$)
	len=LEN(node$)

	'xml node
	IF node${0}=='?' && node${len-1}=='?' THEN RETURN $$Xml
	'regular end
	IF node${0}=='/' THEN RETURN $$RegularEnd
	'irregular node </>
	IF node${len-1}=='/' THEN RETURN $$Irregular
	'entity <!ENTITY>
	IF node${0}=='!' && node${1}=='E'
		IF LEFT$(node$,7)=="!ENTITY" THEN RETURN $$Entity
	END IF
	'other ! declaration
	IF node${0}=='!' THEN RETURN $$Xml
	
	RETURN $$RegularStart

END FUNCTION
'
' ########################
' #####  CleanNode$  #####
' ########################
'
'
'
FUNCTION CleanNode$ (node$)

	node$=TRIM$(node$)
	len=LEN(node$)

	IF node${0}=='<' THEN node$=RIGHT$(node$,len-1)
	IF node${len-1}=='<' THEN node$=LEFT$(node$,len-1):a=1
	IF node${len-1}=='/' THEN node$=LEFT$(node$,len-1)
	IF node${0}=='/' THEN node$=RIGHT$(node$,len-1)
	node$=TRIM$(node$)
	IF node${len-1}=='?' THEN node$=LEFT$(node$,len-1)
	IF node${0}=='?' THEN node$=RIGHT$(node$,len-1)
	IF node${0}=='!' THEN node$=RIGHT$(node$,len-1)
	
	IFZ INSTR(node$,"=")
		RETURN TRIM$(node$)
	ELSE
		p=INSTR(node$,"=")
		node$=TRIM$(LEFT$(node$,p-1))
		p=INSTR(node$," ")
		node$=TRIM$(LEFT$(node$,p-1))
		RETURN TRIM$(node$)
	END IF
	
	RETURN node$

END FUNCTION
'
' ######################
' #####  GetNode$  #####
' ######################
'
'
'
FUNCTION GetNode$ (node$)

	node$=TRIM$(node$)
	len=LEN(node$)

	IF node${0}=='<' THEN node$=RIGHT$(node$,len-1)
	IF node${len-1}=='<' THEN node$=LEFT$(node$,len-1):a=1
	IF node${len-1}=='/' THEN node$=LEFT$(node$,len-1)
	IF node${0}=='/' THEN node$=RIGHT$(node$,len-1)
	node$=TRIM$(node$)
	IF node${len-1}=='?' THEN node$=LEFT$(node$,len-1)
	IF node${0}=='?' THEN node$=RIGHT$(node$,len-1)
	
	RETURN node$

END FUNCTION
'
' #############################
' #####  XmlLoadFromFile  #####
' #############################
'
'
'
FUNCTION XMLNODE XmlLoadFromFile (fileName$)
	
	'make sure the file exists
	IF XstFileExists (fileName$)==-1 THEN RETURN $$FALSE

	'load file
	XstLoadString (fileName$, @data$)

	RETURN XmlLoadFromString (data$)

END FUNCTION
'
' #########################
' #####  XmlGetChild  #####
' #########################
'
'
'
FUNCTION XMLNODE XmlGetChild (XMLNODE parent,child$,instance)

	SHARED XMLNODE Node[]

	child=-1
	nodes=UBOUND(Node[])
	FOR i=0 TO nodes
		IF (Node[i].parent=parent.id) && (CSTRING$(Node[i].name)=child$)
			INC child
			IF (instance<=1) || (child==instance-1) THEN RETURN Node[i]
		END IF
	NEXT i

END FUNCTION
'
' #########################
' #####  XmlGetData$  #####
' #########################
'
'
'
FUNCTION XmlGetData$ (XMLNODE node)
	
	IFT HasChildren(node) THEN RETURN ""
	data$=CSTRING$(node.data)
	XmlTranslateData(@data$)
	RETURN data$

END FUNCTION
'
' ##############################
' #####  XmlGetAttribute$  #####
' ##############################
'
'
'
FUNCTION XmlGetAttribute$ (XMLNODE node,name$)

	SHARED XMLNODE Node[]
	SHARED XMLATTRIBUTE Attribute[]
	
	FOR i=0 TO $$XmlMaxAttributes
		IF node.attribute[i]>-1
			IF CSTRING$(Attribute[node.attribute[i]].name)==name$
				value$=CSTRING$(Attribute[node.attribute[i]].value)
				XmlTranslateData (@value$)
				RETURN value$
			END IF
		END IF
	NEXT i

	RETURN ""

END FUNCTION
'
' ###########################
' #####  GetAttribute$  #####
' ###########################
'
'
'
FUNCTION GetAttribute$ (XMLNODE node,name$)

	SHARED XMLNODE Node[]
	SHARED XMLATTRIBUTE Attribute[]
	
	FOR i=0 TO $$XmlMaxAttributes
		IF node.attribute[i]>-1
			IF CSTRING$(Attribute[node.attribute[i]].name)==name$
				RETURN CSTRING$(Attribute[node.attribute[i]].value)
			END IF
		END IF
	NEXT i

	RETURN ""

END FUNCTION
'
' ##################################
' #####  XmlGetChildInstances  #####
' ##################################
'
'
'
FUNCTION XmlGetChildInstances (XMLNODE parent,child$)

	SHARED XMLNODE Node[]
	
	DIM Child[0]
	child=0
	nodes=UBOUND(Node[])
	FOR i=0 TO nodes
		IF Node[i].parent=parent.id && CSTRING$(Node[i].name)=child$
			REDIM Child[child]
			Child[child]=i
			INC child
		END IF
	NEXT i
	
	RETURN child

END FUNCTION
'
' #######################
' #####  ParseData  #####
' #######################
'
'
'
FUNCTION ParseData (data$)

	SHARED XMLNODE Node[]
	SHARED useEntities, collectXML, collectData, useEvents
	SHARED translateMnemonics
	SHARED xmldata$
	
	SHARED FUNCADDR XmlListener(XLONG,XLONG)
	
	'store data
	xmldata$=data$

	'parse away
	start=-1
	end=-1
	parentNode=-1
	parentName$=""
	cdata=$$FALSE
	comment=$$FALSE
	openComments=0
	inContainer=$$FALSE
	deadNode=$$FALSE
	openNodes=0
	len=LEN(data$)
	FOR i=0 TO len
		
		filePosition=i

		'detect comments
		IF openComments==0 && data${i}=='<' && data${i+1}=='!' && data${i+2}=='-' && data${i+3}=='-'
			comment=$$TRUE
			INC openComments
		END IF
		IF comment==$$TRUE && data${i}=='-' && data${i+1}=='-' && data${i+2}=='>'
			IF openComments=1 THEN comment=$$FALSE
			DEC openComments
			i=i+3
			DO FOR
		END IF
		
		'don't parse comments
		IFF comment
			
			'detect cdata
			IF cdata==$$FALSE && data${i}=='<' && data${i+1}=='!' && data${i+2}=='['
				IF (data${i+3}=='[') || (data${i+3}=='C' && data${i+4}=='D' && data${i+5}=='A' && data${i+6}=='T' && data${i+7}=='A')
					cdata=$$TRUE
				END IF
			END IF
			IF cdata==$$TRUE && data${i}==']' && data${i+1}==']' && data${i+2}=='>'
				cdata=$$FALSE
				i=i+3
				DO FOR
			END IF
			
			'dont parse cdata
			IFF cdata
	
				'detect containers
				a=0
				IF (inContainer==$$TRUE) && (data${i}==''' || data${i}=='"') THEN inContainer=$$FALSE:a=1
				IF (a==0) && (inContainer==$$FALSE) && (data${i}==''' || data${i}=='"') THEN inContainer=$$TRUE
				
				'ensure we are not in a container
				IFF inContainer

					'if this is a node
					IF(data${i}=='<' && start==-1) THEN start=i+1
					IF(data${i}=='>' && start>-1 && end==-1) THEN end=i+2
					
					'define node
					IF start>-1 && end>-1 && start<end
	
						'get name
						name$=MID$ (data$, start, end-start)

						'trim off brackets
						name$=RIGHT$(name$,LEN(name$)-1)
						name$=LEFT$(name$,LEN(name$)-1)
						name$=TRIM$(name$)
						
						'make sure there is something to parse
						IF name$
						
							SELECT CASE DetectNodeType(name$)
									
								CASE $$Xml:
									IFT collectXML THEN AddNode (GetNode$(name$),0,$$XmlNodeType_Xml)
								
								CASE $$Invalid:
								
								CASE $$Entity:
									IFT useEntities
										ParseEntityString (name$)
									END IF
					
								CASE $$RegularStart:
									IFF deadNode
										IF parentNode==-1
											parentName$=CleanNode$ (name$)
											p=AddNode (GetNode$(name$),0,$$XmlNodeType_Regular)
											IF p>-1
												parentNode=p
												Node[parentNode].dataStart=end
											ELSE
												deadNode=$$TRUE
											END IF
										ELSE
											parentName$=CleanNode$ (name$)
											p=AddNode (GetNode$(name$),parentNode,$$XmlNodeType_Regular)
											IF p>-1
												parentNode=p
												Node[parentNode].dataStart=end
											ELSE
												deadNode=$$TRUE
											END IF
										END IF
										INC openNodes
									END IF
									
								CASE $$RegularEnd:
									IF parentNode>-1 && CleanNode$ (name$)==parentName$
										IFF deadNode 
											parentNode=Node[parentNode].parent
											parentName$=CSTRING$(Node[parentNode].name)
										ELSE
											deadNode=$$FALSE
										END IF
										DEC openNodes
									END IF
	
								CASE $$Irregular:
									IFF deadNode
										IF parentNode==-1
											IF AddNode (GetNode$ (name$),0,$$XmlNodeType_Irregular)>-1
												parentName$=""
												parentNode=-1
											END IF
										ELSE
											AddNode (GetNode$ (name$),parentNode,$$XmlNodeType_Irregular)
										END IF
									END IF
	
							END SELECT
						
							'clear start/end positions
							start=-1:end=-1
							
						END IF
						
					END IF
					
				END IF
				
			END IF
			
		END IF
		
	NEXT i
	
	'get data
	IFT collectData
		nodes=UBOUND(Node[])
		FOR i=0 TO nodes
			getData=$$FALSE
			IF (HasChildren (Node[i])==$$FALSE)
				SELECT CASE Node[i].type
					CASE $$XmlNodeType_Regular:name$="</"+CSTRING$(Node[i].name)+">":getData=$$TRUE
				END SELECT
				IFT getData
					pos=INSTR (data$, name$, Node[i].dataStart)
					IF pos THEN Node[i].dataEnd=pos
					IF Node[i].dataStart>-1 && Node[i].dataEnd>Node[i].dataStart
						Xfree(Node[i].data)
						nodeData$=MID$(data$,Node[i].dataStart,Node[i].dataEnd-Node[i].dataStart)
						Node[i].data=CSTRING(nodeData$)
					END IF
				END IF
			END IF
		NEXT i
	END IF
	
	'fire event ending event
	IFT useEvents THEN @XmlListener($$XmlEvent_End,0)

	RETURN $$TRUE
	
END FUNCTION
'
' ###############################
' #####  XmlLoadFromString  #####
' ###############################
'
'
'
FUNCTION XMLNODE XmlLoadFromString (string$)

	SHARED XMLNODE Node[]
	SHARED FUNCADDR XmlListener(XLONG,XLONG)
	SHARED useEvents
	
	'trim up data
	data$=TRIM$(CSIZE$(string$))
	
	'initialize
	XmlDrop ()
	
	'fire event
	IFT useEvents THEN @XmlListener($$XmlEvent_Start,0)
	
	'parse file
	ParseData(data$)
	
	'return root object
	RETURN Node[0]

END FUNCTION
'
' ########################
' #####  XmlAddNode  #####
' ########################
'
'
'
FUNCTION XMLNODE XmlAddNode (XMLNODE parent,name$,type)

	XMLNODE node
	SHARED XMLNODE Node[]
	
	SELECT CASE type
		CASE $$XmlNodeType_Regular,$$XmlNodeType_Irregular:
			n=AddNode (name$,parent.id,type)
			IF n>-1 THEN RETURN Node[n]
	END SELECT

	RETURN node

END FUNCTION
'
' ###########################
' #####  XmlDeleteNode  #####
' ###########################
'
'
'
FUNCTION XmlDeleteNode (lpnode)

	SHARED XMLNODE Node[]
	XMLNODE node, blank
	
	IFZ lpnode THEN RETURN $$FALSE
	XstCopyMemory (lpnode, &node, SIZE(XMLNODE))
	IFZ CSTRING$(node.name) THEN RETURN $$FALSE

	IFF HasChildren (node)
		DestroyNode (node.id)
	ELSE
		nodes=UBOUND(Node[])
		FOR i=0 TO nodes
			IF Node[i].parent==node.id THEN DestroyNode (i)
		NEXT i
	END IF
	
	XstCopyMemory (&blank, lpnode, SIZE(XMLNODE))
	
	RETURN $$TRUE

END FUNCTION
'
' ############################
' #####  XmlAddNodeData  #####
' ############################
'
'
'
FUNCTION XmlAddNodeData (lpnode, data$)

	SHARED XMLNODE Node[]
	XMLNODE node
	
	IFZ lpnode THEN RETURN $$FALSE
	XstCopyMemory (lpnode, &node, SIZE(XMLNODE))
	IFZ CSTRING$(node.name) THEN RETURN $$FALSE
	
	IF Node[node.id].data THEN Xfree(Node[node.id].data)
	Node[node.id].data=CSTRING(data$)
	Node[node.id]=node
	Node[node.id].dataStart=-1
	Node[node.id].dataEnd=-1
	
	XstCopyMemory (&Node[node.id], lpnode, SIZE(XMLNODE))
	
	RETURN $$TRUE

END FUNCTION
'
' #################################
' #####  XmlAddNodeAttribute  #####
' #################################
'
'
'
FUNCTION XmlAddNodeAttribute (lpnode, name$, value$)

	SHARED XMLNODE Node[]
	SHARED XMLATTRIBUTE Attribute[]
	XMLNODE node
	
	IFZ lpnode THEN RETURN $$FALSE
	XstCopyMemory (lpnode, &node, SIZE(XMLNODE))
	IFZ node.name THEN RETURN $$FALSE

	IFF XmlAttributeExist (node,name$)
		FOR i=0 TO $$XmlMaxAttributes
			IF node.attribute[i]>-1
				node.attribute[i]=AddAttribute (name$, value$)
				EXIT FOR
			END IF
		NEXT i
	ELSE
		FOR i=0 TO $$XmlMaxAttributes
			IF CSTRING$(Attribute[node.attribute[i]].name)==name$
				DestroyAttribute (node.attribute[i])
				Attribute[node.attribute[i]].name=CSTRING(name$)
				Attribute[node.attribute[i]].value=CSTRING(value$)
				EXIT FOR
			END IF
		NEXT i
	END IF
	Node[node.id]=node
	
	XstCopyMemory (&node, lpnode, SIZE(XMLNODE))
	
	RETURN $$TRUE
	
END FUNCTION
'
' ####################################
' #####  XmlDeleteNodeAttribute  #####
' ####################################
'
'
'
FUNCTION XmlDeleteNodeAttribute (lpnode,name$)

	SHARED XMLNODE Node[]
	SHARED XMLATTRIBUTE Attribute[]
	XMLNODE node
	
	IFZ lpnode THEN RETURN $$FALSE
	IFZ name$ THEN RETURN $$FALSE
	XstCopyMemory (lpnode, &node, SIZE(XMLNODE))
	IFZ CSTRING$(node.name) THEN RETURN $$FALSE

	FOR i=0 TO $$XmlMaxAttributes
		IF Node[node.id].attribute[i]>-1
			IF CSTRING$(Attribute[Node[node.id].attribute[i]].name)==name$
			
				DestroyAttribute (Node[node.id].attribute[i])
				Node[node.id].attribute[i]=-1
				
				Node[node.id]=node
				XstCopyMemory (&Node[node.id], lpnode, SIZE(XMLNODE))
				
				RETURN $$TRUE
			END IF
		END IF
	NEXT i
	
	RETURN $$FALSE

END FUNCTION
'
' ##########################
' #####  XmlSetParent  #####
' ##########################
'
'
'
FUNCTION XmlSetParent (lpnode,XMLNODE parent)

	SHARED XMLNODE Node[]
	XMLNODE node
	
	IFZ lpnode THEN RETURN $$FALSE
	XstCopyMemory (lpnode, &node, SIZE(XMLNODE))
	IFZ CSTRING$(node.name) THEN RETURN $$FALSE

	node.parent=parent.id

	IFT HasChildren (node)
		upp=UBOUND(Node[])
		FOR i=0 TO upp
			IF Node[i].parent==node.id
				ChangeParent(i, parent.id)
			END IF
		NEXT i
	END IF
	
	Node[node.id]=node
	XstCopyMemory (&Node[node.id], lpnode, SIZE(XMLNODE))
	
	RETURN $$TRUE

END FUNCTION
'
' ##############################
' #####  XmlGetAttributes  #####
' ##############################
'
'
'
FUNCTION XmlGetAttributes (XMLNODE node)

	SHARED XMLNODE Node[]
	
	a=0
	FOR i=0 TO $$XmlMaxAttributes
		IF Node[node.id].attribute[i] THEN INC a
	NEXT i
	
	RETURN a

END FUNCTION
'
' #########################
' #####  HasChildren  #####
' #########################
'
' Used internally as generally this is faster than XmlGetChildren().
'
FUNCTION HasChildren (XMLNODE node)

	SHARED XMLNODE Node[]
	
	nodes=UBOUND(Node[])
	FOR i=0 TO nodes
		IF Node[i].parent==node.id THEN RETURN $$TRUE
	NEXT i
	
	RETURN $$FALSE

END FUNCTION
'
' ###########################
' #####  XmlChildExist  #####
' ###########################
'
'
'
FUNCTION XmlChildExist (XMLNODE parent,child$)

	SHARED XMLNODE Node[]
	
	nodes=UBOUND(Node[])
	FOR i=0 TO nodes
		IF CSTRING$(Node[i].name)==child$ && Node[i].parent==parent.id THEN RETURN $$TRUE
	NEXT i
	
	RETURN $$FALSE

END FUNCTION
'
' ###############################
' #####  XmlAttributeExist  #####
' ###############################
'
'
'
FUNCTION XmlAttributeExist (XMLNODE node,name$)

	SHARED XMLNODE Node[]
	SHARED XMLATTRIBUTE Attribute[]
	
	FOR i=0 TO $$XmlMaxAttributes
		IF Node[node.id].attribute[i]>-1
			IF CSTRING$(Attribute[Node[node.id].attribute[i]].name)==name$ THEN RETURN $$TRUE
		END IF
	NEXT i
	
	RETURN $$FALSE

END FUNCTION
'
' ##########################
' #####  XmlNodeList$  #####
' ##########################
'
'
'
FUNCTION XmlNodeList$ ()

	SHARED XMLNODE Node[]
	
	a$="root node\n"
	nodes=UBOUND(Node[])
	FOR i=1 TO nodes
		a$=a$+CSTRING$(Node[i].name)+" of "+CSTRING$(Node[Node[i].parent].name)+"\n"
	NEXT i
	a$=TRIM$(a$)
	RETURN a$

END FUNCTION

'
' #####################
' #####  XmlDrop  #####
' #####################
'
'
'
FUNCTION XmlDrop ()

	SHARED XMLNODE Node[]
	SHARED Node$[]
	SHARED XMLATTRIBUTE	Attribute[]
	
	SHARED EntityPool_Name$[], EntityPool_Value$[]
	
	'clear node pool
	upp=UBOUND(Node[])
	IF upp>-1
		FOR i=0 TO upp
			DestroyNode (i)
		NEXT i
	END IF
	
	'create data arrays
	DIM Node[]
	DIM Node$[]
	DIM UserNode[]
	DIM EntityPool_Name$[]
	DIM EntityPool_Value$[]
	DIM Attribute[]
	
	'add predefined entities
	AddEntity ("lt",CHR$(60))
	AddEntity ("gt",CHR$(62))
	AddEntity ("amp",CHR$(38))
	AddEntity ("apos",CHR$(39))
	AddEntity ("quot",CHR$(34))

	'add root node
	AddNode ("root",-1,$$XmlNodeType_Undefined)
	
END FUNCTION
'
' ############################
' #####  XmlGetChildren  #####
' ############################
'
'
'
FUNCTION XmlGetChildren (XMLNODE parent)

	SHARED XMLNODE Node[]
	
	a=0
	nodes=UBOUND(Node[])
	FOR i=0 TO nodes
		IF Node[i].parent==parent.id THEN INC a
	NEXT i
	
	RETURN a

END FUNCTION
'
' ########################
' #####  XmlGetNode  #####
' ########################
'
'
'
FUNCTION XMLNODE XmlGetNode (node)

	SHARED XMLNODE Node[]
	
	node=node-1
	IF node>=0 && node<=UBOUND(Node[])
		RETURN Node[node]
	ELSE
		RETURN 0
	END IF
	
END FUNCTION
'
' ###############################
' #####  ParseEntityString  #####
' ###############################
'
'
'
FUNCTION ParseEntityString (string$)

	string$=TRIM$(string$)
	string$=TRIM$(RIGHT$(string$,LEN(string$)-7))
	
	'tokenize line
	quote=$$FALSE
	len=LEN(string$)
	DIM table$[0]
	token=0
	FOR c=0 TO len
		char$=CHR$(string${c})
		SELECT CASE char$
			CASE "=", " ":
				IFT quote THEN table$[token]=table$[token]+char$
			CASE CHR$('"'),CHR$('''):
				INC token:REDIM table$[token]
				IFF quote THEN quote=$$TRUE ELSE quote=$$FALSE
			CASE ELSE:
				table$[token]=table$[token]+char$
		END SELECT
	NEXT c

	'add entities
	upp=UBOUND(table$[])
	FOR i=0 TO upp STEP 2
		IF i+1>upp THEN EXIT FOR
		n$=TRIM$(table$[i])
		v$=table$[i+1]
		AddEntity (n$,v$)
	NEXT i

END FUNCTION
'
' #######################
' #####  AddEntity  #####
' #######################
'
'
'
FUNCTION AddEntity (name$,value$)

	SHARED EntityPool_Name$[], EntityPool_Value$[]
	
	IFF isValidName(name$) THEN RETURN -1
	e=EntityExist (name$)
	IF e>-1
		EntityPool_Value$[e]=value$
		RETURN e
	ELSE
		upp=UBOUND(EntityPool_Name$[])+1
		REDIM EntityPool_Name$[upp]
		REDIM EntityPool_Value$[upp]
		EntityPool_Name$[upp]=name$
		EntityPool_Value$[upp]=value$
		RETURN i
	END IF

	RETURN -1
	
END FUNCTION
'
' ###########################
' #####  XmlGetEntity$  #####
' ###########################
'
'
'
FUNCTION XmlGetEntity$ (name$)

	SHARED EntityPool_Value$[]
	
	e=EntityExist (name$)
	IF e>-1
		value$=EntityPool_Value$[e]
		XmlTranslateData (@value$)
		RETURN value$
	END IF
	
	RETURN ""

END FUNCTION
'
' ########################
' #####  GetEntity$  #####
' ########################
'
'
'
FUNCTION GetEntity$ (name$)

	SHARED EntityPool_Value$[]
	
	e=EntityExist (name$)
	IF e>-1
		RETURN EntityPool_Value$[e]
	END IF
	
	RETURN ""

END FUNCTION
'
' ############################
' #####  XmlEntityExist  #####
' ############################
'
'
'
FUNCTION XmlEntityExist (name$)

	SHARED EntityPool_Name$[]

	upp=UBOUND(EntityPool_Name$[])
	FOR i=0 TO upp
		IF EntityPool_Name$[i]==name$ THEN RETURN $$TRUE
	NEXT i
	
	RETURN $$FALSE

END FUNCTION
'
' #########################
' #####  EntityExist  #####
' #########################
'
'
'
FUNCTION EntityExist (name$)

	SHARED EntityPool_Name$[]

	upp=UBOUND(EntityPool_Name$[])
	FOR i=0 TO upp
		IF EntityPool_Name$[i]==name$ THEN RETURN i
	NEXT i
	
	RETURN -1

END FUNCTION
'
' ##############################
' #####  XmlTranslateData  #####
' ##############################
'
'
'
FUNCTION XmlTranslateData (@data$)

	len=LEN(data$)
	IF len>1
		cdata=$$FALSE
		FOR i=0 TO len
			'make sure it isn't protected <![C
			IF cdata==$$FALSE && data${i}=='<' && data${i+1}=='!' && data${i+2}=='['
				IF data${i+3}=='C' && data${i+4}=='D' && data${i+5}=='A' && data${i+6}=='T' && data${i+7}=='A' && data${i+8}=='['
					cdata=$$TRUE
				END IF
			END IF
			IF cdata==$$TRUE && data${i}==']' && data${i+1}==']' && data${i+2}=='>' THEN cdata=$$FALSE
			IFF cdata
				'comments
				IF data${i}=='<' && data${i+1}=='!' && data${i+2}=='-' && data${i+3}=='-'
					s=i
					e=-1
					FOR c=i+4 TO len
						IF data${c}=='-' && data${c+1}=='-' && data${c+2}=='>' THEN e=c+2:EXIT FOR
					NEXT c
					IF s<e
						s$=LEFT$(data$,s)
						e$=RIGHT$(data$,len-(e+1))
						data$=s$+e$
						len=LEN(data$)
					END IF
				END IF
				'entity references
				IF data${i}=='&'
					cref=$$FALSE
					'characters
					IF data${i+1}=='#'
						char$=""
						FOR c=i+2 TO len
							IF data${c}==';' THEN EXIT FOR
							char$=char$+CHR$(data${c})
						NEXT c
						data$=XstMergeStrings$ (data$, CHR$(XLONG(char$)), i+1, LEN("&#"+char$)+1)
						len=LEN(data$)
						cref=$$TRUE
					END IF
					'entities
					IFF cref
						char$=""
						FOR c=i+1 TO len
							IF data${c}==';' THEN EXIT FOR
							char$=char$+CHR$(data${c})
						NEXT c
						data$=XstMergeStrings$ (data$, XmlGetEntity$ (char$), i+1, LEN("&"+char$)+1)
						len=LEN(data$)
					END IF
				END IF
			END IF
		NEXT i
		'get rid of cdata tags
		IF LEFT$(TRIM$(data$),9)=="<![CDATA[" && RIGHT$(TRIM$(data$),3)=="]]>"
			data$=TRIM$(data$)
			data$=RIGHT$(data$,LEN(data$)-9)
			IF RIGHT$(data$,3)=="]]>" THEN data$=LEFT$(data$,LEN(data$)-3)
		END IF
		
		RETURN $$TRUE
	END IF
	
	RETURN $$FALSE

END FUNCTION
'
' ##########################
' #####  XmlAddEntity  #####
' ##########################
'
'
'
FUNCTION XmlAddEntity (name$,value$)
	
	IF EntityExist (name$)>-1
		AddEntity (name$,value$)
		RETURN $$TRUE
	END IF
	
	RETURN $$FALSE

END FUNCTION
'
' #############################
' #####  XmlDeleteEntity  #####
' #############################
'
'
'
FUNCTION XmlDeleteEntity (name$)

	SHARED EntityPool_Name$[]
	SHARED EntityPool_Value$[]
	
	DIM tempName$[]
	DIM tempValue$[]
	
	e=EntityExist(name$)
	IF e>-1
		EntityPool_Name$[e]=""
		EntityPool_Value$[e]=""
		RETURN $$TRUE
	ELSE
		RETURN $$FALSE
	END IF

END FUNCTION
'
' ##################################
' #####  XmlGetAbsoluteParent  #####
' ##################################
'
'
'
FUNCTION XMLNODE XmlGetAbsoluteParent (XMLNODE this)

	SHARED XMLNODE Node[]
	
	child=this.id
	parent=this.parent
	IF parent<=0 THEN RETURN Node[0]
	upp=UBOUND(Node[])
	DO
		FOR i=0 TO upp
			IF i==Node[parent].parent
				IF i==0 THEN EXIT DO
				parent=i
				EXIT FOR
			END IF
		NEXT i
	LOOP
	
	RETURN Node[parent]

END FUNCTION
'
' #################################
' #####  XmlGetChildFromPath  #####
' #################################
'
'	Retrieves a node using a path.
'
' The first argument is an address to an XMLNODE structure, which specifies a node to start from. The second is
'		the path. Only use Unix path slashes (/). You can optionaly specifiy an instance per path part with a colon. 
'		(i.e. "a/b/c:3") Also don't specifiy the root node, it is assumed if lpstart is zero.
'
FUNCTION XMLNODE XmlGetChildFromPath (lpstart, path$)

	SHARED XMLNODE Node[]
	XMLNODE node
	
	path$=TRIM$(path$)
	IF INSTR (path$, "\\") THEN XstReplace (@path$, "\\", "/", 0)
	IF path${0}=='/' THEN path$=RIGHT$(path$,LEN(path$)-1)
	IF path${LEN(path$)-1}=='/' THEN path$=LEFT$(path$,LEN(path$)-1)
	XstParseStringToStringArray (path$, "/", @path$[])
	IF lpstart
		XstCopyMemory (lpstart, &node, SIZE(XMLNODE))
	ELSE
		node=Node[0]
	END IF
	upp=UBOUND(path$[])
	FOR i=0 TO upp
		part$=path$[i]
		p=RINSTR(part$,":")
		IF p
			node$=LEFT$(part$,p-1)
			instance=XLONG(RIGHT$(part$,LEN(part$)-p))
		ELSE
			node$=part$
			instance=0
		END IF
		node=XmlGetChild(node,node$,instance)
	NEXT i
	
	RETURN node

END FUNCTION
'
' #########################
' #####  isValidName  #####
' #########################
'
'
'
FUNCTION isValidName (string$)
	
	IF TRIM$(string$)
		len=LEN(string$)-1
		FOR i=0 TO len
			char=string${i}
			IF (char<0x41 || char>0x5a) && (char<0x61 || char>0x7a) && (char<0x30 || char>0x39)
				IF (char<>0x3a) && (char<>0x2d) && (char<>0x5f)
					RETURN $$FALSE
				END IF
			END IF
		NEXT i
	ELSE
		RETURN $$FALSE
	END IF
	
	RETURN $$TRUE

END FUNCTION
'
' #####################
' #####  CSTRING  #####
' #####################
'
'
'
FUNCTION CSTRING (string$)
	
	IF string$
		string$=string$+"\0"
		size=SIZE(string$)+1
		addr=Xmalloc (size)
		XstCopyMemory (&string$, addr, size)
		RETURN addr
	ELSE
		RETURN 0
	END IF

END FUNCTION
'
' #########################
' #####  DestroyNode  #####
' #########################
'
'
'
FUNCTION DestroyNode (node)

	SHARED XMLNODE Node[]
	XMLNODE blank
	
	Xfree(Node[node].name)
	Xfree(Node[node].data)

	FOR i=0 TO $$XmlMaxAttributes
		IF Node[node].attribute[i]>-1 THEN DestroyAttribute (Node[node].attribute[i])
	NEXT i
	
	Node[node]=blank
	
END FUNCTION
'
' ########################
' #####  XmlSetFlag  #####
' ########################
'
'
'
FUNCTION XmlSetFlag (flag, value)

	SHARED collectEntities, useEvents, collectXML, collectData
	
	SELECT CASE flag
		CASE $$XmlOption_CollectEntities:collectEntities=value
		CASE $$XmlOption_UseEvents:useEvents=value
		CASE $$XmlOption_CollectXml:collectXML=value
		CASE $$XmlOption_CollectData:collectData=value
	END SELECT

END FUNCTION
'
' ################################
' #####  XmlDefaultSettings  #####
' ################################
'
'
'
FUNCTION XmlDefaultSettings ()

	SHARED useEntities, useEvents, collectXML, collectData
	SHARED FUNCADDR XmlListener(XLONG,XLONG)

	useEntities=$$TRUE
	useEvents=$$TRUE
	collectXML=$$TRUE
	collectData=$$TRUE
	XmlListener=&DefaultListener()

END FUNCTION
'
' ###################
' #####  Hello  #####
' ###################
'
'
'
FUNCTION Hello (code$)

	? code$

END FUNCTION
'
' ############################
' #####  XmlSetListener  #####
' ############################
'
'
'
FUNCTION XmlSetListener (func)

	SHARED FUNCADDR XmlListener(XLONG,XLONG)
	
	IF func THEN XmlListener=func ELSE XmlListener=&DefaultListener ()

END FUNCTION
'
' #############################
' #####  DefaultListener  #####
' #############################
'
'
'
FUNCTION DefaultListener (a,b)

	SELECT CASE a
		CASE $$XmlEvent_AddNode:
		CASE $$XmlEvent_Start:
			'? "start"
		CASE $$XmlEvent_End:'? "end"
	END SELECT

END FUNCTION
'
' ##########################
' #####  AddAttribute  #####
' ##########################
'
'
'
FUNCTION AddAttribute (name$, value$)
	
	SHARED XMLATTRIBUTE Attribute[]

	IF TRIM$(name$)
		IFF isValidName (name$) THEN RETURN -1
		upp=UBOUND(Attribute[])+1
		REDIM Attribute[upp]
		Attribute[upp].name=CSTRING(name$)
		IF TRIM$(value$) THEN lpvalue=CSTRING(value$) ELSE lpvalue=0
		Attribute[upp].value=lpvalue
		RETURN upp
	END IF
	
	RETURN -1

END FUNCTION
'
' ##############################
' #####  DestroyAttribute  #####
' ##############################
'
'
'
FUNCTION DestroyAttribute (attribute)

	SHARED XMLATTRIBUTE Attribute[]

	Xfree(Attribute[attribute].name)
	Xfree(Attribute[attribute].value)
	Attribute[attribute].name=0
	Attribute[attribute].value=0

END FUNCTION
'
' #########################
' #####  XmlGetNodes  #####
' #########################
'
' Returns the number of nodes currently added to the node pool. Note it includes the root node.
'
FUNCTION XmlGetNodes ()

	SHARED XMLNODE Node[]
	
	RETURN UBOUND(Node[])+1

END FUNCTION
'
' #################################
' #####  BinarySearch_STRING  #####
' #################################
'
'	Returns $$FALSE if the function failed to find the value, $$TRUE if it found at least one match.
'	
FUNCTION BinarySearch_STRING (STRING array[], STRING value, matches[])

	'return $$TRUE if array doesn't exist
	IFZ array[] THEN RETURN $$TRUE
	
	'copy array
	XstCopyArray (@array[], @temp$[])
	
	'initialize some variables
	first=0
	last=UBOUND(temp$[])
	m=-1
	
	'sort array
	DIM indices[0]
	XstQuickSort (@temp$[], @indices[], 0, last, $$SortIncreasing)
	
	'array to store our matches
	DIM matches[]

	'loop through array
	DO WHILE(first<=last)
		
		'calculate middle index
		m=(first+last)/2
		
		'test if we've found a match
		IF XstCompareStrings (&temp$[m], $$EQ, &value, mode)
			first=m+1
			upp=UBOUND(matches[])+1
			REDIM matches[upp]
			matches[upp]=indices[m]
		ELSE
			'decide whether value is in the upper or lower half of this section
			IF XstCompareStrings (&temp$[m], $$GT, &value, mode) THEN last=m-1 ELSE first=m+1
		END IF
		
	LOOP
	
	'our result
	IF UBOUND(matches[])==-1 THEN RETURN $$FALSE ELSE RETURN $$TRUE

END FUNCTION
'
' ######################
' #####  FindNode  #####
' ######################
'
'
'
FUNCTION FindNode (name$, instance)

	SHARED Node$[]
	
	BinarySearch_STRING (@Node$[], name$, @matches[])
	RETURN matches[instance]
	
END FUNCTION
'
' ##########################
' #####  XmlGetChildB  #####
' ##########################
'
'
'
FUNCTION XMLNODE XmlGetChildB (XMLNODE parent,child$,instance)
	
	XMLNODE node
	
	SHARED XMLNODE Node[]
	SHARED Node$[]
	
	'do a search on our node array
	BinarySearch_STRING (@Node$[], child$, @matches[])
	
	'return the node
	upp=UBOUND(matches[])
	IF upp==-1 THEN RETURN node'error
	IF instance>upp || instance<0 THEN RETURN node'error
	IF instance<=1 THEN RETURN Node[matches[0]]
	RETURN Node[matches[instance-1]]
	
END FUNCTION
END FUNCTION
'
' ##########################
' #####  ChangeParent  #####
' ##########################
'
'
'
FUNCTION ChangeParent (child, newparent)

	SHARED XMLNODE Node[]
	
	Node[child].parent=newparent
	
	IFT HasChildren (Node[child])
		upp=UBOUND(Node[])
		FOR i=0 TO upp
			IF Node[i].parent==child
				IFF ChangeParent(i, newparent)THEN RETURN $$TRUE
			END IF
		NEXT i
		RETURN $$TRUE
	END IF
	
	RETURN $$FALSE

END FUNCTION
'
' ###############################
' #####  XmlGetNodeAddress  #####
' ###############################
'
'
'
FUNCTION XmlGetNodeAddress (XMLNODE node)

	SHARED XMLNODE Node[]

	RETURN &Node[node.id]

END FUNCTION
'
' ###################################
' #####  XmlGetNodePoolAddress  #####
' ###################################
'
'
'
FUNCTION XmlGetNodePoolAddress ()

	SHARED XMLNODE Node[]
	
	RETURN &Node[]

END FUNCTION
'
' #########################
' #####  XmlCompile$  #####
' #########################
'
'
'
FUNCTION XmlCompile$ (useIndention)

	SHARED XMLNODE Node[]
	SHARED compiledTree$
	SHARED indention
	
	indention=0
	compiledTree$=""
	upp=UBOUND(Node[])
	FOR i=1 TO upp
		IF Node[i].parent==0
			
			IFT useIndention THEN indention$=CHR$('\t', indention)
			
			attributes$=BuildAttributeString$ (i)
			IF attributes$ THEN attributes$=" "+attributes$
			
			SELECT CASE Node[i].type
				CASE $$XmlNodeType_Irregular:
					compiledTree$=compiledTree$+indention$+"<"+CSTRING$(Node[i].name)+attributes$+"/>\n"
				CASE $$XmlNodeType_Regular:
					compiledTree$=compiledTree$+indention$+"<"+CSTRING$(Node[i].name)+attributes$+">\n"
				CASE $$XmlNodeType_Xml:
					compiledTree$=compiledTree$+indention$+"<?"+CSTRING$(Node[i].name)+attributes$+"?>\n"
			END SELECT
			
			INC indention
			
			IFT HasChildren (Node[i])
				Compile(i, useIndention)
			ELSE
				IF Node[i].data
					compiledTree$=compiledTree$+CSTRING$(Node[i].data)+"\n"
				END IF
			END IF
			
			DEC indention
			
			IF Node[i].type==$$XmlNodeType_Regular
				compiledTree$=compiledTree$+indention$+"</"+CSTRING$(Node[i].name)+">\n"
			END IF

		END IF
	NEXT i
	
	RETURN TRIM$(compiledTree$)

END FUNCTION
'
' #####################
' #####  Compile  #####
' #####################
'
'
'
FUNCTION Compile (parent, useCDATA)

	SHARED XMLNODE Node[]
	SHARED compiledTree$
	SHARED indention
	
	upp=UBOUND(Node[])
	FOR i=1 TO upp
		IF Node[i].parent==parent
			
			IFT useIndention THEN indention$=CHR$('\t', indention)
			
			attributes$=BuildAttributeString$ (i)
			IF attributes$ THEN attributes$=" "+attributes$
			
			SELECT CASE Node[i].type
				CASE $$XmlNodeType_Irregular:
					compiledTree$=compiledTree$+indention$+"<"+CSTRING$(Node[i].name)+attributes$+"/>\n"
				CASE $$XmlNodeType_Regular:
					compiledTree$=compiledTree$+indention$+"<"+CSTRING$(Node[i].name)+attributes$+">\n"
				CASE $$XmlNodeType_Xml:
					compiledTree$=compiledTree$+indention$+"<?"+CSTRING$(Node[i].name)+attributes$+"?>\n"
			END SELECT
			
			INC indention
			
			IFT HasChildren (Node[i])
				Compile(i, useIndention)
			ELSE
				IF Node[i].data
					compiledTree$=compiledTree$+CSTRING$(Node[i].data)+"\n"
				END IF
			END IF
			
			DEC indention
			
			IF Node[i].type==$$XmlNodeType_Regular
				compiledTree$=compiledTree$+indention$+"</"+CSTRING$(Node[i].name)+">\n"
			END IF
			
		END IF
	NEXT i

END FUNCTION
'
' ###################################
' #####  BuildAttributeString$  #####
' ###################################
'
'
'
FUNCTION BuildAttributeString$ (node)

	SHARED XMLNODE Node[]
	SHARED XMLATTRIBUTE Attribute[]
	
	string$=""
	FOR i=0 TO $$XmlMaxAttributes
		IF Node[node].attribute[i]>-1
			name$=CSTRING$(Attribute[Node[node].attribute[i]].name)
			value$=CSTRING$(Attribute[Node[node].attribute[i]].value)
			string$=string$+name$+"='"+value$+"' "
		END IF
	NEXT i
	RETURN TRIM$(string$)

END FUNCTION
END PROGRAM