'
'
' ####################
' #####  PROLOG  #####
' ####################
'	Author: Alan Gent November 2005
'	Version: 1.0000 Initial version
' Copyight: Alan Gent 2005
'
' This library is available for free use in any open source, private or commercial product
' provided that the authors name and copyright remain with the code.
' Use of this code is entirely at your own risk.
'
' 
'	INSTALLATION:
'	
'	Unzip the turtle.zip file into your source code folder
'	Compile this file ( turtle.x ) as a DLL using Run / Compile as Library (DLL)
' Build the library using Run / Build or PF10
'	Copy file turtle.dll to your program folder
'	Copy file turtle.dec to your include folder
'	Copy file turtle.lib to your lib folder
'	
'
' 
'	GETTING STARTED:
'	
'	After installing as above:
'	Compile and execute some of the demo programs to confirm correct installation
'	
' 
' 
'	YOUR OWN TURTLE PROGRAMS:
'	
'	The PROLOG must contain an IMPORT "turtle" line
'	Put your turtle instructions in the $$WM_PAINT section

'	Use BeginPaint() to get a valid device context ( hdc ) for the window you want to use
'	and pass it to InitialiseTurtle()

'	You MUST initialise the turtle with InitialiseTurtle(hwnd, hdc, origin) before any drawing
' can be done

'	Be careful to use DOUBLE values wherever these are required. Xblite will convert function parameters
'	automatically but in other cases it is down to you
'
' Note that y values increase UP the screen in the usual mathematical coordinate system
'	
'	Call TerminateTurtle() after drawing to clean up turtle created pens and restore the original
' pen back in to the DC

' When you have finished all you want to do in the $$WM_PAINT section use EndPaint() 
'	
'	Use the demo programs as a template or copy turtle.xxx to your templates directory and add the line
' New Turtle Program, turtle.xxx
' to the file templates.xxx
'	
' 
' 
'	SUMMARY OF FUNCTIONS:
'	
'	Details of function parameters and effects are included with the code for each function below
'	Most turtle functions are quite simple and easily understood from the source code
'	
'	
'	Initialize and set turtle characteristics:
'  
'		InitialiseTurtle (hwnd, hdc, origin)
'		SetTurtleOrigin(origin)
'		SetTurtleRange (xrange, yrange)
'		SetTurtleScaleFactor (DOUBLE xfactor, DOUBLE yfactor)
'		SetTurtleColor (color)
'		SetTurtleWidth (width)
'		SetTurtleStyle (style)
'		SetTurtlePen (style, width, color)
'		SetTurtleDelay (msec)
'	  SetCheckMessages (onoff)
'		TerminateTurtle ()
'
'
'	Move turtle with trail ( if pendown ):
' 
'		Move (DOUBLE distance)
'		MoveTo (DOUBLE x, DOUBLE y)
'		MoveRel (DOUBLE deltax, DOUBLE deltay)
'		Forward (DOUBLE distance)
'		Backward (DOUBLE distance)	
'		Curve (DOUBLE radius, DOUBLE angle, pixelsperline)
'		DrawTurtlePoint ()
'		
'	Move turtle with no trail:
'	
'		Go (DOUBLE distance)
'		GoTo (DOUBLE x, DOUBLE y)
'		GoRel (DOUBLE deltax, DOUBLE deltay)
'		CentreTurtle ()
'		HomeTurtle ()
'	
'	Turn Turtle:
' 
'		Turn (DOUBLE degrees)
'		Right (DOUBLE degrees)
'		Left (DOUBLE  degrees)
'		TurnTo (DOUBLE degrees)
'	
'	Control Pen:
'  
'		PenUp ()
'		PenDown ()
'		TogglePen ()
'	
'	Test / Get turtle position:
'	
'		TurtleOnScreen (beep)
'		DOUBLE GetTurtleLogicalX ()
'		DOUBLE GetTurtleLogicalY ()
'		DOUBLE GetTurtleScreenX ()
'		DOUBLE GetTurtleScreenY ()
'		DOUBLE GetTurtleDegrees ()
'		DOUBLE GetTurtleRadians ()
'	
'	Color Management:
'	
'		RandomColor ()
'		ColorNumber(color)
'		CreateColorSequence (c1,c2,c3,c4,c5,c6,c7,c8)
'		NextColorInSequence ()
'		RandomColorFromSequence ()
'		ColorSequence (index)
'	
'	Lstring creation and drawing:
'	
'		GenerateLstring (in$, rule$, @lstring$)
'		GenerateLstring2 (in$, rule$[], @lstring$)
'		DrawLstring (lstring$, angle#, distance#)
'
'	Turtle Stack:
'	
'		Push ()
'		Pop (trail)
'		Restore (trail)
'		ClearStack ()	
'	
'	Miscellaneous support functions:
'	
'		DOUBLE SinD (DOUBLE degrees)
'		DOUBLE CosD (DOUBLE degrees)
'		CheckMessages ()'
'
' MORE INFORMATION
' If you want to learn more about turtle graphics, fractals etc there is a wealth of material on the
' internet. The following key words will find plenty of useful information
'	"turtle graphics"
' "turtle logo"			- LOGO was a specialist turtle language
' "turtle fractal"
' "Seymour Papert"	- the original creator of turtle graphics
' "Lindenmayer"  		- the 'L' in L-string / L-system
'
'
'
PROGRAM	"turtle"  	' 1-8 char program/file name without .x or any .extent
VERSION	"1.0000"    ' version number - increment before saving altered program
'MAKEFILE "xdll.xxx"
'
' Programs contain:  1: PROLOG          - no executable code - see below
'                    2: Entry function  - start execution at 1st declared func
' * = optional       3: Other functions - everything else - all other functions
'
' The PROLOG contains (in this order):
' * 1. Program name statement             PROGRAM "progname"
' * 2. Version number statement           VERSION "0.0000"
' * 3. Import library statements          IMPORT  "libName"
' * 4. Composite type definitions         TYPE <typename> ... END TYPE
'   5. Internal function declarations     DECLARE/INTERNAL FUNCTION Func (args)
' * 6. External function declarations     EXTERNAL FUNCTION FuncName (args)
' * 7. Shared constant definitions        $$ConstantName = literal or constant
' * 8. Shared variable declarations       SHARED  variable
'
' ******  Comment libraries in/out as needed  *****
'
	IMPORT	"xma"   		' Math library		 : Sin\Cos\Tan...
'	IMPORT	"xcm"   		' Complex library  : complex number library  (trig, etc)
	IMPORT	"xst"   		' Standard library : required by most programs
	IMPORT	"xsx"   		' Standard library : required by most programs
'	IMPORT	"xio"			' Console IO library
	IMPORT  "gdi32"
	IMPORT	"user32"
	IMPORT  "kernel32"
'


DECLARE FUNCTION  Entry ()

EXPORT
DECLARE FUNCTION  InitialiseTurtle (hwnd, hdc, origin)
DECLARE FUNCTION  SetTurtleOrigin(origin)
DECLARE FUNCTION  SetTurtleRange (xrange, yrange)
DECLARE FUNCTION  SetTurtleScaleFactor (DOUBLE xfactor, DOUBLE yfactor)
DECLARE FUNCTION  SetTurtleColor (color)
DECLARE FUNCTION  SetTurtleWidth (width)
DECLARE FUNCTION  SetTurtleStyle (style)
DECLARE FUNCTION  SetTurtlePen (style, width, color)
DECLARE FUNCTION  SetTurtleDelay (msec)
DECLARE FUNCTION  TerminateTurtle ()
DECLARE FUNCTION  CentreTurtle ()
DECLARE FUNCTION  HomeTurtle ()
DECLARE FUNCTION  Move (DOUBLE distance)
DECLARE FUNCTION  MoveTo (DOUBLE x, DOUBLE y)
DECLARE FUNCTION  MoveRel (DOUBLE deltax, DOUBLE deltay)
DECLARE FUNCTION  Go (DOUBLE distance)
DECLARE FUNCTION  GoTo (DOUBLE x, DOUBLE y)
DECLARE FUNCTION  GoRel (DOUBLE deltax, DOUBLE deltay)
DECLARE FUNCTION  Forward (DOUBLE distance)
DECLARE FUNCTION  Backward (DOUBLE distance)
DECLARE FUNCTION  Curve (DOUBLE radius, DOUBLE angle, pixelsperline)
DECLARE FUNCTION  DrawTurtlePoint ()
DECLARE FUNCTION  Turn (DOUBLE degrees)
DECLARE FUNCTION  Right (DOUBLE degrees)
DECLARE FUNCTION  Left (DOUBLE  degrees)
DECLARE FUNCTION  TurnTo (DOUBLE degrees)
DECLARE FUNCTION  PenUp ()
DECLARE FUNCTION  PenDown ()
DECLARE FUNCTION  TogglePen ()
DECLARE FUNCTION  TurtleOnScreen (beep)
DECLARE FUNCTION  DOUBLE GetTurtleLogicalX ()
DECLARE FUNCTION  DOUBLE GetTurtleLogicalY ()
DECLARE FUNCTION  DOUBLE GetTurtleScreenX ()
DECLARE FUNCTION  DOUBLE GetTurtleScreenY ()
DECLARE FUNCTION  DOUBLE GetTurtleDegrees ()
DECLARE FUNCTION  DOUBLE GetTurtleRadians ()
DECLARE FUNCTION  DOUBLE SinD (DOUBLE degrees)
DECLARE FUNCTION  DOUBLE CosD (DOUBLE degrees)
DECLARE FUNCTION  RandomColor ()
DECLARE FUNCTION  ColorNumber(color)
DECLARE FUNCTION  CreateColorSequence (c1,c2,c3,c4,c5,c6,c7,c8)
DECLARE FUNCTION  NextColorInSequence ()
DECLARE FUNCTION  RandomColorFromSequence ()
DECLARE FUNCTION  ColorSequence (index)
DECLARE FUNCTION  DrawLstring (lstring$, angle#, distance#)
DECLARE FUNCTION  GenerateLstring (in$, rule$, @lstring$)
DECLARE FUNCTION  GenerateLstring2 (in$, rule$[], @lstring$)
DECLARE FUNCTION  Push ()
DECLARE FUNCTION  Pop (trail)
DECLARE FUNCTION  Restore (trail)
DECLARE FUNCTION  ClearStack ()
DECLARE FUNCTION  SetCheckMessages (onoff)
DECLARE FUNCTION  CheckMessages ()
END EXPORT
INTERNAL FUNCTION  DOUBLE NormalizeRadians (DOUBLE radians)

'
'
' ######################
' #####  Entry ()  #####
' ######################
'
FUNCTION  Entry ()
	IF LIBRARY(0) THEN RETURN
END FUNCTION


'=====================================================================================================================
'=======================TURTLE FUNCTIONS==============================================================================
'=====================================================================================================================


' ################################
' #####  SetTurtleOrigin ()  #####
' ################################
'
' the value of origin defines the location of point 0,0 as indicated in the diagram below
' sets min and max x, y coordinates for checking by TurtleOnScreen()
'
' *************************************
' *3                7                4*
' *                                   *
' *                                   *
' *                                   *
' *                                   *
' *6                1                8*
' *                                   *
' *                                   *
' *                                   *
' *                                   *
' *2                9                5*
' *************************************
'
FUNCTION  SetTurtleOrigin (origin)
SHARED thwnd, thdc, minx, maxx, miny, maxy, cx, cy
RECT rct


	GetClientRect (thwnd, &rct)
	cx = rct.right 
	cy = rct.bottom 
	
	SetMapMode (thdc, MM_ISOTROPIC)
	
	SELECT CASE origin
		
		CASE 1:
			'origin is centre of window ( all 4 quadrants show )
			minx = -cx \ 2 : maxx = cx \ 2
			miny = -cy \ 2 : maxy = cy \ 2
			SetViewportOrgEx (thdc, cx \ 2, cy \ 2, NULL)
			
		CASE 2:
			minx = 0 : maxx = cx
			miny = 0 : maxy = cy
			'origin is bottom left ( quadrant = +ve x, +ve y )
			SetViewportOrgEx (thdc, 0, cy, NULL)
			
		CASE 3:
			minx = 0 : maxx = cx
			miny = - cy : maxy = 0
			'origin is top left ( quadrant = +ve x, -ve y )
			SetViewportOrgEx (thdc, 0, 0, NULL)
			
		CASE 4:
			minx = - cx : maxx = 0
			miny = - cy : maxy = 0
			'origin is top right ( quadrant = -ve x, -ve y ) 
			SetViewportOrgEx (thdc, cx, 0, NULL)
			
		CASE 5:
			minx = - cx : maxx = 0
			miny =  0 : maxy = cy
			'origin is bottom right ( quadrant = -ve x, +ve y
			SetViewportOrgEx (thdc, cx, cy, NULL)
			
		CASE 6:
			'origin is centre left of window (+ve x, +ve and -ve y)
			minx = 0 : maxx = cx
			miny = -cy \ 2 : maxy = cy \ 2
			SetViewportOrgEx (thdc, 0, cy \ 2, NULL)
			
		CASE 7:
			'origin is centre top of window (-ve y, +ve and -ve x)
			minx = -cx \ 2 : maxx = cx \ 2
			miny = -cy : maxy = 0
			SetViewportOrgEx (thdc, cx \ 2, 0, NULL)
			
		CASE 8:
			'origin is centre right of window (-ve x, +ve and -ve y)
			minx = -cx : maxx = 0
			miny = -cy \ 2 : maxy = cy \ 2
			SetViewportOrgEx (thdc, cx, cy \ 2, NULL)
			
			
		CASE 9:
			'origin is centre bottom of window (+ve y, +ve and -ve x)
			minx = -cx \ 2 : maxx = cx \ 2
			miny = 0 : maxy = cy 
			SetViewportOrgEx (thdc, cx \ 2, cy, NULL)
			
		CASE ELSE:
			minx = -cx \ 2 : maxx = cx \ 2
			miny = -cy \ 2 : maxy = cy \ 2
			'default origin is centre of window as case 1
			SetViewportOrgEx (thdc, cx \ 2, cy \ 2, NULL)
		
	END SELECT
	
END FUNCTION

' ###############################
' #####  SetTurtleDelay ()  #####
' ###############################
' 
' If pendown and last operation drew a line then delay for msec milliseconds
' Enables the action to be slowed down so that it can be followed!
' 
FUNCTION  SetTurtleDelay (msec)
SHARED delay
	delay = ABS(msec)
END FUNCTION

' #################################
' #####  InitialiseTurtle ()  #####
' #################################
' Creates and sets up initial values for a turtle and defines the origin
' You MUST call this before using any of the turtle commands as it 'attaches' the turtle to your window
' Once initialised, the other functions will use hwnd and hdc where needed without your needing to pass
' them in each function call
' 	
' thwnd    = handle of window turtle is in
' thdc     = handle to Device Context for turtle to use
' origin   = 1 to 9, see SetTurtleOrigin()
' tx       = turtle x coordinate (logical)
' ty       = turtle y coordinate (logical)
' sx       = turtle x coordinate (screen - physical)
' sy       = turtle y coordinate (screen - physical)
' ta       = turtle angle in radians measured clockwise from north = 0
' delay    = msec delay to be added after each drawing operation
' pendown  = leave trail only if this is $$TRUE
'	penstyle = style of trail e.g solid, dashed ( default solid )
' penwidth = width of trail in screen pixels  ( default 1 )
' pencolor = color of trail										( default black )
' xsf      = scaling factor for x values ( default 1.0 )
' ysf      = scaling factor for y values ( default -1.0 )
' tstack[] = stack for storing turtle location + bearing with Push(), Pop()
' checkmsg = flag to control if messages are checked for after each Move()



'
FUNCTION  InitialiseTurtle(hwnd, hdc, origin)
SHARED DOUBLE tx, ty, sx, sy, ta, xsf, ysf, tstack[]
SHARED pendown, delay, thwnd, thdc, checkmsg
SHARED penstyle, penwidth, pencolor, oldpen 
SHARED stacksize, stackptr

	IFZ hwnd THEN RETURN $$TRUE
	IFZ hdc  THEN RETURN $$TRUE
	
	'create turtle stack
	stacksize = 600
	IFZ tstack[] THEN DIM tstack[stacksize] : stackptr = 0
	
	'set initial characteristics
	penstyle	= $$PS_SOLID
	penwidth 	= 1
	pencolor	= $$Black
	tpen 			= CreatePen(penstyle, penwidth, pencolor)
	oldpen 		= SelectObject(hdc, tpen)	'TerminateTurtle() reinstates oldpen 
	thwnd 		= hwnd						'store handle to window
	thdc 			= hdc							'store handle to user supplied Devise ContextC
	ta 				= 0								'heading due north
	delay 		= 0								'no delay
	pendown 	= $$TRUE					'pendown
	xsf 			= 1.0							'1 to 1 scaling on x axis
	ysf 			= -1.0						'1 to 1 scaling on y axis and y to increase up the screen
  checkmsg 	= $$TRUE					'check message queue after each move
	
	
	'initialise position
	SetTurtleOrigin(origin)			'set 0,0 point - default is centre of client area
	HomeTurtle ()								'move to origin	
	
END FUNCTION
 
' ###########################
' #####  HomeTurtle ()  #####
' ###########################
'
' Moves the turtle to the origin
'
FUNCTION  HomeTurtle()
SHARED thdc
SHARED DOUBLE tx, ty, sx, sy

	'no scaling required for 0,0
	tx = 0 : sx = 0
	ty = 0 : sy = 0

	MoveToEx(thdc, XLONG(sx), XLONG(sy), NULL)	

END FUNCTION

'
'
' ######################
' #####  PenUp ()  #####
' ######################
' 
' Set pen up so that no trail is drawn
'
FUNCTION  PenUp ()
SHARED pendown

	pendown = $$FALSE

END FUNCTION
'
'
' ######################
' ####  PenDown ()  ####
' ######################
'
' Set pendown ready for drawing
'
FUNCTION  PenDown ()
SHARED pendown

	pendown = $$TRUE

END FUNCTION


' ##########################
' #####  TogglePen ()  #####
' ##########################
'
' If pen is up it is put down
' If pen is down it is set to up
'
FUNCTION  TogglePen ()
SHARED pendown

	pendown = ! pendown

END FUNCTION


' #####################
' #####  Move ()  #####
' #####################
'
' Moves the turtle distance logical units in the direction of the current bearing
' If pendown then leaves a trail and delays if required
' Else, just moves to the new position with no trail and no delay
'
FUNCTION  Move (DOUBLE distance)
SHARED thdc, thwnd, pendown, delay, checkmsg
SHARED DOUBLE tx, ty, sx, sy, ta, xsf, ysf

	IFZ distance THEN RETURN $$TRUE
	tx = tx + distance * Sin(ta) : sx = tx * xsf 
	ty = ty + distance * Cos(ta) : sy = ty * ysf
	
	
	IF pendown THEN
		LineTo(thdc, XLONG(sx), XLONG(sy))
		IF delay THEN XstSleep(delay)
	ELSE
		MoveToEx(thdc, XLONG(sx), XLONG(sy), NULL)
	ENDIF
	
	IF checkmsg THEN	CheckMessages()	

END FUNCTION

' #######################
' #####  MoveTo ()  #####
' #######################
'
' Moves the turtle to x, y
' If pendown then draws a line and delays if required
' If penup no line and no delay ( same as GoTo())
'
FUNCTION  MoveTo (DOUBLE x,DOUBLE y)
SHARED thdc, pendown, delay
SHARED DOUBLE tx, ty, sx, sy, xsf, ysf

	tx = x : sx = tx * xsf		
	ty = y : sy = ty * ysf 
	
	IF pendown THEN
		LineTo(thdc, XLONG(sx), XLONG(sy))
		IF delay THEN XstSleep(delay)
	ELSE
		MoveToEx(thdc, XLONG(sx), XLONG(sy), NULL)
	ENDIF

END FUNCTION

' ########################
' #####  MoveRel ()  #####
' ########################
'
' Moves the turtle to tx + deltax, ty + deltay
' If pendown then draws a line and delays if required
' If penup no line and no delay ( same as GoRel())
'
FUNCTION  MoveRel (DOUBLE deltax,DOUBLE deltay)
SHARED thdc, pendown, delay
SHARED DOUBLE tx, ty

	tempx = tx + deltax 
	tempy = ty + deltay 
	
	MoveTo (tempx, tempy)	'scaling done within MoveTo

END FUNCTION

' ###################
' #####  Go ()  #####
' ###################
'
' Moves the turtle d units in the direction of the current bearing
' never draws a line - even if pendown, never delays
'
FUNCTION  Go (DOUBLE distance)
SHARED pendown

	IFZ distance THEN RETURN $$TRUE
	
	temp = pendown
	pendown = $$FALSE
	
	Move (distance)

	pendown = temp

END FUNCTION


' #####################
' #####  GoTo ()  #####
' #####################
'
' Moves the turtle to x, y
' Never draws a line - even if pendown, never delays
'
FUNCTION  GoTo (DOUBLE x,DOUBLE y)
SHARED thdc
SHARED DOUBLE tx, ty, sx, sy, xsf, ysf

	tx = x : sx = tx * xsf   
	ty = y : sy = ty * ysf  
	
	MoveToEx(thdc, XLONG(sx), XLONG(sy), NULL)

END FUNCTION

' ######################
' #####  GoRel ()  #####
' ######################
'
' Moves the turtle to tx + deltax, ty + deltay
' Never draws a line, never delays
'
FUNCTION  GoRel (DOUBLE deltax,DOUBLE deltay)
SHARED pendown
	temp = pendown
	pendown = $$FALSE

	MoveRel (deltax,deltay)

	pendown = temp

END FUNCTION


' #####################
' #####  Forward  #####
' #####################
'
' Compatability with LOGO
FUNCTION Forward (DOUBLE distance)
	IFZ distance THEN RETURN $$TRUE
	Move (ABS(distance))
END FUNCTION

' ######################
' #####  Backward  #####
' ######################
'
' Compatability with LOGO
FUNCTION Backward (DOUBLE distance)
	IFZ distance THEN RETURN $$TRUE
	Move (-ABS(distance))
END FUNCTION

' #####################
' #####  Turn ()  #####
' #####################
'
' Turn input degrees clockwise if + ve
' Turn input degrees anticlockwise if - ve
'
FUNCTION Turn (DOUBLE degrees)
SHARED DOUBLE ta

	IFZ degrees THEN RETURN $$TRUE
	
	'Normalise angle to range 0-2PI
	ta = NormalizeRadians (ta + (degrees * $$DEGTORAD))

END FUNCTION

' ######################
' #####  Right ()  #####
' ######################
'
' Compatability with LOGO
' Always turn clockwise
'
FUNCTION Right (DOUBLE degrees)
	IFZ degrees THEN RETURN $$TRUE
	Turn (ABS(degrees))
END FUNCTION

' #####################
' #####  Left ()  #####
' #####################
'
' Compatability with LOGO
' Always turn anticlockwise
'
FUNCTION Left (DOUBLE degrees)
	IFZ degrees THEN RETURN $$TRUE
	Turn (-ABS(degrees))
END FUNCTION


' #####################
' ##### TurnTo ()  ####
' #####################
'
' Turn to absolute bearing degrees measured clockwise from due north = 0 degrees
'  
FUNCTION TurnTo (DOUBLE degrees)
SHARED DOUBLE ta

	ta = NormalizeRadians (degrees * $$DEGTORAD)

END FUNCTION


' ################################
' ##### GetTurtleLogicalX ()  ####
' ################################
'
' return turtle's current X coordinate in logical units
'  
FUNCTION DOUBLE GetTurtleLogicalX ()
SHARED DOUBLE tx

	RETURN tx

END FUNCTION


' ################################
' ##### GetTurtleLogicalY ()  ####
' ################################
'
' return turtle's current Y coordinate in logical units
'  
FUNCTION DOUBLE GetTurtleLogicalY ()
SHARED DOUBLE ty

	RETURN ty

END FUNCTION


' ###############################
' ##### GetTurtleRadians ()  ####
' ###############################
'
' return turtle's current bearing in radians
'  
FUNCTION DOUBLE GetTurtleRadians ()
SHARED DOUBLE ta

	RETURN ta

END FUNCTION


' ###############################
' ##### GetTurtleDegrees ()  ####
' ###############################
'
' return turtle's current bearing in degrees
'  
FUNCTION DOUBLE GetTurtleDegrees ()
SHARED DOUBLE ta

	RETURN ta * $$RADTODEG

END FUNCTION




' ##############################
' ##### NormalizeRadians () ####
' ##############################
'
' Normalise angle to positive value in range 0-2PI
' INTERNAL use only, not exported to DLL

FUNCTION  DOUBLE NormalizeRadians (DOUBLE angle)
DOUBLE normangle

	normangle = angle - (FIX(angle / $$TWOPI)) * $$TWOPI 		' reduce to -$$TWOPI to +$$TWOPI
	IF normangle < 0 THEN normangle = normangle + $$TWOPI		' make it positive 0 to $$TWOPI
	RETURN normangle

END FUNCTION
'
' ############################
' #####  SetTurtleRange  #####
' ############################
'
' define the number of logical pixels for screen width (xrange) or height (yrange)
' scale factor = screen pixels / logical pixels
' y scale factor made -ve to make y axis increase towards top of screen
' normally input only (xrange, NULL) or (NULL, yrange). The missing factor defaults to
' the same absolute value as the supplied one for undistorted graphics
' input (xrange, yrange) for graph plotting or deliberate distortion / stretching
' See also SetTurtleScaleFactor() which is often simpler to use but does NOT rescale if 
' the window is resized.
' Note that negative values will reverse the direction of the axis! 
'
FUNCTION SetTurtleRange (xrange, yrange)
SHARED thwnd, cx, cy
SHARED DOUBLE xsf, ysf
RECT rct

	IF (xrange = 0) && (yrange = 0) THEN RETURN $$TRUE
	
	IF xrange <> 0 THEN
		xsf = DOUBLE (cx) / DOUBLE (xrange)
		IF yrange = 0 THEN ysf = - ABS(xsf)
	ENDIF
	
	IF yrange <> 0 THEN
		ysf = - DOUBLE (cy) / DOUBLE (yrange)
		IF xrange = 0 THEN xsf = ABS(ysf)
	ENDIF	

END FUNCTION
'
' ##################################
' #####  SetTurtleScaleFactor  #####
' ##################################
'
' input number of logical pixels per physical pixel on screen for x and y axes
' scale factor > 1 reduces size of output
' scale factor < 1 increases size of output
' y scale factor made -ve to make y axis increase towards top of screen
' for undistorted graphics keep ABS(xfactor) = ABS(yfactor)
' default is 1.0, -1.0
' Negative values reverse the direction of the axis!
' 
FUNCTION SetTurtleScaleFactor (DOUBLE xfactor, DOUBLE yfactor)
SHARED DOUBLE xsf, ysf

	IFZ xfactor THEN
		xsf = 1.0
	ELSE
		xsf = 1 / xfactor
	ENDIF
	
	'make y increase in up direction
	IFZ yfactor THEN
		ysf = -1.0
	ELSE	
		ysf = -1 / yfactor
	ENDIF

END FUNCTION

'
' ############################
' #####  TurtleOnScreen  #####
' ############################
'
' Returns TRUE if turtle is in visible part of screen (client area)
' Returns FALSE if turtle has gone off screen
' If FALSE, optionally emit warning sound if beep not zero
' Silent if beep = 0
'
FUNCTION TurtleOnScreen (beep)
SHARED  minx, maxx, miny, maxy
SHARED DOUBLE sx, sy
	
	IF XLONG(sx) >= minx && XLONG(sx) <= maxx && XLONG(sy) <= -miny && XLONG(sy) >= -maxy  THEN
		RETURN $$TRUE
	ELSE
		IF beep THEN MessageBeep(0)
		RETURN $$FALSE
	ENDIF

END FUNCTION
'
' ##############################
' #####  GetTurtleScreenX  #####
' ##############################
'
' return turtle's current X coordinate in physical pixels
'
FUNCTION DOUBLE GetTurtleScreenX ()
SHARED DOUBLE sx

	RETURN sx

END FUNCTION
'
' ##############################
' #####  GetTurtleScreenY  #####
' ##############################
'
' return turtle's current Y coordinate in physical pixels
'
FUNCTION DOUBLE GetTurtleScreenY ()
SHARED DOUBLE sy

	RETURN sy

END FUNCTION

'
' ##################
' #####  SinD  #####
' ##################
' Convenience supporting function
' Sine function with input in degrees
'
FUNCTION DOUBLE SinD (DOUBLE degrees)
	RETURN Sin(degrees * $$DEGTORAD)
END FUNCTION
'
' ##################
' #####  CosD  #####
' ##################
' Convenience supporting function
' Cosine function with input in degrees
'
FUNCTION DOUBLE CosD (DOUBLE degrees)
	RETURN Cos(degrees * $$DEGTORAD)
END FUNCTION
'
' ##########################
' #####  CentreTurtle  #####
' ##########################
'
' Moves Turtle to centre of client area, irrespective of origin
'
FUNCTION CentreTurtle ()
SHARED thdc, minx, maxx, miny, maxy
SHARED DOUBLE tx, ty, sx, sy, xsf, ysf
	
	sx = DOUBLE((minx + maxx) \ 2) : tx = sx / xsf 
	sy = - DOUBLE((miny + maxy) \ 2) : ty = sy / ysf
	
	MoveToEx(thdc, XLONG(sx), XLONG(sy), NULL)
	
END FUNCTION
'
' #############################
' #####  DrawTurtlePoint  #####
' #############################
'	
' Sets the point at the Turtle position ( If pendown )
' Turtle position is not changed
'
FUNCTION DrawTurtlePoint ()
SHARED thdc, pendown, delay, pencolor, penwidth
SHARED DOUBLE sx, sy

	IF pendown THEN
		IF penwidth > 1 THEN
			Move(-1)						'includes a delay because pendown
			Go(1)
		ELSE
			SetPixelV(thdc, XLONG(sx), XLONG(sy), pencolor)
			XstSleep(delay)			'separate delay required
		ENDIF		

	ELSE
		RETURN $$TRUE
	ENDIF

END FUNCTION
'
' ############################
' #####  SetTurtleColor  #####
' ############################
' Set the colour of the line drawn by the turtle 
'
'
FUNCTION SetTurtleColor (color)
SHARED thdc, penstyle, penwidth, pencolor
	
	pencolor = color
	DeleteObject(SelectObject(thdc, CreatePen(penstyle, penwidth, pencolor)))

END FUNCTION
'
' ############################
' #####  SetTurtleWidth  #####
' ############################
' Set the width of the line drawn by the turtle in screen pixels
' To use styles other than SOLID the width must be 1
'
FUNCTION SetTurtleWidth (width)
SHARED thdc, penstyle, penwidth, pencolor
	
	penwidth = width
	DeleteObject(SelectObject(thdc, CreatePen(penstyle, penwidth, pencolor)))

END FUNCTION
'
' ############################
' #####  SetTurtleStyle  #####
' ############################
'	Sets the style of the line ( trail) drawn by the turtle
' Acceptable styles are those used by Windows and defined in gdi32.dec
' e.g $$PS_SOLID, $$PS_DOT etc.
'
FUNCTION SetTurtleStyle (style)
SHARED thdc, penstyle, penwidth, pencolor
	
	penstyle = style
	DeleteObject(SelectObject(thdc, CreatePen(penstyle, penwidth, pencolor)))
	
END FUNCTION
'
' ##########################
' #####  SetTurtlePen  #####
' ##########################
'
' Set 2 or more attributes of pen in one function
'
FUNCTION SetTurtlePen (style, width, color)
SHARED thdc, penstyle, penwidth, pencolor
	
	penstyle = style
	penwidth = width
	pencolor = color
	DeleteObject(SelectObject(thdc, CreatePen(penstyle, penwidth, pencolor)))

END FUNCTION
'
' #########################
' #####  RandomColor  #####
' #########################
'
' returns a randomly generated 24 bit color
'
FUNCTION RandomColor ()	
	RETURN (XstRandom() AND 0x00FFFFFF)
END FUNCTION
'
' #########################
' #####  ColorNumber  #####
' #########################
'
' returns one of the standard predefined Xblite colors
' does not include white

FUNCTION ColorNumber(color)
STATIC colorarray[]
	IFZ colorarray[] THEN GOSUB initialise
	index = ABS(color MOD 41)
	RETURN colorarray[index]
	
	SUB initialise
		DIM colorarray[41]
		colorarray[0]  = $$Black
		colorarray[1]  = $$DarkBlue            
		colorarray[2]  = $$MediumBlue          
		colorarray[3]  = $$BrightBlue          
		colorarray[4]  = $$LightBlue           
		colorarray[5]  = $$DarkGreen           
		colorarray[6]  = $$MediumGreen        
		colorarray[7]  = $$BrightGreen         
		colorarray[8]  = $$LightGreen          
		colorarray[9]  = $$DarkCyan            
		colorarray[10] = $$MediumCyan          
		colorarray[11] = $$BrightCyan          
		colorarray[12] = $$LightCyan           
		colorarray[13] = $$DarkRed             
		colorarray[14] = $$MediumRed           
		colorarray[15] = $$BrightRed           
		colorarray[16] = $$LightRed            
		colorarray[17] = $$DarkMagenta         
		colorarray[18] = $$MediumMagenta       
		colorarray[19] = $$BrightMagenta       
		colorarray[20] = $$LightMagenta        
		colorarray[21] = $$DarkBrown          
		colorarray[22] = $$MediumBrown         
		colorarray[23] = $$Yellow              
		colorarray[24] = $$BrightYellow        
		colorarray[25] = $$LightYellow         
		colorarray[26] = $$DarkGrey            
		colorarray[27] = $$MediumGrey         
		colorarray[28] = $$LightGrey           
		colorarray[29] = $$BrightGrey         
		colorarray[30] = $$DarkSteel           
		colorarray[31] = $$Steel              
		colorarray[32] = $$BrightSteel         
		colorarray[33] = $$MediumOrange        
		colorarray[34] = $$BrightOrange        
		colorarray[35] = $$LightOrange         
		colorarray[36] = $$MediumAqua          
		colorarray[37] = $$BrightAqua          
		colorarray[38] = $$DarkViolet          
		colorarray[39] = $$MediumViolet        
		colorarray[40] = $$BrightViolet        
	END SUB
END FUNCTION              
'
' #################################
' #####  CreateColorSequence  #####
' #################################
' Create a sequence of up to 8 colors
' First negative number terminates the sequence
' Once created this set of colours can be accessed by:
'				NextColorInSequence ()
'				RandomColorFromSequence ()
'				ColorSequence (index)
'
'
FUNCTION CreateColorSequence (c1,c2,c3,c4,c5,c6,c7,c8)
SHARED cseqarray[], maxindex, csindex
	IFZ cseqarray THEN	DIM cseqarray[8]
	
	maxindex = -1
	csindex = -1
	IF c1 >= 0 THEN
		cseqarray[0] = c1: INC maxindex: csindex = 0
	ELSE
		RETURN
	ENDIF
	
	IF c2 >= 0 THEN
		cseqarray[1] = c2: INC maxindex
	ELSE
		RETURN
	ENDIF
	
	IF c3 >= 0 THEN
		cseqarray[2] = c3: INC maxindex
	ELSE
		RETURN
	ENDIF
	
	IF c4 >= 0 THEN
		cseqarray[3] = c4: INC maxindex
	ELSE
		RETURN
	ENDIF
	
	IF c5 >= 0 THEN
		cseqarray[4] = c5: INC maxindex
	ELSE
		RETURN
	ENDIF
	
	IF c6 >= 0 THEN
		cseqarray[5] = c6: INC maxindex
	ELSE
		RETURN
	ENDIF
	
	IF c7 >= 0 THEN
		cseqarray[6] = c7: INC maxindex
	ELSE
		RETURN
	ENDIF
	
	IF c8 >= 0 THEN
		cseqarray[7] = c8: INC maxindex
	ELSE
		RETURN
	ENDIF	

END FUNCTION
'
' #################################
' #####  NextColorInSequence  #####
' #################################
'
' Return the next color from a sequence set up by CreateColorSequence()
' Cycles round continuously
' Defaults to Black if sequence has not been initialized
'
FUNCTION NextColorInSequence ()
SHARED cseqarray[], maxindex, csindex
	IFZ cseqarray[] THEN RETURN $$Black
	IF csindex > -1 THEN
		color = cseqarray[csindex]
		IF csindex = maxindex THEN
			csindex = 0
		ELSE
			INC csindex
		ENDIF
	ELSE
		color = $$Black
	ENDIF
	RETURN color
END FUNCTION
'
' #####################################
' #####  RandomColorFromSequence  #####
' #####################################
'
' Returns a color selected at random from a sequence set up by CreateColorSequence()
' Defaults to Black if sequence has not been initialized
'
FUNCTION RandomColorFromSequence ()
SHARED cseqarray[], maxindex
	IFZ cseqarray[] THEN RETURN $$Black
	IF maxindex < 0 THEN RETURN $$Black
	RETURN cseqarray[(XstRandom() AND 0x7FFFFFFF) MOD (maxindex + 1)]
END FUNCTION
'
' ###########################
' #####  ColorSequence  #####
' ###########################
'
' returns a color indexed from a sequence set up by CreateColorSequence()
' Defaults to Black if sequence has not been initialized
'
FUNCTION ColorSequence (index)
SHARED cseqarray[], maxindex
	IFZ cseqarray[] THEN RETURN $$Black
	IF maxindex < 0 THEN RETURN $$Black
	RETURN cseqarray[(index AND 0x7FFFFFFF) MOD (maxindex + 1)]
END FUNCTION
'
' #########################
' #####  DrawLstring  #####
' #########################
'
' Executes Turtle commands from an Lstring
' Each move will be by the amount specified in distance#
' Each turn will be by the angle specified in angle#
' See GenerateLstring() and GenerateLstring2() for creation of Lstring 

FUNCTION DrawLstring (lstring$, angle#, distance#)

	IFZ lstring$  THEN RETURN $$TRUE
	IFZ angle#    THEN RETURN $$TRUE
	IFZ distance# THEN RETURN $$TRUE
	
	FOR index = 0 TO LEN(lstring$) - 1
		SELECT CASE lstring${index}
			CASE 'F': Forward(distance#)
			CASE 'B': Backward(distance#)
			CASE '+': Turn(+angle#)
			CASE '-': Turn(-angle#)
			CASE 'f': Go(distance#)
			CASE 'b': Go(-distance#)
			CASE '[': Push()
			CASE ']': Pop(0)
			CASE ELSE: 'Ignore other characters
		END SELECT
	NEXT index
	
END FUNCTION
'
' #############################
' #####  GenerateLstring  #####
' #############################
' An Lstring is a string containing characters that are defined for the L-system of fractal representation
' rule$ has format "X>ABC..."
' 
' Each occurrence of X in in$ is replaced by ABC... in lstring$
' Other characters preserved 
' This can be long running so periodically calls CheckMessages()
' For multiple rules use GenerateLstring2()
'
' See also DrawLstring()
'
'
FUNCTION GenerateLstring (in$, rule$, @lstring$)
	IFZ in$   THEN RETURN $$TRUE
	IFZ rule$ THEN RETURN $$TRUE
	
	
	FOR i = 0 TO LEN(in$) - 1
		IF i MOD 1000 = 0 THEN CheckMessages()
		IF in${i} = rule${0} THEN			
			lstring$ = lstring$ + RIGHT$(rule$,LEN(rule$) - 2)
		ELSE
			lstring$ = lstring$ + CHR$(in${i})	
		ENDIF		
	NEXT i

END FUNCTION

' ##############################
' #####  GenerateLstring2  #####
' ##############################
'
' Like GenerateLstring but takes an array of rules as input rather than a single rule
' Each string in the array holds a separate rule
' This can be long running so periodically calls CheckMessages()
'
FUNCTION GenerateLstring2 (in$, rule$[], @lstring$)
	IFZ in$   THEN RETURN $$TRUE
	IFZ rule$[] THEN RETURN $$TRUE
	
	FOR i = 0 TO LEN(in$) - 1
		IF i MOD 1000 = 0 THEN CheckMessages()
		match = $$FALSE
		FOR j = 0 TO UBOUND(rule$[])
			IF in${i} = ASC(rule$[j]) THEN
				lstring$ = lstring$ + RIGHT$(rule$[j], LEN(rule$[j]) - 2)
				match = $$TRUE
				EXIT FOR 			'Each input character must only match one rule
			ENDIF
		NEXT j
		IF match = $$FALSE THEN lstring$ = lstring$ + CHR$(in${i})
	NEXT i

END FUNCTION
'
' ##################
' #####  Push  #####
' ##################
'
' Store turtle position and bearing on a stack
' Last pushed value can be reinstated with Pop()
' Stack expands automatically as required
'
FUNCTION Push ()
SHARED DOUBLE tstack[], tx, ty, ta
SHARED stacksize, stackptr

	IF stackptr > stacksize - 3 THEN
		stacksize = stacksize + 300
		REDIM tstack[stacksize]
	ENDIF
	
	tstack[stackptr] = tx : INC stackptr
	tstack[stackptr] = ty : INC stackptr
	tstack[stackptr] = ta : INC stackptr

END FUNCTION
'
' #################
' #####  Pop  #####
' #################
'
' Retrieve turtle position and bearing from stack 
' If trail = 0 GoTo() uses position without drawing a trail
' If trail <> 0 uses MoveTo() and draws a trail
'
FUNCTION Pop (trail)
SHARED DOUBLE tstack[], tx, ty, ta
SHARED stacksize, stackptr

	IF stackptr < 3 THEN RETURN $$TRUE
	DEC stackptr : ta = tstack[stackptr] 
	DEC stackptr : ty = tstack[stackptr] 
	DEC stackptr : tx = tstack[stackptr]
	
	IFZ trail THEN
		GoTo(tx,ty)
	ELSE
		MoveTo(tx,ty)
	ENDIF 

END FUNCTION




'
' ####################
' #####  Curve   #####
' ####################
' Draws a section of the arc of a circle
' radius  =  radius of circle in LOGICAL units
' angle   =  angle (degrees) subtended at centre of circle by start and end of arc
'            +ve values draw the arc clockwise, -ve values anticlockwise
' the curve is drawn as a series of straight lines forming a polygon approximating the arc
' pixelsperline controls the number of PHYSICAL pixels in each edge of the polygon
' 1 means each physical pixel is in effect computed separately
' n means the circle is approximated to the requisite number of lines of length n physical pixels
' 0 means set n to the default of 5 physical pixel edges - good compromise between speed and precision
' The function compensates for scaling up / down to ensure a good physical curve
' Note, if scaling factors for x and y axes are not the same, the curve becomes an arc of an ellipse!

FUNCTION Curve (DOUBLE radius, DOUBLE angle, pixelsperline)
SHARED pendown, delay
SHARED DOUBLE xsf, ysf

	IFZ radius THEN RETURN $$TRUE
	IFZ angle  THEN RETURN $$TRUE
	radius = ABS(radius)
	IF pixelsperline < 0 THEN	pixelsperline = 1 'best possible resolution 
	IF pixelsperline = 0 THEN pixelsperline = 5 'default setting for good curves at speed
	
	'calculate logical pixels for full circle
		lpc# = radius * $$TWOPI
	
	'calculate physical pixels for circle allowing for scaling
		ppc# = lpc# * Sqrt(((xsf * xsf) + (ysf * ysf)) / 2)
	
	'calculate number of physical pixels in required arc
		ppa# = ABS(ppc# * angle / 360)

	'calculate number of steps for required pixelsperline
		steps = XLONG(ppa# / pixelsperline)
		IF steps < 2 THEN steps = 2
 
	'calculate length of edge of polygon of required radius that has steps sides in the given angle
		stepspercircle# = ABS(DOUBLE(steps * 360 / angle))
		edgelength# = 2 * radius * SinD(180 / stepspercircle#)
	
	'calculate angle to turn per step (can be negative for counterclockwise)
		anglestep# = angle / steps
	
	'turn off delay whilst drawing the curve
		IF delay THEN
			old_delay = delay
			delay = 0
		END IF
	
	'draw the curve
	
	'First and last moves are at anglestep# / 2 to entry and exit bearing
	'Produces a smoother transition at entry and exit and symmetrical geometry
		Turn(anglestep#/2)
		Move(edgelength#)
			FOR i = 1 TO steps - 1 
				Turn(anglestep#)
				Move(edgelength#)
			NEXT i
		Turn(anglestep#/2)
	
	
	
	'restore delay and then delay if required
		IF old_delay THEN
			delay = old_delay
			IF pendown THEN XstSleep(delay)
		END IF 

END FUNCTION

'
' #####################
' #####  Restore  #####
' #####################
'
' GoTo() the last position on stack - do NOT Pop() stack
' Useful for repeatedly returning to a saved position
' If trail = 0  (FALSE) uses GoTo() position without trail
' If trail <> 0 (TRUE ) uses MoveTo() position and draws a trail 
'
'
FUNCTION Restore (trail)
SHARED DOUBLE tstack[], tx, ty, ta
SHARED stackptr

	ta = tstack[stackptr - 1] 
	ty = tstack[stackptr - 2] 
	tx = tstack[stackptr - 3]
	
	IFZ trail THEN
		GoTo(tx,ty)
	ELSE
		MoveTo(tx,ty)
	ENDIF 

END FUNCTION
'
' ########################
' #####  ClearStack  #####
' ########################
'
' Reset stack pointer and shrink stack to normal size if necessary
'
FUNCTION ClearStack ()
SHARED DOUBLE tstack[]
SHARED stacksize, stackptr

	stackptr = 0
	IF stacksize > 600 THEN
		stacksize = 600
		REDIM tstack[stacksize]
	ENDIF
	
END FUNCTION
'
' ##############################
' #####  SetCheckMessages  #####
' ##############################
' Turn on / off the checking and actioning of messages after each Move() operation
' onoff = 0 ($$FALSE) turns checking OFF
' any other value ($$TRUE) turns it ON
'
FUNCTION SetCheckMessages (onoff)
SHARED checkmsg

	IF onoff THEN
		checkmsg = $$TRUE
	ELSE
		checkmsg = $$FALSE
	ENDIF

END FUNCTION

'
' ###########################
' #####  CheckMessages  #####
' ###########################
' Checks and actions all messages from the message queue for the turtle window
' Allows a long running or looping program to be closed
' If SetCheckMessages ( see above ) is ON, this function is invoked after each Move()
' Can also be called directly at other points in your program 
'
FUNCTION CheckMessages ()
SHARED thwnd
MSG msg
DO WHILE (PeekMessageA(&msg,thwnd,0,0,$$PM_REMOVE))
	TranslateMessage(&msg)
	DispatchMessageA(&msg)
LOOP
END FUNCTION

'
' #############################
' #####  TerminateTurtle  #####
' #############################
' Call this to clean up when you have finished drawing
' Deletes turtle pen and reinstates the pen saved at initialisation

FUNCTION TerminateTurtle ()
SHARED oldpen, thdc, owndc
	DeleteObject(SelectObject(thdc, oldpen))	
END FUNCTION


END PROGRAM
