'
' ###########################################
' #####  Xmal LONGDOUBLE MATHS LIBRARY  #####
' ###########################################
'
' COPYRIGHT ALAN GENT December 2010
' VERSION 1.00 Released December 2010

 
PROGRAM "xmal"
VERSION "1.00"
CONSOLE
MAKEFILE "xdll.xxx"
'
	IMPORT "xst"							' Standard library : required by most programs
	IMPORT "xsx"							' Xblite
	IMPORT "kernel32"					' Exceptions
	IMPORT "xio"							' Console
	
EXPORT


'Initialisation / Version
DECLARE FUNCTION 						Xmal ()
DECLARE FUNCTION 						XmalVersion$ ()

'Library Control - angles
DECLARE FUNCTION 						XmalSetAngleUnits(type)											' Set degrees or radians for angles
DECLARE FUNCTION 						XmalGetAngleUnits()													' Get current setting
 
'Library Control - exceptions and FPU detail
DECLARE FUNCTION 						XmalSetSafetyChecks(bTrueOrFalse)						' Set level of parameter checking and LOCAL exception catching
DECLARE FUNCTION            XmalSetMode(mode)														' Set IEEE754, STANDARD or FREESTYLE mode
DECLARE FUNCTION						XmalSetGlobalExceptions(bTrueOrFalse)				' Set GLOBAL exception catching On / Off
DECLARE FUNCTION USHORT 		XmalSetFPUExceptionMasks(mask$)             ' Easily set / unset the 6 exception masks
DECLARE FUNCTION 						XmalGetFPUExceptions()       								' Get 6 FPU exception bits from status word
DECLARE FUNCTION 						XmalClearFPUExceptions(bAction)             ' Clear FPU exceptions with or without actioning them

'Library Control - Hex / Bin formatting
DECLARE FUNCTION 						XmalSetFormatting(bTrueOrFalse)							' Control formatting of HexS$(), HexD$(), HexL$(), BinS$(), BinD$() and BinL$()

'Library Control - Trace facility
DECLARE FUNCTION 						XmalSetTraceLevel(traceLevel)								' Control level of tracing, trace size, display
DECLARE FUNCTION 						XmalSetTraceSize(numFunc)										' Control level of tracing, trace size, display
DECLARE FUNCTION 						XmalSetTraceDisplay(funcPerLine)						' Control level of tracing, trace size, display
DECLARE FUNCTION 						XmalClearTrace()														' Empty trace
DECLARE FUNCTION 						XmalShowTrace()															' Display trace in full
DECLARE FUNCTION 						XmalShowTraceRolledUp()											' Display trace with consecutive calls to same function rolled up

'Library Control - approximations
DECLARE FUNCTION 						XmalSetApproxZERO (limit##)									' Set limit for IsApproxZERO()
DECLARE FUNCTION 						XmalSetApproxINF (limit##)                  ' Set limit for IsApproxINF()
DECLARE FUNCTION 						XmalSetApproxEQ (limitFraction##)           ' Set limit for IsApproxEQ()
DECLARE FUNCTION 						XmalSetDenormals(bTrueOrFalse)							' Determine whether NextHigher() and NextLower() include denormals

'Debuggging control
DECLARE FUNCTION 						XmalSetProcessorDisplayMode(mode)           ' Determine how %_ShowFPU and %_ShowCPU use the console
	
'Factorials
DECLARE FUNCTION LONGDOUBLE Factorial(n)																' Factorial 

'Permutations /Combinations / Binomial coefficients
DECLARE FUNCTION LONGDOUBLE nPr(n,r)																		' Permutations
DECLARE FUNCTION LONGDOUBLE nCr(n,r)																		' Combinations, Binomial

'Chain Product
DECLARE FUNCTION LONGDOUBLE Product(n1, n2)													' Product n1 * (n1 + 1) * (n1 + 2) ..... * n2

'Trig functions
DECLARE FUNCTION LONGDOUBLE Sin (LONGDOUBLE a)											' Sine
DECLARE FUNCTION LONGDOUBLE Cos (LONGDOUBLE a)											' Cosine
DECLARE FUNCTION LONGDOUBLE Tan (LONGDOUBLE a)											' Tangent
DECLARE FUNCTION LONGDOUBLE Sec (LONGDOUBLE a)											' Secant
DECLARE FUNCTION LONGDOUBLE Csc (LONGDOUBLE a)											' Cosecant
DECLARE FUNCTION LONGDOUBLE Cot (LONGDOUBLE a)											' Cotangent
DECLARE FUNCTION 						SinCos (LONGDOUBLE a, LONGDOUBLE @sine, LONGDOUBLE @cosine)  ' Sine and Cosine in 1 function

'Inverse Trig Functions
DECLARE FUNCTION LONGDOUBLE Asin (LONGDOUBLE x)											' Arcsine
DECLARE FUNCTION LONGDOUBLE Acos (LONGDOUBLE x)											' Arccosine
DECLARE FUNCTION LONGDOUBLE Atan (LONGDOUBLE x)											' Arctangent											
DECLARE FUNCTION LONGDOUBLE Atan2 (LONGDOUBLE y, LONGDOUBLE x)			' Arctangent - 2 argument
DECLARE FUNCTION LONGDOUBLE Asec (LONGDOUBLE x)											' Arcsecant 
DECLARE FUNCTION LONGDOUBLE Acsc (LONGDOUBLE x)											' Arccosecant
DECLARE FUNCTION LONGDOUBLE Acot (LONGDOUBLE x)											' Arccotangent

'Roots
DECLARE FUNCTION LONGDOUBLE Sqrt (LONGDOUBLE x)											' Square root of x
DECLARE FUNCTION LONGDOUBLE Cbrt (LONGDOUBLE x)											' Cube root of x
DECLARE FUNCTION LONGDOUBLE NthRoot (LONGDOUBLE x, n)								' Nth root of x

'Exponentiation / Power
DECLARE FUNCTION LONGDOUBLE Exp2 (LONGDOUBLE x)											' 2 ** x, pairs with Log2()
DECLARE FUNCTION LONGDOUBLE Exp (LONGDOUBLE x)											' e ** x, pairs with Log()
DECLARE FUNCTION LONGDOUBLE Exp10 (LONGDOUBLE x)										' 10 ** x, pairs with Log10()
DECLARE FUNCTION LONGDOUBLE Power (LONGDOUBLE x, LONGDOUBLE y)			' x ** y, pairs with Logb()
DECLARE FUNCTION LONGDOUBLE Power2 (p2)															' 2 ** p, as Exp2 but faster because p must be Integer
DECLARE FUNCTION LONGDOUBLE Power10 (p10)														' 10 ** p, as Exp10Lbut faster / more accurate because p must be Integer
DECLARE FUNCTION LONGDOUBLE PowerX (LONGDOUBLE x, p)								' x ** p, p is an integer and optimised multiplication is used not logs

'Logarithms
DECLARE FUNCTION LONGDOUBLE Log2 (LONGDOUBLE x)											' log x base 2, pairs with Exp2()
DECLARE FUNCTION LONGDOUBLE Log (LONGDOUBLE x)											' log x base e, pairs with Exp()
DECLARE FUNCTION LONGDOUBLE Ln  (LONGDOUBLE x)											' log x base e, pairs with Exp() - SYNONYM for Log()
DECLARE FUNCTION LONGDOUBLE Log1p (LONGDOUBLE x)										' log base e (1 + x) for very small x
DECLARE FUNCTION LONGDOUBLE Log10 (LONGDOUBLE x)										' log x base 10, pairs with Exp10()
DECLARE FUNCTION LONGDOUBLE Logb (LONGDOUBLE x, LONGDOUBLE b)				' log x to base b ( b must be > 1), pairs with Power()

'Hyperbolic functions
DECLARE FUNCTION LONGDOUBLE Sinh (LONGDOUBLE x)											' Hyperbolic Sine
DECLARE FUNCTION LONGDOUBLE Cosh (LONGDOUBLE x)											' Hyperbolic Cosine
DECLARE FUNCTION LONGDOUBLE Tanh (LONGDOUBLE x)											' Hyperbolic Tangent
DECLARE FUNCTION LONGDOUBLE Csch (LONGDOUBLE x)											' Hyperbolic Cosecant
DECLARE FUNCTION LONGDOUBLE Sech (LONGDOUBLE x)											' Hyperbolic Secant
DECLARE FUNCTION LONGDOUBLE Coth (LONGDOUBLE x)											' Hyperbolic Cotangent

'Inverse Hyperbolic functions
DECLARE FUNCTION LONGDOUBLE Asinh (LONGDOUBLE x)										' Inverse Hyperbolic Sine
DECLARE FUNCTION LONGDOUBLE Acosh (LONGDOUBLE x)										' Inverse Hyperbolic Cosine
DECLARE FUNCTION LONGDOUBLE Atanh (LONGDOUBLE x)										' Inverse Hyperbolic Tangent
DECLARE FUNCTION LONGDOUBLE Acsch (LONGDOUBLE x)										' Inverse Hyperbolic Cosecant
DECLARE FUNCTION LONGDOUBLE Asech (LONGDOUBLE x)										' Inverse Hyperbolic Secant
DECLARE FUNCTION LONGDOUBLE Acoth (LONGDOUBLE x)										' Inverse Hyperbolic Cotangent

'Miscellaneous
DECLARE FUNCTION LONGDOUBLE Hypot (LONGDOUBLE x, LONGDOUBLE y)											' Hypot (Pythag) 2 argument
DECLARE FUNCTION LONGDOUBLE Hypot3 (LONGDOUBLE x, LONGDOUBLE y, LONGDOUBLE z)				' Hypot (Pythag) 3 argument

'AP and GP
DECLARE FUNCTION LONGDOUBLE SumAP	(LONGDOUBLE a, LONGDOUBLE d, n1, n2)				' Sum of Arithmetic Progression
DECLARE FUNCTION LONGDOUBLE ProdAP (LONGDOUBLE a, LONGDOUBLE d, n1, n2)				' Product of Arithmetic Progression by multiplication
DECLARE FUNCTION LONGDOUBLE ProdAP2(LONGDOUBLE f, LONGDOUBLE l, n)						' Product of Arithmetic Progression using Gamma function
DECLARE FUNCTION LONGDOUBLE SumGP (LONGDOUBLE a, LONGDOUBLE r, n1, n2)				' Sum of Geometric Progression
DECLARE FUNCTION LONGDOUBLE ProdGP (LONGDOUBLE a, LONGDOUBLE r, n1, n2)				' Product of Geometric Progression


'Round and Mod as xma but with longdouble input
DECLARE FUNCTION LONGDOUBLE Fround	(LONGDOUBLE x, p10)                       ' Round a longdouble to specified power of 10
DECLARE FUNCTION LONGDOUBLE Fmod 		(LONGDOUBLE x, LONGDOUBLE y)							' Remainder after dividing x by y

'C compatibility
'lower case C names have been capitalised for Xblite
DECLARE FUNCTION LONGDOUBLE Ldexp (LONGDOUBLE x, exponent)                    ' Generate longdouble from mantissa and exponent
DECLARE FUNCTION LONGDOUBLE Frexp (LONGDOUBLE x, @exponent)                   ' Split longdouble into mantissa and exponent
DECLARE FUNCTION LONGDOUBLE Modf 	(LONGDOUBLE x, LONGDOUBLE @integer)         ' Split longdouble into fraction and integer parts
DECLARE FUNCTION LONGDOUBLE CopySign (LONGDOUBLE x, LONGDOUBLE y)             ' Return x with the sign of y

'General helper / utility functions
DECLARE FUNCTION 						IsValid		(LONGDOUBLE x)                          ' Test if longdouble is a processable value
DECLARE FUNCTION 						IsNormal		(LONGDOUBLE x)                        ' Test if longdouble is a normal value including 0
DECLARE FUNCTION 						IsPositive	(LONGDOUBLE x)                        ' Test if longdouble is positive          
DECLARE FUNCTION 						IsNegative	(LONGDOUBLE x)                        ' Test if longdouble is negative
DECLARE FUNCTION 						IsNAN		(LONGDOUBLE x, sign)                      ' Test if longdouble is a NAN 
DECLARE FUNCTION 						IsSNAN	(LONGDOUBLE x, sign)                      ' Test if longdouble is a signalling NAN
DECLARE FUNCTION 						IsQNAN	(LONGDOUBLE x, sign)                      ' Test if longdouble is a quiet NAN
DECLARE FUNCTION 						IsINF		(LONGDOUBLE x, sign)                      ' Test if longdouble is infinity
DECLARE FUNCTION 						IsZERO	(LONGDOUBLE x, sign)                      ' Test if longdouble is zero
DECLARE FUNCTION 						IsDEN		(LONGDOUBLE x, sign)                      ' Test if longdouble is a denormal
DECLARE FUNCTION 						IsINDEFINITE(LONGDOUBLE x)                        ' Test if longdouble is INDEFINITE
DECLARE FUNCTION  					GetClass (LONGDOUBLE x)                           ' Test if longdouble is NORMAL, DENORMAL, ZERO, NAN 
DECLARE FUNCTION 						GetOriginOfNAN(LONGDOUBLE nan, @funcNum, @func$)  ' Determine source of a NAN

'Extract portions of longdouble
DECLARE FUNCTION						GetSignBit	(LONGDOUBLE x)                        ' Extract sign bit from longdouble
DECLARE FUNCTION 						GetExponent	(LONGDOUBLE x)                        ' Extract exponent from longdouble including offset
DECLARE FUNCTION 						GetTrueExponent	(LONGDOUBLE x)                    ' Extract exponent from longdouble excluding offset
DECLARE FUNCTION USHORT			Lsexp(LONGDOUBLE x)                        				' Extract sign and exponent word from longdouble					
DECLARE FUNCTION 						Lhigh(LONGDOUBLE x)                        				' Extract high half of mantissa
DECLARE FUNCTION 						Llow(LONGDOUBLE x)                                ' Extract low half of mantissa
DECLARE FUNCTION LONGDOUBLE GetMantissa	(LONGDOUBLE x)												' Return mantissa as longdouble with exponent of 0
DECLARE FUNCTION LONGDOUBLE	Extract (LONGDOUBLE x, @exponent)									' Split longdouble into mantissa and exponent

'Increment / Decrement by 1 or more representables
DECLARE FUNCTION 						NextValue(LONGDOUBLE @x, cSteps)									' Change x to value cSteps representables from x 
DECLARE FUNCTION LONGDOUBLE NextToward (LONGDOUBLE x, LONGDOUBLE y)           ' Return value 1 representable from x in direction of y
DECLARE FUNCTION            NextHigher (LONGDOUBLE @x)												' Change x to next higher representable
DECLARE FUNCTION            NextLower  (LONGDOUBLE @x)												' Change x to next lower representable




'Display FLOATS in Hex / Bin
'Control format with XmalSetFormatting()
'DECLARE FUNCTION 						HexS$ (SINGLE x)
'DECLARE FUNCTION 						HexD$ (DOUBLE x)
DECLARE FUNCTION 						HexL$ (LONGDOUBLE x)                              ' Create HEX string representation of longdouble
'DECLARE FUNCTION 						BinS$ (SINGLE x)
'DECLARE FUNCTION 						BinD$ (DOUBLE x)
DECLARE FUNCTION 						BinL$ (LONGDOUBLE x)                              ' Create BIN string representation of longdouble

'Exactly equal test
DECLARE FUNCTION 						IsIdentical(LONGDOUBLE x, LONGDOUBLE y)           ' Test if 2 longdoubles of any class are the same

'Approximate tests
DECLARE FUNCTION 						IsApproxZERO (LONGDOUBLE x)                       ' Test approximately 0
DECLARE FUNCTION 						IsApproxINF (LONGDOUBLE x)                        ' Test approximately INF
DECLARE FUNCTION 						IsApproxEQ (LONGDOUBLE x, LONGDOUBLE y)           ' Test x approx = y
DECLARE FUNCTION 						IsApproxLT (LONGDOUBLE x, LONGDOUBLE y)           ' Test x approx < y
DECLARE FUNCTION 						IsApproxLTE (LONGDOUBLE x, LONGDOUBLE y)          ' Test x approx <= y
DECLARE FUNCTION 						IsApproxGT (LONGDOUBLE x, LONGDOUBLE y)           ' Test x approx > y
DECLARE FUNCTION 						IsApproxGTE (LONGDOUBLE x, LONGDOUBLE y)          ' Test x approx >= y


'MAKE / Image substitutes
'Substitutes for MAKE() and ability to load longdouble IMAGE, create specials etc
DECLARE FUNCTION LONGDOUBLE Lmake (USHORT sexp, hi, lo)                       ' Make longdouble from 3 parts. Like DMAKE()
DECLARE FUNCTION LONGDOUBLE Limage (longImage$)                               ' Make longdouble from HEX image. Like 0d....
DECLARE FUNCTION LONGDOUBLE Lbuild (LONGDOUBLE mantissa, exponent)            ' Make longdouble from mantissa and exponent
DECLARE FUNCTION LONGDOUBLE Lmax()																						' Make maximum positive normal longdouble value
DECLARE FUNCTION LONGDOUBLE Lmin()																						' Make smallest positive normal longdouble value
DECLARE FUNCTION LONGDOUBLE LmaxDen()																					' Make maximum positive denormal longdouble value
DECLARE FUNCTION LONGDOUBLE LminDen()																					' Make smallest positive denormal longdouble value
DECLARE FUNCTION LONGDOUBLE QNAN(sign)																				' Make Quiet NAN
DECLARE FUNCTION LONGDOUBLE SNAN(sign)																				' Make Signalling NAN
DECLARE FUNCTION LONGDOUBLE INF(sign)																				  ' Make an INFINITY
DECLARE FUNCTION LONGDOUBLE UNS(sign)																				  ' Make typical UNSUPPORTED

'SAFE Conversions from longdouble to smaller types
DECLARE FUNCTION SINGLE     SafeLDtoSingle (LONGDOUBLE x, @errFlag)           ' Convert longdouble to SINGLE with range checks
DECLARE FUNCTION DOUBLE 		SafeLDtoDouble (LONGDOUBLE x, @errFlag)           ' Convert longdouble to DOUBLE with range checks
DECLARE FUNCTION 						SafeLDConvert (LONGDOUBLE x, lpVariable, varType) ' Convert longdouble to any numeric / float with range checks

'ADVANCED
DECLARE FUNCTION LONGDOUBLE Gamma (LONGDOUBLE x)														  ' Gamma function
DECLARE FUNCTION LONGDOUBLE LogGamma (LONGDOUBLE x)													  ' Log of Gamma function for large x 
DECLARE FUNCTION LONGDOUBLE LogFactorial (GIANT n)													  ' Log of Factorial for large n


'SPLIT TO Xmalp??
DECLARE FUNCTION LONGDOUBLE Beta(LONGDOUBLE x, LONGDOUBLE y)								  ' Beta function
DECLARE FUNCTION LONGDOUBLE LogBeta(LONGDOUBLE x, LONGDOUBLE y)							  ' Log of Beta function for large x, y
DECLARE FUNCTION LONGDOUBLE Erf(LONGDOUBLE x)																  ' Error function
DECLARE FUNCTION LONGDOUBLE Erfc(LONGDOUBLE x)															  ' Complimentary error function
DECLARE FUNCTION LONGDOUBLE En(n, LONGDOUBLE x)													      ' Exponential integral
DECLARE FUNCTION LONGDOUBLE Ei(LONGDOUBLE x)																  ' Exponential integral - the other one!
DECLARE FUNCTION LONGDOUBLE Fibonacci(n)																		  ' nth number of Fibonacci series



'WIP YET TO BE STARTED / COMPLETED
'DECLARE FUNCTION LONGDOUBLE Si()
'DECLARE FUNCTION LONGDOUBLE Ci()
'DECLARE FUNCTION LONGDOUBLE BesselJ0()
'DECLARE FUNCTION LONGDOUBLE BesselJ1()
'DECLARE FUNCTION LONGDOUBLE BesselJn()
'DECLARE FUNCTION LONGDOUBLE BesselY0()
'DECLARE FUNCTION LONGDOUBLE BesselY1()
'DECLARE FUNCTION LONGDOUBLE BesselYn()

'*********************************************************************************************
'INTERNAL INTERNAL INTERNAL INTERNAL INTERNAL
DECLARE FUNCTION 						XmalExcept()																		' Exception Catcher
DECLARE FUNCTION 						FunctionNumberToName$(funcNum)									' Decode function number to function name()
DECLARE FUNCTION 						DisplayProcessor(lpDataArea, type)							' Show FPU / CPU
DECLARE FUNCTION LONGDOUBLE PqNAN()																					' Create Positive QNAN with function number payload in last byte
DECLARE FUNCTION LONGDOUBLE PsNAN()																					' Create Positive SNAN with function number payload in last byte
DECLARE FUNCTION LONGDOUBLE NqNAN()																					' Create Negative QNAN with function number payload in last byte
DECLARE FUNCTION LONGDOUBLE NsNAN()																					' Create Negative SNAN with function number payload in last byte
DECLARE FUNCTION 						FpuOpcodeToName$( UBYTE op1, UBYTE op2)					' Disassemble 2 bytes of FPU opcode to INSTRUCTION MNEMONIC


'*********************************************************************************************
'*********************************************************************************************
' ############################
' #####  xmal constants  #####
' ############################
	
	'Mathematical constants to 20+ sig digits

	$$PIDIV8			= 0.3926990816987241548078##  		' pi/8
	$$PIDIV4      = 0.7853981633974483096157##			' pi/4	(45 deg)  
	$$3PIDIV8			= 1.1780972450961724644235##			' 3pi/8 
	$$PIDIV2     	= 1.5707963267948966192313##			' pi/2  (90 deg)
	$$5PIDIV8			= 1.9634954084936207740392##			' 5pi/8 
	$$3PIDIV4			=	2.3561944901923449288470##			' 3pi/4	(135 deg) 
	$$7PIDIV8			= 2.7488935718910690836548##			' 7pi/8 
	$$PI					= 3.1415926535897932384626##  		' pi		(180 deg)
	$$PIDIV6			=	0.5235987755982988730771##			' pi/6	(30 deg)
	$$PIDIV3			= 1.0471975511965977461542##			' pi/3	(60 deg)
	$$2PIDIV3			= 2.0943951023931954923084##			' 2pi/3	(120 deg)
	$$5PIDIV6			= 2.6179938779914943653855##			' 5pi/6	(150 deg)
	$$TWOPI       = 6.2831853071795864769253##			' 2pi		(360 deg)
	$$LNSQRT2PI  	= 0.9189385332046727417803##			' ln(sqrt(2 * pi))
	$$2DIVSQRTPI  = 1.1283791670955125738962##			' 2 / Sqrt(pi)
	$$1DIVSQRTPI  = 0.5641895835477562869481##			' 1 / Sqrt(pi)
	
	$$E           = 2.7182818284590452353603##  		' e
	$$LOG2E       = 1.4426950408889634073599##			' log2 of e
	$$LOG2TEN			= 3.3219280948873623478703##			' log2 of 10
	$$LOG10E      = 0.4342944819032518276511##			' log10 of e
	$$LOG10TWO		= 0.3010299956639811952137##  		' log10 of 2
	$$LN2         = 0.6931471805599453094172##			' loge (ln) of 2
	$$LN10        = 2.3025850929940456840180##			' loge (ln) of 10
	
	$$RADTODEG    = 57.295779513082320876798##  		' degrees in 1 radian
	$$RADTOMIN		= 3437.7467707849392526079##			' minutes in 1 radian
	$$RADTOSEC		= 206264.80624709635515647##			' seconds in 1 radian
  $$DEGTORAD    = 0.0174532925199432957692##  		' radians in 1 degree
	$$MINTORAD		= 2.9088820866572159615395d-4##		' radians in 1 minute
	$$SECTORAD		= 4.8481368110953599358991d-6##		'	radians in 1 second

	$$EULER				= 0.5772156649015328606065## 			' Euler's constant

	
	'xmal library control	
	$$DEGREES   	= 0							
	$$RADIANS  		= -1
	$$ON					= -1 'TRUE
	$$OFF					= 0  'FALSE
	
	$$IEEE754     = 0
	$$TEST        = 1
	$$STANDARD    = 2
	$$FREESTYLE   = 3

	
	
	$$MAX_EXPONENT 	= 16383													'maximum true binary exponent for a NORMAL
	$$MIN_EXPONENT	= -16382												'minimum true binary exponent for a NORMAL
	$$LOG1PLIMIT		= 0.2928932188134524755992## 		'1 - Sqrt(2)/2. Range check for flog1p
  $$EPSILON				= 5.5d-19##											'smallest positive increment such that (1 + $$EPSILON) > 1 is TRUE
	$$XMININ				= 1d-4932##											'smallest positive number for which 1 / $$MININ is representable without overflow
	$$LOGGAMMAMAX  	= 1d4928##											'highest number for which LogGamma() can return a valid result
	$$FRTLNGMAX  		= 1d1232##											'approx 4th root of $$LOGGAMMAMAX       
	$$PNT68      	 	= 0.6796875##										'boundary value in LogGamma()
	$$MAXFACTORIAL  = 1754													'maximum number for which factorial can be held in LONGDOUBLE
	$$TWOPOW63			= 9.223372036854775806d+18##		'2 to the power 63 - limit of some Trig functions
	
	'xma compatible NANs
	$$NNAN			=	0dFFFFFFFFFFFFFFFF
	$$PNAN			= 0d7FFFFFFFFFFFFFFF
	
	'Exception flags 
	$$Invalid		= 1
	$$Denormal  = 2
	$$Zero			= 4
	$$Overflow	= 8
	$$Underflow	= 16
	$$Precision	= 32
	$$Any = 63
	

	
	
END EXPORT	
	

'****************************************************************************************************************
'****************************************************************************************************************
' XMAL PREFIXED FUNCTIONS THAT CONTROL THE LIBRARY ITSELF
'****************************************************************************************************************
'****************************************************************************************************************


' #####################
' #####  Xmal ()  #####
' #####################
'
FUNCTION Xmal ()
'EXCEPTION_DATA exception 
'FLOATING_SAVE_AREA fsa
UBYTE #XmalTrace[]

	'initialisation executed when xmal.dll is first loaded
	XstSetFPUPrecision($$64_BITS)									'make sure FPU is in extended precision mode. This is normal for Xblite but - just in case it's been changed
	
	XstGetExceptionFunction(@except)														'if there is no exception handler......
	IFZ except THEN	XstSetExceptionFunction(&XmalExcept())			'install the Xmal GLOBAL exception catcher
	
	#APPROX_ZERO_LIMIT## 			= 1d-16##	
	#APPROX_INFINITY_LIMIT## 	= 1d+18##
	#APPROX_EQUAL_FRACTION##  = 1d-16##
	
	#ANGLE_UNITS 							= $$RADIANS				'default is to work in RADIANS. Change with XmalSetAngleUnits()
	
	#SAFETY_CHECKS 						= 0								'default is do safety checks. Set by function call below
	#MODE											= 0         			'immediately set to default STANDARD mode below
	XmalSetMode($$STANDARD)
	
	#FORMAT_HEX         			= $$TRUE					'default is to FORMAT Hex output. Change with XmalSetFormatting()
	#DENORMAL                 = $$TRUE          'Switch to control if NextHigher() and NextLower() include denormals
	
	#FUNC_NUM									= -1							'always keep track of last executed function for exception reporting
	
	DIM #XmalTrace[19]													'create trace buffer for default 20 entries
	#TRACE_ADDRESS						= &#XmalTrace[]
	#TRACE_WRAP								= 20
	#TRACE_FUNC_PER_LINE			= 10
	
	#TRACE										= $$TRUE					'maintain trace of major function calls
	#TRACE2										= $$FALSE					'trace most functions but not helper and Xmal functions
	#TRACE3										= $$FALSE					'trace nearly everything
	
	#PAUSE_ON_DISPLAY         = $$TRUE
	#CLEAR_ON_DISPLAY         = $$TRUE
	
	
	
	
	'Initialise global variables for special values in lieu of $$constant
	'These are for internal use only
	'Note: NANs are generated with payload by special internal functions
	
	#PINF## = 0##
ASM fld  t[pinf]
ASM fstp t[%#PINF.xmal]

	#NINF## = 0##
ASM fld  t[ninf]
ASM fstp t[%#NINF.xmal]

	#LMAX## = 0##
ASM fld  t[max_longdouble]
ASM fstp t[%#LMAX.xmal]

	#LMIN## = 0##
ASM fld  t[min_longdouble]
ASM fstp t[%#LMIN.xmal]

	#LMAXDEN## = 0##
ASM fld  t[max_denormal]
ASM fstp t[%#LMAXDEN.xmal]

	#LMINDEN## = 0##
ASM fld  t[min_denormal]
ASM fstp t[%#LMINDEN.xmal]

	#TWOPOW63## = 0##
ASM fld  t[twopow63]
ASM fstp t[%#TWOPOW63##.xmal]	
	
END FUNCTION



' #############################
' #####  XmalVersion$ ()  #####
' #############################
'
FUNCTION XmalVersion$()
	RETURN "Xmal Version = 1.00"
END FUNCTION


' #################################
' #####  XmalSetAngleUnits()  #####
' #################################
'
'
FUNCTION XmalSetAngleUnits(type)
'set whether to use degrees or radians with trig and inverse trig functions

	#FUNC_NUM = 107
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF
	
	SELECT CASE type
		CASE $$DEGREES, $$RADIANS : #ANGLE_UNITS = type
		CASE ELSE 								: RETURN $$TRUE
	END SELECT

END FUNCTION



' #################################
' #####  XmalGetAngleUnits()  #####
' #################################
'
'
FUNCTION XmalGetAngleUnits()
'return current units of angle - radians or degrees

	#FUNC_NUM = 108
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF	
	
	RETURN #ANGLE_UNITS
	
END FUNCTION

' ###################################
' #####  XmalSetSafetyChecks()  #####
' ###################################
'
'
FUNCTION XmalSetSafetyChecks(bTrueOrFalse)
'turn additional safety checking ON ($$TRUE) or OFF ($$FALSE)
'turn local exception handling ON ($$TRUE) to provide default fix to exception or OFF ($$FALSE)

	#FUNC_NUM = 121
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF
	
	IF #MODE <> $$FREESTYLE THEN RETURN $$TRUE		' Function disabled in IEEE754 / STANDARD / TEST modes
	
	SELECT CASE bTrueOrFalse
		CASE $$TRUE:  #SAFETY_CHECKS = $$TRUE
		CASE $$FALSE: #SAFETY_CHECKS = $$FALSE
	END SELECT
	
END FUNCTION



' ###########################
' #####  XmalSetMode()  #####
' ###########################
'
'
FUNCTION XmalSetMode(mode)
' Set library mode 

	#FUNC_NUM = 197
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF	
	
	SELECT CASE mode
		'mode 3 is set temporarily to allow XmalSetFPUExceptionMasks() to operate
		CASE $$IEEE754:
			#SAFETY_CHECKS = $$FALSE													' Turn off safety checks. Let FPU handle everything
			#MODE = 3: XmalSetFPUExceptionMasks("A+")					' All exceptions masked	
			
		CASE $$TEST:
			#SAFETY_CHECKS = $$FALSE
			#MODE = 3: XmalSetFPUExceptionMasks("A-D+P+")    	' Only Denormal and Precision exceptions masked												   
			
		CASE $$STANDARD, $$FREESTYLE:
			#SAFETY_CHECKS = $$TRUE														' Reset to default configuration
			#MODE = 3: XmalSetFPUExceptionMasks("")     			' Xblite default masks
			
		CASE ELSE
			RETURN $$TRUE
		
	END SELECT
	
	'mode is valid so set it
	#MODE = mode
	
END FUNCTION



' #######################################
' #####  XmalSetGlobalExceptions()  #####
' #######################################
'
'
FUNCTION XmalSetGlobalExceptions(bTrueOrFalse)
'turn GLOBAL exception catching ON ($$TRUE) or OFF ($$FALSE)

	#FUNC_NUM = 122
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF	
	
	SELECT CASE bTrueOrFalse
		CASE $$TRUE:  RETURN XstSetExceptionFunction(&XmalExcept())
		CASE $$FALSE: RETURN XstSetExceptionFunction(0)
	END SELECT
	
END FUNCTION



' #################################
' #####  XmalSetFormatting()  #####
' #################################
'
'
FUNCTION XmalSetFormatting(bTrueOrFalse)
'turn BinL$() and HexL$() formatting ON ($$TRUE) or OFF ($$FALSE)

	#FUNC_NUM = 123
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF
	
	SELECT CASE bTrueOrFalse
		CASE $$TRUE:  #FORMAT_HEX = $$TRUE
		CASE $$FALSE: #FORMAT_HEX = $$FALSE
		CASE ELSE RETURN
	END SELECT
	
END FUNCTION


' ########################################
' #####  XmalSetFPUExceptionMasks()  #####
' ########################################
'
'
FUNCTION USHORT XmalSetFPUExceptionMasks(mask$)
'Sets or Unsets mask bits in FPU Control Word

	#FUNC_NUM = 124
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF
	
	IF #MODE <> $$FREESTYLE THEN RETURN $$TRUE		' Function disabled in IEEE754 / STANDARD / TEST modes
	
ASM 	fstcw w[temp]					;get control word to modify
ASM 	fstcw w[temp + 2]			;and a copy to keep unchanged

	IFZ mask$ THEN		'reset masks to default Xblite settings
ASM 	and w[temp], 1111111111000000b		;unset all masks 
ASM 	or	w[temp], 0000000000100011b		;reset to standard with U, O, Z unmasked
		
	ELSE
		mask$ = UCASE$(mask$)
		len = LEN(mask$)
		i_max = len - 1
		
		aa = INCHR(mask$, "A")	'ALL
		pp = INCHR(mask$, "P")	'PRECISION
		uu = INCHR(mask$, "U")	'UNDERFLOW
		oo = INCHR(mask$, "O")	'OVERFLOW
		zz = INCHR(mask$, "Z")	'DIVIDE BY ZERO		
		dd = INCHR(mask$, "D")	'DENORMAL		
		ii = INCHR(mask$, "I")	'INVALID
		

		SELECT CASE ALL TRUE
			CASE aa
				IF aa <= i_max THEN
					sign = mask${aa}
					SELECT CASE sign
						CASE '+','1'
ASM							or  w[temp], 0000000000111111b
						CASE '-','0'
ASM							and w[temp], 1111111111000000b							
					END SELECT
				ENDIF						
			CASE pp		'PRECISION EXCEPTION MASK
				IF pp <= i_max THEN
					sign = mask${pp}
					SELECT CASE sign
						CASE '+','1'
ASM							bts w[temp], 5
						CASE '-','0'
ASM							btr w[temp], 5							
					END SELECT
				ENDIF
				
			CASE uu		'UNDERFLOW EXCEPTION MASK
				IF uu <= i_max THEN
					sign = mask${uu}
					SELECT CASE sign
						CASE '+','1'
ASM							bts w[temp], 4
						CASE '-','0'
ASM							btr w[temp], 4							
					END SELECT
				ENDIF
				
			CASE oo		'OVERFLOW EXCEPTION MASK
				IF oo <= i_max THEN
					sign = mask${oo}
					SELECT CASE sign
						CASE '+','1'
ASM							bts w[temp], 3
						CASE '-','0'
ASM							btr w[temp], 3							
					END SELECT
				ENDIF
				
			CASE zz		'DIVIDE BY ZERO EXCEPTION MASK
				IF zz <= i_max THEN
					sign = mask${zz}
					SELECT CASE sign
						CASE '+','1'
ASM							bts w[temp], 2
						CASE '-','0'
ASM							btr w[temp], 2							
					END SELECT
				ENDIF
				
			CASE dd		'DENORMAL EXCEPTION MASK
				IF dd <= i_max THEN
					sign = mask${dd}
					SELECT CASE sign
						CASE '+','1'
ASM							bts w[temp], 1
						CASE '-','0'
ASM							btr w[temp], 1							
					END SELECT
				ENDIF
				
			CASE ii		'INVALID OPERATION EXCEPTION MASK
				IF ii <= i_max THEN
					sign = mask${ii}
					SELECT CASE sign
						CASE '+','1'
ASM							bts w[temp], 0
						CASE '-','0'
ASM							btr w[temp], 0							
					END SELECT
				ENDIF
				
		END SELECT		
	ENDIF

	'load modified control word
ASM fldcw w[temp]

	'return the old control word from second copy	
ASM xor eax, eax
ASM mov ax, w[temp + 2]
ASM jmp end.XmalSetFPUExceptionMasks.xmal	
	
END FUNCTION

'
' #################################
' #####  XmalSetTraceLevel () #####
' #################################
'
'
'
FUNCTION XmalSetTraceLevel (traceLevel)
'Set trace level. 0 = OFF / 1 = Level 1 functions only / 2 = Most functions / 3 = All functions

	SELECT CASE traceLevel
		CASE 0
			' trace OFF but single function trace for major functions is still on
			' this enables Exception function to report last function executed
			#TRACE 	= $$FALSE
			#TRACE2	= $$FALSE
			#TRACE3 = $$FALSE
		CASE 1
			' function trace for major functions only
			#TRACE 	= $$TRUE
			#TRACE2	= $$FALSE
			#TRACE3 = $$FALSE
		CASE 2
			' function trace for more functions
			#TRACE 	= $$TRUE
			#TRACE2	= $$TRUE
			#TRACE3 = $$FALSE			
		CASE 3
			' function trace for all functions
			#TRACE 	= $$TRUE
			#TRACE2	= $$TRUE
			#TRACE3 = $$TRUE			
		CASE ELSE
			' leave unchanged
			RETURN $$TRUE
	END SELECT
END FUNCTION


'
' ################################
' #####  XmalSetTraceSize () #####
' ################################
'
'
'
FUNCTION XmalSetTraceSize (numFunc)
'Set NUMBER OF FUNCTIONS to TRACE
UBYTE #XmalTrace[]
	
	IF numFunc > 2 THEN 
		IF numFunc > 1000 THEN numFunc = 1000			' sensible upper limit
		IF numFunc = #TRACE_WRAP THEN RETURN 			' it hasn't changed
		
		DIM #XmalTrace[numFunc - 1]				'	resize and clear it
ASM mov d[trace_index], 0							;	reset the ASM index
		#TRACE_ADDRESS = &#XmalTrace[]		' may have moved if made bigger
		#TRACE_WRAP = numFunc							' update wrap 
	ENDIF
END FUNCTION


'
' ###################################
' #####  XmalSetTraceDisplay () #####
' ###################################
'
'
'
FUNCTION XmalSetTraceDisplay (funcPerLine)	
'Set FUNCTIONS per LINE for DISPLAY

	IF funcPerLine < 1 THEN RETURN
	#TRACE_FUNC_PER_LINE = funcPerLine

END FUNCTION


'
' #############################
' #####  XmalShowTrace () #####
' #############################
'
'
'
FUNCTION XmalShowTrace()
'Show trace without roll up
AUTOX index
UBYTE #XmalTrace[],func

	PRINT "\r\n Xmal Function Call Full Trace:\r\n\r\n ";
	
'Get start value of index	
ASM mov eax, d[trace_index]
ASM mov d[XmalShowTrace.index], eax

	upper = UBOUND(#XmalTrace[])
	wrap  = upper + 1
	out = 0
	
	FOR n = 0 TO upper
		
		func = #XmalTrace[index]
		IF func > 0 THEN
			
			PRINT FunctionNumberToName$(func);
			out = out + 1
			
			IF n < upper THEN
				IF (out MOD #TRACE_FUNC_PER_LINE) THEN
					PRINT ", ";
				ELSE
					PRINT ",\r\n ";
				ENDIF
			ENDIF
			
		ENDIF
		
		'next buffer entry
		INC index
		index = index MOD wrap
		
	NEXT n
	

	IFZ out THEN PRINT " Trace is EMPTY"	
	PRINT "\n"
	
END FUNCTION

'
' #####################################
' #####  XmalShowTraceRolledUp () #####
' #####################################
'
'
'
FUNCTION XmalShowTraceRolledUp()
'Show trace with rollup
AUTOX index
UBYTE #XmalTrace[],func

	PRINT "\r\n Xmal Function Call Rolled Up Trace:\r\n"
	
'Get start value of index	
ASM mov eax, d[trace_index]
ASM mov d[XmalShowTraceRolledUp.index], eax

	upper = UBOUND(#XmalTrace[])
	wrap  = upper + 1
	out = 0
	first = $$TRUE
	rollCount = 1
	
	FOR n = 0 TO upper
		func = #XmalTrace[index]
		
		IF func > 0 THEN
			
			IF first THEN
				lastFunc = func
				first = $$FALSE
				GOTO nextIndex
			ENDIF
			
			IF func = lastFunc THEN
				INC rollCount
				GOTO nextIndex
			ENDIF
			
			
			PRINT FunctionNumberToName$(lastFunc);
			out = out + 1
			lastFunc = func
			
			IF rollCount > 1 THEN
				PRINT " x " + STRING(rollCount);
				rollCount = 1
			ENDIF
			
			IF n <= upper THEN
				IF (out MOD #TRACE_FUNC_PER_LINE) THEN
					PRINT ", ";
				ELSE
					PRINT ",\r\n ";
				ENDIF
			ENDIF
			
		ENDIF
		
nextIndex:
		'next buffer entry
		INC index
		index = index MOD wrap
		
	NEXT n
	

	'Epilogue - do last entry
	IF func <> 0 THEN
		PRINT FunctionNumberToName$(func);
		out = out + 1	
		IF rollCount > 1 THEN	PRINT " x " + STRING(rollCount);
	ENDIF

	IFZ out THEN PRINT "Trace is EMPTY"
	PRINT "\n"
	
END FUNCTION

'
' ##############################
' #####  XmalClearTrace () #####
' ##############################
'
'
'
FUNCTION XmalClearTrace()
'Destroys any existing trace record and starts again
UBYTE #XmalTrace[]

'Empty and reset the trace array
	address = &#XmalTrace[]
ASM mov esi, d[XmalClearTrace.address]		;	address of first byte in array
ASM mov edi, d[esi - 8]								; get number of elements from header
ASM add edi, esi											; point to byte after last
ASM call %_ZeroMemory									; wipe it to 0

'reset the ASM pointer	
ASM mov d[trace_index], 0
END FUNCTION

'
' ################################
' #####  XmalSetDenormals () #####
' ################################
'
'
'
FUNCTION 	XmalSetDenormals(bTrueOrFalse)	
' Determine whether NextHigher() and NextLower() include denormals

	#FUNC_NUM = 123
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF
	
	SELECT CASE bTrueOrFalse
		CASE $$TRUE:  #DENORMAL = $$TRUE
		CASE $$FALSE: #DENORMAL = $$FALSE
		CASE ELSE RETURN
	END SELECT

END FUNCTION



'
' ###########################################
' #####  XmalSetProcessorDisplayMode () #####
' ###########################################
'
'
'
FUNCTION 	XmalSetProcessorDisplayMode(mode)	
' mode 0. Console display does not pause for %_ShowFPU, %_ShowCPU
' mode 1. Console pauses but display accumulates
' mode 2. Console pauses and clears when enter pressed
	
	SELECT CASE mode
		CASE 0
			#PAUSE_ON_DISPLAY = $$FALSE
		CASE 1
			#PAUSE_ON_DISPLAY = $$TRUE
			#CLEAR_ON_DISPLAY = $$FALSE
		CASE 2
			#PAUSE_ON_DISPLAY = $$TRUE
			#CLEAR_ON_DISPLAY = $$TRUE			
	END SELECT

END FUNCTION


'
' #####################################
' #####  XmalClearFPUExceptions() #####
' #####################################
'
'
FUNCTION XmalClearFPUExceptions(bAction)
'Clears all FPU exceptions
	IF bAction THEN
ASM fclex
	ELSE
ASM fnclex
  ENDIF
END FUNCTION



'
' ####################################
' #####  XmalTestFPUExceptions() #####
' ####################################
'
'
FUNCTION XmalGetFPUExceptions()
'Get 6 FPU exception flags from status word

ASM xor eax, eax
ASM fstsw ax
ASM and ax, 111111b
ASM jmp end.XmalGetFPUExceptions.xmal
 
END FUNCTION


'
' ########################################
' #####  XmalTestFPUConditionCodes() #####
' ########################################
'
'

'****************************************************************************************************************
'****************************************************************************************************************
' FACTORIALS, PERMUTATIONS, COMBINATIONS, PRODUCT
'****************************************************************************************************************
'****************************************************************************************************************

' ####################
' #####  nPr ()  #####
' ####################
'
FUNCTION LONGDOUBLE nPr(n,r)
'Returns number of ways of arranging n items in groups of r where order matters - nPr
'If nPr exceeds maximum LONGDOUBLE then returns INF(+1) if in SAFE_MODE, otherwise crashes
 
	#FUNC_NUM = 29
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF	


	IF n < 0 THEN RETURN PqNAN()
	IF r < 0 THEN RETURN PqNAN()
	IF r > n THEN RETURN PqNAN()

	
	'n and r both positive with r < n	
	IF n > $$MAXFACTORIAL THEN
		'too big for factorial - use Product() instead. This will return INF(+1) if numbers get too big
		RETURN Product(n, n - r + 1)
	ELSE
		IF n = r THEN
			RETURN Factorial(n)
		ELSE
			RETURN Factorial(n) / Factorial(n - r)
		ENDIF
	ENDIF 
	
END FUNCTION



' ###################
' #####  nCr()  #####
' ###################
'
'
FUNCTION LONGDOUBLE nCr(n,r)
'Returns number of ways of selecting r items from a group of n where order does not matter - nCr
'n and r are integers so do not need Validity checking

	#FUNC_NUM = 28
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	

	IF n < 0 THEN RETURN PqNAN()
	IF r < 0 THEN RETURN PqNAN()
	IF r > n THEN RETURN PqNAN()

	
	'simple case
	IF n = r THEN RETURN 1
	
	permutations## = nPr(n,r)
	
	IF IsValid(permutations##) THEN
		IF r <= $$MAXFACTORIAL THEN
			RETURN permutations## / Factorial(r)
		ELSE
			RETURN Exp(Log(permutations##) - LogGamma(r + 1))
		ENDIF				
	ELSE
		'too big for nPr. Use LogGamma() - still only works if result is in range.
		RETURN Exp(LogGamma(n + 1) + LogGamma(n - r + 1) - LogGamma(r + 1))		
	ENDIF
	
END FUNCTION


' #########################
' #####  Factorial()  #####
' #########################
'
'
FUNCTION LONGDOUBLE Factorial(n)
'Returns n! as a longdouble provided n is between 0 and $$MAXFACTORIAL inclusive; otherwise returns +/- INF
'n is an integer so does not need Validity checking

'Result is accurate to 19 digits - limited by the precision of the longdouble mantissa
'For values up to 52, the factorial is looked up directly from a table

'For values above 52, looks up factorial of nearest multiple of 10 from a table
'If n is a multiple of 10, this factorial is returned immediately
'If n > nearest multiple of 10, additional multipliers are applied
'If n < nearest multiple of 10, additional divisions are applied
'e.g. factorial 320 is returned directly from the table
'e.g. factorial 324 is calculated by looking up factorial 320 from the table then mutiplying by (321 * 322 * 323 * 324)
'     factorial 327 is calculated by looking up factorial 330 from the table then dividing by (328 * 329 * 330) 
AUTOX base

	#FUNC_NUM = 25
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF	
	
	SELECT CASE TRUE
		CASE n < 0 	: RETURN PqNAN()
		CASE n < 2 	: RETURN 1##
		CASE n < 53
			'retrieve the factorial directly from fact_low_table
ASM mov		d[Factorial.base], ADDR fact_low_table 				; point to table
			RETURN ROUND(LONGDOUBLEAT(base + n * 10))			
			
		CASE n <= $$MAXFACTORIAL
			'use fact_high_table and work up or down from nearest multiple of 10
			nDiv10 = n \ 10
			rem    = n MOD 10
			'If rem 6 to 9 work downwards from higher multiple of 10
			IF rem >= 6 THEN INC nDiv10
			nearest10  = nDiv10 * 10
	
			'retrieve the factorial of nearest multiple of 10 (table starts at 50)
ASM mov		d[Factorial.base], ADDR fact_high_table 				; point to table
			fact## = LONGDOUBLEAT(base + nearest10 - 50)
	
			SELECT CASE TRUE
				CASE rem = 0 : RETURN ROUND(fact##)
				CASE rem < 6 : RETURN ROUND(fact## * Product(nearest10 + 1, n)) 
				CASE ELSE    : RETURN ROUND(fact## / Product(n + 1, nearest10)) 
			END SELECT
			
		CASE ELSE RETURN #PINF##
	END SELECT
	
END FUNCTION


' #######################
' #####  Product()  #####
' #######################
'
'

FUNCTION LONGDOUBLE Product(n1,n2)
'returns n1 * (n1 + 1) * (n1 + 2) * .............. (n2 - 1) * n2
'If overflow occurs:
'	If running in SAFE_MODE then returns INF
' If running with SAFE_MODE off, throws exception

EXCEPTION_DATA exception

	#FUNC_NUM = 27
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
	IF n1 = 0 RETURN 0##
	IF n2 = 0 RETURN 0##
	IF n1 = n2 THEN RETURN n1									' only 1 term	
	IF n1 > n2 THEN SWAP n1,n2								' makes subsequent processing simpler
	IF ((n1 < 0) && (n2 > 0)) THEN RETURN 0		' product includes 0 so will always be 0
	
	IF #SAFETY_CHECKS THEN
		XstTry(SUBADDRESS(ChainMultiply), SUBADDRESS(ProductExcept), @exception)	
'return here from protected sub with either VALID RESULT or INF(+1) on TOS
ASM jmp > end.Product.xmal
		
	ELSE
		GOSUB ChainMultiply
ASM jmp > end.Product.xmal
		
	ENDIF

	SUB ChainMultiply
ASM fild 		d[Product.n1]	
		FOR multiplier = (n1 + 1) TO n2
ASM fimul 	d[Product.multiplier]
		NEXT multiplier
		
ASM fwait										;detect any exception after the last fimul
		EXIT SUB								'no exception, exit with valid result. Note, cannot RETURN from protected SUB		

badReturn:									'if exception occurs, return INF(+1)
ASM fld		t[pinf]						
	END SUB
	

	SUB ProductExcept
		'only exception should be FPU OVERFLOW
		exception.address  = GOADDRESS(badReturn)
		exception.response = $$ExceptionRetry
	END SUB

END FUNCTION

'****************************************************************************************************************
'****************************************************************************************************************
' TRIGONOMETRIC FUNCTIONS
'****************************************************************************************************************
'****************************************************************************************************************


' ###################
' #####  Sin()  #####
' ###################
'
'
FUNCTION LONGDOUBLE Sin(LONGDOUBLE a)
'Returns Sine of angle a
'Validity of a handled by FPU instructions

	#FUNC_NUM = 26
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM fld 	t[Sin.a]					; load input value

	IF #ANGLE_UNITS = $$DEGREES THEN
ASM fld 	t[DegToRad]				; convert degrees to radians if necessary
ASM fmulp st1, st
	ENDIF
	
ASM fsin

	'need to check for completion of fsin. If not complete will have input value on stack
ASM fstsw		ax
ASM fwait
ASM and ax, 0000010000000000b			; extract C2
ASM test ax, ax
ASM jz	> Sin_OK
 
	'Remove input and return NAN
ASM fstp		t[temp]								; send input a to dustbin
	RETURN PqNAN()
	
ASM Sin_OK:
ASM jmp > end.Sin.xmal
END FUNCTION


' ###################
' #####  Cos()  #####
' ###################
'
'
FUNCTION LONGDOUBLE Cos(LONGDOUBLE a)
'Returns Cosine of angle a
'Validity of a handled by FPU instructions
LONGDOUBLE radians

	#FUNC_NUM = 1
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF	

ASM fld t[Cos.a]					; load input value

	IF #ANGLE_UNITS = $$DEGREES THEN
ASM fld t[DegToRad]				; convert degrees to radians if necessary
ASM fmulp st1, st
	ENDIF

ASM fcos

	'need to check for completion of fcos. If not complete will have input value on stack
ASM fstsw		ax
ASM fwait
ASM and ax, 0000010000000000b			; extract C2
ASM test ax, ax
ASM jz	> Cos_OK
 
	'Remove input and return NAN
ASM fstp		t[temp]								; send input a to dustbin
	RETURN PqNAN()
	
ASM Cos_OK:
ASM jmp end.Cos.xmal

END FUNCTION


' ###################
' #####  Tan()  #####
' ###################
'
'
FUNCTION LONGDOUBLE Tan(LONGDOUBLE a)
'Returns Tangent of angle a
'Validity of a handled by FPU instructions

	#FUNC_NUM = 2
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM fld t[Tan.a]					; load input value

	IF #ANGLE_UNITS = $$DEGREES THEN
ASM fld t[DegToRad]				; convert degrees to radians if necessary
ASM fmulp st1, st
	ENDIF
	
ASM fptan

	'need to check for completion of fptan. If not complete will only be 1 value on stack
	'if complete, will also push 1 onto stack
ASM fstsw		ax
ASM fwait
ASM and ax, 0000010000000000b			; extract C2
ASM test ax, ax
ASM jz	> Tan_OK
 
	'Remove input and return NAN
ASM fstp		t[temp]								; send input a to dustbin
	RETURN PqNAN()
	
ASM Tan_OK:
ASM fxch													; remove the unwanted 1
ASM fstp st1
ASM jmp > end.Tan.xmal

END FUNCTION


' ######################
' #####  SinCos()  #####
' ######################
'
'
FUNCTION SinCos (LONGDOUBLE a, LONGDOUBLE sine, LONGDOUBLE cosine)
'Sets @sine to Sin(a) and @cosine to Cos(a) in 1 efficient function
'Validity of a handled by FPU instructions

	#FUNC_NUM = 2
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM fld t[SinCos.a]							; load input value

	IF #ANGLE_UNITS = $$DEGREES THEN
ASM fld t[DegToRad]									; convert degrees to radians if necessary
ASM fmulp st1, st
	ENDIF
	
ASM fsincos												; C2 = 0 and st = cos, st1 = sin OR C2 = 1 and st = radians

	'need to check for completion of fsincos. If not complete will only be 1 value on stack
	'this would cause a stack error if try to pop 2 values
ASM fstsw		ax
ASM fwait
ASM and ax, 0000010000000000b			; extract C2
ASM test ax, ax
ASM jz	> SinCos_OK
 
	'Remove input and return with @sin, @cosine unchanged
ASM fstp		t[temp]								; send input a to dustbin
	RETURN $$TRUE

	
	'Completed ok with 2 values on stack so continue	
ASM SinCos_OK:
ASM fstp		t[SinCos.cosine]		; pop it into @cosine
ASM fstp		t[SinCos.sine]			; pop it into @sine
 
END FUNCTION


' ###################
' #####  Sec()  #####
' ###################
'
'
FUNCTION LONGDOUBLE Sec (LONGDOUBLE a)
'If Cosine of a is not zero return Secant of a, otherwise INF
'N.B. degrees to radians taken care of within Cos() if required
'Validity of a handled by FPU instructions

	#FUNC_NUM = 4
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	

	
 	x## = Cos(a)
	IF IsNAN(x##,0) THEN RETURN PqNAN()
	
	IF x## = 0## THEN
		RETURN #PINF##
	ELSE
		RETURN 1## / x##
	ENDIF
	
END FUNCTION


' ###################
' #####  Csc()  #####
' ###################
'
'
FUNCTION LONGDOUBLE Csc (LONGDOUBLE a)
'If Sine of a is not zero return Cosecant of a, otherwise INF
'N.B. degrees to radians taken care of within Sin() if required
'Validity of a handled by FPU instructions

	#FUNC_NUM = 3
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
	x## = Sin(a)
	IF IsNAN(x##,0) THEN RETURN PqNAN()

	IF x## = 0## THEN
		RETURN #PINF##
	ELSE
		RETURN 1## / x##
	ENDIF


END FUNCTION


' ###################
' #####  Cot()  #####
' ###################
'
'
FUNCTION LONGDOUBLE Cot (LONGDOUBLE a)
'If Tangent of a is not zero return Cotangent of a, otherwise INF
'N.B. degrees to radians taken care of within Tan() if required
'Validity of a handled by FPU instructions

	#FUNC_NUM = 5
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
	x## = Tan(a)
	IF IsNAN(x##,0) THEN RETURN PqNAN()

	IF x## = 0## THEN
		RETURN #PINF##
	ELSE
		RETURN 1## / x##
	ENDIF

END FUNCTION


'****************************************************************************************************************
'****************************************************************************************************************
' INVERSE TRIGONOMETRIC FUNCTIONS
'****************************************************************************************************************
'****************************************************************************************************************

' ####################
' #####  Atan()  #####
' ####################
'
'
FUNCTION LONGDOUBLE Atan (LONGDOUBLE x)
'Returns principal angle (-pi/2 to pi/2) having a Tangent of x
'x can be from -inf to +inf
'Validity of a handled by FPU instructions

	#FUNC_NUM = 8
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM fld 	t[Atan.x]
ASM fld1
ASM fpatan						; radians

	IF #ANGLE_UNITS = $$DEGREES THEN
ASM fld t[RadToDeg]
ASM fmulp st1, st			; degrees	
	ENDIF

ASM jmp > end.Atan.xmal

END FUNCTION




' #####################
' #####  Atan2()  #####
' #####################
'
'
FUNCTION LONGDOUBLE Atan2 (LONGDOUBLE y, LONGDOUBLE x)
'Returns angle having a Tangent of (y / x)
'The signs of y and x are used to determine the quadrant of the returned angle as follows
'Atan(y/x) = principal angle ( -90 degrees to +90 degrees )

'If x > 0, y = any 		Atan2(y,x) = Atan(y/x)
'If x < 0, y >= 0     Atan2(y,x) = 180 + Atan(y/x)				
'If x < 0, y < 0      Atan2(y,x) = -180 + Atan(y/x)
'If x = 0, y > 0      Atan2(y,x) = 90
'If x = 0, y < 0      Atan2(y,x) = -90
'If x = 0, y = 0      Atan2(y,x) = undefined (this function returns +NAN)

'+ve results indicate angles measured anti-clockwise from the +x axis
'-ve results indicate angles measured clockwise from the +x axis
'Validity of x and y handled by FPU instructions

	#FUNC_NUM = 9
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
	
ASM fld t[Atan2.y]
ASM fld t[Atan2.x]
ASM fpatan

	IF #ANGLE_UNITS = $$DEGREES THEN
ASM fld t[RadToDeg]
ASM fmulp st1, st			; degrees	
	ENDIF
	
ASM jmp end.Atan2.xmal

END FUNCTION


' ####################
' #####  Asin()  #####
' ####################
'
'
FUNCTION LONGDOUBLE Asin (LONGDOUBLE x)
'returns principal angle (-pi/2 to pi/2) having Sine of x
'x can be -1 to + 1. If outside this range then returns $$INDEFINITE ( because of Sqrt(-ve) )
'2 to 3 times faster than Asin() in xma

	#FUNC_NUM = 6
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :

	IF x = 0## 	THEN RETURN 0##

	IF #ANGLE_UNITS = $$RADIANS THEN
		IF x = -1## THEN RETURN -$$PIDIV2
		IF x = 1##  THEN RETURN $$PIDIV2
	ELSE
		IF x = -1## THEN RETURN -90
		IF x = 1##  THEN RETURN 90
	ENDIF

ASM fld 	t[Asin.x]		;	x
ASM fld1							;	x, 1
ASM fld 	st1					; x, 1, x
ASM fmul 	st, st2			;	x*x, 1, x
ASM fsubp st1, st			;	(1 - x*x), x
ASM fsqrt							; Sqrt(1 - x*x), x		- Sqrt will return $$INDEFINITE if x out of range
ASM fpatan						; radians

	IF #ANGLE_UNITS = $$DEGREES THEN
ASM fld t[RadToDeg]		
ASM fmulp st1, st			; degrees
	ENDIF

ASM jmp > end.Asin.xmal

END FUNCTION


' ####################
' #####  Acos()  #####
' ####################
'
'
FUNCTION LONGDOUBLE Acos (LONGDOUBLE x)
'returns principal angle (0 to pi) having Cosine of x

	#FUNC_NUM = 7
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF	

ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
	
	IF x = 1##  THEN RETURN 0##

	IF #ANGLE_UNITS = $$RADIANS THEN
		IF x = -1## THEN RETURN $$PI
		IF x = 0##  THEN RETURN $$PIDIV2
	ELSE
		IF x = -1## THEN RETURN 180##
		IF x = 0##  THEN RETURN 90##
	ENDIF


ASM fld 	t[Acos.x]	;	x
ASM fld1							;	x, 1
ASM fld 	st1					; x, 1, x
ASM fmul 	st, st2 
ASM fsubp st1, st
ASM fsqrt							; x, Sqrt(1 - x*x)
ASM fxch
ASM fpatan						; radians

	IF #ANGLE_UNITS = $$DEGREES THEN
ASM fld t[RadToDeg]		
ASM fmulp st1, st			; degrees
	ENDIF

ASM jmp > end.Acos.xmal

END FUNCTION


' ####################
' #####  Acot()  #####
' ####################
'
'
FUNCTION LONGDOUBLE Acot (LONGDOUBLE x)
'returns principal angle (-pi/2 to pi/2) having Tangent of x
'Note - this is different to Acot() in xma which returns 0 to pi

	#FUNC_NUM = 12
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF

ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
	
	IF x = 0## THEN
		IF #ANGLE_UNITS = $$DEGREES THEN
			RETURN 90##
		ELSE
			RETURN $$PIDIV2
		ENDIF
	ENDIF
	
	RETURN Atan(1##/x)		' radians or degrees
	
END FUNCTION


' ####################
' #####  Acsc()  #####
' ####################
'
'
FUNCTION LONGDOUBLE Acsc (LONGDOUBLE x) 
'returns principal angle (-pi/2 to pi/2) having Cosecant of x

	#FUNC_NUM = 10
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
		
	IF #SAFETY_CHECKS THEN
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :		
		IF (ABS(x) < 1##) THEN RETURN PqNAN()
	ENDIF
	
	RETURN Asin(1##/x)		' Asin() takes care of degrees or radians

END FUNCTION


' ####################
' #####  Asec()  #####
' ####################
'
'
FUNCTION LONGDOUBLE Asec (LONGDOUBLE x)
'returns principal angle (0 to pi) having Secant of x

	#FUNC_NUM = 11
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
		
	IF #SAFETY_CHECKS THEN
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
		IF (ABS(x) < 1##) THEN RETURN PqNAN()
	ENDIF
	
	RETURN Acos(1##/x)		' Acos() takes care of degrees or radians

END FUNCTION


'****************************************************************************************************************
'****************************************************************************************************************
' ROOTS      ROOTS      ROOTS
'****************************************************************************************************************
'****************************************************************************************************************

' ####################
' #####  Sqrt()  #####
' ####################
'
'

FUNCTION LONGDOUBLE Sqrt (LONGDOUBLE x)
'If x is +ve returns Sqrt(x) otherwise -NAN
'Validity of x handled by FPU instruction

	#FUNC_NUM = 42
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
	IF #SAFETY_CHECKS THEN
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
		IF x < 0## THEN RETURN NqNAN()
	ENDIF
	
ASM fld 	t[Sqrt.x]
ASM fsqrt
ASM fwait										
ASM jmp > end.Sqrt.xmal

END FUNCTION

'
' #####################
' #####  Cbrt ()  #####
' #####################
'
'
FUNCTION LONGDOUBLE Cbrt (LONGDOUBLE x)
'returns cube root of x
LONGDOUBLE root

	#FUNC_NUM = 43
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF

	IF #SAFETY_CHECKS THEN
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
	ENDIF
	
	negative = $$FALSE
	IF x < 0## THEN negative = $$TRUE

	SELECT CASE x
		CASE 0## RETURN 0##
		CASE 1## RETURN 1##
		CASE -1## RETURN -1##
		CASE ELSE
ASM btr		w[Cbrt.x + 8], 15					; clear sign bit to make sure x is +ve													
ASM fld1
ASM fld 	t[Cbrt.x]
ASM fyl2x														; log2(x)
ASM fld		t[onethird]
ASM fmul														; log2(x) / 3
ASM call 	%_two2IplusF							; antilog2(log2(x) / 3)
ASM fstp	t[Cbrt.root]							; collect the answer in variable root
	END SELECT

	'give the root 1 refinement with Newton-Raphson; this helps eliminate small errors from use of log() / antilog()
	root = (0.666666666666666666667## * root) + (0.3333333333333333333333## * (x / (root * root)))
	
	IF negative THEN RETURN -root
	RETURN root
	
END FUNCTION


' #######################
' #####  NthRoot()  #####
' #######################
'
'
FUNCTION LONGDOUBLE NthRoot (LONGDOUBLE x, n)
'returns Nth root of x
LONGDOUBLE root

	#FUNC_NUM = 44
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :	
	IF n < 1 THEN RETURN PqNAN()
	IF n = 1 THEN RETURN x
	IF x = 0## THEN RETURN 0## 


	negative = $$FALSE
	IF x < 0## THEN
		'n must be odd or result is imaginary
		IFZ (n MOD 2) THEN RETURN PqNAN()
		negative = $$TRUE
	ENDIF

	SELECT CASE x
		CASE 0## RETURN 0##
		CASE 1## RETURN 1##
		CASE -1## RETURN -1##
		CASE ELSE
		power## = 1##/n
ASM btr		w[NthRoot.x + 8], 15				; clear sign bit to make sure x is +ve													
ASM fld1
ASM fld 	t[NthRoot.x]
ASM fyl2x															; log2(x)
ASM fld		t[NthRoot.power]
ASM fmul															; log2(x) / n
ASM call 	%_two2IplusF								; antilog2(log2(x) / n)
ASM fstp	t[NthRoot.root]							; collect the answer in variable root
	END SELECT

	'give the root 1 refinement with Newton-Raphson; this helps reduce small errors from use of log() / antilog()
	'Note - must use PowerX() not Power()
	
	root = (1##/n) * ((x/PowerX(root,n-1)) + (n-1)*root)

	IF negative THEN RETURN -root
	RETURN root
	
END FUNCTION


'****************************************************************************************************************
'****************************************************************************************************************
'POWERS, EXPONENTS, LOGS
'****************************************************************************************************************
'****************************************************************************************************************

' ####################
' #####  Exp2()  #####
' ####################
'
'
FUNCTION LONGDOUBLE Exp2 (LONGDOUBLE x)
'return 2 ** x

	#FUNC_NUM = 36
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
		
	IF #SAFETY_CHECKS THEN
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
		IF x > $$MAX_EXPONENT THEN RETURN #PINF##
		IF x < $$MIN_EXPONENT THEN RETURN 0##
	ENDIF

	IF x = 0## THEN RETURN 1##

ASM fld 	t[Exp2.x]				
ASM call 	%_two2IplusF
ASM jmp > end.Exp2.xmal

END FUNCTION


' ######################
' #####  Power2()  #####
' ######################
'
'
FUNCTION LONGDOUBLE Power2(p2)
'returns 2 ** p2 significantly faster than Exp2() BUT p2 must be an INTEGER
'Since p2 is integer, no Validity check needed

	#FUNC_NUM = 39
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
	IF #SAFETY_CHECKS THEN
		IF p2 > $$MAX_EXPONENT THEN RETURN #PINF##
		IF p2 < $$MIN_EXPONENT THEN RETURN 0##
	ENDIF
	
	IF p2 = 0 THEN RETURN 1##

ASM fild 	d[Power2.p2]
ASM fld1
ASM fscale
ASM fstp	st1
ASM jmp > end.Power2.xmal

END FUNCTION


' #######################
' #####  Power10()  #####
' #######################
'
'
FUNCTION LONGDOUBLE Power10(p10)
' 10 ** p10, as Exp10 but faster because p10 must be Integer between +4932 and -4932
' Also substantially more accurate
' accurate values for 10 to the power 1000, 2000, 3000, 4000, 100 to 900, 10 to 90 and 1 to 9 are stored in tables
' using these, for example, 10 ** 3875 is calculated as (10 ** 3000) * (10 ** 800) * (10 ** 70) * (10 ** 5)
' Since p10 is integer, no Validity check needed
AUTOX LONGDOUBLE result

	#FUNC_NUM = 40
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
	'This check is necessary independent of SAFE_MODE setting
	IF p10 > 4932   THEN RETURN #PINF##
	IF p10 <= -4932 THEN RETURN 0##

	
	invert = $$FALSE
	IF p10 < 0 THEN
		invert = $$TRUE
		p10 = -p10
	ENDIF
	
'initialise result on FPU
ASM fld1

	IF p10 >= 1000 THEN
	'multiply by 10 ** thousands
		thousands = p10 \ 1000
		p10  = p10 MOD 1000		
ASM mov 	esi, ADDR thousands
ASM mov 	eax, d[Power10.thousands]
ASM dec 	eax
ASM imul	eax, 10
ASM fld		t[esi + eax]
ASM fmul
	ENDIF
	
	IF p10 >= 100 THEN
		hundreds = p10 \ 100
		p10  = p10 MOD 100
	'multiply by 10 ** hundreds		
ASM mov 	esi, ADDR hundreds
ASM mov 	eax, d[Power10.hundreds]
ASM dec 	eax
ASM imul 	eax, 10
ASM fld		t[esi + eax]
ASM fmul  
	ENDIF
	
	IF p10 >= 10 THEN
	'multiply by 10 ** tens
		tens = p10 \ 10
		p10  = p10 MOD 10			
ASM mov 	esi, ADDR tens
ASM mov 	eax, d[Power10.tens]
ASM dec 	eax
ASM imul	eax, 10
ASM fld		t[esi + eax]
ASM fmul	
	ENDIF
	
	IF p10 >= 1 THEN
	'multiply by 10 ** units
ASM mov 	esi, ADDR units
ASM mov 	eax, d[Power10.p10]
ASM dec 	eax
ASM imul	eax, 10
ASM fld		t[esi + eax]
ASM fmul  
	ENDIF
	
	IF invert THEN
	'return 1 / result
ASM fld1
ASM fdivrp st1, st 		
	ENDIF
	
ASM jmp end.Power10.xmal
	
END FUNCTION



' ######################
' #####  PowerX()  #####
' ######################
'
'
FUNCTION LONGDOUBLE PowerX(LONGDOUBLE x, p)
'calculates (x ** p) by direct mutiplication rather than using logs
'speed and accuracy optimised by minimising the number of actual multiplications
'e.g x ** 10 is not calculated as x * x * x * x * x * x * x * x * x * x but as follows:
'1st, x * x is calculated giving x ** 2 and temporary result set to this
'this is then used to calculate x ** 4 then x ** 8 by squaring twice
'the original x ** 2 is then multiplied by x ** 8 to give the required x ** 10
 
EXCEPTION_DATA exception
AUTOX	bitTop

	#FUNC_NUM = 41
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF

ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
	
	invert = $$FALSE
	IF p < 0 THEN
		p = -p
		invert = $$TRUE
	ENDIF
	
	overFlow = $$FALSE

ASM mov 	eax, d[PowerX.p]
ASM bsr		eax, eax										;find the highest bit set to 1
ASM mov		d[PowerX.bitTop], eax

	result## = 1##
	factor## = x
	
	
	IF #SAFETY_CHECKS THEN
		
		FOR i = 0 TO bitTop									'look at each bit in turn to see which factors have to be applied to result##
			
ASM mov 	eax, d[PowerX.i]
ASM bt		d[PowerX.p], eax						
ASM jnc		>
			XstTry (SUBADDRESS(Multiply), SUBADDRESS(Exception1), @exception)
			IF overFlow THEN GOTO PowerX_overflow
ASM :
			IF i = bitTop THEN RETURN result##
			XstTry (SUBADDRESS(Factor), SUBADDRESS(Exception2), @exception)

		NEXT i
		
	ELSE
		
		FOR i = 0 TO bitTop									'look at each bit in turn to see which factors have to be applied to result##
ASM mov 	eax, d[PowerX.i]
ASM bt		d[PowerX.p], eax						
ASM jnc		>
			GOSUB Multiply
ASM :
			IF i = bitTop THEN
				IF invert THEN RETURN 1 / result##
				RETURN result##
			ENDIF
			
			GOSUB Factor
		NEXT i
		
	ENDIF		


SUB Multiply
	'for set bits only, multiply in the relevant factor
	result## = result## * factor##
ASM fwait												;check for exceptions
	EXIT SUB
	
badReturn1:
	overFlow = $$TRUE
END SUB

SUB Factor
	'square up factor, x**2,  x**4, x**8,  x**16 ....... 
	factor## = factor## * factor##
ASM fwait												;check for exceptions
	EXIT SUB
	
badReturn2:
	overFlow = $$TRUE
END SUB

SUB Exception1			
	'only exception should be FPU OVERFLOW in Multiply SUB
	exception.address  = GOADDRESS(badReturn1)
	exception.response = $$ExceptionRetry
END SUB

SUB Exception2
	'only exception should be FPU OVERFLOW in Factor SUB
	exception.address  = GOADDRESS(badReturn2)
	exception.response = $$ExceptionRetry
END SUB

PowerX_overflow:
	IF invert THEN
		RETURN 0##
	ELSE
		RETURN #PINF##
	ENDIF


END FUNCTION



' ###################
' #####  Exp()  #####
' ###################
'
'
FUNCTION LONGDOUBLE Exp (LONGDOUBLE x)
'returns e ** x

	#FUNC_NUM = 35
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
	IF #SAFETY_CHECKS THEN
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
		'make sure e ** x can be represented as LONGDOUBLE
		IF x > 	11356.52340629414395## THEN RETURN #PINF##
		IF x < -11355.13711193302406## THEN RETURN 0##
	ENDIF
	
	IF x = 0## THEN RETURN 1##

ASM fld 	t[Exp.x]
ASM fldl2e
ASM fmul						; TOS = Log2(e ** x)
ASM call 	%_two2IplusF
ASM jmp > end.Exp.xmal

END FUNCTION 


' #####################
' #####  Exp10()  #####
' #####################
'
'
FUNCTION LONGDOUBLE Exp10 (LONGDOUBLE x)
'returns 10 ** x

	#FUNC_NUM = 37
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
	IF #SAFETY_CHECKS THEN
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
		IF x > 	4932.075448958667902## THEN RETURN #PINF##
		IF x < -4931.47338896733994##  THEN RETURN 0##
	ENDIF

	IF x = 0## THEN RETURN 1##

ASM fld 	t[Exp10.x]
ASM fldl2t
ASM fmul											; TOS = Log2(10 ** x)
ASM call 	%_two2IplusF
ASM jmp > end.Exp10.xmal

END FUNCTION





' #####################
' #####  Log10()  #####
' #####################
'
'
FUNCTION LONGDOUBLE Log10 (LONGDOUBLE x)
'returns log base 10 of x

	#FUNC_NUM = 32
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
		
	IF #SAFETY_CHECKS THEN
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
		IF x <= 0## THEN RETURN NqNAN()
	ENDIF
	
ASM fldlg2
ASM fld 	t[Log10.x]
ASM fyl2x
ASM jmp > end.Log10.xmal
END FUNCTION



' ##################
' #####  Ln()  #####
' ##################
'
'
FUNCTION LONGDOUBLE Ln (LONGDOUBLE x)
'SYNONYM for Log(): Shows in trace as Log()
	RETURN Log(x)
END FUNCTION

 
' ###################
' #####  Log()  #####
' ###################
'
'
FUNCTION LONGDOUBLE Log (LONGDOUBLE x)
'returns log base e of x

	#FUNC_NUM = 30
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
		
	IF #SAFETY_CHECKS THEN
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
		IF x <= 0## THEN RETURN NqNAN()
	ENDIF
	
ASM fldln2
ASM fld 	t[Log.x]
ASM fyl2x
ASM jmp > end.Log.xmal

END FUNCTION


' ######################
' #####  Log1p()  #####
' ######################
'
'
FUNCTION LONGDOUBLE Log1p (LONGDOUBLE x)
'returns log base e (1 + x) for small x in limited range - see below
'for very small x, Log1p(x) is more accurate than Log(1 + x)

	#FUNC_NUM = 34
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :

	'check x is in range permitted by the fyl2xp1 instruction below 
	IF x >  $$LOG1PLIMIT 	THEN RETURN PqNAN()		'approx  >  0.2929   	
	IF x < -$$LOG1PLIMIT	THEN RETURN NqNAN()		'approx  < -0.2929

	
ASM fldln2
ASM fld 		t[Log1p.x]
ASM fyl2xp1											
ASM jmp > 	end.Log1p.xmal

END FUNCTION


' ####################
' #####  Log2()  #####
' ####################
'
'
FUNCTION LONGDOUBLE Log2 (LONGDOUBLE x)
'returns log base 2 of x

	#FUNC_NUM = 31
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF	
	
	IF #SAFETY_CHECKS THEN
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
		IF x <= 0## THEN RETURN PqNAN()
	ENDIF
	
ASM fld1
ASM fld 	t[Log2.x]
ASM fyl2x
ASM jmp > end.Log2.xmal

END FUNCTION


' ####################
' #####  Logb()  #####
' ####################
'
'
FUNCTION LONGDOUBLE Logb (LONGDOUBLE x, LONGDOUBLE b)
'returns log base b of x

	#FUNC_NUM = 33
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF

ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
ASM call %_TestArg2isNAN
ASM jnc >
	RETURN b
ASM :
	
	IF x <= 0## THEN RETURN NqNAN()
	IF b = 1## THEN RETURN NqNAN()
	IF b <=0 THEN RETURN NqNAN

	RETURN (Log2(x) / Log2(b))
	
END FUNCTION
	

' #####################
' #####  Power()  #####
' #####################
'
'
FUNCTION LONGDOUBLE Power (LONGDOUBLE x, LONGDOUBLE y)
'Returns x ** y

	#FUNC_NUM = 38
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
	IF #SAFETY_CHECKS THEN
ASM call %_TestArg1isNAN
ASM jnc >
		RETURN x
ASM :
ASM call %_TestArg2isNAN
ASM jnc >
		RETURN y
ASM :
	ENDIF
	
	'Check x for special conditions 
	IF x = 0## THEN RETURN 0##		'0 ** anything = 0
	IF x = 1## THEN RETURN 1##		'1 ** anything = 1
	
	negate = $$FALSE			'initialise flag to handle sign of x
	
	'special considerations if x is -ve
	IF x < 0## THEN
		'If x is -ve then y must be an integer
		IF INT(y) <> y THEN RETURN PqNAN()
		
		'If y is even result will be +ve otherwise negative 
		IF (y MOD 2##) <> 0## THEN negate = $$TRUE
		
		'check for special case of -1
		IF x = -1## THEN
			IF negate THEN RETURN -1## ELSE RETURN 1##
		ENDIF
		
	ENDIF
	
	'Check y for special conditions 
	IF y = 0## THEN
		IF  negate THEN RETURN -1## ELSE RETURN 1##		'+/- anything ** 0 = +/- 1
	ENDIF
	
	IF y = 1## THEN RETURN x												'x ** 1 = x
	

	'No short cuts so do it the hard way!    ABS(result) = antilog(y * log(x))
	xx## = Exp2(y * Log2(ABS(x)))									
	
	'if necessary, change the sign
	IF negate THEN RETURN -xx## ELSE RETURN xx##
	
END FUNCTION

'***********************************************************************************************************************
'***********************************************************************************************************************
' HELPER FUNCTIONS       HELPER FUNCTIONS
'***********************************************************************************************************************
'***********************************************************************************************************************

' ########################
' #####  GetClass()  #####
' ########################
'
'
FUNCTION  GetClass (LONGDOUBLE x)

'returns type as follows:
' 1 = NORMAL
' 2 = ZERO
' 3 = DENORMAL
' 4 = INFINITY
' 5 = NAN / Indefinite
 
 	#FUNC_NUM = 104 
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF	
	
ASM mov 	ax, w[ebp + 16]			; get exponent
ASM btr 	ax, 15							; clear the sign
ASM cmp 	ax, 0
ASM je		> DenOrZero
ASM cmp		ax, 0x7FFF
ASM je		> NanOrInf

	RETURN 1		'Normal number
	
ASM DenOrZero:
ASM cmp		d[ebp + 12], 0			; mantissa high
ASM jne >	Denormal 
ASM cmp		d[ebp + 8], 0				; mantissa low
ASM jne > Denormal

	RETURN 2		'Zero
	
ASM Denormal:
	RETURN 3		'Denormal		
	
	
ASM NanOrInf:
ASM cmp		d[ebp + 12], 0x80000000			; mantissa high
ASM jne > Nan 
ASM cmp		d[ebp + 8], 0								; mantissa low
ASM jne > Nan
	
	RETURN 4		'Infinity
	
ASM Nan:
	RETURN 5		'NAN / indefinite		


END FUNCTION



' #######################
' #####  IsNormal()  #####
' #######################
'
'
FUNCTION  IsNormal (LONGDOUBLE x)
 
 	#FUNC_NUM = 103 
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF	
	
ASM mov 	ax, w[ebp + 16]			; get exponent
ASM and 	ax, 0x7FFF					; clear the sign
ASM cmp 	ax, 0x7FFF
ASM jne		> 

	RETURN $$FALSE							' NAN or INF
	
ASM :
ASM cmp   ax, 0
ASM je >

	RETURN $$TRUE								' NORMAL <> 0
	
ASM :													; distinguish between ZERO and Denormal
ASM cmp 	d[ebp + 8], 0
ASM je >

	RETURN $$FALSE							' DENORMAL
	
ASM :
ASM cmp 	d[ebp + 12], 0
ASM jne >

	RETURN $$TRUE								' ZERO is NORMAL
	
ASM :

	RETURN $$FALSE							' DENORMAL

END FUNCTION



' #######################
' #####  IsValid()  #####
' #######################
'
'
FUNCTION  IsValid (LONGDOUBLE x)
 
 	#FUNC_NUM = 105 
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF	
	
ASM mov 	ax, w[ebp + 16]			; get exponent
ASM and 	ax, 0x7FFF					; clear the sign
ASM cmp 	ax, 0x7FFF
ASM jne		> 

	RETURN $$FALSE							' NAN or INF
	
ASM :

	RETURN $$TRUE								' NORMAL, ZERO or DENORMAL

END FUNCTION

''
' #####################
' #####  IsNAN () #####
' #####################
'
'


FUNCTION IsNAN(LONGDOUBLE x, sign)
'Test to see if x is NAN. 
'If sign = +1 then must be +NAN, if sign = -1 then must be -NAN, if sign = 0 then can be +NAN or -NAN

	#FUNC_NUM = 185
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
	'Firstly, exponent (excluding sign bit) must be all 1's
ASM mov ax, w[IsNAN.x + 8]
ASM and ax, 0x7FFF
ASM cmp ax, 0x7FFF				
ASM jnz > notNAN

	'Secondly, mantissa must NOT be 0b100000000 ...000 (this would be an INF). Anything else is a NAN
ASM cmp d[IsNAN.x + 4], 0x80000000
ASM jne > gotNAN
ASM cmp d[IsNAN.x],0
ASM je  > notNAN
ASM gotNAN:

	'It is a NAN, now sort out the signs
	actSign = +1										'assume +1, then test actual sign
ASM bt  w[IsNAN.x + 8], 15 
ASM jnc >													
	actSign = -1										'it is actually -1 
ASM :
	IF sign = 0 THEN RETURN $$TRUE					' don't care about sign
	IF sign = actSign THEN RETURN $$TRUE		' signs match OK
	
ASM notNAN:
	RETURN $$FALSE
	
END FUNCTION


	
'END FUNCTION


'
' ######################
' #####  IsSNAN () #####
' ######################
'
'
FUNCTION IsSNAN(LONGDOUBLE x, sign)
'Test to see if x is SNAN. 
'If sign = +1 then must be +SNAN, if sign = -1 then must be -SNAN, if sign = 0 then can be +SNAN or -SNAN

	#FUNC_NUM = 186
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
 
	'See if it is a NAN of any kind
	IFF IsNAN(x, sign) THEN RETURN $$FALSE

	'It is a NAN but test if it is QUIET or SIGNALLING
	'Test whether 2nd bit in mantissa is 1 (QUIET) or 0 (SIGNALLING)	
ASM bt d[IsSNAN.x + 4], 30
ASM jc > notSNAN

	'It is SNAN, now sort out the signs
	IF sign = 0 THEN RETURN $$TRUE					' don't care about sign
	
	actSign = +1										'assume +1, then test actual sign
ASM bt  w[IsSNAN.x + 8], 15 
ASM jnc >													
	actSign = -1										'it is actually -1 
ASM :

	IF sign = actSign THEN RETURN $$TRUE		' signs match OK
	
ASM notSNAN:
	RETURN $$FALSE
END FUNCTION



'
' ############################
' #####  IsINDEFINITE () #####
' ############################
'
'
FUNCTION IsINDEFINITE(LONGDOUBLE x)
'Test to see if x is the special value INDEFINITE


	#FUNC_NUM = 192
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
ASM cmp 	w[ebp + 16], 0xFFFF						; test exponent
ASM jne		> notIndefinite 
ASM cmp		d[ebp + 12], 0xC0000000				; mantissa high
ASM jne		> notIndefinite 
ASM cmp   d[ebp + 8], 0
ASM jne		> notIndefinite 
	RETURN $$TRUE
	
ASM notIndefinite:
	RETURN $$FALSE
	
END FUNCTION


'
' ######################
' #####  IsQNAN () #####
' ######################
'
'
FUNCTION IsQNAN(LONGDOUBLE x, sign)
'Test to see if x is QNAN. 
'If sign = +1 then must be +QNAN, if sign = -1 then must be -QNAN, if sign = 0 then can be +QNAN or -QNAN

	#FUNC_NUM = 187
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF 
	
	'See if it is a NAN of any kind
	IFF IsNAN(x, sign) THEN RETURN $$FALSE

	'It is a NAN but test if it is QUIET or SIGNALLING
	'Test whether 2nd bit in mantissa is 1 (QUIET) or 0 (SIGNALLING)	
ASM bt d[IsQNAN.x + 4], 30
ASM jnc > notQNAN

	'It is QNAN, now sort out the signs
	IF sign = 0 THEN RETURN $$TRUE					' don't care about sign
	
	actSign = +1										'assume +1, then test actual sign
ASM bt  w[IsQNAN.x + 8], 15 
ASM jnc >													
	actSign = -1										'it is actually -1 
ASM :

	IF sign = actSign THEN RETURN $$TRUE		' signs match OK
	
ASM notQNAN:
	RETURN $$FALSE
END FUNCTION

'
' ##############################
' #####  GetOriginOfNAN () #####
' ##############################
'
'
FUNCTION GetOriginOfNAN(LONGDOUBLE nan, funcNum, func$)
'Return origin / source of a NAN

ASM call %_TestArg1isNAN
ASM jc >
	funcNum = -1: func$ = "NOTaNAN"					'input nan is not a NAN
	RETURN $$TRUE
ASM :
	
	IF IsINDEFINITE(nan) THEN								'generated internally by FPU
		funcNum = 255: func$ = "FPU"
		RETURN 
	ENDIF

ASM mov ax, w[GetOriginOfNAN.nan]					;get last 2 bytes
ASM cmp ax, 0xF800
ASM jne >
	funcNum = 254: func$ = "CONST"					'set from $$ constant
	RETURN
ASM :

ASM cmp ax, 0xFFFF
ASM jne >	Xmal
ASM mov ax, w[GetOriginOfNAN.nan + 6]			;created by QNAN or SNAN - examine bit 2 of mantissa
ASM bt	ax, 14
ASM jnc >
	funcNum = 179: func$ = "QNAN()"				
	RETURN
ASM :
	funcNum = 180: func$ = "SNAN()"				
	RETURN

ASM Xmal:
ASM and ax, 0x00FF
ASM cwde 
ASM mov d[GetOriginOfNAN.funcNum], eax 
	func$ = FunctionNumberToName$(funcNum)	'created by Xmal function
	RETURN
	
	funcNum = -1: func$ = "UNKNOWN"					'unknown
	RETURN $$FALSE

END FUNCTION



' ##########################
' #####  IsNegative()  #####
' ##########################
'
'
FUNCTION IsNegative(LONGDOUBLE x)
'return $$TRUE if x is negative, otherwise $$FALSE
'perform test in situ with stack parameter for speed

	#FUNC_NUM = 109
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF	
	
ASM mov 	ax, w[ebp + 16]						; get sign and exponent
ASM bt 		ax, 15										; test sign bit
ASM jnc >														; sign bit = 0 so $$FALSE
ASM mov 	eax, -1										; sign bit = 1 so $$TRUE
ASM jmp 	end.IsNegative.xmal				
ASM :																; returns 0 by default

END FUNCTION


' ##########################
' #####  IsPositive()  #####
' ##########################
'
'
FUNCTION IsPositive(LONGDOUBLE x)
'return $$TRUE if x is positive (including zero), otherwise $$FALSE
'perform test in situ with stack parameter for speed

	#FUNC_NUM = 110
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
ASM mov 	ax, w[ebp + 16]						; get sign and exponent
ASM bt 		ax, 15										; test sign bit
ASM jc >														; sign bit = 1 so $$FALSE
ASM mov 	eax, -1										; sign bit = 0 so $$TRUE
ASM jmp 	end.IsPositive.xmal				
ASM :																; returns 0 by default

END FUNCTION


'
' #####################
' #####  IsINF () #####
' #####################
'
'
FUNCTION IsINF(LONGDOUBLE x, sign)
'Test to see if x is INF. 
'If sign = +1 then must be +INF, if sign = -1 then must be -INF, if sign = 0 then can be +INF or -INF

	#FUNC_NUM = 188
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
	IF sign >= 0 THEN
		'test versus +ve INF. Return $$TRUE if matches
ASM	mov esi, ADDR pinf	
	
ASM	mov ax, w[IsINF.x + 8]		;sign and exponent
ASM	cmp ax, w[esi + 8]
ASM	jne > notPlusINF
ASM	mov eax, d[IsINF.x + 4]		;1st half mantissa
ASM	cmp eax, d[esi + 4]
ASM	jne > notPlusINF		
ASM	mov eax, d[IsINF.x]				;2nd half mantissa
ASM	cmp eax, d[esi]
ASM	jne > notPlusINF
		RETURN $$TRUE
	ENDIF

ASM notPlusINF:
	
	IF sign <= 0 THEN
		'test versus -ve INF
ASM	mov esi, ADDR ninf	
	
ASM	mov ax, w[IsINF.x + 8]		;sign and exponent
ASM	cmp ax, w[esi + 8]
ASM	jne > notNegINF
ASM	mov eax, d[IsINF.x + 4]		;1st half mantissa
ASM	cmp eax, d[esi + 4]
ASM	jne > notNegINF		
ASM	mov eax, d[IsINF.x]				;2nd half mantissa
ASM	cmp eax, d[esi]
ASM	jne > notNegINF
		RETURN $$TRUE		
	ENDIF	
	
ASM notNegINF:
	RETURN $$FALSE
	
END FUNCTION


'
' ######################
' #####  IsZERO () #####
' ######################
'
'
FUNCTION IsZERO(LONGDOUBLE x, sign)
'Test to see if x is ZERO. 
'If sign = +1 then must be +ZERO, if sign = -1 then must be -ZERO, if sign = 0 then can be +ZERO or -ZERO

	#FUNC_NUM = 191
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
	IF sign >= 0 THEN
		'test for +ZERO. Return $$TRUE if matches
		
ASM	cmp w[IsZERO.x + 8], 0		;sign and exponent must be 0
ASM	jne > notPlusZERO
ASM	cmp d[IsZERO.x + 4], 0		;1st half mantissa must be 0
ASM	jne > notPlusZERO		
ASM	cmp d[IsZERO.x], 0				;2nd half mantissa must be 0
ASM	jne > notPlusZERO
		RETURN $$TRUE
	ENDIF
	
ASM notPlusZERO:
	
	IF sign <= 0 THEN
		'test versus -ve ZERO. Return $$TRUE if matches
		
ASM	cmp w[IsZERO.x + 8], 0x8000		;sign and exponent must be 0x8000
ASM	jne > notNegZERO
ASM	cmp d[IsZERO.x + 4], 0				;1st half mantissa must be 0
ASM	jne > notNegZERO		
ASM	cmp d[IsZERO.x], 0						;2nd half mantissa must be 0
ASM	jne > notNegZERO
		RETURN $$TRUE		
	ENDIF	
	
ASM notNegZERO:
	RETURN $$FALSE
	
END FUNCTION


'
' #####################
' #####  IsDEN () #####
' #####################
'
'
FUNCTION IsDEN(LONGDOUBLE x, sign)
'Test to see if x is DENORMAL. 
'If sign = +1 then must be +DEN, if sign = -1 then must be -DEN, if sign = 0 then can be +DEN or -DEN

	#FUNC_NUM = 189
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
	'Firstly, check first bit in mantissa is 0
ASM bt d[IsDEN.x + 4], 31
ASM jc > notDEN

	'Secondly, exponent (excluding sign bit) must be 0
ASM test w[IsDEN.x + 8], 0x7FFF				
ASM jnz > notDEN

	'It IS denormal, now sort out the signs
	actSign = +1										'assume +1, then test actual sign
ASM bt  w[IsDEN.x + 8], 15 
ASM jnc >													
	actSign = -1										'it is actually -1 
ASM :
	IF sign = 0 THEN RETURN $$TRUE					' don't care about sign
	IF sign = actSign THEN RETURN $$TRUE		' signs match OK
	
ASM notDEN:
	RETURN $$FALSE
	
END FUNCTION

'
' #########################
' #####  NextValue () #####
' #########################
'
'
FUNCTION NextValue(LONGDOUBLE x, cSteps)
'Change x to the value cSteps normal representables away
'If cSteps is +ve, moves towards INF of the same sign as x. That is, away from 0
'If cSteps is -ve, moves towards 0
'The following logic depends on cSteps being XLONG and so only directly affecting the low half of the mantissa

	#FUNC_NUM = 131
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF

	IF cSteps = 0 THEN RETURN $$TRUE
	IFF IsValid(x) THEN RETURN $$TRUE
	
	IF x = 0## THEN 
		'Deal with special case of +0 and -0
		SELECT CASE ALL TRUE
			CASE cSteps <= -1
				'-cSteps means go closer to 0 but we are there so we can't
				RETURN $$TRUE
				
			CASE cSteps >= +1
				'special move to first normal with higher exponent and mantissa than 0
				'If +0 then move towards +INF, if -0 then move towards -INF 
ASM mov d[NextValue.x], 0
ASM mov d[NextValue.x + 4], 0x80000000
ASM or  w[NextValue.x + 8], 0x0001				;keep sign of 0 and set exponent

			CASE cSteps > 1
				'first special step will have been done above, now do remaining (cSteps - 1) normal steps
				DEC cSteps
				GOSUB PluscSteps
				
		END SELECT
		
	ELSE
		'x is not 0, test for denormal
ASM bt	d[NextValue.x + 4], 31
ASM jc > normal

		'Deal with x is denormal
		SELECT CASE ALL TRUE
			CASE cSteps <= -1
				'-cSteps means go closer to 0 so go right to 0 skipping all other denormals
ASM mov d[NextValue.x], 0
ASM mov d[NextValue.x + 4], 0
ASM and w[NextValue.x + 8], 0x8000				;keep denormals sign and keep exponent of 0

			CASE cSteps >= +1
				'special move to first normal with higher exponent and mantissa than 0
				'If +0 then move towards +INF, if -0 then move towards -INF 
ASM mov d[NextValue.x], 0
ASM mov d[NextValue.x + 4], 0x80000000
ASM or  w[NextValue.x + 8], 0x0001				;keep denormals sign and set exponent

			CASE cSteps > 1
				'first special step will have been done above, now do remaining (cSteps - 1) normal steps
				DEC cSteps
				GOSUB PluscSteps
				
		END SELECT
		RETURN 'from denormal	

ASM normal:		
		IF cSteps > 0 THEN
			GOSUB PluscSteps
		ELSE
			GOSUB NegcSteps
		ENDIF
	ENDIF



SUB PluscSteps
	'Move to larger mantissas and exponents	i.e. towards +INF for +ve x and -INF for -ve x 
ASM mov eax,	d[NextValue.cSteps]
ASM clc
ASM add d[NextValue.x], eax									; adjust mant low 
ASM adc d[NextValue.x + 4], 0								; mant high
ASM cmp d[NextValue.x + 4], 0								; have we denormalised
ASM jne >																		; no
ASM mov d[NextValue.x + 4], 0x80000000			; yes so normalise and adjust exponent
ASM add w[NextValue.x + 8], 1
ASM :
END SUB		


	
SUB NegcSteps		
	'Move to smaller mantissas and exponents i.e. towards 0 for +ve or -ve x
ASM mov eax,	d[NextValue.cSteps]
ASM neg eax														; cSteps = ABS(cSteps)
ASM clc
ASM sub d[NextValue.x], eax									; adjust mant low 
ASM sbb d[NextValue.x + 4], 0								; mant high
ASM sbb w[NextValue.x + 8], 0								; adjust exponent
ASM cmp d[NextValue.x + 4], 0x80000000			; have we denormalised
ASM jae >																		; no
ASM add d[NextValue.x + 4], 0x80000000			; yes so normalise and adjust exponent 
ASM sub w[NextValue.x + 8], 1
ASM :
	'Need to see if we have reached or overshot denormal zone or 0 itself
ASM mov ax, w[NextValue.x + 8]			; get exponent just generated
ASM and ax, 0x7FFF									; remove the sign
ASM cmp ax, 0												; is it 0 indicating denormal or 0
ASM jne >														; no so OK
ASM mov d[NextValue.x], 0						; yes so set x = 0
ASM mov d[NextValue.x + 4], 0			 
ASM :		
END SUB
	
END FUNCTION


'
' ##########################
' #####  NextToward () #####
' ##########################
'
'
FUNCTION LONGDOUBLE NextToward(LONGDOUBLE x, LONGDOUBLE y)
'Returns next representable number after x in the direction of (towards) y
'IEEE754 requires NAN return if either argument is NAN

	#FUNC_NUM = 132
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
ASM call %_TestArg2isNAN
ASM jnc >
	RETURN y
ASM :

	IF y = x THEN RETURN y
	
	temp## = x
	
	IF y > x THEN
		SELECT CASE TRUE
			CASE x = 0##
				'Move towards +INF
				temp## = +0##
				NextValue(@temp##, +1)
		
			CASE x > 0##
				'Move towards +INF
				NextValue(@temp##, +1)
			
			CASE x < 0##
				'Move towards 0
				NextValue(@temp##, -1)
			
		END SELECT
		
	ELSE
		' x > y		
		SELECT CASE TRUE
			CASE x = 0##
				'Move towards -INF
				temp## = -0##
				NextValue(@temp##, +1)
			
			CASE x > 0##
				'Move towards 0
				NextValue(@temp##, -1)
		
			CASE x < 0##
				'Move towards INF
				NextValue(@temp##, +1)
			
		END SELECT
		
	ENDIF
	
	RETURN temp##
	
END FUNCTION


'
' ##########################
' #####  NextHigher () #####
' ##########################
'
'

FUNCTION NextHigher(LONGDOUBLE x)
' Change x to the next higher representation - i.e. towards +INF
' By default, include Denormals. Switch Denormals off with XmalSetDenormals()

	#FUNC_NUM = 133
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
	IF IsPositive(x) THEN
		class = GetClass(x)
		SELECT CASE class
			CASE 1
				' Normal <> 0
				IF x = #LMAX## THEN
					x = #PINF##
				ELSE
					NextValue(@x,+1)
				ENDIF
				
			CASE 2
				' Normal = 0
				IF #DENORMAL THEN
					x = #LMINDEN##
				ELSE
					x = #LMIN##
				ENDIF
				
			CASE 3
				' Denormal
				IF #DENORMAL THEN
					IF x < #LMAXDEN## THEN
ASM clc
ASM add d[NextHigher.x], 1									; adjust mant low 
ASM adc d[NextHigher.x + 4], 0							; mant high
					ELSE
						x = #LMIN##
					ENDIF
				ELSE
					x = #LMIN##
				ENDIF
				
			CASE 4
				' Infinity - cannot go any higher
				RETURN $$TRUE
				
			CASE 5
				' NAN
				RETURN $$TRUE
			
		END SELECT
		
	ELSE
		temp## = -x
		ret = NextLower(@temp##)
		x = -temp##
		RETURN ret
	ENDIF

END FUNCTION


'
' #########################
' #####  NextLower () #####
' #########################
'
'
FUNCTION NextLower(LONGDOUBLE x)
' Change x to the next lower representation - i.e. towards -INF
' By default, include Denormals. Switch Denormals off with XmalSetDenormals()

	#FUNC_NUM = 134
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
	IF IsPositive(x) THEN
		class = GetClass(x)
		SELECT CASE class
			CASE 1
				' Normal <> 0
				IF x = #LMIN## THEN
					IF #DENORMAL THEN
						x = #LMAXDEN##
					ELSE
						x = 0##
					ENDIF
				ELSE
					NextValue(@x,-1)
				ENDIF
				
			CASE 2
				' Normal = 0
				IF #DENORMAL THEN
					x = -#LMINDEN##
				ELSE
					x = -#LMIN##
				ENDIF
				
			CASE 3
				' Denormal
				IF #DENORMAL THEN
					IF x > #LMINDEN## THEN
ASM clc
ASM sub d[NextLower.x], 1									; adjust mant low 
ASM sbb d[NextLower.x + 4], 0							; mant high
					ELSE
						x = 0##
					ENDIF
				ELSE
					x = 0##
				ENDIF
				
			CASE 4
				' Infinity
				x = #LMAX##
				RETURN 
				
			CASE 5
				' NAN
				RETURN $$TRUE
			
		END SELECT
		
	ELSE
		temp## = -x
		ret = NextHigher(@temp##)
		x = -temp##
		RETURN ret
	ENDIF
	
END FUNCTION



'***********************************************************************************************************************
'***********************************************************************************************************************
' APPROXIMATE TESTS        APPROXIMATE TESTS
'***********************************************************************************************************************
'***********************************************************************************************************************


' ############################
' #####  IsApproxZERO()  #####
' ############################
'
'
FUNCTION IsApproxZERO(LONGDOUBLE x)
'return $$TRUE if x is close to zero. Limit can be changed with XmalSetApproxZERO()
'this test works with a FIXED / ABSOLUTE limit

	#FUNC_NUM = 111
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
	'limit checks 
	RETURN (ABS(x) < #APPROX_ZERO_LIMIT##) 
	
END FUNCTION


' ###########################
' #####  IsApproxINF()  #####
' ###########################
'
'
FUNCTION IsApproxINF(LONGDOUBLE x)
'return $$TRUE if x is close to infinity. Limit can be changed with XmalSetApproxINF()
'this test works with a FIXED / ABSOLUTE limit

	#FUNC_NUM = 112
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF	
	
	RETURN (ABS(x) > #APPROX_INFINITY_LIMIT##) 
	
END FUNCTION


' ##########################
' #####  IsApproxEQ()  #####
' ##########################
'
'
FUNCTION IsApproxEQ(LONGDOUBLE x, LONGDOUBLE y)
'return $$TRUE if x and y are Approx equal. Limit can be changed with XmalSetApproxEQ()
'this is a RELATIVE test that adjusts to the magnitude of the values to be compared

	#FUNC_NUM = 113
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF	

	'full precision checks
	IF #APPROX_EQUAL_FRACTION## = 0## THEN RETURN (x = y)

	'if they are exactly equal then return $$TRUE
	IF (x = y) THEN RETURN $$TRUE

	'deal with special cases where x or y are 0
	IF x = 0## THEN RETURN ABS(y) < #APPROX_ZERO_LIMIT##
	IF y = 0## THEN RETURN ABS(x) < #APPROX_ZERO_LIMIT##	

	'this algorithm only works effectively if both x and y are non-zero because magnitude## must be non-zero
	difference## 		=  ABS(x - y)																										'absolute difference between the 2 values
	magnitude## 		= (ABS(x) + ABS(y)) / 2##																				'average absolute magnitude of the values to be compared

	IF difference## < ((#APPROX_EQUAL_FRACTION##) * magnitude##) THEN RETURN $$TRUE		'test (difference / magnitude) < specified fraction ( but * is faster! )

END FUNCTION



' ###########################
' #####  IsApproxLTE()  #####
' ###########################
'
'
FUNCTION IsApproxLTE(LONGDOUBLE x, LONGDOUBLE y)
'return $$TRUE if x is less than or nearly equal to y. Limit can be changed with XmalSetApproxEQ()

	#FUNC_NUM = 114
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
	IF x < y THEN RETURN $$TRUE	
	RETURN IsApproxEQ(x,y)

	
END FUNCTION


' ###########################
' #####  IsApproxGTE()  #####
' ###########################
'
'
FUNCTION IsApproxGTE(LONGDOUBLE x, LONGDOUBLE y)
'return $$TRUE if x is greater than y or nearly equal to y. Limit can be changed with XmalSetApproxEQ()

	#FUNC_NUM = 115
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
	IF x > y THEN RETURN $$TRUE	
	RETURN IsApproxEQ(x,y)
	
END FUNCTION


' ##########################
' #####  IsApproxGT()  #####
' ##########################
'
'
FUNCTION IsApproxGT(LONGDOUBLE x, LONGDOUBLE y)
'return $$TRUE if x is greater than y and not nearly equal to y. Limit can be changed with XmalSetApproxEQ()

	#FUNC_NUM = 116
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
	IF IsApproxEQ(x,y) THEN RETURN $$FALSE
	RETURN x > y
	
END FUNCTION



' ##########################
' #####  IsApproxLT()  #####
' ##########################
'
'
FUNCTION IsApproxLT(LONGDOUBLE x, LONGDOUBLE y)
'return $$TRUE if x is less than y and not nearly equal. Limit can be changed with XmalSetApproxEQ()

	#FUNC_NUM = 117
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
	IF IsApproxEQ(x,y) THEN RETURN $$FALSE
	RETURN x < y
	
END FUNCTION



' #################################
' #####  XmalSetApproxZERO()  #####
' #################################
'
'
FUNCTION XmalSetApproxZERO(limit##)
'set the limit to be used when testing for near 0. This is an absolute value

	#FUNC_NUM = 118 
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF	
	
	#APPROX_ZERO_LIMIT## = ABS(limit##)

END FUNCTION


' ################################
' #####  XmalSetApproxINF()  #####
' ################################
'
'
FUNCTION XmalSetApproxINF(limit##)
'set the limit to be used when testing for Approx infinity. This is an absolute value

	#FUNC_NUM = 119 
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF
	
	#APPROX_INFINITY_LIMIT## = ABS(limit##)
	
END FUNCTION


' ###############################
' #####  XmalSetApproxEQ()  #####
' ###############################
'
'
FUNCTION XmalSetApproxEQ(limitFraction##)
'set the limit to be used when testing for Approx 0. 
'this is a fraction / ratio that is applied to the values to be tested to calculate an absolute value.
'if set to 0, then exact equality is required

	#FUNC_NUM = 120
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF
	
	limitFraction## = ABS(limitFraction##)
	IF limitFraction## > 0.5## THEN RETURN		'keep it sensible
	
	#APPROX_EQUAL_FRACTION## = limitFraction##
	
END FUNCTION


'******************************************************************************************************************
'******************************************************************************************************************
'   Fmod()    Fround()
'******************************************************************************************************************
'******************************************************************************************************************


' #####################
' #####  Fmod ()  #####
' #####################
'
'
FUNCTION LONGDOUBLE Fmod (LONGDOUBLE x, LONGDOUBLE y)
'returns remainder after dividing x by y

	#FUNC_NUM = 129
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
ASM fld    t[Fmod.y]  				; st0 = y
ASM fld    t[Fmod.x]  					; st1 = y, st0 = x

ASM fmod_continue:
ASM		fprem											; partial remainder
ASM		fstsw 	ax 								; store status word in ax
ASM 	fwait											; wait for completion
ASM		sahf  										; copy the condition bits to the CPU flag register
ASM		jp <	fmod_continue				; test the C2 flag - go round again if not complete
ASM	fstp 		st1
ASM	jmp			end.Fmod.xmal				; return with modulus in st(0)

END FUNCTION

' ######################
' #####  Fround()  #####
' ######################
'
'
FUNCTION  LONGDOUBLE Fround (LONGDOUBLE x, p10)
'returns x rounded to specified power of 10

	#FUNC_NUM = 106
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
  pTen## = Power10(p10)
  RETURN (ROUND(x / pTen##)) * pTen##
	
END FUNCTION



'***********************************************************************************************************************
'***********************************************************************************************************************
'    HYPERBOLIC FUNCTIONS       HYPERBOLIC FUNCTIONS
'***********************************************************************************************************************
'***********************************************************************************************************************


' ####################
' #####  Cosh()  #####
' ####################
'
'
FUNCTION LONGDOUBLE Cosh (LONGDOUBLE x)
'returns the hyperbolic cosine of x

'magic number changed from 19 in xma to 25 because of the extra precision required
'e ** 25 roughly equal 10 ** 10. e ** -25 roughly equals 10 ** -10
'range is 10 ** 20+ so the small number has no effect on the large number within the 19 digit accuracy of longdouble

	#FUNC_NUM = 14
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
	
	SELECT CASE TRUE
		CASE (x = 0##)		: RETURN (1##)
		CASE (x >= 25##)	: RETURN (Exp(x) * 0.5##)
		CASE (x <= -25##)	: RETURN (Exp(-x) * 0.5##)
		CASE ELSE					: RETURN ((Exp(x) + Exp(-x)) * 0.5##)
	END SELECT
	
END FUNCTION


' ####################
' #####  Sinh()  #####
' ####################
'
'
FUNCTION LONGDOUBLE Sinh (LONGDOUBLE x)
'returns the hyperbolic sine of x
'magic number changed from 19 in xma to 25 because of the extra precision required
'Exp used instead of Expmo

	#FUNC_NUM = 13
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
	
	SELECT CASE TRUE
		CASE (x = 0##)		: RETURN 0##
		CASE (x >= 25##)  : RETURN Exp(x) * 0.5##
		CASE (x <= -25##)	: RETURN Exp(-x) * -0.5##
		CASE ELSE					: RETURN (Exp(x) - Exp(-x)) * 0.5## 
	END SELECT
	
END FUNCTION


' ####################
' #####  Tanh()  #####
' #################### 
'
'
FUNCTION LONGDOUBLE Tanh (LONGDOUBLE x)
'returns the hyperbolic tangent of x

'magic number changed from 19 in xma to 25 because of the extra precision required
'Exp() used instead of Expmo()

	#FUNC_NUM = 15
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
	
	SELECT CASE TRUE
		CASE (x = 0##)			:	RETURN 0##
		CASE (x >= 25##)		:	RETURN 1##
		CASE (x <= -25##)		:	RETURN -1##
		CASE ELSE	
			ePlusX##  = Exp(x)
			eMinusX## = Exp(-x)
			RETURN (ePlusX## - eMinusX##) / (ePlusX## + eMinusX##) 
	END SELECT
	
END FUNCTION


' ####################
' #####  Sech()  #####
' ####################
'
'
FUNCTION LONGDOUBLE Sech (LONGDOUBLE x)
'returns the hyperbolic secant of x

	#FUNC_NUM = 17
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
	
	RETURN (1## / Cosh(x))		'Cosh(x) cannot be 0
	
END FUNCTION


' #####################
' #####  Csch ()  #####
' #####################
'
'
FUNCTION LONGDOUBLE Csch (LONGDOUBLE x)
'returns the hyperbolic cosecant of x

	#FUNC_NUM = 16
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :

	IF x = 0## THEN RETURN #PINF##
	RETURN (1## / Sinh(x))
	
END FUNCTION


' #####################
' #####  Coth ()  #####
' #####################
'
'
FUNCTION LONGDOUBLE Coth (LONGDOUBLE x)
'returns the hyperbolic cotangent of x
'magic number changed from 19 in xma to 25 because of the extra precision required

	#FUNC_NUM = 18
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
	
	SELECT CASE TRUE
		CASE (x = 0##)					: RETURN #PINF##
    CASE (ABS(x) < 1d-4##)	: RETURN 1##/x + x/3##
		CASE (x >= 25##)				: RETURN 1##
		CASE (x <= -25##)				: RETURN -1##
		CASE ELSE
			ePlusX##  = Exp(x)
			eMinusX## = Exp(-x)
			RETURN (ePlusX## + eMinusX##) / (ePlusX## - eMinusX##)
	END SELECT
	
END FUNCTION

'***********************************************************************************************************************
'***********************************************************************************************************************
'    INVERSE HYPERBOLIC FUNCTIONS       INVERSE HYPERBOLIC FUNCTIONS
'***********************************************************************************************************************
'***********************************************************************************************************************


' #####################
' #####  Asinh()  #####
' #####################
'
FUNCTION LONGDOUBLE Asinh (LONGDOUBLE x)
'returns the hyperbolic arcsine of x
'boundaries unchanged from double version

	#FUNC_NUM = 19
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
	
  SELECT CASE TRUE
    CASE (x < -1d8##)				: RETURN (-Log(-2##*x - 0.5##/x))
    CASE (ABS(x) < 1d-4##)  : RETURN (x*(1## - x*x/6##))
    CASE (x > 1d8##)        : RETURN (Log(2##*x + 0.5##/x))
    CASE ELSE             	:	RETURN (Log(x + Sqrt(x*x + 1##)))
  END SELECT
	
END FUNCTION



' ######################
' #####  Acosh ()  #####
' ######################
'
'
FUNCTION LONGDOUBLE Acosh (LONGDOUBLE x)
'returns the hyperbolic arccosine of x
'boundary changed from 1d20 to 1d10 which gives 19 digit accuracy

	#FUNC_NUM = 20
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
	
  SELECT CASE TRUE
	  CASE x < 1##		: RETURN PqNAN()
    CASE x > 1d10##	: RETURN (Log(2## * x))
		CASE ELSE				: RETURN (Log(x + Sqrt(x*x - 1##)))
  END SELECT
	
END FUNCTION


' ######################
' #####  Atanh ()  #####
' ######################
'
'
FUNCTION LONGDOUBLE Atanh (LONGDOUBLE x)
'returns the hyperbolic arctangent of x
'boundaries unchanged from double version

	#FUNC_NUM = 21
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
	
	SELECT CASE TRUE
		CASE ABS(x) < 1d-5##	: RETURN x * (1## + x*x/3##)
    CASE ABS(x) < 1##     : RETURN Log((1## + x) / (1## - x)) * 0.5##
		CASE x = 1##					: RETURN #PINF##
		CASE x = -1##					: RETURN #NINF##
		CASE ELSE							: RETURN PqNAN()
	END SELECT
	
END FUNCTION


' ######################
' #####  Acsch ()  #####
' ######################
'
'
FUNCTION LONGDOUBLE Acsch (LONGDOUBLE x)
'returns the hyperbolic arccosecant of x

	#FUNC_NUM = 22
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
	
	IF x = 0## THEN
		RETURN PqNAN()
	ELSE
		RETURN Asinh(1## / x)
	ENDIF
	
END FUNCTION


' #####################  
' #####  Asech()  #####
' #####################
'
'
FUNCTION LONGDOUBLE Asech (LONGDOUBLE x)
'returns the hyperbolic arcsecant of x

	#FUNC_NUM = 23
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
	
	SELECT CASE TRUE
		CASE x <= 0##		: RETURN PqNAN()
		CASE x > 1##		: RETURN PqNAN()
		CASE ELSE				: RETURN Acosh(1## / x)
	END SELECT
	
END FUNCTION


' ######################
' #####  Acoth ()  #####
' ######################
'
'
FUNCTION LONGDOUBLE Acoth (LONGDOUBLE x)
'returns the hyperbolic arccotangent of x

	#FUNC_NUM = 24
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
	
	IF ABS(x) > 1## THEN
		RETURN Atanh(1## / x)
	ELSE		
		RETURN PqNAN()
	ENDIF
	
END FUNCTION



'***********************************************************************************************************************
'***********************************************************************************************************************
'    HYPOT FUNCTIONS       HYPOT FUNCTIONS
'***********************************************************************************************************************
'***********************************************************************************************************************


' ######################
' #####  Hypot ()  #####
' ######################
'
'
FUNCTION LONGDOUBLE Hypot (LONGDOUBLE x, LONGDOUBLE y) LONGDOUBLE
'returns Sqrt(x*x + y*y) with lowest possible risk of overflow

	#FUNC_NUM = 50
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
ASM call %_TestArg2isNAN
ASM jnc >
	RETURN y
ASM :
	
	IF x = 0## THEN RETURN y
	IF y = 0## THEN RETURN x
	
	ax = ABS(x)
	ay = ABS(y)
	
	IF ax > ay THEN
		yy = ay / ax 
		RETURN ax * Sqrt(1.0## + (yy * yy))
	ELSE
		xx = ax / ay 
		RETURN ay * Sqrt(1.0## + (xx * xx))		
	ENDIF

END FUNCTION



' #######################
' #####  Hypot3 ()  #####
' #######################
'
'
FUNCTION LONGDOUBLE Hypot3 (LONGDOUBLE x, LONGDOUBLE y, LONGDOUBLE z) LONGDOUBLE
'returns Sqrt(x*x + y*y + z*z) with lowest possible risk of overflow

	#FUNC_NUM = 51
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
ASM call %_TestArg2isNAN
ASM jnc >
	RETURN y
ASM :
ASM call %_TestArg3isNAN
ASM jnc >
	RETURN z
ASM :
	
	IF x = 0 THEN RETURN Hypot(y,z)
	IF y = 0 THEN RETURN Hypot(x,z)
	IF z = 0 THEN RETURN Hypot(x,y)
	
	ax = ABS(x)
	ay = ABS(y)
	az = ABS(z)
	
	max = ax: method = 1
	IF ay > max THEN max = ay: method = 2
	IF az > max THEN max = az: method = 3
	
	SELECT CASE method
		CASE 1
			'max = ax
			yy = ay/max
			zz = az/max
			RETURN ax * Sqrt(1.0## + (yy * yy) + (zz * zz))
		CASE 2
			'max = ay
			xx = ax/max
			zz = az/max
			RETURN ay * Sqrt(1.0## + (xx * xx) + (zz * zz))
		CASE 3
			'max = az
			xx = ax/max
			yy = ay/max
			RETURN az * Sqrt(1.0## + (xx * xx) + (yy * yy))
		CASE ELSE RETURN PqNAN()
	END SELECT 

END FUNCTION




'***********************************************************************************************************************
'***********************************************************************************************************************
'    DETAIL
'***********************************************************************************************************************
'***********************************************************************************************************************


' ########################
' #####  Extract ()  #####
' ########################
'
'
FUNCTION LONGDOUBLE Extract (LONGDOUBLE x, exponent)
'returns mantissa (significand) of x. Sign of mantissa is same as original x
'returns unbiassed (true) exponent of x in exponent
'NOTE mantissa in range 1 <= mantissa < 2 for longdoubles

	#FUNC_NUM = 52
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF

ASM fld 	t[Extract.x]
ASM fxtract
ASM fxch  
ASM fistp	d[Extract.exponent]
ASM jmp > end.Extract.xmal

END FUNCTION

' ##########################
' #####  GetSignBit()  #####
' ##########################
'
'
FUNCTION GetSignBit(LONGDOUBLE x)
'returns actual sign bit. 0 if +ve, 1 if -ve
'work directly with variable on stack

	#FUNC_NUM = 125
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF	
	
ASM xor 	eax, eax
ASM mov 	ax, w[GetSignBit.x + 8]		; get sign and exponent
ASM and 	ax, 8000h									; isolate the sign bit
ASM rol		ax, 1											; move sign from bit 15 to bit 0
ASM jmp 	end.GetSignBit.xmal				; RETURN eax			

END FUNCTION


' ###########################
' #####  GetExponent()  #####
' ###########################
'
'
FUNCTION GetExponent(LONGDOUBLE x)
'returns actual exponent of x including offset
'work directly with variable on stack

	#FUNC_NUM = 126
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
ASM xor 	eax, eax
ASM mov 	ax, w[ebp + 16]						; get sign and exponent
ASM and 	ax, 7FFFh									; clear the sign bit to leave exponent
ASM jmp 	end.GetExponent.xmal			; RETURN eax			

END FUNCTION


' ###############################
' #####  GetTrueExponent()  #####
' ###############################
'
'
FUNCTION GetTrueExponent(LONGDOUBLE x)
'returns true exponent of x without offset
'work directly with variable on stack

	#FUNC_NUM = 127
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
ASM xor 	eax, eax
ASM mov 	ax, w[ebp + 16]						; get sign and exponent
ASM and 	ax, 7FFFh									; clear the sign bit to leave exponent
ASM sub		ax, 3FFFh									; subtract offset 
ASM jmp 	end.GetTrueExponent.xmal	; RETURN eax

END FUNCTION


' ###########################
' #####  GetMantissa()  #####
' ###########################
'
'
FUNCTION LONGDOUBLE GetMantissa(LONGDOUBLE x)
'returns mantissa of x with same sign as x
'modify input value on stack!

	#FUNC_NUM = 128
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
ASM mov 	ax, w[ebp + 16]						; get sign and exponent
ASM and 	ax, 8000h									; clear the exponent but leave sign bit
ASM or		ax, 3FFFh									; set exponent to 0 true
ASM mov 	w[ebp + 16], ax						; modify x
ASM fld		t[GetMantissa.x]					; load x to TOS 
ASM jmp 	end.GetMantissa.xmal			; RETURN mantissa on TOS

END FUNCTION



'***********************************************************************************************************************
'***********************************************************************************************************************
'    HEX DISPLAY FUNCTIONS       HEX DISPLAY FUNCTIONS
'***********************************************************************************************************************
'***********************************************************************************************************************

' ######################
' #####  HexS$ ()  #####
' ######################
'
'
'FUNCTION HexS$ (SINGLE x)
'returns HEX$() style representation of x in big endian format as stored in FPU register
'DO NOT TRACE
	
'	RETURN HEX$(XLONGAT(&x),8) 

'END FUNCTION



' ######################
' #####  HexD$ ()  #####
' ######################
'
'
'FUNCTION HexD$ (DOUBLE x)
'returns Hex$() style representation of x in big endian format as stored in FPU register
'by default, a "-" is added to separate the sign and exponent from the mantissa for clarity
'this formatting can be turned on / off with XmalSetFormatting()

'DO NOT TRACE 

'	address = &x
	
'	IF #FORMAT_HEX THEN
'		out$ = LEFT$(HEX$(USHORTAT(address + 6), 4), 3) + "-" + RIGHT$(HEX$(GIANTAT(address),16),13)
'	ELSE
'		out$ = HEX$(GIANTAT(address), 16)
'	ENDIF  
	
'	RETURN out$
	
'END FUNCTION



' ######################
' #####  HexL$ ()  #####
' ######################
'
'
FUNCTION HexL$ (LONGDOUBLE x)
'returns Hex$() style representation of x in big endian format as stored in FPU register
'by default, a "-" is added to separate the sign and exponent from the mantissa for clarity
'this formatting can be turned on / off with XmalSetFormatting()

	#FUNC_NUM = 130 'DO NOT TRACE 
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF
	
	address = &x

	out$ = HEX$(USHORTAT(address + 8), 4)
	IF #FORMAT_HEX THEN out$ = out$ + "-"
	out$ = out$ + HEX$(GIANTAT(address),16) 
	
	RETURN out$
	
END FUNCTION


' ######################
' #####  BinS$ ()  #####
' ######################
'
'
'FUNCTION BinS$ (SINGLE x)
'returns BIN$() style representation of x in big endian format as stored in FPU register
'sign , exponent and mantissa are separated by "-" for clarity
'this formatting can be turned on / off with XmalSetFormattingL()

'DO NOT TRACE

'	temp$ = BIN$(XLONGAT(&x), 32)
	
'	IFF #FORMAT_HEX THEN
	 
'		RETURN temp$ 
		
'	ELSE
		
'		sign$ = LEFT$(temp$) + "-"
'		exp$  = MID$(temp$, 2, 8) + "-"
'		mant$ = "(1)" + MID$(temp$, 10, 7) + " " + MID$(temp$, 17, 8) + " " + MID$(temp$, 25,32)
		
'	ENDIF

'	RETURN sign$ + exp$ + mant$
	
'END FUNCTION



' ######################
' #####  BinD$ ()  #####
' ######################
'
'
'FUNCTION BinD$ (DOUBLE x)
'returns BIN$() style representation of x in big endian format as stored in FPU register
'sign , exponent and mantissa are separated by "-" for clarity
'this formatting can be turned on / off with XmalSetFormattingL()

'DO NOT TRACE

'	temp$ = BIN$(GIANTAT(&x), 64)
	
'	IFF #FORMAT_HEX THEN
	 
'		RETURN temp$ 
		
'	ELSE
		
'		sign$ = LEFT$(temp$) + "-"
'		exp$  = MID$(temp$, 2, 11) + "-"
'		mant1$ = "(1)" + MID$(temp$, 13, 8) + " " + MID$(temp$, 21, 8) + " " + MID$(temp$, 29,8 ) + " "
'		mant2$ = MID$(temp$, 37, 8) + " " + MID$(temp$, 45, 8) + " " + MID$(temp$, 53,8 ) + " " + RIGHT$(temp$,4)
'	ENDIF

'	RETURN sign$ + exp$ + mant1$ + mant2$
	
'END FUNCTION


' ######################
' #####  BinL$ ()  #####
' ######################
'
'
FUNCTION BinL$ (LONGDOUBLE x)
'returns BIN$() style representation of x in big endian format as stored in FPU register
'sign , exponent and mantissa are separated by "-" for clarity
'each of the 8 bytes of the mantissa are separated by a space
'this formatting can be turned on / off with XmalSetFormattingL()

	#FUNC_NUM = 132
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF
	
	address = &x
	
	IFF #FORMAT_HEX THEN
	 
		RETURN BIN$(USHORTAT(address + 8), 16) + BIN$(GIANTAT(address),64)
		
	ELSE
		
		sexp$ = BIN$(USHORTAT(address + 8), 16)
		sign$ = LEFT$(sexp$) + "-"
		exp$  = RIGHT$(sexp$, 15) + "-"
		mant$ = BIN$(GIANTAT(address),64)
		
		man$ = ""
		FOR i = 0 TO 6 
			man$ = man$ + MID$(mant$, i * 8 + 1, 8) + " "			'space after first 7
		NEXT i
		man$ = man$ + MID$(mant$, 57, 8)										'no space on last one
		
	ENDIF

	RETURN sign$ + exp$ + man$
	
END FUNCTION

'***********************************************************************************************************************
'***********************************************************************************************************************
'    XBLITE INTRINSIC LOOKALIKES FOR LONGDOUBLE equivalents to DMAKE(), DHIGH(), DLOW(), 0dxxx image
'***********************************************************************************************************************
'***********************************************************************************************************************

' ######################
' #####  Lmake ()  #####
' ######################
'
'
FUNCTION LONGDOUBLE Lmake(USHORT sexp, hi, lo)
'allows a longdouble to be built in SMAKE()/DMAKE() style

	#FUNC_NUM = 53
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF	

ASM lea 	esi, temp
ASM mov 	ax, w[Lmake.sexp]
ASM mov 	w[esi + 8], ax
ASM mov 	eax, d[Lmake.hi]
ASM mov 	d[esi + 4], eax
ASM mov 	eax, d[Lmake.lo]
ASM mov 	d[esi], eax
ASM fld 	t[esi]
ASM jmp 	end.Lmake.xmal

END FUNCTION



' ######################
' #####  Lhigh ()  #####
' ######################
'
'
FUNCTION Lhigh(LONGDOUBLE x)
'Returns high 4 bytes of longdouble mantissa
	#FUNC_NUM = 194
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF
	
ASM mov 	eax, d[Lhigh.x + 4]
ASM jmp		> end.Lhigh.xmal
END FUNCTION


' #####################
' #####  Llow ()  #####
' #####################
'
'
FUNCTION Llow(LONGDOUBLE x)
'Returns low 4 bytes of longdouble mantissa
	#FUNC_NUM = 195
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF	
	
ASM mov 	eax, d[Llow.x]
ASM jmp		> end.Llow.xmal
END FUNCTION


' ######################
' #####  Lsexp ()  #####
' ######################
'
'
FUNCTION USHORT Lsexp(LONGDOUBLE x)
'Returns sign and exponent word from longdouble

	#FUNC_NUM = 193
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF
	
ASM movzx 	eax, w[Lsexp.x + 8]
ASM jmp		> end.Lsexp.xmal
END FUNCTION


' #######################
' #####  Limage ()  #####
' #######################
'
'
FUNCTION LONGDOUBLE Limage(longImage$)
'create longdouble from Image. 
'can build any extended format including SPECIALS
'longImage$ is in HEX optionally prefixed with 0x, 0l, 0L and must consist of 20 hexits

	#FUNC_NUM = 55
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF	
	
	IFZ longImage$ THEN RETURN PqNAN()
	len = LEN(longImage$)
	
	SELECT CASE len
		CASE 20:																							'no prefix
		CASE 22:
			first2$ = LEFT$(longImage$, 2)											'extra 2 characters must be a valid prefix
			SELECT CASE first2$
				CASE "0x", "0X", "0l", "0L"
					longImage$ = LCLIP$(longImage$, 2)							'remove prefix
				CASE ELSE RETURN PqNAN()			
			END SELECT
		CASE ELSE RETURN PqNAN()						'string not valid length
	END SELECT
	
	'Validate input string
	longImage$ = UCASE$(longImage$)

	FOR i = 0 TO LEN(longImage$) - 1
		ch = longImage${i}
		IF (ch >= '0') && (ch <= '9') DO NEXT
		IF (ch >= 'A') && (ch <= 'F') DO NEXT
		'not a valid hexit
		RETURN PqNAN()
	NEXT i
	
	se$ = "0x" + LEFT$(longImage$, 4)			' sign and exponent 
	mh$ = "0x" + MID$(longImage$, 5, 8)		' mantissa high 
	ml$ = "0x" + RIGHT$(longImage$, 8)		' mantissa low 
	
	RETURN Lmake(USHORT(se$), XLONG(mh$), XLONG(ml$))

END FUNCTION



' #######################
' #####  Lbuild ()  #####
' #######################
'
'
FUNCTION LONGDOUBLE Lbuild(LONGDOUBLE mantissa, exponent)
'returns mantissa * (2 ** exponent)
'note - mantissa is not constrained on input but will be normalised to 1 <= mantissa < 2 in the longdouble result
EXCEPTION_DATA exception

	#FUNC_NUM = 56
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF	
	
	IF #SAFETY_CHECKS THEN		
		XstTry(SUBADDRESS(Build), SUBADDRESS(BuildExcept), @exception)	
		RETURN result##		
	ELSE		
		RETURN mantissa * Power2(exponent)		
	ENDIF

	SUB Build
		result## = mantissa * Power2(exponent)
ASM fwait										;detect any exception
		EXIT SUB								'no exception, exit with valid result.		 
badReturn:									'if exception occurs, return result## set in BuildExcept
	END SUB
	

	SUB BuildExcept
		'only exception should be FPU OVERFLOW
		exception.address  = GOADDRESS(badReturn)
		exception.response = $$ExceptionRetry
		SELECT CASE exception.code
			CASE $$ExceptionUnderflow
				result## = 0##
			CASE $$ExceptionFloatOverflow
				result## = #PINF##
			CASE ELSE
				result## = PqNAN()
		END SELECT
	END SUB
	
END FUNCTION



' #####################
' #####  Lmax ()  #####
' #####################
'
'
FUNCTION LONGDOUBLE Lmax()
' Make maximum positive normal longdouble value

	#FUNC_NUM = 136
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
ASM fld t[max_longdouble]
ASM jmp end.Lmax.xmal
END FUNCTION



' #####################
' #####  Lmin ()  #####
' #####################
'
'
FUNCTION LONGDOUBLE Lmin()
' Make smallest positive normal longdouble value

	#FUNC_NUM = 135
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF

ASM fld t[min_longdouble]
ASM jmp end.Lmin.xmal
END FUNCTION



' ########################
' #####  LmaxDen ()  #####
' ########################
'
'
FUNCTION LONGDOUBLE LmaxDen()
' Make maximum positive denormal longdouble value

	#FUNC_NUM = 138
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
ASM fld t[max_denormal]
ASM jmp end.LmaxDen.xmal
END FUNCTION



' ########################
' #####  LminDen ()  #####
' ########################
'
'
FUNCTION LONGDOUBLE LminDen()
' Make smallest positive denormal longdouble value

	#FUNC_NUM = 137
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
ASM fld t[min_denormal]
ASM jmp end.LminDen.xmal
END FUNCTION



'*********************************************************************************************************************************
'*********************************************************************************************************************************
'  C COMPATIBILITY FUNCTIONS           C COMPATIBILITY FUNCTIONS
'*********************************************************************************************************************************
'*********************************************************************************************************************************

' #######################
' #####  Ldexp ()  ######
' #######################
'

FUNCTION LONGDOUBLE Ldexp(LONGDOUBLE x, exponent)
'mimics C function ldexpl()
'returns x * (2 ** exponent)
'note that x can be any number - it is not constrained to be within the range of a normalised mantissa

	#FUNC_NUM = 57
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
	RETURN x * Power2(exponent)
	
END FUNCTION



' ######################
' #####  Frexp ()  #####
' ######################
'
'
FUNCTION LONGDOUBLE Frexp(LONGDOUBLE x, exponent)
'mimics C function frexpl()
'extract the mantissa and exponent such that x = mantissa * (2 ** exponent)
'the mantissa is returned as the function value and the exponent in the input variable exponent
'mantissa is normalised so that that 0.5 <= mantissa < 1
 
	#FUNC_NUM = 58
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
	mantissa## = Extract (x, @exponent)						'get mantissa## and exponent with 1 <= mantissa## < 2 - i.e LONGDOUBLE format	
	INC exponent																	'rebase to 0.5 <= mantissa## < 1 as defined for this C function
	RETURN mantissa## / 2

END FUNCTION




' #####################
' #####  Modf ()  #####
' #####################
'
' 
FUNCTION LONGDOUBLE Modf(LONGDOUBLE x, LONGDOUBLE integer)
'mimics C function modf()
'returns fractional part of x and puts integer part in integer
'each part has same sign as x

	#FUNC_NUM = 59
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
	integer = FIX(x)
	RETURN x - integer

END FUNCTION


'
' ########################
' #####  CopySign () #####
' ########################
'
'
FUNCTION LONGDOUBLE CopySign (LONGDOUBLE x, LONGDOUBLE y)
'Return x with the sign of y - mimics C function copysign

	#FUNC_NUM = 72
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
ASM and 	w[ebp + 16], 7FFFh				; clear the sign bit of x
ASM mov 	bx, w[ebp + 28]						; get sign and exponent of y
ASM and 	bx, 8000h									; isolate just the sign bit from y
ASM or		w[ebp + 16], bx						; mask sign from y into x
ASM fld		t[CopySign.x]							; load modified x onto FPU TOS
ASM jmp   end.CopySign.xmal					; RETURN

END FUNCTION



'*********************************************************************************************************************************
'*********************************************************************************************************************************
'  CONVERSIONS      CONVERSIONS
'*********************************************************************************************************************************
'*********************************************************************************************************************************

' ###############################
' #####  SafeLDtoSingle ()  #####
' ###############################
'
' 
FUNCTION SINGLE SafeLDtoSingle(LONGDOUBLE x, errFlag)
'If x is in range of SINGLE then it is returned as a single and errFlag is set to $$FALSE
'Otherwise, the function returns NAN and errFlag is set to $$TRUE

	#FUNC_NUM = 60
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
	IF x < 0## THEN negative = $$TRUE
	errFlag = 0
	
ASM fld 	t[SafeLDtoSingle.x]
ASM fabs																; work with |x|
ASM fld		d[max_single]
ASM fcomip
ASM jb > 	bad_value_S										; too big
ASM fld		d[min_single]
ASM fcomip													
ASM ja > 	bad_value_S										; too small

'ok to convert
	IF negative THEN				
ASM fchs																; change sign back to -ve if necessary
	ENDIF
ASM jmp		end.SafeLDtoSingle.xmal				; RETURN x already on TOS

ASM bad_value_S:
	errFlag = -1													' signal ERROR
ASM fld		t[pnan]
ASM fstp  st1														; replace x with nan on TOS and tidy stack 
ASM jmp		end.SafeLDtoSingle.xmal				; RETURN NAN
END FUNCTION



' ###############################
' #####  SafeLDtoDouble ()  #####
' ###############################
'
' 
FUNCTION DOUBLE SafeLDtoDouble(LONGDOUBLE x, errFlag)
'If x is in range of DOUBLE then it is returned as a double and errFlag is set to $$FALSE
'Otherwise, the function returns NAN and errFlag is set to $$TRUE

	#FUNC_NUM = 61
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
	IF x < 0## THEN negative = $$TRUE
	errFlag = 0
	
ASM fld 	t[SafeLDtoDouble.x]
ASM fabs																; work with |x|
ASM fld		q[max_double]
ASM fcomip
ASM jb > 	bad_value_D										; too big
ASM fld	 	q[min_double]
ASM fcomip
ASM ja > 	bad_value_D										; too small

'ok to convert
	IF negative THEN				
ASM fchs																; change sign back to -ve if necessary
	ENDIF
ASM jmp		end.SafeLDtoDouble.xmal				; RETURN x already on TOS

ASM bad_value_D:
	errFlag = -1													' signal ERROR
ASM fld		t[pnan]
ASM fstp  st1														; replace x with nan on TOS and tidy stack 
ASM jmp		end.SafeLDtoDouble.xmal				; RETURN NAN

END FUNCTION


' ##############################
' #####  SafeLDConvert ()  #####
' ##############################
'
' 
FUNCTION SafeLDConvert(LONGDOUBLE x, lpVariable, varType)
'If x is in range of requested type, convert it and update variable at lpVariable, RETURN $$FALSE
'Otherwise, store 0 at lpVariable and RETURN $$TRUE

	#FUNC_NUM = 62
	IF #TRACE2 THEN
ASM call %_XmalTrace
	ENDIF
	
	IFF IsValid(x) THEN
		'NAN,INF etc have no equivalent for integers
		SELECT CASE varType
			CASE $$SINGLE, $$DOUBLE
				'OK carry on below
			CASE $$GIANT
				GOTO badValue8
			CASE ELSE
				GOTO badValue4
		END SELECT
	ENDIF
	
	IF varType < $$SINGLE THEN x = ROUND(x)			'integer types only need to be rounded off
	
	SELECT CASE varType
		'test for most common / biggest types first
		CASE $$DOUBLE
			IF ABS(x) >  1.79769313486231d+308## THEN GOTO badValue8
			IF ABS(x) <  2.2250738585073d-308##  THEN GOTO badValue8
			'convert via FPU and update the variable 
ASM fld		t[SafeLDConvert.x]
ASM mov 	esi, d[SafeLDConvert.lpVariable]
ASM fstp 	q[esi]
ASM fwait																	;generate exception if anything wrong - should never happen!
			RETURN
			
		CASE $$SINGLE 
			IF ABS(x) > 3.402823d38## THEN GOTO badValue4
			IF ABS(x) < 1.175494d-38## THEN GOTO badValue4
			'convert via FPU and update the variable
ASM fld		t[SafeLDConvert.x]
ASM mov 	esi, d[SafeLDConvert.lpVariable]
ASM fstp  d[esi]
ASM fwait																	;generate exception if anything wrong - should never happen!
			RETURN
				
		CASE $$SLONG, $$XLONG
			IF x < -2.147483648d+9## THEN GOTO badValue4
			IF x >  2.147483647d+9## THEN GOTO badValue4  
			
		CASE $$GIANT
			IF x > 	9.223372036854775805d+18## THEN GOTO badValue8
			IF x < -9.223372036854775806d+18## THEN GOTO badValue8
			'convert via FPU and update the variable
ASM fld		t[SafeLDConvert.x]
ASM mov 	esi, d[SafeLDConvert.lpVariable]
ASM fistp q[esi]
ASM fwait																	;generate exception if anything wrong - should never happen!

			RETURN

		CASE $$ULONG
			IF x < 0##				 			THEN GOTO badValue4
			IF x > 4.294967295d+9## THEN GOTO badValue4
			'convert via FPU and update the variable 
ASM fld		t[SafeLDConvert.x]
ASM mov 	esi, d[SafeLDConvert.lpVariable]
ASM fistp q[temp]											; need to go via giant because fistp d[] always works in signed arithmetic as XLONG / SLONG
ASM fwait															; generate exception if anything wrong - should never happen!
ASM mov 	eax, d[temp]								; just use bottom half of giant
ASM mov		d[esi], eax
 
			RETURN
			
		CASE $$USHORT
			IF x < 0## 			THEN GOTO badValue4
			IF x > 65535##	THEN GOTO badValue4  
			
		CASE $$SSHORT
			IF x < -32768## THEN GOTO badValue4
			IF x >  32767##	THEN GOTO badValue4  
			
		CASE $$UBYTE
			IF x < 0## 		THEN GOTO badValue4
			IF x > 255## 	THEN GOTO badValue4  
			
		CASE $$SBYTE
			IF x < -128## THEN GOTO badValue4
			IF x >  127## THEN GOTO badValue4
			
		CASE ELSE
			'invalid type, do nothing to variable but return error  
			RETURN $$TRUE
	END SELECT

'Valid SLONG, XLONG, USHORT, SSHORT, UBYTE, SBYTE all drop through to here
'These are actually stored as 4 bytes so fistp d[esi] works for all of them
'This will not work for xSHORT or xBYTE ARRAY elements!	
ASM fld		t[SafeLDConvert.x]
ASM mov 	esi, d[SafeLDConvert.lpVariable]
ASM fistp d[esi]
ASM fwait																;generate exception if anything wrong - should never happen!
	RETURN

'Invalid SINGLE, SLONG, XLONG, ULONG, USHORT, SSHORT, UBYTE, SBYTE all end up here
badValue4:
ASM mov 	esi, d[SafeLDConvert.lpVariable]
ASM mov 	d[esi], 0
	RETURN $$TRUE

'Invalid GIANT, DOUBLE end up here
badValue8:
ASM mov 	esi, d[SafeLDConvert.lpVariable]
ASM mov 	d[esi], 0
ASM mov		d[esi + 4], 0  
	RETURN $$TRUE

END FUNCTION


'*********************************************************************************************************************************
'*********************************************************************************************************************************
'  ARITHMETIC AND GEOMETRIC PROGRESSIONS           ARITHMETIC AND GEOMETRIC PROGRESSIONS 
'*********************************************************************************************************************************
'*********************************************************************************************************************************

' ###################### 
' #####  SumAP ()  #####
' ###################### 
'
'
FUNCTION LONGDOUBLE SumAP(LONGDOUBLE a, LONGDOUBLE d, n1, n2)
'return sum of the arithmetic progression a, a + d, a + 2d .... from term n1 to term n2 (1 based)
'a and d do not need Validity check

	#FUNC_NUM = 45
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
	IF n1 < 1 THEN RETURN PqNAN()
	IF n2 < 0 THEN RETURN PqNAN()
	
	IF (n2 = 0) || (n1 = n2) THEN RETURN a + (n1 - 1) * d
	IF n2 < n1 THEN SWAP n1, n2
	
	IF n1 = 1 THEN
		RETURN (a + ((n2 - 1) * d / 2##)) * n2
	ELSE
		RETURN (a + (((n1 + n2 - 2) * d / 2##))) * (n2 - n1 + 1)
	ENDIF 

END FUNCTION


' ####################### 
' #####  ProdAP ()  #####
' ####################### 
'
'
FUNCTION LONGDOUBLE ProdAP(LONGDOUBLE a, LONGDOUBLE d, n1, n2)
'returns product of arithmetic series from term n1 to term n2 (1 based)
'a and d do not need Validity check

EXCEPTION_DATA exception

	#FUNC_NUM = 46
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
	IF n1 < 1 THEN RETURN PqNAN()
	IF n2 < 0 THEN RETURN PqNAN()
	
	IF (n2 = 0) || (n1 = n2) THEN RETURN a + (n1 - 1) * d
	IF n2 < n1 THEN SWAP n1, n2
	
	result## = 1##
	factor## = a + (n1 - 1) * d
	
	IF #SAFETY_CHECKS THEN
		XstTry(SUBADDRESS(evaluate), SUBADDRESS(ProdAPExcept), @exception)
	ELSE
		GOSUB evaluate
	ENDIF
	
	RETURN result##
	
	SUB evaluate
		FOR i = 1 TO n2 - n1 + 1
			result## = result## * factor##
			factor## = factor## + d
ASM 	fwait												;check for exceptions each loop
		NEXT i
		EXIT SUB									

badReturn:												'if exception occurs, return INF(+1)
	result## = #PINF##
	END SUB
	
	SUB ProdAPExcept
		'only exception will be FPU OVERFLOW
		exception.address  = GOADDRESS(badReturn)
		exception.response = $$ExceptionRetry
	END SUB

END FUNCTION


' ######################## 
' #####  ProdAP2 ()  #####
' ######################## 
'
'
FUNCTION LONGDOUBLE ProdAP2(LONGDOUBLE f, LONGDOUBLE l, n)
'returns product of series a, a + d, a + 2d .... for n terms
'starting with term of value f(irst) = a + xd and ending with term of value l(ast) = a + (x + n - 1)d over total n terms

	#FUNC_NUM = 47
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
	IF n < 2 THEN RETURN PqNAN()
	
	d## = (l - f) / LONGDOUBLE(n - 1)
	fraction## = f / d##
	
	'If fraction## is negative whole number, reverse the sequence otherwise Gamma() will not work
	IF ((fraction## < 0##) && (Fmod(fraction##, 1##) = 0##)) THEN
		SWAP f, l
		d## = - d##
		fraction## = f / d##
	ENDIF
	
	RETURN Power(d##, n) * (Gamma(fraction## + n) / Gamma(fraction##))

END FUNCTION



' ###################### 
' #####  SumGP ()  #####
' ###################### 
'
'
FUNCTION LONGDOUBLE SumGP(LONGDOUBLE a, LONGDOUBLE r, n1, n2)
'return sum of the geometric progression a, a*r, a*(r**2) .... from term n1 to term n2 (-1 means to infinity) (1 based)

	#FUNC_NUM = 48
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
	IF n1 < 1 THEN RETURN PqNAN()			'No such term
	IF r = 0## THEN RETURN PqNAN()		'Not a GP
	
	'n2 checks - can be -1, 0 or more
	SELECT CASE TRUE
		CASE (n2 < -1)
			RETURN PqNAN()
			
		CASE (n2 = -1)
			'Sum to infinity, must have suitable r to converge
			IF ABS(r) >= 1## THEN RETURN PqNAN()
			
		CASE (n2 = 0) || (n2 = n1)
			'Return n1th term = a * (r ** (n1-1))
			RETURN a * Power(r, n1 - 1)
		
		CASE n2 < n1
			'n2 is 1 or more but <> n1
			'don't allow SWAP because n2 special in allowing -1 and 0 but n1 doesn't
			RETURN PqNAN()
			
	END SELECT

	IF n2 = -1 THEN
		'r already checked for convergence
		IF n1 = 1 THEN
			'Sum 1 to infinty
			RETURN a / (1## - r)
		ELSE
			'Sum n1 to Infinity
			RETURN (a * Power(r, n1 - 1)) / (1## - r)
		ENDIF
		
	ELSE
		'r <> 0 otherwise no restriction
		IF n1 = 1 THEN
			'Sum 1 to n2
			IF r > 1## THEN
				RETURN a * (Power(r, n2) - 1##) / (r - 1##)
			ELSE
				RETURN a * (1## - Power(r, n2)) / (1## - r)
			ENDIF
			
		ELSE
			'Sum n1 to n2
			IF r > 1## THEN
				RETURN (a * Power(r, n1 - 1)) * (Power(r, n2 - n1 + 1) - 1##) / (r - 1##)
			ELSE
				RETURN (a * Power(r, n1 - 1)) * (1## - Power(r, n2 - n1 + 1)) / (1## - r)
			ENDIF
			
		ENDIF
		
	ENDIF	

END FUNCTION



' ####################### 
' #####  ProdGP ()  #####
' ####################### 
'
'
FUNCTION LONGDOUBLE ProdGP(LONGDOUBLE a, LONGDOUBLE r, n1, n2)
'returns product of term in series a, a*(r), a*(r**2) ... between terms n1 and n2 (1 based)
EXCEPTION_DATA exception

	#FUNC_NUM = 49
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
	IF n1 < 1 THEN RETURN PqNAN()						
	IF n2 < 0 THEN RETURN PqNAN()

	IF r = 0## THEN RETURN PqNAN()		'Not a GP
	IF n2 < n1 THEN SWAP n1, n2
	
	IF (n2 = 0) || (n1 = n2) THEN RETURN a * Power(r, n1 - 1)

	pa = n2 - n1 + 1																			'power to raise a by
	pr = ROUND(SumAP(n1 - 1, 1##, 1, n2 - n1 + 1))				'power to raise r by
	
	
	IF #SAFETY_CHECKS THEN
		XstTry(SUBADDRESS(evaluate), SUBADDRESS(ProdGPExcept), @exception)
	ELSE
		GOSUB evaluate
	ENDIF
	
	RETURN result##
	
	SUB evaluate
		result## = Power(a, pa) * Power(r, pr)
ASM fwait										;check for exceptions 
		EXIT SUB		
badReturn:									'if exception occurs, return INF(+1)
	result## = #PINF##
	END SUB
	
	SUB ProdGPExcept
		'only exception will be FPU OVERFLOW
		exception.address  = GOADDRESS(badReturn)
		exception.response = $$ExceptionRetry
	END SUB

END FUNCTION



'*******************************************************************************************************************
'*******************************************************************************************************************
'  SPECIAL / ADVANCED FUNCTIONS
'*******************************************************************************************************************
'*******************************************************************************************************************


' ######################
' #####  Gamma ()  #####
' ######################
'
'
'
FUNCTION LONGDOUBLE Gamma (LONGDOUBLE x) LONGDOUBLE
'Returns Gamma(x)
'Converted from FORTRAN version on netlib.org/specfun/gamma
'all variables are LONGDOUBLE by default except:
XLONG parity, i, n, Paddress, Qaddress, Caddress

	#FUNC_NUM = 63
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :

'Set up pointers to coefficient tables
ASM mov d[Gamma.Paddress], ADDR gammaP
ASM mov d[Gamma.Qaddress], ADDR gammaQ
ASM mov d[Gamma.Caddress], ADDR gammaC

	parity = $$FALSE
	fact = 1
	n = 0
	y = x

	IF y <= 0 THEN
		y = -x
		y1 = FIX(y)
		res = y - y1
		IF res <> 0 THEN
			IF (y1 <> (FIX(y1 * 0.5##) * 2)) THEN parity = $$TRUE
			fact = -$$PI / Sin($$PI * res)
			y = y + 1##
		ELSE
			RETURN #PINF##
		ENDIF
	ENDIF 
	
	IF y <= $$EPSILON THEN
		
		IF y >= $$XMININ THEN
			RETURN 1## / y
		ELSE
			RETURN #PINF##
		ENDIF
		
	ELSE
		
		IF y < 12## THEN
			
			y1 = y
			IF y < 1## THEN
				z = y
				y = y + 1##
			ELSE
				n = INT(y) - 1
				y = y - LONGDOUBLE(n)
				z = y - 1##
			ENDIF
			
			xnum = 0##
			xden = 1##
			
			FOR i = 0 TO 7
				xnum = (xnum + LONGDOUBLEAT(Paddress + i * 10)) * z
				xden = (xden * z) + LONGDOUBLEAT(Qaddress + i * 10)
			NEXT i
			
			res = xnum / xden + 1##
			
			IF (y1 < y) THEN
				res = res / y1
			ELSE				
				FOR i = 1 TO n
					res = res * y
					y = y + 1##
				NEXT i
			ENDIF
			
		ELSE
			
			IF y <= 1755 THEN
				ysq = y * y
				sum = LONGDOUBLEAT(Caddress + 60)
				
				FOR i = 0 TO 5
					sum = sum / ysq + LONGDOUBLEAT(Caddress + i * 10)
				NEXT i
				
				sum = sum / y - y + $$LNSQRT2PI
				sum = sum + (y - 0.5##) * Log(y)
				res = Exp(sum)
			ELSE
				RETURN #PINF##
			ENDIF
			
		ENDIF
		
	ENDIF
	
	IF parity THEN res = -res
	IF fact <> 1## THEN res = fact / res
	
	RETURN res
	
	
ASM .data
ASM align 8
ASM gammaP:
ASM DT -1.71618513886549492533811e0,  2.47656508055759199108314e1
ASM DT -3.79804256470945635097577e2,  6.29331155312818442661052e2
ASM DT  8.66966202790413211295064e2, -3.14512729688483675254357e4
ASM DT -3.61444134186911729807069e4,  6.64561438202405440627855e4
ASM gammaQ:  
ASM DT -3.08402300119738975254353e1,  3.15350626979604161529144e2
ASM DT -1.01515636749021914166146e3, -3.10777167157231109440444e3
ASM DT  2.25381184209801510330112e4,  4.75584627752788110767815e3
ASM DT -1.34659959864969306392456e5, -1.15132259675553483497211e5
ASM gammaC:
ASM DT -1.910444077728e-03 
ASM DT  8.4171387781295e-04
ASM DT -5.952379913043012e-04
ASM DT  7.93650793500350248e-04
ASM DT -2.777777777777681622553e-03
ASM DT  8.333333333333333331554247e-02
ASM DT  5.7083835261e-03 
ASM .code

END FUNCTION




'
' #########################
' #####  LogGamma ()  #####
' #########################
'
'
'
FUNCTION LONGDOUBLE LogGamma (LONGDOUBLE x) LONGDOUBLE
'Returns Ln(Gamma(x)) without intermediate calculation of Gamma(x) itself. This allows for very large values of x
'Converted from FORTRAN version on netlib.org/specfun/algama
'Precision 18 to 19 digits

'all variables are LONGDOUBLE by default except:
XLONG D1address, P1address, Q1address, D2address, P2address, Q2address, D4address, P4address, Q4address, Caddress
XLONG i

	#FUNC_NUM = 64
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
	
'Set up pointers to coefficient tables
ASM mov d[LogGamma.D1address], ADDR LogGammaD1
ASM mov d[LogGamma.P1address], ADDR LogGammaP1
ASM mov d[LogGamma.Q1address], ADDR LogGammaQ1
ASM mov d[LogGamma.D2address], ADDR LogGammaD2
ASM mov d[LogGamma.P2address], ADDR LogGammaP2
ASM mov d[LogGamma.Q2address], ADDR LogGammaQ2
ASM mov d[LogGamma.D4address], ADDR LogGammaD4
ASM mov d[LogGamma.P4address], ADDR LogGammaP4
ASM mov d[LogGamma.Q4address], ADDR LogGammaQ4
ASM mov d[LogGamma.Caddress],  ADDR LogGammaC

	y = x
	IF (y > 0) && (y <= $$LOGGAMMAMAX) THEN
		IF y <= $$EPSILON THEN 
			res = -Log(y)
		ELSE
			IF y <= 1.5## THEN
				IF (y < $$PNT68) THEN
					corr = -Log(y)
					xm1 = y 
				ELSE
					corr = 0##
					xm1 = (y - 0.5##) - 0.5##
				ENDIF
				
				IF ((y <= 0.5##) || (y >= $$PNT68)) THEN
					xden = 1##
					xnum = 0##
					FOR i = 0 TO 7
						xnum = xnum * xm1 + LONGDOUBLEAT(P1address + i * 10)
						xden = xden * xm1 + LONGDOUBLEAT(Q1address + i * 10)
					NEXT i
					res = corr + (xm1 * (LONGDOUBLEAT(D1address) + xm1 * (xnum / xden)))
				ELSE
					xm2 = (y - 0.5##) - 0.5##
					xden = 1##
					xnum = 0##
					FOR i = 0 TO 7
						xnum = xnum * xm2 + LONGDOUBLEAT(P2address + i * 10)
						xden = xden * xm2 + LONGDOUBLEAT(Q2address + i * 10)
					NEXT i
					res = corr + xm2 * (LONGDOUBLEAT(D2address) + xm2 * (xnum / xden))
				ENDIF
				
			ELSE
				IF (y <= 4##) THEN	
					xm2 = y - 2##
					xden = 1##
					xnum = 0##
					FOR i = 0 TO 7
						xnum = xnum * xm2 + LONGDOUBLEAT(P2address + i * 10)
						xden = xden * xm2 + LONGDOUBLEAT(Q2address + i * 10)
					NEXT i
					res = xm2 * (LONGDOUBLEAT(D2address) + xm2 * (xnum / xden))
				ELSE
					IF (y <= 12##) THEN
						xm4 = y - 4##
						xden = -1##
						xnum = 0##
						FOR i = 0 TO 7
							xnum = xnum * xm4 + LONGDOUBLEAT(P4address + i * 10)
							xden = xden * xm4 + LONGDOUBLEAT(Q4address + i * 10)
						NEXT i
						res = LONGDOUBLEAT(D4address) + xm4 * (xnum / xden) 
					ELSE	
						res = 0##
						IF (y <= $$FRTLNGMAX) THEN
							res = LONGDOUBLEAT(Caddress + 60)
							ysq = y * y
							FOR i = 0 TO 5
								res = res / ysq + LONGDOUBLEAT(Caddress + i * 10)
							NEXT i
						ENDIF
						res = res / y
						corr = Log(y)
						res = res + $$LNSQRT2PI - (0.5## * corr)
						res = res + y * (corr - 1##)
					ENDIF
				ENDIF	
			ENDIF
		ENDIF
		RETURN res			
	ELSE
		IF y <= 0 THEN
			RETURN PqNAN()
		ELSE
			RETURN #PINF##
		ENDIF
	ENDIF
	

ASM .data
ASM align 8
ASM LogGammaD1:
ASM DT -5.772156649015328605195174e-1
ASM LogGammaP1:  
ASM DT  4.945235359296727046734888e0, 2.018112620856775083915565e2
ASM DT  2.290838373831346393026739e3, 1.131967205903380828685045e4
ASM DT  2.855724635671635335736389e4, 3.848496228443793359990269e4
ASM DT  2.637748787624195437963534e4, 7.225813979700288197698961e3
ASM LogGammaQ1:
ASM DT  6.748212550303777196073036e1, 1.113332393857199323513008e3
ASM DT  7.738757056935398733233834e3, 2.763987074403340708898585e4
ASM DT  5.499310206226157329794414e4, 6.161122180066002127833352e4
ASM DT  3.635127591501940507276287e4, 8.785536302431013170870835e3 
ASM LogGammaD2:
ASM DT  4.227843350984671393993777e-1 
ASM LogGammaP2:  
ASM DT  4.974607845568932035012064e0, 5.424138599891070494101986e2 
ASM DT  1.550693864978364947665077e4, 1.847932904445632425417223e5 
ASM DT  1.088204769468828767498470e6, 3.338152967987029735917223e6 
ASM DT  5.106661678927352456275255e6, 3.074109054850539556250927e6
ASM LogGammaQ2:
ASM DT  1.830328399370592604055942e2, 7.765049321445005871323047e3
ASM DT  1.331903827966074194402448e5, 1.136705821321969608938755e6
ASM DT  5.267964117437946917577538e6, 1.346701454311101692290052e7
ASM DT  1.782736530353274213975932e7, 9.533095591844353613395747e6
ASM LogGammaD4:
ASM DT  1.791759469228055000094023e0
ASM LogGammaP4:  
ASM DT  1.474502166059939948905062e4,  2.426813369486704502836312e6
ASM DT  1.214755574045093227939592e8,  2.663432449630976949898078e9
ASM DT  2.940378956634553899906876e10, 1.702665737765398868392998e11
ASM DT  4.926125793377430887588120e11, 5.606251856223951465078242e11
ASM LogGammaQ4:
ASM DT  2.690530175870899333379843e3,  6.393885654300092398984238e5
ASM DT  4.135599930241388052042842e7,  1.120872109616147941376570e9
ASM DT  1.488613728678813811542398e10, 1.016803586272438228077304e11
ASM DT  3.417476345507377132798597e11, 4.463158187419713286462081e11
ASM LogGammaC:
ASM DT -1.910444077728e-03
ASM DT  8.4171387781295e-04
ASM DT -5.952379913043012e-04
ASM DT  7.93650793500350248e-04
ASM DT -2.777777777777681622553e-03
ASM DT  8.333333333333333331554247e-02 
ASM DT  5.7083835261e-03 
ASM .code
END FUNCTION


'
' #############################
' #####  LogFactorial ()  #####
' #############################
'
'
'
FUNCTION LONGDOUBLE LogFactorial(GIANT n)
'returns natural log of Factorial(n)

	#FUNC_NUM = 65
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
	
	SELECT CASE TRUE
		CASE n < 0								: RETURN NqNAN()
		CASE n = 0								: RETURN 0##
		CASE n = 1								: RETURN 0##
		CASE n <= $$MAXFACTORIAL	: RETURN Log(Factorial(n))
		CASE ELSE									: RETURN LogGamma(LONGDOUBLE(n) + 1##)
	END SELECT	

END FUNCTION


'
' #####################
' #####  Beta ()  #####
' #####################
'
'
'
FUNCTION LONGDOUBLE Beta(LONGDOUBLE x, LONGDOUBLE y)
'Returns Beta(x,y)

	#FUNC_NUM = 66
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
ASM call %_TestArg2isNAN
ASM jnc >
	RETURN y
ASM :
	
	IF (x <= 0##) || (y <= 0##) THEN
		RETURN PqNAN()
	ELSE
		RETURN Exp(LogGamma(x) + LogGamma(y) - LogGamma( x + y))
	ENDIF
	
END FUNCTION

'
' ########################
' #####  LogBeta ()  #####
' ########################
'
'
'
FUNCTION LONGDOUBLE LogBeta(LONGDOUBLE x, LONGDOUBLE y)
'Returns LogBeta(x,y)

	#FUNC_NUM = 66
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
ASM call %_TestArg2isNAN
ASM jnc >
	RETURN y
ASM :
	
	IF (x <= 0##) || (y <= 0##) THEN
		RETURN PqNAN()
	ELSE
		RETURN LogGamma(x) + LogGamma(y) - LogGamma( x + y)
	ENDIF
	
END FUNCTION


'
' ####################
' #####  Erf ()  #####
' ####################
'
'
'
FUNCTION LONGDOUBLE Erf(LONGDOUBLE x) LONGDOUBLE
'returns value of Error Function for x. erf(x)
'based on FORTRAN routine at netlib.org/specfun
XLONG i, erfAaddr, erfBaddr, erfCaddr, erfDaddr, erfPaddr, erfQaddr

	#FUNC_NUM = 67
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :

'Set up pointers to coefficient tables
ASM mov d[Erf.erfAaddr], ADDR erfA
ASM mov d[Erf.erfBaddr], ADDR erfB
ASM mov d[Erf.erfCaddr], ADDR erfC
ASM mov d[Erf.erfDaddr], ADDR erfD
ASM mov d[Erf.erfPaddr], ADDR erfP
ASM mov d[Erf.erfQaddr], ADDR erfQ

	y = ABS(x)
	IF y <= 0.46875## THEN
		'evaluate erf
		ysq = 0##
		IF y > #EPSILON## THEN ysq = y * y
		xnum = LONGDOUBLEAT(erfAaddr + 40) * ysq
		xden = ysq
		FOR i = 0 TO 2
			xnum = (xnum + LONGDOUBLEAT(erfAaddr + i * 10)) * ysq
			xden = (xden + LONGDOUBLEAT(erfBaddr + i * 10)) * ysq
		NEXT i
		result = x * (xnum + LONGDOUBLEAT(erfAaddr + 30)) / (xden + LONGDOUBLEAT(erfBaddr + 30))
		RETURN result
		
	ELSE
		IF y <= 4## THEN
			'evaluate erfc
			xnum = y * LONGDOUBLEAT(erfCaddr + 80)
			xden = y
			FOR i = 0 TO 6
				xnum = (xnum + LONGDOUBLEAT(erfCaddr + i * 10)) * y
				xden = (xden + LONGDOUBLEAT(erfDaddr + i * 10)) * y
			NEXT i
			result = (xnum + LONGDOUBLEAT(erfCaddr + 70)) / (xden + LONGDOUBLEAT(erfDaddr + 70))
			ysq = FIX(y * 16##) / 16##
			del = (y - ysq) * (y + ysq)
			result = Exp(-ysq * ysq) * Exp(-del) * result

			
		ELSE
			'evaluate erfc for |x| > 4. XBIG = 100. XINF = 3 * 10**4932. XMAX = 1 * 10**4932
			result = 0##
			IF y >= 100## THEN GOTO fixup
			ysq = 1## / (y * y)
			xnum = LONGDOUBLEAT(erfPaddr + 50) * ysq 
			xden = ysq
			FOR i = 0 TO 3
				xnum = (xnum + LONGDOUBLEAT(erfPaddr + i * 10)) * ysq
				xden = (xden + LONGDOUBLEAT(erfQaddr + i * 10)) * ysq
			NEXT i
			result = ysq * (xnum + LONGDOUBLEAT(erfPaddr + 40)) / (xden + LONGDOUBLEAT(erfQaddr + 40))
			result = ($$1DIVSQRTPI - result) / y
			ysq = FIX(y * 16##) / 16##
			del = (y - ysq) * (y + ysq)
			result = Exp(-ysq * ysq) * Exp(-del) * result
			
		ENDIF
		
		
	ENDIF
	
fixup:
	result = (0.5## - result) + 0.5##
	IF x < 0## THEN result = - result
	RETURN result

	
ASM .data
'  Coefficients for approximation to  erf  in first interval
ASM erfA:
ASM DT 3.16112374387056560e00, 1.13864154151050156e02
ASM DT 3.77485237685302021e02, 3.20937758913846947e03, 1.85777706184603153e-1
ASM erfB:
ASM DT 2.36012909523441209e01, 2.44024637934444173e02, 1.28261652607737228e03, 2.84423683343917062e03

'  Coefficients for approximation to  erfc  in second interval
ASM erfC:
ASM DT 5.64188496988670089e-1
ASM DT 8.88314979438837594e0, 6.61191906371416295e01, 2.98635138197400131e02
ASM DT 8.81952221241769090e02, 1.71204761263407058e03, 2.05107837782607147e03, 1.23033935479799725e03, 2.15311535474403846e-8
ASM erfD:
ASM DT 1.57449261107098347e01, 1.17693950891312499e02, 5.37181101862009858e02, 1.62138957456669019e03
ASM DT 3.29079923573345963e03, 4.36261909014324716e03, 3.43936767414372164e03, 1.23033935480374942e03

'  Coefficients for approximation to  erfc  in third interval
ASM erfP:
ASM DT 3.05326634961232344e-1
ASM DT 3.60344899949804439e-1
ASM DT 1.25781726111229246e-1
ASM DT 1.60837851487422766e-2
ASM DT 6.58749161529837803e-4
ASM DT 1.63153871373020978e-2
ASM erfQ:
ASM DT 2.56852019228982242e00, 1.87295284992346047e00, 5.27905102951428412e-1
ASM DT 6.05183413124413191e-2
ASM DT 2.33520497626869185e-3
ASM .code
END FUNCTION	


'
' #####################
' #####  Erfc ()  #####
' #####################
'
'
'
FUNCTION LONGDOUBLE Erfc(LONGDOUBLE x)
'Return the complimentary error function
	#FUNC_NUM = 68
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
ASM call %_TestArg1isNAN
ASM jnc >
	RETURN x
ASM :
	
	res## = 1## - Erf(ABS(x))
	IF x >=0 THEN
		RETURN res##
	ELSE
		RETURN 2## - res##
	ENDIF
	
END FUNCTION


'
' ###################
' #####  En ()  #####
' ###################
'
'
'
FUNCTION LONGDOUBLE En(n, LONGDOUBLE x) LONGDOUBLE
'Based on expint() in Numerical Recipes in C
XLONG i, ii, nm1

	#FUNC_NUM = 69
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
	IF IsNAN(x,0) THEN RETURN x
	
	IF n < 0 THEN RETURN PqNAN()
	IF x < 0## THEN RETURN PqNAN()
	
	nm1 = n - 1
	IF n = 0 THEN RETURN Exp(-x) / x
	IF x = 0## THEN RETURN 1## / nm1
	
	IF x > 1## THEN
		'Use Lentz's algorithm
		
		b = x + n
		c = 1d4931##				'near maximum for LONGDOUBLE
		d = 1## / b
		h = d
		
		FOR i = 1 TO 150
			a = -i * (nm1 + i)
			b = b + 2##
			d = 1## / (a * d + b)
			c = b + (a / c)
			del = c * d
			h = h * del
			IF (ABS(del - 1##) < 1d-19##) THEN RETURN h * Exp(-x)
		NEXT i
		'Exceeded 150 iterations
		RETURN PqNAN()			
		
	ELSE
		'(x <= 1) Use series expansion
		
		IF (nm1 <> 0) THEN
			ans = 1## / nm1
		ELSE
			ans = -Log(x) - $$EULER
		ENDIF
		
		fact = 1##
		FOR i = 1 TO 100
			fact = fact * (-x / i)
			IF (i <> nm1) THEN
				del = -fact/(i - nm1)
			ELSE
				psi = -$$EULER
				FOR ii = 1 TO nm1
					psi = psi + (1## / ii)
					del = fact * (-Log(x) + psi)
				NEXT ii
			ENDIF
			ans = ans + del
			IF (ABS(del) < ABS(ans) * $$EPSILON) THEN RETURN ans
		NEXT i
		'Exceeded 100 iterations
		RETURN PqNAN()	
		
	ENDIF
	

END FUNCTION


'
' ###################
' #####  Ei ()  #####
' ###################
'
'
'
FUNCTION LONGDOUBLE Ei(LONGDOUBLE x) LONGDOUBLE
'Based on ei() in Numerical Recipes in C
XLONG i, maxit

	#FUNC_NUM = 70
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
	IF x > 11357## THEN RETURN #PINF##								'result too big for LONGDOUBLE
	IF x < 0## THEN RETURN PqNAN()
	IF x < 1d-1000## THEN RETURN Log(x) + $$EULER			'special for very very small x
	
	maxit = 70
	IF x <= -Log($$EPSILON) THEN
		'Use power series for x < ~42
		sum = 0##
		fact = 1##
		FOR i = 1 TO maxit
			fact = fact * (x/i)
			term = fact / i
			sum = sum + term
			IF (term < $$EPSILON * sum) THEN RETURN sum + Log(x) + $$EULER
		NEXT i
		'Exceeded max iterations
		RETURN PqNAN()
		
	ELSE
		'Use asymptotic series
		sum = 0##
		term = 1##
		FOR i = 1 TO maxit
			prev = term
			term = term * (i/x)
			IF term < $$EPSILON THEN EXIT FOR
			IF term < prev THEN
				sum = sum + term			'still converging
			ELSE
				sum = sum - prev			'diverging, unwind and stop
				EXIT FOR
			ENDIF
		NEXT i
		
		IF i > maxit THEN RETURN PqNAN()
		RETURN Exp(x) * ((1## + sum) / x)
		
	ENDIF
	
END FUNCTION


'
' #########################
' #####  Fibonacci () #####
' #########################
'
'
'
FUNCTION LONGDOUBLE Fibonacci(n)
'returns the nth number from the Fibonacci series - 1,1,2,3,5,8,13,21,34,55.........
'n is 1 based and is limited to 1 to 23599
'based on Binet's formula Fibonacci(n) = ((1 + Sqrt(5))**n - (1 - Sqrt(5))**n) / ((2**n) * Sqrt(5))
'To maximise the range of n that can be handlled and the accuracy, this has been rearranged to
'(((1 + Sqrt(5))/2)**n - (-1**n)*((Sqrt(5) - 1)/2)**n) * (1/Sqrt(5))
'Each of the constant expressions has been evaluated to high precision and the Log of these values used below

	#FUNC_NUM = 71
	IF #TRACE THEN
ASM call %_XmalTrace
	ENDIF
	
	IF n < 1 THEN RETURN NqNAN()
	IF n < 3 THEN RETURN 1##
	IF n > 23599 THEN RETURN #PINF##

	fib1## = Exp(0.4812118250596034475## * n)
	fib2## = Exp(-0.4812118250596034475## * n)
	
	'Take care of the (-1**n)
	IF n MOD 2 THEN
		RETURN ROUND((fib1## + fib2##) * 0.44721359549995793928##) 
	ELSE
		RETURN ROUND((fib1## - fib2##) * 0.44721359549995793928##)
	ENDIF
	
END FUNCTION




'**************************************************************************************************************************
'**************************************************************************************************************************
'  INTERNAL FUNCTIONS NOT EXPORTED                   INTERNAL FUNCTIONS NOT EXPORTED
'**************************************************************************************************************************
'**************************************************************************************************************************
'
' #####################################
' #####  FunctionNumberToName$ () #####
' #####################################
'
'

FUNCTION FunctionNumberToName$(funcNum)
'Decode Trace function number to (possibly abbreviated) Function name
'INTERNAL USE ONLY
AUTOX maxFunc, buffer, i, stringAddress

	IF (funcNum < 0) || (funcNum > 255) THEN RETURN ""
	
ASM movzx ebx, b[FunctionNumberToName$.funcNum]					; function number from buffer
ASM lea 	esi, d[funcNameTable + ebx * 8]								; address of function name
ASM mov 	edi, 8																				; length of name always 8	
ASM	call	%_CompositeStringToString											; create an Xblite string from fixed length table entry 
ASM mov	 	d[FunctionNumberToName$.stringAddress], esi		; address of Xblite string created
	
	func$ = ""																	' empty string, handle only
	XLONGAT(&&func$) = stringAddress						' set func$ handle to point to our string (Xblite will release it)
	RETURN func$ + "()"

END FUNCTION


'
' ##########################
' #####  XmalExcept () #####
' ##########################
'
'
FUNCTION XmalExcept()
'GLOBAL Exception Catcher for Xmal library
EXCEPTION_RECORD ExRec
CONTEXT Context
	
  XstGetException (@exception)
  XstExceptionNumberToName (exception, @exception$)
  program$ = XstGetProgramFileName$ ()
	exception$ = " Exception number " + STRING(exception) + " = " + exception$ + " occurred"
	
	PRINT "\r\n XMAL GLOBAL EXCEPTION CATCHER"
	PRINT " Program = "; program$
	PRINT exception$
	
	PRINT " Last Xmal Function called = "; FunctionNumberToName$(#FUNC_NUM)
	
	IF #TRACE THEN
		'Show function call history from Trace Buffer
		XmalShowTrace() 
	ENDIF

	
	XstGetExceptionInformation (@ExRec, @Context)
	lpDataArea = &Context
	
	SELECT CASE exception
		CASE 6, 7, 8, 9, 10, 11, 20
			'FPU type exceptions
			'some FPU exceptions such as stack overflow must be cleared otherwise the FPU is unusable
			'It is needed to build the FPU display so clear with fnclex
ASM fnclex	
			PRINT "\r\n\r\n FPU Details at exception point:"
			DisplayProcessor(lpDataArea,1)
			
		CASE ELSE
			'CPU / memory / other type exceptions	
			PRINT "\r\n\r\n CPU Details at exception point:"
			DisplayProcessor(lpDataArea,0)
			
	END SELECT
	
	RETURN $$ExceptionTerminate

END FUNCTION


'
' ################################
' #####  DisplayProcessor () #####
' ################################
'
'
FUNCTION DisplayProcessor(lpDataArea, type)
'Type = 0, Display CPU, lpDataArea points to start of CONTEXT structure. Use within XmalExcept() with Integer EXCEPTION
'Type = 1, Display FPU, lpDataArea points to start of CONTEXT structure. Use within XmalExcept() with Float EXCEPTION
'Type = 2, Display CPU, lpDataArea points to structure formed by PUSHAD, PUSHFD. Use with ASM routine %_ShowCPU
'Type = 3, Display FPU, lpDataArea points to FLOATING_SAVE_AREA filled by fsave or fnsave. Use with ASM routine %_ShowFPU
'INTERNAL USE ONLY
UBYTE op[]
DIM op[1]
opAddr = &op[]

	'If there is no console, create one
	hStdOut = XioGetStdOut()
	IF hStdOut = -1 THEN
		consoleCreated = $$TRUE
		XioCreateConsole ("XmalExcept() Generated Console", 50)
	ELSE
		consoleCreated = $$FALSE
	ENDIF


	SELECT CASE type
		CASE 0
			'Display CPU from CONTEXT data
			'Get register values and eflags from context area
			
			edi = XLONGAT(lpDataArea + 156)
			esi = XLONGAT(lpDataArea + 160)
			ebx = XLONGAT(lpDataArea + 164)
			edx = XLONGAT(lpDataArea + 168)
			ecx = XLONGAT(lpDataArea + 172)
			eax = XLONGAT(lpDataArea + 176)
			ebp = XLONGAT(lpDataArea + 180)
			eip = XLONGAT(lpDataArea + 184)
			esp = XLONGAT(lpDataArea + 196)
			eflgs = XLONGAT(lpDataArea + 192)
			
			GOSUB DisplayCPU
			
		CASE 1
			'Display FPU from CONTEXT data. Adjust address to point to FSA within the context
			fsaAddr = lpDataArea + 28
			GOSUB DisplayFPU
			
		CASE 2
			'Display CPU from PUSHed data
			'Get register values and eflags from pushad, pushfd data
			'lpDataArea points to base of stack where eflags is
			'used by %_ShowCPU
			
			eflgs = XLONGAT(lpDataArea)
			edi  	= XLONGAT(lpDataArea + 4)
			esi  	= XLONGAT(lpDataArea + 8)
			ebp  	= XLONGAT(lpDataArea + 12)
			esp  	= XLONGAT(lpDataArea + 16) - 4		'adjust for %_ShowCPU's return address which was already on stack when pushad was done
			ebx  	= XLONGAT(lpDataArea + 20)
			edx  	= XLONGAT(lpDataArea + 24)
			ecx  	= XLONGAT(lpDataArea + 28)
			eax  	= XLONGAT(lpDataArea + 32)
			eip 	= 0  'not available through push 
			
			GOSUB DisplayCPU
			
		
		CASE 3
			'Display FPU from FLOATING_SAVE_AREA. Used by %_ShowFPU
			fsaAddr = lpDataArea
			GOSUB DisplayFPU
			
	END SELECT
	
	IF #PAUSE_ON_DISPLAY THEN
		a$ = INLINE$("\r\n Press ENTER to continue >  ")
		IF #CLEAR_ON_DISPLAY THEN
			XioClearConsole(hStdOut)
		ELSE
			PRINT
		ENDIF
	ENDIF

SUB DisplayCPU
	'Display CPU from CONTEXT data

	sep$ 	= SPACE$(4)
	
	PRINT "\r\n CPU DISPLAY:\r\n"
	PRINT	" EAX = " + HEX$(eax,8) + RJUST$(STRING(eax),14) + sep$ + " EBX = " + HEX$(ebx,8) + RJUST$(STRING(ebx),14) 
	PRINT	" ECX = " + HEX$(ecx,8) + RJUST$(STRING(ecx),14) + sep$ + " EDX = " + HEX$(edx,8) + RJUST$(STRING(edx),14) 
	PRINT	" EDI = " + HEX$(edi,8) + RJUST$(STRING(edi),14) + sep$ + " ESI = " + HEX$(esi,8) + RJUST$(STRING(esi),14) 
	PRINT	" EBP = " + HEX$(ebp,8) + RJUST$(STRING(ebp),14) + sep$ + " ESP = " + HEX$(esp,8) + RJUST$(STRING(esp),14) 
	IF eip THEN PRINT " EIP = " + HEX$(eip,8) + RJUST$(STRING(eip),14)		'eip only known if via exception and CONTEXT
	PRINT

'build output flag strings
	eflgsBin$ = BIN$(eflgs,32)
	flgDet$ = " CF = " + MID$(eflgsBin$, 32, 1) + "  PF = " + MID$(eflgsBin$, 30, 1) _
						+ "  AF = " + MID$(eflgsBin$, 28, 1) + "  ZF = " + MID$(eflgsBin$, 26, 1) _
						+ "  SF = " + MID$(eflgsBin$, 25, 1) + "  DF = " + MID$(eflgsBin$, 22, 1) _
						+ "  OF = " + MID$(eflgsBin$, 21, 1)

	PRINT " Flags:\r\n" + flgDet$ 
	PRINT

END SUB
	

SUB DisplayFPU

	'Display FPU from FLOAT_SAVE_AREA  data
	control$ = RIGHT$(BIN$(XLONGAT(fsaAddr),32),16)
	status$  = RIGHT$(BIN$(XLONGAT(fsaAddr + 4),32),16)
	tag$     = RIGHT$(BIN$(XLONGAT(fsaAddr + 8),32),16)
	tos$		 = MID$(status$,3,3)
	TOS			 = XLONG("0b" + tos$) 
	
	PRINT "\r\n FPU DISPLAY:\r\n"
	
	PRINT "                RRRI RC PC RR        PUOZDI (Masks)"
	PRINT " CONTROL WORD = " + LEFT$(control$,4) + " " + MID$(control$,5,2) + " " + MID$(control$,7,2) + " " + MID$(control$,9,2) + "        " + RIGHT$(control$,6)
	PRINT 
	PRINT "                B C3 TOP C2 C1 C0 ES PUOZDI (Exceptions)" 
	PRINT " STATUS  WORD = " + CHR$(status${0}) + "  " + CHR$(status${1}) + " " + MID$(status$,3,3) + "  " + CHR$(status${5}) + "  " + CHR$(status${6}) _
	                     + "  " + CHR$(status${7}) + " " + MID$(status$,9,2) + " " + RIGHT$(status$,6) + "\r\n"



	regBase = fsaAddr + 28			'Start of register save area within CONTEXT	
	FOR i = 0 TO 7
		'for register st0 to st7
		ld## = LONGDOUBLEAT(regBase + i * 10)
		physReg = (TOS + i) MOD 8								'physical register corresponding to STi
		t$ = MID$(tag$,15 - physReg * 2, 2) 		'extract tag for physical register
		SELECT CASE t$													'and decode iot
			CASE "00": tagName$ = " [VALID  ] = "
			CASE "01": tagName$ = " [ZERO   ] = "
			CASE "10": tagName$ = " [SPECIAL] = "
			CASE "11": tagName$ = " [EMPTY  ] = "
		END SELECT
		
		PRINT " ST" + STRING(i) + tagName$ + HexL$(ld##) + "  " + STRING(ld##)
		
	NEXT i
	
	
	IF type = 1
		'Display OPCODE at Code offset for exceptions only
		
		offset = XLONGAT(fsaAddr + 12)
		
		IF offset THEN
ASM mov esi, d[DisplayProcessor.offset]
ASM cs mov ax, w[esi]												; cs:offset >> instruction
ASM mov esi, d[DisplayProcessor.opAddr]
ASM mov w[esi], ax
	
			hexOp$ = HEX$(op[0],2) + " " + HEX$(op[1],2)	
			PRINT "\r\n OPCODE = " + hexOp$ + " >>> " + FpuOpcodeToName$(op[0], op[1])
		ENDIF
	ENDIF
	
	PRINT

END SUB

END FUNCTION


'
' ################################
' #####  FpuOpcodeToName$ () #####
' ################################
'
' INTERNAL
FUNCTION FpuOpcodeToName$( UBYTE op1, UBYTE op2)
'DISASSEMBLE FPU instruction from 2 bytes of op code
'Return string with instruction or "xxx" if decode fails
'Does not process SIB bytes if any

UBYTE ext, xx, yy

	IF op2 <= 0xBF THEN
		'st, memory style. ext is extension of opcode in op1
		ext = EXTU(op2,3,3)		
	ELSE
		'st, sti style. xx and yy determines instruction and register
		xx = EXTU(op2,4,4)
		yy = EXTU(op2,4,0)
	ENDIF

	SELECT CASE op1
		CASE 0xD8
			IF op2 <= 0xBF THEN
				'one op is memory
				SELECT CASE ext
					CASE 0: RETURN "FADD m32real"
					CASE 1: RETURN "FMUL m32real"
					CASE 2: RETURN "FCOM m32real"
					CASE 3: RETURN "FCOMP m32real"
					CASE 4: RETURN "FSUB m32real"
					CASE 5: RETURN "FSUBR m32real"
					CASE 6: RETURN "FDIV m32real"
					CASE 7: RETURN "FDIVR m32real"
				END SELECT
			ELSE
				'both ops are stack
				SELECT CASE xx
					CASE 0xC
						IF yy <= 0x7 THEN RETURN "FADD st, st" + STRING(yy)
						RETURN "FMUL st" + STRING(yy - 8)
					CASE 0xD
						IF yy <= 0x7 THEN RETURN "FCOM st" + STRING(yy)
						RETURN "FCOMP st" + STRING(yy - 8)
					CASE 0xE
						IF yy <= 0x7 THEN RETURN "FSUB st, st" + STRING(yy)
						RETURN "FSUBR st, st" + STRING(yy - 8)
					CASE 0xF
						IF yy <= 0x7 THEN RETURN "FDIV st, st" + STRING(yy)
						RETURN "FDIVR st, st" + STRING(yy - 8)
				END SELECT
			ENDIF
			
		CASE 0xD9
			IF op2 <= 0xBF THEN
				'one op is memory
				SELECT CASE ext
					CASE 0: RETURN "FLD m32real"
					CASE 1: RETURN "aaa"
					CASE 2: RETURN "FST m32real"
					CASE 3: RETURN "FSTP m32real"
					CASE 4: RETURN "FLDENV m28byte"
					CASE 5: RETURN "FLDCW m2byte"
					CASE 6: RETURN "FSTENV m28byte"
					CASE 7: RETURN "FSTCW m2byte"
				END SELECT
			ELSE
				'both ops are stack
				SELECT CASE xx
					CASE 0xC
						IF yy <= 0x7 THEN RETURN "FLD st" + STRING(yy)
						RETURN "FXCH st" + STRING(yy - 8)
					CASE 0xD
						IF yy = 0x0 THEN RETURN "FNOP"
						RETURN "bbb"
					CASE 0xE
						SELECT CASE yy
							CASE 0   : RETURN "FCHS"
							CASE 1   : RETURN "FABS"
							CASE 4   : RETURN "FTST"
							CASE 5   : RETURN "FXAM"
							CASE 8   : RETURN "FLD1"
							CASE 9   : RETURN "FLDL2T"
							CASE 0xA : RETURN "FLDL2E"
							CASE 0xB : RETURN "FLDPI"
							CASE 0xC : RETURN "FLDLG2"
							CASE 0xD : RETURN "FLDLN2"
							CASE 0xE : RETURN "FLDZ"
							CASE ELSE  RETURN "ccc"
						END SELECT
					CASE 0xF
						SELECT CASE yy
							CASE 0   : RETURN "F2XM1"
							CASE 1   : RETURN "FYL2X"
							CASE 2	 : RETURN "FPTAN"
							CASE 3	 : RETURN "FPATAN"
							CASE 4   : RETURN "FXTRACT"
							CASE 5   : RETURN "FPREM1"
							CASE 6	 : RETURN "FDECSTP"
							CASE 7	 : RETURN "FINCSTP"
							CASE 8   : RETURN "FPREM"
							CASE 9   : RETURN "FYL2XP1"
							CASE 0xA : RETURN "FSQRT"
							CASE 0xB : RETURN "FSINCOS"
							CASE 0xC : RETURN "FRNDINT"
							CASE 0xD : RETURN "FSCALE"
							CASE 0xE : RETURN "FSIN"
							CASE 0xF : RETURN "FCOS"
							CASE ELSE  RETURN "ddd"
						END SELECT
					CASE ELSE RETURN "eee"
				END SELECT
			ENDIF	
			
		CASE 0xDA
			IF op2 <= 0xBF THEN
				'one op is memory
				SELECT CASE ext
					CASE 0: RETURN "FIADD m32int"
					CASE 1: RETURN "FIMUL m32int"
					CASE 2: RETURN "FICOM m32int"
					CASE 3: RETURN "FICOMP m32int"
					CASE 4: RETURN "FISUB m32int"
					CASE 5: RETURN "FISUBR m32int"
					CASE 6: RETURN "FIDIV m32int"
					CASE 7: RETURN "FIDIVR m32int"
				END SELECT
			ELSE
				'both ops are stack
				SELECT CASE xx
					CASE 0xC
						IF yy <= 0x7 THEN RETURN "FCMOVB"
						RETURN "FCMOVE"
					CASE 0xD
						IF yy = 0x0 THEN RETURN "FCMOVBE"
						RETURN "FCMOVU"
					CASE 0xE
						IF yy = 0x9 THEN RETURN "FUCOMPP"
					CASE ELSE
						RETURN "fff"
				END SELECT
			ENDIF	
			
		CASE 0xDB
			IF op2 <= 0xBF THEN
				'one op is memory
				SELECT CASE ext
					CASE 0: RETURN "FILD m32int"
					CASE 1: RETURN "FISTTP m32int"
					CASE 2: RETURN "FIST m32int"
					CASE 3: RETURN "FISTP m32int"
					CASE 4: RETURN "ggg"
					CASE 5: RETURN "FLD m80real"
					CASE 6: RETURN "hhh"
					CASE 7: RETURN "FSTP m80real"
				END SELECT
			ELSE
				'both ops are stack
				SELECT CASE xx
					CASE 0xC
						IF yy <= 0x7 THEN RETURN "FCMOVNB st" + STRING(yy)
						RETURN "FCMOVNE + st" + STRING(yy - 8)
					CASE 0xD
						IF yy = 0x0 THEN RETURN "FCMOVNBE st" + STRING(yy)
						RETURN "FCMOVNU st" + STRING(yy - 8)
					CASE 0xE
						IF yy = 0x2 THEN RETURN "FCLEX"
						IF yy = 0x3 THEN RETURN "FINIT"
						IF yy > 0x7 THEN RETURN "FUCOMI"
					CASE ELSE
						RETURN "iii"
				END SELECT
			ENDIF	
			
		CASE 0xDC
			IF op2 <= 0xBF THEN
				'one op is memory
				SELECT CASE ext
					CASE 0: RETURN "FADD m64real"
					CASE 1: RETURN "FMUL m64real"
					CASE 2: RETURN "FCOM m64real"
					CASE 3: RETURN "FCOMP m64real"
					CASE 4: RETURN "FSUB m64real"
					CASE 5: RETURN "FSUBR m64real"
					CASE 6: RETURN "FDIV m64real"
					CASE 7: RETURN "FDIVR m64real"
				END SELECT
			ELSE
				'both ops are stack
				SELECT CASE xx
					CASE 0xC
						IF yy <= 0x7 THEN RETURN "FADD st" + STRING(yy) + ", st"
						RETURN "FMUL st, st" + STRING(yy - 8)
					CASE 0xE
						IF yy <= 0x7 THEN RETURN "FSUBR st" + STRING(yy) + ", st"
						RETURN "FSUB st" + STRING(yy - 8) + ", st"
					CASE 0xF
						IF yy <= 0x7 THEN RETURN "FDIVR st" + STRING(yy) + ", st"
						RETURN "FDIV st" + STRING(yy - 8) + ", st"
					CASE ELSE
						RETURN "jjj"
				END SELECT
			ENDIF
			
		CASE 0xDD
			IF op2 <= 0xBF THEN
				'one op is memory
				SELECT CASE ext
					CASE 0: RETURN "FLD m64real"
					CASE 1: RETURN "FISTTP m64int"
					CASE 2: RETURN "FST m64real"
					CASE 3: RETURN "FSTP m64real"
					CASE 4: RETURN "FRSTOR m108byte"
					CASE 5: RETURN "kkk"
					CASE 6: RETURN "FSAVE m108byte"
					CASE 7: RETURN "FSTSW m2byte"
				END SELECT
			ELSE
				'both ops are stack
				SELECT CASE xx
					CASE 0xC
						IF yy <= 0x7 THEN RETURN "FFREE st" + STRING(yy)
						RETURN "lll"
					CASE 0xD
						IF yy <= 0x7 THEN RETURN "FST st" + STRING(yy)
						RETURN "FSTP st" + STRING(yy - 8)
					CASE 0xE
						IF yy <= 0x7 THEN RETURN "FUCOM st" + STRING(yy)
						RETURN "FUCOMP st" + STRING(yy - 8)
					CASE 0xF
						RETURN "mmm"
				END SELECT
			ENDIF
			
		CASE 0xDE
			IF op2 <= 0xBF THEN
				'one op is memory
				SELECT CASE ext
					CASE 0: RETURN "FIADD m16int"
					CASE 1: RETURN "FIMUL m16int"
					CASE 2: RETURN "FICOM m16int"
					CASE 3: RETURN "FICOMP m16int"
					CASE 4: RETURN "FISUB m16int"
					CASE 5: RETURN "FISUBR m16int"
					CASE 6: RETURN "FIDIV m16int"
					CASE 7: RETURN "FIDIVR m16int"
				END SELECT
			ELSE
				'both ops are stack
				SELECT CASE xx
					CASE 0xC
						IF yy <= 0x7 THEN RETURN "FADDP st" + STRING(yy) + ", st"
						RETURN "FMULP st" + STRING(yy - 8) + ", st"
					CASE 0xD
						IF yy = 0x9 THEN RETURN "FCOMPP"
					CASE 0xE
						IF yy <= 0x7 THEN RETURN "FSUBRP st" + STRING(yy) + ", st"
						RETURN "FSUBP st" + STRING(yy - 8) + ", st"
					CASE 0xF
						IF yy <= 0x7 THEN RETURN "FDIVRP st" + STRING(yy) + ", st"
						RETURN "FDIVP st" + STRING(yy - 8) + ", st"
				END SELECT
			ENDIF
			
		CASE 0xDF
			IF op2 <= 0xBF THEN
				'one op is memory
				SELECT CASE ext
					CASE 0: RETURN "FILD m16int"
					CASE 1: RETURN "FISTTP m16int"
					CASE 2: RETURN "FIST m16int"
					CASE 3: RETURN "FISTP m16int"
					CASE 4: RETURN "FBLD m80dec"
					CASE 5: RETURN "FILD m64int"
					CASE 6: RETURN "FBSTP m80dec"
					CASE 7: RETURN "FISTP m64int"
				END SELECT
			ELSE
				'both ops are stack
				SELECT CASE xx
					CASE 0xE
						IF yy = 0x0 THEN RETURN "FSTSW m2byte"
						IF yy > 0x7 THEN RETURN "FUCOMIP st" + STRING(yy - 8)
					CASE 0xF
						IF yy <= 0x7 THEN RETURN "FCOMIP st" + STRING(yy)
				END SELECT
			ENDIF			
	END SELECT
	'Should never get here
	RETURN "nnn"
END FUNCTION

'
' #####################
' #####  PqNAN () #####
' #####################
'
'
FUNCTION LONGDOUBLE PqNAN()
'Return + QNAN with current function number payloaded in last byte
'INTERNAL USE ONLY
ASM mov eax, d[%#FUNC_NUM.xmal]	
ASM mov b[pnan], al
ASM fld		t[pnan]
ASM mov b[pnan], 0xFF
ASM jmp end.PqNAN.xmal

END FUNCTION

'
' #####################
' #####  NqNAN () #####
' #####################
'
'
FUNCTION LONGDOUBLE NqNAN()
'Return - QNAN with current function number payloaded in last byte
'INTERNAL USE ONLY
ASM mov eax, d[%#FUNC_NUM.xmal]	
ASM mov b[nnan], al
ASM fld		t[nnan]
ASM mov b[nnan], 0xFF
ASM jmp end.NqNAN.xmal

END FUNCTION

'
' #####################
' #####  PsNAN () #####
' #####################
'
'
FUNCTION LONGDOUBLE PsNAN()
'Return + SNAN with current function number payloaded in last byte
'INTERNAL USE ONLY
ASM mov eax, d[%#FUNC_NUM.xmal]	
ASM mov b[psnan], al
ASM fld		t[psnan]
ASM mov b[psnan], 0xFF
ASM jmp end.PsNAN.xmal

END FUNCTION

'
' #####################
' #####  NsNAN () #####
' #####################
'
'
FUNCTION LONGDOUBLE NsNAN()
'Return - SNAN with current function number payloaded in last byte
'INTERNAL USE ONLY
ASM mov eax, d[%#FUNC_NUM.xmal]	
ASM mov b[nsnan], al
ASM fld		t[nsnan]
ASM mov b[nsnan], 0xFF
ASM jmp end.NsNAN.xmal

END FUNCTION


'*********************************************************************************************************************************
'*********************************************************************************************************************************
'   NAN / INF GENERATION
'*********************************************************************************************************************************
'*********************************************************************************************************************************
'
' ####################
' #####  QNAN () #####
' ####################
'
'
FUNCTION LONGDOUBLE QNAN(sign)
'Return +NAN or -NAN (QUIET)

	#FUNC_NUM = 179
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF
	
	IF sign >= 0 THEN
ASM fld		t[pnan]
	ELSE
ASM fld		t[nnan]
	ENDIF
ASM jmp end.QNAN.xmal

END FUNCTION



'
' ####################
' #####  SNAN () #####
' ####################
'
'
FUNCTION LONGDOUBLE SNAN(sign)
'Return +NAN or -NAN (SIGNALLING)

	#FUNC_NUM = 180
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF
	
	IF sign >= 0 THEN
ASM fld		t[psnan]
	ELSE
ASM fld		t[nsnan]
	ENDIF
ASM jmp end.SNAN.xmal

END FUNCTION





'
' ###################
' #####  INF () #####
' ###################
'
'
FUNCTION LONGDOUBLE INF(sign)
'Return +INF or -INF

	#FUNC_NUM = 182
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF
	
	IF sign >= 0 THEN
ASM fld		t[pinf]
	ELSE
ASM fld		t[ninf]
	ENDIF
ASM jmp end.INF.xmal

END FUNCTION




'
' ###################
' #####  UNS () #####
' ###################
'
'
FUNCTION LONGDOUBLE UNS(sign)
'Return +UNS or -UNS - 1 typical value of many unsupported formats

	#FUNC_NUM = 184
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF
	
	
	IF sign >= 0 THEN
ASM fld		t[puns]
	ELSE
ASM fld		t[nuns]
	ENDIF
ASM jmp end.INF.xmal

END FUNCTION



'
' ###########################
' #####  IsIdentical () #####
' ###########################
'
'
FUNCTION IsIdentical(LONGDOUBLE x, LONGDOUBLE y)
'Return TRUE if x is identical to y, otherwise false
'x and y can be ANY longdouble class - NORMAL, ZERO, DENORMAL, INFINITY, NAN, UNS
'compares 80 bits of x with 80 bits of y

	#FUNC_NUM = 196
	IF #TRACE3 THEN
ASM call %_XmalTrace
	ENDIF
	
ASM mov ax, w[IsIdentical.x + 8]	
ASM cmp ax, w[IsIdentical.y + 8]
ASM jne > 
ASM mov eax, d[IsIdentical.x + 4]	
ASM cmp eax, d[IsIdentical.y + 4]
ASM jne > 
ASM mov eax, d[IsIdentical.x]	
ASM cmp eax, d[IsIdentical.y]
ASM jne >
ASM mov eax, -1										; return TRUE
ASM jmp > end.IsIdentical.xmal
ASM :															; return FALSE
END FUNCTION

'**********************************************************************************************************
'**********************************************************************************************************
'**********************************************************************************************************
'**********************************************************************************************************

ASM jmp >> end_program.xmal		; When Xblite drops through to here it must jump over this code!

'**********************************************************************************************************
'ASSEMBLER SUB-ROUTINE %_two2IplusF
'**********************************************************************************************************
'Assembler sub routine used by all Exp..() functions and CbrtL()
'Evaluates 2**(I + F) - where I is integer and F is fraction
'On entry, single value of (I + F) is on TOS

ASM %_two2IplusF:
ASM fstcw		w[temp]							; save control word
ASM fstcw		w[temp + 2]					; and a second copy
ASM and			w[temp],0xF3FF 			; ensure rounding mode is truncate 
ASM fldcw		w[temp]			
ASM fld			st0									; duplicate TOS (I + F)
ASM frndint											; truncate - this is now I
ASM fldcw		w[temp + 2]					; restore original control word / rounding mode
ASM fxch
ASM fsub		st0,st1							; evaluate F = (I + F) - I
ASM f2xm1												; compute (2 ** F) - 1
ASM fld1												; add 1 to give
ASM fadd												; (2 ** F)
ASM fscale											; multiply by (2 ** I) giving 2 ** (I + F) = RESULT    <<<<<<<<<<<<<<< CAN OVERFLOW
ASM fstp st1										; overwrite excess value										 
ASM ret



'**********************************************************************************************************
'ASSEMBLER SUB-ROUTINE %_XmalTrace
'**********************************************************************************************************
'TRACE
'Maintain circular buffer of traced function calls. Number set by XmalSetTraceSize()

ASM %_XmalTrace:
ASM mov eax, d[%#FUNC_NUM.xmal]							; collect current function number in eax
ASM mov ebx, d[%#TRACE_ADDRESS.xmal]				; address of base of trace array
ASM add ebx, d[trace_index]									
ASM mov b[ebx], al													; store function number 
ASM inc	d[trace_index]											; move to next slot
ASM mov eax, d[%#TRACE_WRAP.xmal]											
ASM cmp d[trace_index], eax									; wrap round if necessary
ASM jl >
ASM mov d[trace_index], 0
ASM :
ASM ret 																		; return

'**********************************************************************************************************
'ASSEMBLER SUB-ROUTINE %_ShowFPU
'**********************************************************************************************************
'ShowFPU
'Assembly level call to Show FPU that preserves all CPU registers and flags
'EXPORTED for use in debugging

ASM EXPORT %_ShowFPU:
ASM pushad																	; save CPU state
ASM pushfd

ASM mov eax, ADDR floatSave									; populate FLOATING_SAVE_AREA buffer
ASM fnsave [eax]

ASM push 3																	; type 3 = display FPU from fsa
ASM push eax 																; pointer to fsa
ASM call _DisplayProcessor@8

ASM mov eax, ADDR floatSave									; restore FPU
ASM frstor [eax]

ASM popfd																		; restore CPU state
ASM popad
ASM ret


'**********************************************************************************************************
'ASSEMBLER SUB-ROUTINE %_ShowCPU
'**********************************************************************************************************
'ShowCPU
'Assembly level call to display CPU that preserves all CPU registers and flags
'EXPORTED for use in debugging

ASM EXPORT %_ShowCPU:
ASM pushad										; save all CPU data
ASM pushfd

ASM mov eax, esp							; save value of esp before push below
ASM push 2										; type 2 = display CPU from push structure
ASM push eax									; address of saved esp =>> pushed data
ASM call _DisplayProcessor@8	; parameters removed by function

ASM popfd											; restore all CPU data and balance stack
ASM popad
ASM ret 


'**********************************************************************************************************
'ASSEMBLER SUB-ROUTINES %_TestArg1isNAN and family
'**********************************************************************************************************
'Test longdouble function parameter on stack to see if NAN has been input
'3 different entry points for 1st, 2nd, 3rd parameters
'The 3 parameters are assumed to be the first / second / third function parameters and all longdouble
'RETURNS with carry flag set if parameter is a NAN, otherwise carry flag clear

ASM %_TestArg3isNAN:
ASM lea 	esi, [ebp + 32]						; base of parameter 3
ASM jmp >
ASM %_TestArg2isNAN:
ASM lea 	esi, [ebp + 20]						; base of parameter 2
ASM jmp >
ASM %_TestArg1isNAN:
ASM lea 	esi, [ebp + 8]						; base of parameter 1
ASM :
ASM mov 	ax, w[esi + 8]						; get sign + exponent
ASM and 	ax, 0x7FFF								; clear the sign
ASM cmp 	ax, 0x7FFF								; Is it NAN / INF
ASM je > 
ASM clc															; carry = 0 to signal not a nan
ASM ret

ASM :																; see if it is infinity or nan
ASM cmp		d[esi + 4], 0x80000000		; test mantissa high
ASM je > 														
ASM stc															; carry = 1 to signal nan
ASM ret

ASM :																; could still be infinity
ASM cmp		d[esi], 0									; test mantissa low
ASM je 	>														
ASM stc															; carry = 1 to signal nan
ASM ret

ASM :																; it is infinity												
ASM clc															; carry = 0 to signal not a nan
ASM ret
	
'**********************************************************************************************************
'ASSEMBLER CONSTANTS
'**********************************************************************************************************

'Special values in longdouble formats for use with asm routines
ASM .data
ASM align 8
ASM pnan:
ASM DT 0x7FFFFFFFFFFFFFFFFFFF		; positive NAN		'typical value of many - QUIET NAN
ASM nnan:
ASM DT 0xFFFFFFFFFFFFFFFFFFFF		; negative NAN		'typical value of many - QUIET NAN
ASM indefinite:
ASM DT 0xFFFFC000000000000000		; negative NAN		'special value INDEFINITE
ASM psnan:
ASM DT 0x7FFFBFFFFFFFFFFFFFFF		; positive NAN		'typical value of many - SIGNALLING NAN
ASM nsnan:
ASM DT 0xFFFFBFFFFFFFFFFFFFFF		; negative NAN		'typical value of many - SIGNALLING NAN
ASM pinf:
ASM DT 0x7FFF8000000000000000		; positive INF		'unique
ASM ninf:
ASM DT 0xFFFF8000000000000000		; negative INF		'unique
ASM puns:
ASM DT 0x7FFF7FFFFFFFFFFFFFFF		; positive UNS		'typical value of several
ASM nuns:
ASM DT 0xFFFF7FFFFFFFFFFFFFFF		; negative UNS		'typical value of several
ASM pidiv2:
ASM DT 1.5707963267948966192		; radians == 90 degrees
ASM twopi:
ASM DT 6.2831853071795864769		; pi * 2
ASM RadToDeg:
ASM DT 57.295779513082320877		; degrees == 1 radian
ASM DegToRad:
ASM DT 0.017453292519943295769	; radians == 1 degree
ASM onethird:
ASM DT 0.33333333333333333333		; 1/3
ASM twopow63:
ASM DT 0x403E8000000000000000		; 2 to the power 63


'values for direct loading to FPU in ASM. Mix of integer, single and double
ASM ninety:
ASM DD 90											; 90
ASM two:
ASM DD 2											; 2
ASM max_single:
ASM DD 7F7FFFFFh							; 3.402823e38
ASM min_single:
ASM DD 00800000h							; 1.175495e-38
ASM max_double:
ASM DQ 7FEFFFFFFFFFFFFFh			; 1.79769313486231d+308
ASM min_double:  
ASM DQ 0010000000000000h			; 2.2250738585073d-308
ASM max_longdouble: 
ASM DT 7FFEFFFFFFFFFFFFFFFFh  ; 1.189731495357231764d+4932
ASM min_longdouble:
ASM DT 00018000000000000000h  ; 3.362103143112093506d-4932
ASM max_denormal: 
ASM DT 00007FFFFFFFFFFFFFFFh  ; 
ASM min_denormal:
ASM DT 00000000000000000001h  ; 

'**********************************************************************************************************
'ASSEMBLER DATA / VARIABLES / TABLES
'**********************************************************************************************************
ASM .data
ASM align 8

'***********************************************************************************************************
'Work area to hold up to 12 bytes
ASM temp:
ASM DB 12 dup 0

'And special area for FLOATING_SAVE_AREA data
ASM align 8
ASM floatSave:
ASM DB 112 dup 0

'***********************************************************************************************************
'TRACE INDEX
'***********************************************************************************************************
'index to next entry in trace buffer
'used by %_XmalTrace
ASM trace_index:			; points to next slot to write to (post increment)
ASM DD 0

'***********************************************************************************************************
'FUNCTION NAME TABLE
'*********************************************************************************************************
'List of function names as fixed 8 byte strings (Not Xblite format)
'These have been abbreviated where necessary
'xxxxxxxx = spare
'DDDDDDDD = deleted function - can be reused
'00000000 = do not use 0 entry

ASM funcNameTable:
ASM DQ "00000000", "Cos", "Tan", "Csc", "Sec", "Cot"												; #FUNC_NUM 0 to 5
ASM DQ "Asin", "Acos", "Atan", "Atan2", "Acsc", "Asec", "Acot"							; #FUNC_NUM 6 to 12
ASM DQ "Sinh", "Cosh", "Tanh", "Csch", "Sech", "Coth"												; #FUNC_NUM 13 to 18
ASM DQ "Asinh", "Acosh", "Atanh", "Acsch", "Asech", "Acoth"									; #FUNC_NUM 19 to 24
ASM DQ "Factoral", "Sin", "Product", "nCr", "nPr"														; #FUNC_NUM 25 to 29
ASM DQ "Log", "Log2", "Log10", "Logb", "Log1p"															; #FUNC_NUM 30 to 34
ASM DQ "Exp", "Exp2", "Exp10", "Power", "Power2", "Power10", "PowerX"				; #FUNC_NUM 35 to 41 
ASM DQ "Sqrt", "Cbrt", "NthRoot"																					  ; #FUNC_NUM	42 to 44
ASM DQ "SumAP", "ProdAP", "ProdAP2", "SumGP", "ProdGP", "Hypot"							; #FUNC_NUM 45 to 50	
ASM DQ "Hypot3", "Extract", "Lmake", "xxxxxxxx", "Limage", "Lbuild"					; #FUNC_NUM 51 to 56				
ASM DQ "Ldexp", "Frexp", "Modf", "SafeLDSi", "SafeLDDo", "SafeLDCt"					; #FUNC_NUM 57 to 62
ASM DQ "Gamma", "LogGamma", "LogFctrl", "Beta", "Erf", "Erfc", "En"					; #FUNC_NUM 63 to 69
ASM DQ "Ei", "Fibonnac", "CopySign", "ProdGP2"															; #FUNC_NUM 70 to 73
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 74 to 78
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 79 to 83
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 84 to 88 							
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 89 to 93							
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 94 to 98
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "IsNormal"						; #FUNC_NUM 99 to 103						
ASM DQ "GetClass", "IsValid",  "Fround",   "SetAngle", "GetAngle"						; #FUNC_NUM 104 to 108 							
ASM DQ "IsNeg",    "IsPos",    "IsNear0",  "IsNearIn", "IsNearEQ"						; #FUNC_NUM 109 to 113							
ASM DQ "IsNrLTE",  "IsNrGTE",  "IsNearGT", "IsNearLT", "SetNear0"						; #FUNC_NUM 114 to 118							
ASM DQ "SetNearI", "SetNrEQ",  "SetSafe",  "SetGlobE", "SetDenor"						; #FUNC_NUM 119 to 123							
ASM DQ "SetFPUEx", "GetSign",  "GetExp",   "GetTrExp", "GetManti"						; #FUNC_NUM 124 to 128 							
ASM DQ "Fmod",     "HexL$",    "NextValu", "NextTowd", "NextHigh"						; #FUNC_NUM 129 to 133
ASM DQ "NextLow",  "Lmin",     "Lmax",     "LminDen",  "LmaxDen"						; #FUNC_NUM 134 to 138
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 139 to 143							
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 144 to 148							
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 149 to 153							
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 154 to 158							
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 159 to 163							
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 164 to 168
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 169 to 173							
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 174 to 178
ASM DQ "QNAN",     "SNAN",     "xxxxxx",   "INF",      "xxxxxx", "UNS"			; #FUNC_NUM 179 to 184
ASM DQ "IsNAN",    "IsSNAN",   "IsQNAN",   "IsINF",    "IsDEN", "xxxxxx", "IsZERO"  	  ; #FUNC_NUM 185 to 191
ASM DQ "IsINDEF",  "Lsexp",    "Lhigh",    "Llow",     "IsIdent"												; #FUNC_NUM 192 to 196
ASM DQ "Set754",   "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 197 to 201
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 202 to 206 																				 																				
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 207 to 211 																				 																				
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 212 to 216 																				 																				
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 217 to 221 																				 																				
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 222 to 226 																				 																				
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 227 to 231 																				 																				
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 232 to 236
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 237 to 241 
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 242 to 246
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "xxxxxxxx"						; #FUNC_NUM 247 to 251 																				 																				
ASM DQ "xxxxxxxx", "xxxxxxxx", "xxxxxxxx", "CONST",    "FPU"								; #FUNC_NUM 252 to 255																		 																				 																				 																				 																				 																																								 																				 																				 																				 																				 																				



'*************************************************************************************************************************
'Used with Factorial()
'Table of pre-calculated factorials in LONGDOUBLE format for values 0, 1, 2 ............, 51, 52
' 0 to 52
ASM fact_low_table:
ASM DT 0x3FFF8000000000000000, 0x3FFF8000000000000000, 0x40008000000000000000, 0x4001C000000000000000, 0x4003C000000000000000
ASM DT 0x4005F000000000000000, 0x4008B400000000000000, 0x400B9D80000000000000, 0x400E9D80000000000000, 0x4011B130000000000000
ASM DT 0x4014DD7C000000000000, 0x40189845400000000000, 0x401BE467E00000000000, 0x401FB994660000000000, 0x4023A261D94000000000
ASM DT 0x4027983BBBAC00000000, 0x402B983BBBAC00000000, 0x402FA1BF7766C0000000, 0x4033B5F7665398000000, 0x4037D815C98344800000
ASM DT 0x403C870D9DF20AD00000, 0x4040B141DF4DAE310000, 0x4044F3BA930ACF836000, 0x4049AF2E19AFC5266D00, 0x404E83629343D3DCD1C0
ASM DT 0x4052CD4A0619FB0907BC, 0x4057A6CC24F51BF75649, 0x405C8CBC3F2ECF98B0CE, 0x4060F6496E91EB4B3568, 0x4065DF328C343D3C2864
ASM DT 0x406AD13F6370F96865DF, 0x406FCAB56855719D22AF, 0x4074CAB56855719D22AF, 0x4079D10B13981D2A0BC5, 0x407EDE1BC4D19EFCAC81
ASM DT 0x4083F2EE5F4545E45CAC, 0x408988A61596F7507421, 0x408E9E0008F68DF50648, 0x4093BBA00AA4C892F773, 0x4098E4AB0CF8D4731D96
ASM DT 0x409E8EEAE81B84C7F27E, 0x40A3B71CF96342202EB2, 0x40A8F056075246CA3D4B, 0x40AEA179CCEB478FE12D, 0x40B3DE0779C38265D59E
ASM DT 0x40B99C1D419D77AF9A33, 0x40BEE06A0E525C0C6DA8, 0x40C4A4CDE2847B992089, 0x40C9F734D3C6B965B0CD, 0x40CFBD44722425F1DB5E
ASM DT 0x40D593DD792C3DA4F361, 0x40DAEBA8F91E823EE3E2, 0x40E0BF794A68C9D31928
'Table of pre-calculated factorials in LONGDOUBLE format for values 50, 60, 70, 80 ............, 1740, 1750
ASM fact_high_table:
' 50 to 190
ASM DT 0x40D593DD792C3DA4F361, 0x410F8C5AB93F804FA92E, 0x414BAF3FD62B6EB73E4A, 0x4189E30C84A76D5FF4F2, 0x41C9FF8289544D0A8F23
ASM DT 0x420BD984B2761CAEE121, 0x424EFAD78A5EE2397413, 0x4293B2FB1C054EC8AF87, 0x42D9928E91763389BCF9, 0x432081358E035B52A4EC
ASM DT 0x4367E83ED3F65B165EF0, 0x43B0CAEAF9EC9476EC85, 0x43FAA55BC3220C31C792, 0x4444F252E6DBC6600B2E, 0x44909A89B0B0A81E4471
' 200 to 390
ASM DT 0x44DCA6A15C2404521D73, 0x452993F53C772CA44F05, 0x4576D34ECD8E808E300C, 0x45C4ED838EC128490FB6, 0x4613CE04A39647808399
ASM DT 0x4663876EC3D60A43DCC4, 0x46B384BA0AF0690EA7FB, 0x4703BEF69851632AAA27, 0x4754C6D69E3F871B2A07, 0x47A693DEDF9874013927
ASM DT 0x47F89B2A10E4A1719F8E, 0x484AE31E5238A719BA51, 0x489DE56419C45CAC2DF2, 0x48F19E4506E761359E72, 0x494593CA271CF8878BFF
ASM DT 0x4999B91EE248C490345A, 0x49EE9A3A078A5FE0221E, 0x4A43A991F9BB3FBA813C, 0x4A98F43708D2ABB0DB74, 0x4AEEE4B9E91BFAECE6BB
' 400 to 590
ASM DT 0x4B458A5FEBB00AB6CB80, 0x4B9BD6EB7FB37C52D68A, 0x4BF2D4F2AFA25A3C5774, 0x4C4A85D2AD57178FCCB4, 0x4CA1D42E8A97E3F624F8
ASM DT 0x4CF9D3158190257E9050, 0x4D52831817B0A94B907D, 0x4DAACA542D5A86E53A19, 0x4E03C11D392D4B07D51F, 0x4E5CE2F74C9DE357057F
ASM DT 0x4EB6A38A524C0D1E9AEA, 0x4F108FE6E70C4FA5C2B9, 0x4F6A9A05F62B3CECD5AA, 0x4FC4C7C756A2079CDE0A, 0x501F9C7101A14F08000A
ASM DT 0x507A9367721E7E81E376, 0x50D5A68E8D8359CF3DFE, 0x5130E0F572E289AFC676, 0x518CB506D353527CA995, 0x51E8AD0FA9ED99AA5DB0
' 600 to 790
ASM DT 0x5244C3FA5B144C3658A0, 0x52A18312749731DF66D0, 0x52FDCE87C5FC8636DF9D, 0x535ABF2BBCB1A3991D8B, 0x53B7CF5E64D2C28F5396
ASM DT 0x5415837998C43A3F4C2B, 0x5472C26B4695E590107C, 0x54D0A73F554D08DC0696, 0x552EA703B2DF2A8F3240, 0x558CC12F13B96FC925DA
ASM DT 0x55EB812372C6A1319182, 0x5649C725378600BB92EE, 0x56A8B0C25B07BEF2C416, 0x5707B43FB697C9476863, 0x5766D2C6183B3D433553
ASM DT 0x57C68D0D7B8AC99C249C, 0x5825D7B28C204861341D, 0x5885BC1961267D084F56, 0x58E5BAC3D6050A9201DA, 0x5945D2CA14F4349C4E07
' 800 to 990
ASM DT 0x59A686FE52705CAF49D4, 0x5A06C3E975C5D56E5DF2, 0x5A67A0D4324B2FF9AB7F, 0x5AC8952443F5A2BAB3D9, 0x5B299C00569C8282AD50
ASM DT 0x5B8AB7CB17F938342B40, 0x5BEBF38DE7F948D97D7B, 0x5C4DB542AD4DB8ABA706, 0x5CAF9752646BF9261924, 0x5D118D85B51214F39570
ASM DT 0x5D739416705B75DAD1A8, 0x5DD5AD286339FDECD2A3, 0x5E37E1FA25C9D501DCD6, 0x5E9AA4603DA62B67FE3E, 0x5EFD85221486E03B1F33
ASM DT 0x5F5FEFD9E66F9E39D036, 0x5FC2F006D6416892DA21, 0x602685479181A52BE53A, 0x6089A413BB27B337E642, 0x60ECDFADF3625622840B
' 1000 to 1190
ASM DT 0x6150A8A9DCCCE25A6DD2, 0x61B48C8C4908E7C992F1, 0x6218814D71B3905C2D79, 0x627C833438489436885F, 0x62E092B35817B857185C
ASM DT 0x6344B4930C4DD4B1B8CC, 0x63A8F477F047EFB4E593, 0x640DB5D97930ECD47BF2, 0x64729483EB42360FA04A, 0x64D7850D92AA2A2A9406
ASM DT 0x653C82A5FCE83BE81E10, 0x65A18C7D98E74A465AEA, 0x6606A54F5B7C445D00AB, 0x666BD4AC0A858D6DC54F, 0x66D195744AE602C7672D
ASM DT 0x6736E54E6B94EA4EB88E, 0x679CBFE2BBE72C5E92F8, 0x6802AF05A09B56090ECA, 0x6868ADE0F3235C4A4EE1, 0x68CEBC038748AEEB4B43
' 1200 to 1390
ASM DT 0x6934DD1CD04F0705D057, 0x699B8D5041D21153AA40, 0x6A01C42EE04B8BC530F5, 0x6A6893CE7155DAA54770, 0x6ACEF192D837FB62B6B7
ASM DT 0x6B35D5FC75C196023D2E, 0x6B9CCD546D10512ACB52, 0x6C03D54A84F75973DDD6, 0x6C6AEFB46F97A390B4A5, 0x6CD291A2AF057DD2C34C
ASM DT 0x6D39BF3933EF8D89160C, 0x6DA1879337FFD0B279EE, 0x6E08CF7C601B22E40E92, 0x6E70AB4291FE3901D965, 0x6ED89864A3C4960FB922
ASM DT 0x6F40921B8EC8BC9B8A3D, 0x6FA896D906DB2415838E, 0x7010A79ECF945CA9F5DF, 0x7078C85B4B1C7641187C, 0x70E180BD2EAD72A78979
' 1400 to 1590
ASM DT 0x7149B1C75A0B565DB478, 0x71B283D5AD4F33906575, 0x721AD1E4D95302A43B7D, 0x7283B345C4CE3B44A3EA, 0x72ECA4349666B720299D
ASM DT 0x7355A13781009B916412, 0x73BEA99475A2B4B96867, 0x7427BF04A94FE6877F15, 0x7490E64EDCDD34C96817, 0x74FA948AA4E6E82826D8
ASM DT 0x7563CCE7EEF513BFEA75, 0x75CD97119D8898088BA1, 0x7636EDFFFCE45E3E96AE, 0x76A0C838D3AE49A7DE54, 0x770AB3D04FD2A61C9BCA
ASM DT 0x7774AC50EC63C20B0336, 0x77DEB021E07D6A5F3354, 0x7848BFF29DA98E9C73EB, 0x78B2DEEF9FF0A445A1D1, 0x791D89EB6CA469A485B7
' 1600 to 1750
ASM DT 0x7987B5B8D6F69568FA45, 0x79F1FEDF8AADC8253744, 0x7A5CBE2F7D2F3880127F, 0x7AC796F2FF2EED09160A, 0x7B31FEC563AC021AF48D
ASM DT 0x7B9CE48394E32B07F14A, 0x7C07D9C3E73F4D51323E, 0x7C72DC675B2360B24614, 0x7CDDECD5F7108B8EC9A0, 0x7D49870C5C9E34457D1D
ASM DT 0x7DB4A36652CB7084533C, 0x7E1FD1AD8F526551F981, 0x7E8B8EA1937D3BFAA49E, 0x7EF6CDA8CC0E14118F19, 0x7F629D17199C9EC76282
ASM DT 0x7FCDFE2CB8874FBD6EE4
 

'***********************************************************************************************************
'SELECTED POWERS OF TEN
'*********************************************************************************************************
'Tables for use in Power10
ASM align 8
ASM thousands:	
ASM DT 	1.0e1000, 1.0e2000, 1.0e3000, 1.0e4000
ASM hundreds:
ASM DT 	1.0e100, 1.0e200, 1.0e300, 1.0e400, 1.0e500, 1.0e600, 1.0e700, 1.0e800, 1.0e900
ASM tens:
ASM DT 	1.0e10, 1.0e20, 1.0e30, 1.0e40, 1.0e50, 1.0e60, 1.0e70, 1.0e80, 1.0e90
ASM units:
ASM DT 	1.0e1, 1.0e2, 1.0e3, 1.0e4, 1.0e5, 1.0e6, 1.0e7, 1.0e8, 1.0e9

'****************************************************************************************************
'END OF TABLES
'****************************************************************************************************

ASM .code
END PROGRAM